/*
 *      base64.c
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static int base64_decode_char(char c);

static int base64_decode_char(char c)
{
	if ((c >= 'A') && (c <= 'Z')) {
		return ((int) c) - 65;
	} else if ((c >= 'a') && (c <= 'z')) {
		return ((int) c) - 97 + 26;
	} else if ((c >= '0') && (c <= '9')) {
		return ((int) c) - 48 + 26 + 26;
	} else {
		switch (c) {
			case '+':
				return 62;
			case '/':
				return 63;
			case '=':
				return 0;
			default:
				return -1;
		}
	}
}

char *base64_decode(char *inbuf, char *outbuf, size_t outsize)
{
	int i = 0;
	int k = 0;
	int len;
	int code;
	
	if ((inbuf == NULL) || (outbuf == NULL) || (outsize == 0)) return NULL;
	
	len = strlen(inbuf);
	memset(outbuf, 0, outsize);
	
	while (1) {
		while ((i < len) && (inbuf[i] <= 32)) i++;
		
		if (i == len) if (k == 0) return NULL; else break;
		
		code = (base64_decode_char(inbuf[i]) << 18)
			+ (base64_decode_char(inbuf[i + 1]) << 12)
			+ (base64_decode_char(inbuf[i + 2]) << 6)
			+ (base64_decode_char(inbuf[i + 3]));
		
		outbuf[k] = (code >> 16) & 255;
		if ((inbuf[i + 2] == '=') || (outsize == k + 1)) break;
		outbuf[k+1] = (code >> 8) & 255;
		if ((inbuf[i + 3] == '=') || (outsize == k + 2)) break;
		outbuf[k+2] = code & 255;
		if (outsize == k + 3) break;
		
		i += 4; k += 3;
	}
	
	return outbuf;
}

