/*
 *      compression.c
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zlib.h>

int compression_gzip_http_compress(char *buffer, size_t bufsize)
{
	z_stream zs;
	int rc;
	char *outbuffer;
		
	memset((void *)&zs, 0, sizeof(zs));
	zs.zalloc = Z_NULL;
	zs.zfree = Z_NULL;
	zs.opaque = Z_NULL;
	
	zs.next_in = buffer;
	zs.avail_in = bufsize;
		
	if ((outbuffer = calloc(1, bufsize)) == NULL) return -1;
	
	zs.next_out = outbuffer;
	zs.avail_out = bufsize;
		
	deflateInit2(&zs, Z_DEFAULT_COMPRESSION, Z_DEFLATED, 15|16, 8, Z_DEFAULT_STRATEGY);
	
	do {
		rc = deflate(&zs, (zs.avail_in == 0) ? Z_FINISH : Z_STREAM_END);
		if ((rc != Z_OK) && (rc != Z_STREAM_END)) {
			free(outbuffer);
			return -1;
		}
	} while ((rc == Z_OK) && !((rc == Z_STREAM_END) && (zs.avail_out == bufsize) && (zs.avail_in == 0)));
	
	deflateEnd(&zs);
	
	if (zs.total_out <= bufsize) {
		memcpy(buffer, outbuffer, zs.total_out);
		free(outbuffer);
	} else {
		free(outbuffer);
		return -1;
	}
	
	return zs.total_out;
}

int compression_zlib_compress(char *buffer, size_t bufsize, size_t datasize)
{
	char *outbuffer;
	unsigned long comprlen;
	
	if ((outbuffer = calloc(1, bufsize)) == NULL) return bufsize;
	
	comprlen = bufsize;
	
	if (compress(outbuffer, &comprlen, buffer, datasize) == Z_OK) {
		memcpy(buffer, outbuffer, comprlen);
		free(outbuffer);
		return comprlen;
	}
	
	free(outbuffer);
	
	return datasize;
}

int compression_zlib_decompress(char *buffer, size_t bufsize, size_t datasize)
{
	char *outbuffer;
	unsigned long comprlen;
	int bytes;
	
	if ((outbuffer = calloc(1, bufsize)) == NULL) return bufsize;
	
	comprlen = bufsize;
	
	if (uncompress(outbuffer, &comprlen, buffer, datasize) == Z_OK) {
		if (comprlen > bufsize) {
			bytes = bufsize;
		} else {
			bytes = comprlen;
		}
		memcpy(buffer, outbuffer, bytes);
		free(outbuffer);
		return bytes;
	}
	
	free(outbuffer);
	return datasize;
}
