/*
 *      gost.c
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

enum {
	GOST_CYPHER_ENCRYPTION = 0,
	GOST_CYPHER_DECRYPTION
};

static unsigned char extable[8][16] = {
	{0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF},
	{0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF},
	{0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF},
	{0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF},
	{0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF},
	{0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF},
	{0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF},
	{0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF}
};

static unsigned char stdsyncro[8] = {
	0x07, 0x06, 0x05, 0x04, 0x03, 0x02, 0x01, 0x00
};

const unsigned int C1 = 0x1010101;
const unsigned int C2 = 0x1010104;

unsigned int N1 = 0, N2 = 0, CM1 = 0, CM2 = 0, NI1 = 0, NI2 = 0, CMI1 = 0, CMI2 = 0;
unsigned int SG[2] = {0, 0};

void gost_simple_crypt(unsigned int *block, unsigned int *key, char action)
{
	int round;
	int c = 0, n = 7, q = 0;
	unsigned char exbyte = 0, fbyte = 0, sbyte = 0;	
	
	if ((block == NULL) || (key == NULL)) return;
	
	N1 = *(block+0);
	N2 = *(block+1);
	
	for (round=0; round<32; round++) {
		if (action == GOST_CYPHER_ENCRYPTION) {
			if (round == 24) c = 7;
		} else {
			if (round == 8) c = 7; 
		}
		
		CM1 = key[c] + N1;
		
		n = 7;
		for (q=3; q>=0; q--) {
			exbyte = *((unsigned char *)&CM1 + q);
			fbyte = extable[n][(exbyte & 0xF0) >> 4];
			n--;
			sbyte = extable[n][(exbyte & 0x0F)];
			n--;
			exbyte = (fbyte << 4) | sbyte;
			*((unsigned char *)&CM1 + q) = exbyte;
		}
		
		CM1 = (CM1 << 11) | (CM1 >> 21);
		CM2 = N2 ^ CM1;
		
		if (round < 31) {
			N2 = N1;
			N1 = CM2;
		}
		if (action == GOST_CYPHER_ENCRYPTION) {
			if (round < 24) {
				c++;
				if (c > 7) c = 0;
			} else {
				c--;
				if (c < 0) c = 7;
			}
		} else {
			if (round < 8) {
				c++;
				if (c > 7) c = 0;
			} else {
				c--;
				if (c < 0) c = 7;
			}
		}
	}
		
	N2 = CM2;
	
	*(block+0) = N1;
	*(block+1) = N2;
}

void gost_imito_clear(void)
{
	CMI1 = CMI2 = 0;
}

void gost_imito_proceed(unsigned int *block, unsigned int *key)
{
	int round;
	int c = 0, n = 7, q = 0;
	unsigned char exbyte = 0, fbyte = 0, sbyte = 0;	
	
	if ((block == NULL) || (key == NULL)) return;
	
	NI1 = *(block+0);
	NI2 = *(block+1);
	
	CMI1 = CMI1 ^ NI1; 
	CMI2 = CMI2 ^ NI2;
			
	for (round=0; round<16; round++) {
				
		CMI1 = key[c] + NI1;
		
		n = 7;
		for (q=3; q>=0; q--) {
			exbyte = *((unsigned char *)&CMI1 + q);
			fbyte = extable[n][(exbyte & 0xF0) >> 4];
			n--;
			sbyte = extable[n][(exbyte & 0x0F)];
			n--;
			exbyte = (fbyte << 4) | sbyte;
			*((unsigned char *)&CMI1 + q) = exbyte;
		}
		
		CMI1 = (CMI1 << 11) | (CMI1 >> 21);
		CMI2 = NI2 ^ CMI1;
		
		NI2 = NI1;
		NI1 = CMI2;
		
		c++;
		if (c > 7) c = 0;
	}
		
	NI2 = CMI2;	
}

unsigned int gost_imito_get(void)
{
	return CMI1; 
}

void gost_gamma_crypt_init(unsigned int *key, unsigned char *syncro)
{
	if (key == NULL) return;
	
	if (syncro == NULL) {
		memcpy((char *)&SG, &stdsyncro[0], 8);
	} else {
		memcpy((char *)&SG, &syncro[0], 8);
	}
	
	gost_simple_crypt((unsigned int *)&SG, key, GOST_CYPHER_ENCRYPTION);
}

void gost_gamma_crypt(unsigned int *block, unsigned int *key)
{
	unsigned int SB[2] = {0, 0};
	
	if ((block == NULL) || (key == NULL)) return;
			
	SG[0] = (SG[0] + C1);
	SG[1] = (SG[1] + C2 - 1) + 1;
	
	SB[0] = SG[0]; SB[1] = SG[1];
	gost_simple_crypt((unsigned int *)&SB, key, GOST_CYPHER_ENCRYPTION);
	
	*(block+0) = *(block+0) ^ SB[0];
	*(block+1) = *(block+1) ^ SB[1];
}

void gost_gamma_feedback_crypt_init(unsigned char *syncro)
{
	if (syncro == NULL) {
		memcpy((char *)&SG, &stdsyncro[0], 8);
	} else {
		memcpy((char *)&SG, &syncro[0], 8);
	}
}

void gost_gamma_feedback_crypt(unsigned int *block, unsigned int *key, char action)
{
	unsigned int B[2] = {0, 0};
	unsigned int SB[2] = {0, 0};
	
	if ((block == NULL) || (key == NULL)) return;
	
	SB[0] = SG[0]; SB[1] = SG[1];
	gost_simple_crypt((unsigned int *)&SB, key, GOST_CYPHER_ENCRYPTION);
	
	B[0] = *(block+0) ^ SB[0];
	B[1] = *(block+1) ^ SB[1];
	
	if (action == GOST_CYPHER_ENCRYPTION) {
		SG[0] = B[0];
		SG[1] = B[1];
	} else {
		SG[0] = *(block+0);
		SG[1] = *(block+1);
	}
	
	*(block+0) = B[0];
	*(block+1) = B[1];
}
	
void gost_hash_calculate(unsigned char *H, unsigned char *M)
{
	unsigned char C[32][4];
	unsigned char S[32], S2[32], U[32], V[32], W[32], TB[32], K[32][4];
	unsigned int i, i2, round;
	unsigned int *sivec = (unsigned int *)&S, *s2ivec = (unsigned int *)&S2;
	
	if ((H == NULL) || (M == NULL)) return;
	
	memset(C, 0, sizeof(C));
	C[2][ 0] = 0x00; C[2][ 1] = 0xFF; C[2][ 2] = 0x00; C[2][ 3] = 0xFF; C[2][ 4] = 0x00;
	C[2][ 5] = 0xFF; C[2][ 6] = 0x00; C[2][ 7] = 0xFF; C[2][ 8] = 0xFF; C[2][ 9] = 0x00;
	C[2][10] = 0xFF; C[2][11] = 0x00; C[2][12] = 0xFF; C[2][13] = 0x00; C[2][14] = 0xFF;
	C[2][15] = 0x00; C[2][16] = 0x00; C[2][17] = 0xFF; C[2][18] = 0xFF; C[2][19] = 0x00;
	C[2][20] = 0xFF; C[2][21] = 0x00; C[2][22] = 0x00; C[2][23] = 0xFF; C[2][24] = 0xFF;
	C[2][25] = 0x00; C[2][26] = 0x00; C[2][27] = 0x00; C[2][28] = 0xFF; C[2][29] = 0xFF;
	C[2][30] = 0x00; C[2][31] = 0xFF;
	
	memcpy(U, H, sizeof(U));
	memcpy(V, M, sizeof(V));
	
	memset(K, 0, sizeof(K));
	
	for (i = 0; i < 32; i++) {
		W[i] = U[i] ^ V[i];
	}
	
	//P(W, K[0]);
	for(i = 0; i < 4; i++) {
		for(i2 = 0; i2 < 8; i2++) {
			K[i + 4*i2][0] = W[8*i + i2];
		}
	}
	
	for (round = 1; round < 4; round++) {
		//A(U, TB);
		for (i = 0; i < 24; i++) {
			TB[i] = U[i + 8];
		}
		for (i = 0; i < 8; i++) {
			TB[i + 24] = U[i] ^ U[i + 8];
		}
		
		for (i = 0; i < 32; i++) {
			U[i] = TB[i] ^ C[round][i];
		}
		
		//A(V, TB);
		for (i = 0; i < 24; i++) {
			TB[i] = V[i + 8];
		}
		for (i = 0; i < 8; i++) {
			TB[i + 24] = V[i] ^ V[i + 8];
		}
		
		//A(TB, V);
		for (i = 0; i < 24; i++) {
			V[i] = TB[i + 8];
		}
		for (i = 0; i < 8; i++) {
			V[i + 24] = TB[i] ^ TB[i + 8];
		}
		
		for (i = 0; i < 32; i++) {
			W[i] = U[i] ^ V[i];
		}
		
		//P(W, K[round]);
		for(i = 0; i < 4; i++) {
			for(i2 = 0; i2 < 8; i2++) {
				K[i + 4*i2][round] = W[8*i + i2];
			}
		}
	}
	
	memcpy(S, H, sizeof(S));
	
	for (i = 0; i < 32; i += 8) {
		gost_simple_crypt((unsigned int *)&S[i], (unsigned int *)&K[i >> 3], GOST_CYPHER_ENCRYPTION);
	}
		
	//psi(S, 12); 
	memset(S2, 0, sizeof(S2));
	s2ivec[0] = sivec[6];
	s2ivec[1] = sivec[7];
	s2ivec[2] = (sivec[0] << 16) ^ (sivec[0] >> 16) ^ (sivec[0] & 0xffff) ^ (sivec[1] & 0xffff) ^
		(sivec[1] >> 16) ^ (sivec[2] << 16) ^ sivec[6] ^ (sivec[6] << 16) ^ (sivec[7] & 0xffff0000) ^
		(sivec[7] >> 16);
	s2ivec[3] = (sivec[0] & 0xffff) ^ (sivec[0] << 16) ^ (sivec[1] & 0xffff) ^ (sivec[1] << 16) ^
		(sivec[1] >> 16) ^ (sivec[2] << 16) ^ (sivec[2] >> 16) ^ (sivec[3] << 16) ^ sivec[6] ^
		(sivec[6] << 16) ^ (sivec[6] >> 16) ^ (sivec[7] & 0xffff) ^ (sivec[7] << 16) ^ (sivec[7] >> 16);
	s2ivec[4] = (sivec[0] & 0xffff0000) ^ (sivec[0] << 16) ^ (sivec[0] >> 16) ^ (sivec[1] & 0xffff0000) ^ 
		(sivec[1] >> 16) ^ (sivec[2] << 16) ^ (sivec[2] >> 16) ^ (sivec[3] << 16) ^ (sivec[3] >> 16) ^ 
		(sivec[4] << 16) ^ (sivec[6] << 16) ^ (sivec[6] >> 16) ^(sivec[7] & 0xffff) ^ (sivec[7] << 16) ^
		(sivec[7] >> 16);
	s2ivec[5] = (sivec[0] << 16) ^ (sivec[0] >> 16) ^ (sivec[0] & 0xffff0000) ^ (sivec[1] & 0xffff) ^
		sivec[2] ^ (sivec[2] >> 16) ^ (sivec[3] << 16) ^ (sivec[3] >> 16) ^ (sivec[4] << 16) ^
		(sivec[4] >> 16) ^ (sivec[5] << 16) ^ (sivec[6] << 16) ^ (sivec[6] >> 16) ^ (sivec[7] & 0xffff0000) ^
		(sivec[7] << 16) ^ (sivec[7] >> 16);
	s2ivec[6] = sivec[0] ^ (sivec[1] >> 16) ^ (sivec[2] << 16) ^ sivec[3] ^ (sivec[3] >> 16) ^
		(sivec[4] << 16) ^ (sivec[4] >> 16) ^ (sivec[5] << 16) ^ (sivec[5] >> 16) ^ sivec[6] ^
		(sivec[6] << 16) ^ (sivec[6] >> 16) ^ (sivec[7] << 16);
	s2ivec[7] = (sivec[0] & 0xffff0000) ^ (sivec[0] << 16) ^ (sivec[1] & 0xffff) ^ (sivec[1] << 16) ^
		(sivec[2] >> 16) ^ (sivec[3] << 16) ^ sivec[4] ^ (sivec[4] >> 16) ^ (sivec[5] << 16) ^ (sivec[5] >> 16) ^
		(sivec[6] >> 16) ^ (sivec[7] & 0xffff) ^ (sivec[7] << 16) ^ (sivec[7] >> 16);
	memcpy(S, S2, sizeof(S));
	for (i = 0; i < 32; i++) {
		S[i] ^= M[i];
	}
		
	//psi(S, 1 );
	s2ivec[0] = (sivec[1] << 16) ^ (sivec[0] >> 16);
	s2ivec[1] = (sivec[2] << 16) ^ (sivec[1] >> 16);
	s2ivec[2] = (sivec[3] << 16) ^ (sivec[2] >> 16);
	s2ivec[3] = (sivec[4] << 16) ^ (sivec[3] >> 16);
	s2ivec[4] = (sivec[5] << 16) ^ (sivec[4] >> 16);
	s2ivec[5] = (sivec[6] << 16) ^ (sivec[5] >> 16);
	s2ivec[6] = (sivec[7] << 16) ^ (sivec[6] >> 16);
	s2ivec[7] = (sivec[0] & 0xffff0000) ^ (sivec[0] << 16) ^ (sivec[1] & 0xffff0000) ^ 
		(sivec[1] << 16) ^ (sivec[6] << 16) ^ (sivec[7] & 0xffff0000) ^ (sivec[7] >> 16);
	memcpy(S, S2, sizeof(S));
	for (i = 0; i < 32; i++) {
		S[i] ^= H[i];
	}
	
	//psi(S, 61);
	s2ivec[0] = (sivec[0] & 0xffff0000) ^ (sivec[0] << 16) ^ (sivec[0] >> 16) ^ (sivec[1] >> 16) ^ (sivec[1] & 0xffff0000) ^
		(sivec[2] << 16) ^ (sivec[3] >> 16) ^ (sivec[4] << 16) ^ (sivec[5] >> 16) ^ sivec[5] ^ (sivec[6] >> 16) ^ (sivec[7] << 16) ^
		(sivec[7] >> 16) ^ (sivec[7] & 0xffff);
	s2ivec[1] = (sivec[0] << 16) ^ (sivec[0] >> 16) ^ (sivec[0] & 0xffff0000) ^ (sivec[1] & 0xffff) ^ sivec[2] ^ (sivec[2] >> 16) ^
		(sivec[3] << 16) ^ (sivec[4] >> 16) ^ (sivec[5] << 16) ^ (sivec[6] << 16) ^ sivec[6] ^ (sivec[7] & 0xffff0000) ^ (sivec[7] >> 16);
	s2ivec[2] = (sivec[0] & 0xffff) ^ (sivec[0] << 16) ^ (sivec[1] << 16) ^ (sivec[1] >> 16) ^ (sivec[1] & 0xffff0000) ^ (sivec[2] << 16) ^
		(sivec[3] >> 16) ^ sivec[3] ^ (sivec[4] << 16) ^ (sivec[5] >> 16) ^ sivec[6] ^ (sivec[6] >> 16) ^ (sivec[7] & 0xffff) ^ (sivec[7] << 16) ^
		(sivec[7] >> 16);
	s2ivec[3] = (sivec[0] << 16) ^ (sivec[0] >> 16) ^ (sivec[0] & 0xffff0000) ^ (sivec[1] & 0xffff0000) ^ (sivec[1] >> 16) ^
		(sivec[2] << 16) ^ (sivec[2] >> 16) ^ sivec[2] ^ (sivec[3] << 16) ^ (sivec[4] >> 16) ^ sivec[4] ^ (sivec[5] << 16) ^ (sivec[6] << 16) ^
		(sivec[7] & 0xffff) ^ (sivec[7] >> 16);
	s2ivec[4] = (sivec[0] >> 16) ^ (sivec[1] << 16) ^ sivec[1] ^ (sivec[2] >> 16) ^ sivec[2] ^ (sivec[3] << 16) ^ (sivec[3] >> 16) ^ sivec[3] ^
		(sivec[4] << 16) ^ (sivec[5] >> 16) ^ sivec[5] ^ (sivec[6] << 16) ^ (sivec[6] >> 16) ^ (sivec[7] << 16);
	s2ivec[5] = (sivec[0] << 16) ^ (sivec[0] & 0xffff0000) ^ (sivec[1] << 16) ^ (sivec[1] >> 16) ^ (sivec[1] & 0xffff0000) ^
		(sivec[2] << 16) ^ sivec[2] ^ (sivec[3] >> 16) ^ sivec[3] ^ (sivec[4] << 16) ^ (sivec[4] >> 16) ^ sivec[4] ^ (sivec[5] << 16) ^
		(sivec[6] << 16) ^ (sivec[6] >> 16) ^ sivec[6] ^ (sivec[7] << 16) ^ (sivec[7] >> 16) ^ (sivec[7] & 0xffff0000);
	s2ivec[6] = sivec[0] ^ sivec[2] ^ (sivec[2] >> 16) ^ sivec[3] ^ (sivec[3] << 16) ^ sivec[4] ^ (sivec[4] >> 16) ^ (sivec[5] << 16) ^
		(sivec[5] >> 16) ^ sivec[5] ^ (sivec[6] << 16) ^ (sivec[6] >> 16) ^ sivec[6] ^ (sivec[7] << 16) ^ sivec[7];
	s2ivec[7] = sivec[0] ^ (sivec[0] >> 16) ^ (sivec[1] << 16) ^ (sivec[1] >> 16) ^ (sivec[2] << 16) ^ (sivec[3] >> 16) ^ sivec[3] ^
		(sivec[4] << 16) ^ sivec[4] ^ (sivec[5] >> 16) ^ sivec[5] ^ (sivec[6] << 16) ^ (sivec[6] >> 16) ^ (sivec[7] << 16) ^ sivec[7];
	
	memcpy(H, S2, sizeof(S2));
}

void gost_hash_new(char *string, unsigned char *hash)
{
	unsigned int i, tmp, stringlen;
	unsigned int srcbuf[8], hashsum[8], hashbuf[8], hashlen[8];
	unsigned int pos = 0, bytes = 0;
	
	if ((string == NULL) || (hash == NULL)) return;
	
	if ((stringlen = strlen(string)) == 0) return;
	
	memset(hashsum, 0, sizeof(hashsum));
	memset(hashbuf, 0, sizeof(hashbuf));
	
	pos = 0;
	
	while (pos < stringlen) {
		if ((stringlen - pos) >= sizeof(srcbuf)) {
			bytes = sizeof(srcbuf);
		} else {
			bytes = stringlen - pos;
		}
		memset(srcbuf, 0, sizeof(srcbuf));
		memcpy(&srcbuf[0], &string[pos], bytes);
		
		tmp = 0;
		for (i = 0; i < 32; i++) {
			tmp += (*((unsigned char *)&srcbuf + i) + *((unsigned char *)&hashsum + i));
			*((unsigned char *)&hashsum + i) = (tmp & 0xff);
			tmp >>= 8;
		}
		
		gost_hash_calculate((unsigned char *)&hashbuf, (unsigned char *)&srcbuf);
		pos += bytes;
	}
    
	memset(hashlen, 0, sizeof(hashlen));
	tmp = stringlen << 3;
	for (i = 0; i < 32; i++) {
		*((unsigned char *)&hashlen + i) = (tmp & 0xff);
		tmp >>= 8;
	}
	
	gost_hash_calculate((unsigned char *)&hashbuf, (unsigned char *)&hashlen);
	gost_hash_calculate((unsigned char *)&hashbuf, (unsigned char *)&hashsum);
	
	memcpy(hash, hashbuf, 32);
}

char gost_segment_encrypt(char *indata, char *outdata, size_t length, unsigned int *key, unsigned int *imito)
{
	unsigned int pos, bytes;
	unsigned int block[2] = {0, 0};
	
	if ((indata == NULL) || (length == 0) || (outdata == NULL)) return 0;
	
	gost_imito_clear();
	gost_gamma_crypt_init((unsigned int *)key, (unsigned char *)&stdsyncro);
	pos = 0;
	
	while (pos < length) {
		if ((length - pos) >= sizeof(block)) {
			bytes = sizeof(block);
		} else {
			bytes = length - pos;
		}
		memset(block, 0, sizeof(block));
		memcpy(&block[0], &indata[pos], bytes);
		
		gost_imito_proceed((unsigned int *)&block, (unsigned int *)key);
		gost_gamma_crypt((unsigned int *)&block, (unsigned int *)key);
		
		memcpy(&outdata[pos], &block[0], bytes);
		
		pos += bytes;
	}
	
	*imito = gost_imito_get();
	
	return 1;
}

char gost_segment_decrypt(char *indata, char *outdata, size_t length, unsigned int *key, unsigned int *imito)
{
	int i;
	unsigned int pos, bytes;
	unsigned int block[2] = {0, 0};
	
	if ((indata == NULL) || (length == 0) || (outdata == NULL)) return 0;
	
	gost_imito_clear();
	gost_gamma_crypt_init((unsigned int *)key, (unsigned char *)&stdsyncro);
	pos = 0;
	
	while (pos < length) {
		if ((length - pos) >= sizeof(block)) {
			bytes = sizeof(block);
		} else {
			bytes = length - pos;
		}
		memset(block, 0, sizeof(block));
		memcpy(&block[0], &indata[pos], bytes);
		
		gost_gamma_crypt((unsigned int *)&block, (unsigned int *)key);
		if (bytes < sizeof(block)) 
			for (i=bytes; i<sizeof(block); i++) 
				*((unsigned char *)&block + i) = 0;
		gost_imito_proceed((unsigned int *)&block, (unsigned int *)key);
		
		memcpy(&outdata[pos], &block[0], bytes);
		
		pos += bytes;
	}
	
	if (*imito != gost_imito_get()) return 0;
	
	return 1;
}

