/*
 *      iso639.c
 *      
 *      Copyright 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*Reference: 
	ISO 639 Codes
	Source: http://www.loc.gov/standards/iso639-2/php/English_list.php
	        http://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
*/

#include <string.h>
#include <stdlib.h>
#include "iso639.h"

typedef struct {
	const char *family;
	const char *name;
	const char *nativename;
	const char *table1;
	const char *table2t;
	const char *table2b;
	const char *table3;
} iso639_table;


static iso639_table iso639table[] = {
	{"Northwest Caucasian", "Abkhaz",                             "аҧсуа",                                    "ab", "abk", "abk", "abk"},
	{"Afro-Asiatic",        "Afar",                               "Afaraf",                                   "aa", "aar", "aar", "aar"},
	{"Indo-European",       "Afrikaans",                          "Afrikaans",                                "af", "afr", "afr", "afr"},
	{"Niger–Congo",         "Akan",                               "Akan",                                     "ak", "aka", "aka", "aka + 2"},
	{"Indo-European",       "Albanian",                           "Shqip",                                    "sq", "sqi", "alb", "sqi + 4"},
	{"Afro-Asiatic",        "Amharic",                            "አማርኛ",                                     "am", "amh", "amh", "amh"},
	{"Afro-Asiatic",        "Arabic",                             "العربية",                                  "ar", "ara", "ara", "ara + 30"},
	{"Indo-European",       "Aragonese",                          "Aragonés",                                 "an", "arg", "arg", "arg"},
	{"Indo-European",       "Armenian",                           "Հայերեն",                                  "hy", "hye", "arm", "hye"},
	{"Indo-European",       "Assamese",                           "অসমীয়া",                                     "as", "asm", "asm", "asm"},
	{"Northeast Caucasian", "Avaric",                             "авар мацӀ, магӀарул мацӀ",                 "av", "ava", "ava", "ava"},
	{"Indo-European",       "Avestan",                            "avesta",                                   "ae", "ave", "ave", "ave"},
	{"Aymaran",             "Aymara",                             "aymar",                                    "aru", "ay", "aym", "aym + 2"},
	{"Turkic",              "Azerbaijani",                        "azərbaycan dili",                          "az", "aze", "aze", "aze + 2"},
	{"Niger–Congo",         "Bambara",                            "bamanankan",                               "bm", "bam", "bam", "bam"},
	{"Turkic",              "Bashkir",                            "башҡорт теле",                             "ba", "bak", "bak", "bak"},
	{"Language isolate",    "Basque euskara",                     "euskera",                                  "eu", "eus", "baq", "eus"},
	{"Indo-European",       "Belarusian",                         "Беларуская",                               "be", "bel", "bel", "bel"},
	{"Indo-European",       "Bengali",                            "বাংলা",                                      "bn", "ben", "ben", "ben"},
	{"Indo-European",       "Bihari",                             "भोजपुरी",                                     "bh", "bih", "bih", "--"},
	{"Creole",              "Bislama",                            "Bislama",                                  "bi", "bis", "bis", "bis"},
	{"Indo-European",       "Bosnian",                            "bosanski jezik",                           "bs", "bos", "bos", "bos"},
	{"Indo-European",       "Breton",                             "brezhoneg",                                "br", "bre", "bre", "bre"},
	{"Indo-European",       "Bulgarian",                          "български език",                           "bg", "bul", "bul", "bul"},
	{"Sino-Tibetan",        "Burmese",                            "ဗမာစာ",                                      "my", "mya", "bur", "mya"},
	{"Indo-European",       "Catalan; Valencian",                 "Català",                                   "ca", "cat", "cat", "cat"},
	{"Austronesian",        "Chamorro",                           "Chamoru",                                  "ch", "cha", "cha", "cha"},
	{"Northeast Caucasian", "Chechen",                            "нохчийн мотт",                             "ce", "che", "che", "che"},
	{"Niger–Congo",         "Chichewa; Chewa; Nyanja",            "chiCheŵa, chinyanja",                      "ny", "nya", "nya", "nya"},
	{"Sino-Tibetan",        "Chinese",                            "中文 (Zhōngwén), 汉语, 漢語",                 "zh", "zho", "chi", "zho + 13"},
	{"Turkic",              "Chuvash",                            "чӑваш чӗлхи",                              "cv", "chv", "chv", "chv"},
	{"Indo-European",       "Cornish",                            "Kernewek",                                 "kw", "cor", "cor", "cor"},
	{"Indo-European",       "Corsican",                           "corsu, lingua corsa",                      "co", "cos", "cos", "cos"},
	{"Algonquian",          "Cree",                               "ᓀᐦᐃᔭᐍᐏᐣ",                                  "cr", "cre", "cre", "cre + 6"},
	{"Indo-European",       "Croatian",                           "hrvatski",                                 "hr", "hrv", "hrv", "hrv"},
	{"Indo-European",       "Czech",                              "česky, čeština",                           "cs", "ces", "cze", "ces"},
	{"Indo-European",       "Danish",                             "dansk",                                    "da", "dan", "dan", "dan"},
	{"Indo-European",       "Divehi; Dhivehi; Maldivian",         "ދިވެހި",                            "dv", "div", "div", "div"},
	{"Indo-European",       "Dutch",                              "Nederlands, Vlaams",                       "nl", "nld", "dut", "nld"},
	{"Sino-Tibetan",        "Dzongkha",                           "རྫོང་ཁ",                                       "dz", "dzo", "dzo", "dzo"},
	{"Indo-European",       "English",                            "English",                                  "en", "eng", "eng", "eng"},
	{"Constructed",         "Esperanto",                          "Esperanto",                                "eo", "epo", "epo", "epo"},
	{"Uralic",              "Estonian",                           "eesti, eesti keel",                        "et", "est", "est", "est + 2"},
	{"Niger–Congo",         "Ewe",                                "Eʋegbe",                                   "ee", "ewe", "ewe", "ewe"},
	{"Indo-European",       "Faroese",                            "føroyskt",                                 "fo", "fao", "fao", "fao"},
	{"Austronesian",        "Fijian",                             "vosa Vakaviti",                            "fj", "fij", "fij", "fij"},
	{"Uralic",              "Finnish",                            "suomi, suomen kieli",                      "fi", "fin", "fin", "fin"},
	{"Indo-European",       "French",                             "français, langue française",               "fr", "fra", "fre", "fra"},
	{"Niger–Congo",         "Fula; Fulah; Pulaar; Pular",         "Fulfulde, Pulaar, Pular",                  "ff", "ful", "ful", "ful + 9"},
	{"Indo-European",       "Galician",                           "Galego",                                   "gl", "glg", "glg", "glg"},
	{"South Caucasian",     "Georgian",                           "ქართული",                                  "ka", "kat", "geo", "kat"},
	{"Indo-European",       "German",                             "Deutsch",                                  "de", "deu", "ger", "deu"},
	{"Indo-European",       "Greek, Modern",                      "Ελληνικά",                                 "el", "ell", "gre", "ell"},
	{"Tupian",              "Guaraní",                            "Avañe'ẽ",                                  "gn", "grn", "grn", "grn + 5"},
	{"Indo-European",       "Gujarati",                           "ગુજરાતી",                                     "gu", "guj", "guj", "guj"},
	{"Creole",              "Haitian; Haitian Creole",            "Kreyòl ayisyen",                           "ht", "hat", "hat", "hat"},
	{"Afro-Asiatic",        "Hausa",                              "Hausa, هَوُسَ",                               "ha", "hau", "hau", "hau"},
	{"Afro-Asiatic",        "Hebrew (modern)",                    " עברית",                                     "he", "heb", "heb", "heb"},
	{"Niger–Congo",         "Herero",                             "Otjiherero",                               "hz", "her", "her", "her"},
	{"Indo-European",       "Hindi",                              "हिन्दी, हिंदी",                                   "hi", "hin", "hin", "hin"},
	{"Austronesian",        "Hiri Motu",                          "Hiri Motu",                                "ho", "hmo", "hmo", "hmo"},
	{"Uralic",              "Hungarian",                          "Magyar",                                   "hu", "hun", "hun", "hun"},
	{"Constructed",         "Interlingua",                        "Interlingua",                              "ia", "ina", "ina", "ina"},
	{"Austronesian",        "Indonesian",                         "Bahasa Indonesia",                         "id", "ind", "ind", "ind"},
	{"Constructed",         "Interlingue",                        "Occidental, Interlingue",                  "ie", "ile", "ile", "ile"},
	{"Indo-European",       "Irish",                              "Gaeilge",                                  "ga", "gle", "gle", "gle"},
	{"Niger–Congo",         "Igbo",                               "Asụsụ Igbo",                               "ig", "ibo", "ibo", "ibo"},
	{"Eskimo–Aleut",        "Inupiaq",                            "Iñupiaq, Iñupiatun",                       "ik", "ipk", "ipk", "ipk + 2"},
	{"Constructed",         "Ido",                                "Ido",                                      "io", "ido", "ido", "ido"},
	{"Indo-European",       "Icelandic",                          "Íslenska",                                 "is", "isl", "ice", "isl"},
	{"Indo-European",       "Italian",                            "Italiano",                                 "it", "ita", "ita", "ita"},
	{"Eskimo–Aleut",        "Inuktitut",                          "ᐃᓄᒃᑎᑐᑦ",                                    "iu", "iku", "iku", "iku + 2"},
	{"Japonic",             "Japanese",                           "日本語 (にほんご)",                           "ja", "jpn", "jpn", "jpn"},
	{"Austronesian",        "Javanese",                           "basa Jawa",                                "jv", "jav", "jav", "jav"},
	{"Eskimo–Aleut",        "Kalaallisut, Greenlandic",           "kalaallisut, kalaallit oqaasii",           "kl", "kal", "kal", "kal"},
	{"Dravidian",           "Kannada",                            "ಕನ್ನಡ",                                       "kn", "kan", "kan", "kan"},
	{"Nilo-Saharan",        "Kanuri",                             "Kanuri",                                   "kr", "kau", "kau", "kau + 3"},
	{"Indo-European",       "Kashmiri",                           "कश्मीरी, كشميري‎",                              "ks", "kas", "kas", "kas"},
	{"Turkic",              "Kazakh",                             "Қазақ тілі",                               "kk", "kaz", "kaz", "kaz"},
	{"Austro-Asiatic",      "Khmer",                              "ខ្មែរ, ខេមរភាសា, ភាសាខ្មែរ",                      "km", "khm", "khm", "khm"},
	{"Niger–Congo",         "Kikuyu, Gikuyu",                     "Gĩkũyũ",                                   "ki", "kik", "kik", "kik"},
	{"Niger–Congo",         "Kinyarwanda",                        "Ikinyarwanda",                             "rw", "kin", "kin", "kin"},
	{"Turkic",              "Kirghiz, Kyrgyz",                    "кыргыз тили",                              "ky", "kir", "kir", "kir"},
	{"Uralic",              "Komi",                               "коми кыв",                                 "kv", "kom", "kom", "kom + 2"},
	{"Niger–Congo",         "Kongo",                              "KiKongo",                                  "kg", "kon", "kon", "kon + 3"},
	{"Language isolate",    "Korean",                             "한국어 (韓國語), 조선어 (朝鮮語) ",                  "ko", "kor", "kor", "kor"},
	{"Indo-European",       "Kurdish",                            "Kurdî, كوردی‎",                             "ku", "kur", "kur", "kur + 3"},
	{"Niger–Congo",         "Kwanyama, Kuanyama",                 "Kuanyama",                                 "kj", "kua", "kua", "kua"},
	{"Indo-European",       "Latin",                              "latine, lingua latina",                    "la", "lat", "lat", "lat"},
	{"Indo-European",       "Luxembourgish, Letzeburgesch",       "Lëtzebuergesch",                           "lb", "ltz", "ltz", "ltz"},
	{"Niger–Congo",         "Luganda",                            "Luganda",                                  "lg", "lug", "lug", "lug"},
	{"Indo-European",       "Limburgish, Limburgan, Limburger",   "Limburgs",                                 "li", "lim", "lim", "lim"},
	{"Niger–Congo",         "Lingala",                            "Lingála",                                  "ln", "lin", "lin", "lin"},
	{"Tai–Kadai",           "Lao",                                "ພາສາລາວ",                                   "lo", "lao", "lao", "lao"},
	{"Indo-European",       "Lithuanian",                         "lietuvių kalba",                           "lt", "lit", "lit", "lit"},
	{"Niger–Congo",         "Luba-Katanga",                       "--",                                       "lu", "lub", "lub", "lub"},
	{"Indo-European",       "Latvian",                            "latviešu valoda",                          "lv", "lav", "lav", "lav"},
	{"Indo-European",       "Manx",                               "Gaelg, Gailck",                            "gv", "glv", "glv", "glv"},
	{"Indo-European",       "Macedonian",                         "македонски јазик",                         "mk", "mkd", "mac", "mkd"},
	{"Austronesian",        "Malagasy",                           "Malagasy fiteny",                          "mg", "mlg", "mlg", "mlg + 10"},
	{"Austronesian",        "Malay",                              "bahasa Melayu, بهاس ملايو‎",                 "ms", "msa", "may", "msa + 13"},
	{"Dravidian",           "Malayalam",                          "മലയാളം",                                   "ml", "mal", "mal", "mal"},
	{"Afro-Asiatic",        "Maltese",                            "Malti",                                    "mt", "mlt", "mlt", "mlt"},
	{"Austronesian",        "Māori",                              "te reo Māori",                             "mi", "mri", "mao", "mri"},
	{"Indo-European",       "Marathi (Marāṭhī)",                  "मराठी",                                      "mr", "mar", "mar", "mar"},
	{"Austronesian",        "Marshallese",                        "Kajin M̧ajeļ",                              "mh", "mah", "mah", "mah"},
	{"Mongolic",            "Mongolian",                          "монгол",                                   "mn", "mon", "mon", "mon + 2"},
	{"Austronesian",        "Nauru",                              "Ekakairũ Naoero",                          "na", "nau", "nau", "nau"},
	{"Dené–Yeniseian",      "Navajo, Navaho",                     "Diné bizaad, Dinékʼehǰí",                  "nv", "nav", "nav", "nav"},
	{"Indo-European",       "Norwegian Bokmål",                   "Norsk bokmål",                             "nb", "nob", "nob", "nob"},
	{"Niger–Congo",         "North Ndebele",                      "isiNdebele",                               "nd", "nde", "nde", "nde"},
	{"Indo-European",       "Nepali",                             "नेपाली",                                      "ne", "nep", "nep", "nep"},
	{"Niger–Congo",         "Ndonga",                             "Owambo",                                   "ng", "ndo", "ndo", "ndo"},
	{"Indo-European",       "Norwegian Nynorsk",                  "Norsk nynorsk",                            "nn", "nno", "nno", "nno"},
	{"Indo-European",       "Norwegian",                          "Norsk",                                    "no", "nor", "nor", "nor + 2"},
	{"Sino-Tibetan",        "Nuosu",                              "ꆈꌠ꒿ Nuosuhxop",                         "ii", "iii", "iii", "iii"},
	{"Niger–Congo",         "South Ndebele",                      "isiNdebele",                               "nr", "nbl", "nbl", "nbl"},
	{"Indo-European",       "Occitan",                            "Occitan",                                  "oc", "oci", "oci", "oci"},
	{"Algonquian",          "Ojibwe, Ojibwa",                     "ᐊᓂᔑᓈᐯᒧᐎᓐ",                                "oj", "oji", "oji", "oji + 7"},
	{"Indo-European",       "Old Church Slavonic",                "ѩзыкъ словѣньскъ",                        "cu", "chu", "chu", "chu"},
	{"Afro-Asiatic",        "Oromo",                              "Afaan Oromoo",                             "om", "orm", "orm", "orm + 4"},
	{"Indo-European",       "Oriya",                              "ଓଡ଼ିଆ ",                                     "or", "ori", "ori", "ori"},
	{"Indo-European",       "Ossetian, Ossetic",                  "ирон æвзаг",                               "os", "oss", "oss", "oss"},
	{"Indo-European",       "Panjabi, Punjabi",                   "ਪੰਜਾਬੀ, پنجابی‎",                              "pa", "pan", "pan", "pan"},
	{"Indo-European",       "Pāli",                               "पाऴि",                                       "pi", "pli", "pli", "pli"},
	{"Indo-European",       "Persian",                            "فارسی",                                    "fa", "fas", "per", "fas + 2"},
	{"Indo-European",       "Polish",                             "polski",                                   "pl", "pol", "pol", "pol"},
	{"Indo-European",       "Pashto, Pushto",                     "پښتو ",                                   "ps", "pus", "pus", "pus + 3"},
	{"Indo-European",       "Portuguese",                         "Português",                                "pt", "por", "por", "por"},
	{"Quechuan",            "Quechua",                            "Runa Simi, Kichwa",                        "qu", "que", "que", "que + 44"},
	{"Indo-European",       "Romansh",                            "rumantsch grischun",                       "rm", "roh", "roh", "roh"},
	{"Niger–Congo",         "Kirundi",                            "Ikirundi",                                 "rn", "run", "run", "run"},
	{"Indo-European",       "Romanian, Moldavian, Moldovan",      "română",                                   "ro", "ron", "rum", "ron"},
	{"Indo-European",       "Russian",                            "русский язык",                             "ru", "rus", "rus", "rus"},
	{"Indo-European",       "Sanskrit (Saṁskṛta)",                "संस्कृतम्",                                    "sa", "san", "san", "san"},
	{"Indo-European",       "Sardinian",                          "sardu",                                    "sc", "srd", "srd", "srd + 4"},
	{"Indo-European",       "Sindhi",                             "सिन्धी, سنڌي، سندھی",                          "sd", "snd", "snd", "snd"},
	{"Uralic",              "Northern Sami",                      "Davvisámegiella",                          "se", "sme", "sme", "sme"},
	{"Austronesian",        "Samoan",                             "gagana fa\'a Samoa",                       "sm", "smo", "smo", "smo"},
	{"Creole",              "Sango",                              "yângâ tî sängö",                           "sg", "sag", "sag", "sag"},
	{"Indo-European",       "Serbian",                            "српски језик",                             "sr", "srp", "srp", "srp"},
	{"Indo-European",       "Scottish Gaelic; Gaelic",            "Gàidhlig",                                 "gd", "gla", "gla", "gla"},
	{"Niger–Congo",         "Shona",                              "chiShona",                                 "sn", "sna", "sna", "sna"},
	{"Indo-European",       "Sinhala, Sinhalese",                 "සිංහල",                                     "si", "sin", "sin", "sin"},
	{"Indo-European",       "Slovak",                             "slovenčina",                               "sk", "slk", "slo", "slk"},
	{"Indo-European",       "Slovene",                            "slovenščina",                              "sl", "slv", "slv", "slv"},
	{"Afro-Asiatic",        "Somali",                             "Soomaaliga, af Soomaali",                  "so", "som", "som", "som"},
	{"Niger–Congo",         "Southern Sotho",                     "Sesotho",                                  "st", "sot", "sot", "sot"},
	{"Indo-European",       "Spanish; Castilian",                 "español, castellano",                      "es", "spa", "spa", "spa"},
	{"Austronesian",        "Sundanese",                          "Basa Sunda",                               "su", "sun", "sun", "sun"},
	{"Niger–Congo",         "Swahili",                            "Kiswahili",                                "sw", "swa", "swa", "swa + 2"},
	{"Niger–Congo",         "Swati",                              "SiSwati",                                  "ss", "ssw", "ssw", "ssw"},
	{"Indo-European",       "Swedish",                            "svenska",                                  "sv", "swe", "swe", "swe"},
	{"Dravidian",           "Tamil",                              "தமிழ்",                                      "ta", "tam", "tam", "tam"},
	{"Dravidian",           "Telugu",                             "తెలుగు",                                     "te", "tel", "tel", "tel"},
	{"Indo-European",       "Tajik",                              "тоҷикӣ, toğikī, تاجیکی‎",                   "tg", "tgk", "tgk", "tgk"},
	{"Tai–Kadai",           "Thai",                               "ไทย",                                       "th", "tha", "tha", "tha"},
	{"Afro-Asiatic",        "Tigrinya",                           "ትግርኛ",                                      "ti", "tir", "tir", "tir"},
	{"Sino-Tibetan",        "Tibetan Standard, Tibetan, Central", "བོད་ཡིག",                                      "bo", "bod", "tib", "bod"},
	{"Turkic",              "Turkmen",                            "Türkmen, Түркмен",                         "tk", "tuk", "tuk", "tuk"},
	{"Austronesian",        "Tagalog",                            "Wikang Tagalog, ᜏᜒᜃᜅ᜔ ᜆᜄᜎᜓᜄ᜔","tl", "tgl", "tgl", "tgl"},
	{"Niger–Congo",         "Tswana",                             "Setswana",                                 "tn", "tsn", "tsn", "tsn"},
	{"Austronesian",        "Tonga (Tonga Islands)",              "faka Tonga",                               "to", "ton", "ton", "ton"},
	{"Turkic",              "Turkish",                            "Türkçe",                                   "tr", "tur", "tur", "tur"},
	{"Niger–Congo",         "Tsonga",                             "Xitsonga",                                 "ts", "tso", "tso", "tso"},
	{"Turkic",              "Tatar",                              "татарча, tatarça, تاتارچا",                "tt", "tat", "tat", "tat"},
	{"Niger–Congo",         "Twi",                                "Twi",                                      "tw", "twi", "twi", "twi"},
	{"Austronesian",        "Tahitian",                           "Reo Tahiti",                               "ty", "tah", "tah", "tah"},
	{"Turkic",              "Uighur, Uyghur",                     "Uyƣurqə, ئۇيغۇرچە‎",                         "ug", "uig", "uig", "uig"},
	{"Indo-European",       "Ukrainian",                          "українська",                               "uk", "ukr", "ukr", "ukr"},
	{"Indo-European",       "Urdu",                               "اردو",                                     "ur", "urd", "urd", "urd"},
	{"Turkic",              "Uzbek",                              "O'zbek, Ўзбек, أۇزبېك",                    "uz", "uzb", "uzb", "uzb + 2"},
	{"Niger–Congo",         "Venda",                              "Tshivenḓa",                                "ve", "ven", "ven", "ven"},
	{"Austro-Asiatic",      "Vietnamese",                         "Tiếng Việt",                               "vi", "vie", "vie", "vie"},
	{"Constructed",         "Volapük",                            "Volapük",                                  "vo", "vol", "vol", "vol"},
	{"Indo-European",       "Walloon",                            "Walon",                                    "wa", "wln", "wln", "wln"},
	{"Indo-European",       "Welsh",                              "Cymraeg",                                  "cy", "cym", "wel", "cym"},
	{"Niger–Congo",         "Wolof",                              "Wollof",                                   "wo", "wol", "wol", "wol"},
	{"Indo-European",       "Western Frisian",                    "Frysk",                                    "fy", "fry", "fry", "fry"},
	{"Niger–Congo",         "Xhosa",                              "isiXhosa",                                 "xh", "xho", "xho", "xho"},
	{"Indo-European",       "Yiddish",                            "ייִדיש",                                       "yi", "yid", "yid", "yid + 2"},
	{"Niger–Congo",         "Yoruba",                             "Yorùbá",                                   "yo", "yor", "yor", "yor"},
	{"Tai–Kadai",           "Zhuang, Chuang",                     "Saɯ cueŋƅ, Saw cuengh",                    "za", "zha", "zha", "zha + 16"},
	{"Niger–Congo",         "Zulu",                               "isiZulu",                                  "zu", "zul", "zul", "zul"},
	{NULL,                  NULL,                                 NULL,                                       NULL,  NULL,  NULL,  NULL}
}; 	

int iso639_get_lang_by_locale(char *locale, iso639lang_t *language)
{
	char *end;
	int numsym, i;
	char code[3];
	
	if ((locale == NULL) || (language == NULL)) return -1;
	
	if ((end = strchr(locale, '_')) == NULL) return -1;
	if ((numsym = end - locale) != 2) return -1;
	
	strncpy(code, locale, numsym);
	code[numsym] = '\0';
	
	i = 0;
	while (iso639table[i].table1 != NULL) {
		if (strncasecmp(iso639table[i].table1, (char *)&code, 2) == 0) {
			language->family = iso639table[i].family;
			language->name = iso639table[i].name;
			language->nativename = iso639table[i].nativename;
			return 1;
		}
		i++;
	}
	
	return 0;
}
