/*
 *      log.h
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LOG_H__
#define __LOG_H__

#include <sys/time.h>
#include <stdarg.h>

typedef int (*log_read_func)(int flags, time_t ts, char *entry);
typedef int (*log_add_func)(int flags, time_t ts, char *entry);

struct log_t {
	int filefd;
	char *filename;
	int silent;
	unsigned int readlines;
	time_t readtime;
	log_read_func readfunc;
	log_add_func addfunc;
};

enum {
	LOG_EVENT_MESSAGE = 0x00000001,
	LOG_EVENT_WARNING = 0x00000002,
	LOG_EVENT_ERROR = 0x00000004,
	LOG_EVENT_PRIVELEGED = 0x00000008,
	LOG_EVENT_UNDEFINED = 0x00000010,
	LOG_OUTPUT_STDOUT = 0x00000020,
	LOG_OUTPUT_STDERR = 0x00000040,
	LOG_OUTPUT_SILENT = 0x00000080
};

int log_add_message_static(struct log_t *log, int flags, char *message);
int log_add_message(struct log_t *log, int event, char *format, ...);
int log_read_messages(struct log_t *log);
struct log_t *log_init_instance(char *program, char *filename, int readonly, int silent);
void log_set_callbacks(struct log_t *log, log_read_func readfunc, log_add_func addfunc);
void log_free_instance(struct log_t *log);

#endif /* __LOG_H__ */
