/*
 *      odf.c
 *      
 *      Copyright 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <stdarg.h>
#include <pwd.h>

#include "zip.h"
#include "odf.h"

static char odf_mimetype_text_template[] =
	"application/vnd.oasis.opendocument.text";

static char odf_manifest_xml_template[] = 
	"<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
	"<manifest:manifest xmlns:manifest=\"urn:oasis:names:tc:opendocument:xmlns:manifest:1.0\">"
	"	<manifest:file-entry manifest:media-type=\"application/vnd.oasis.opendocument.text\" manifest:full-path=\"/\"/>"
	"	<manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"content.xml\"/>"
	"	<manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"meta.xml\"/>"
	"	<manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"settings.xml\"/>"
	"	<manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"styles.xml\"/>"
	"</manifest:manifest>";

static char odf_meta_xml_template[] = 
	"<?xml version=\"1.0\" ?>"
	"<office:document-meta office:version=\"1.0\""
	" xmlns:dc=\"http://purl.org/dc/elements/1.1/\""
	" xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\""
	" xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\""
	" xmlns:ooo=\"http://openoffice.org/2004/office\""
	" xmlns:xlink=\"http://www.w3.org/1999/xlink\">"
	"	<office:meta>\n"
	"		<meta:initial-creator>%s</meta:initial-creator>\n"
	"		<meta:creation-date>%s</meta:creation-date>\n"
	"		<dc:date>%s</dc:date>\n"
	"		<dc:creator>%s</dc:creator>\n"
	"		<meta:editing-duration>%s</meta:editing-duration>\n"
	"		<meta:editing-cycles>1</meta:editing-cycles>\n"
	"		<meta:generator>%s</meta:generator>\n"
	"	</office:meta>\n"
	"</office:document-meta>";
	
static char odf_settings_xml_template[] = 
	"<?xml version=\"1.0\" ?>"
	"<office:document-settings office:version=\"1.0\""
	" xmlns:config=\"urn:oasis:names:tc:opendocument:xmlns:config:1.0\""
	" xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\""
	" xmlns:ooo=\"http://openoffice.org/2004/office\""
	" xmlns:xlink=\"http://www.w3.org/1999/xlink\" />";

static char odf_styles_xml_template[] = 
	"<?xml version=\"1.0\" ?>"
	"<office:document-styles office:version=\"1.0\""
	" xmlns:chart=\"urn:oasis:names:tc:opendocument:xmlns:chart:1.0\""
	" xmlns:dc=\"http://purl.org/dc/elements/1.1/\""
	" xmlns:dom=\"http://www.w3.org/2001/xml-events\""
	" xmlns:dr3d=\"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0\""
	" xmlns:draw=\"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0\""
	" xmlns:fo=\"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0\""
	" xmlns:form=\"urn:oasis:names:tc:opendocument:xmlns:form:1.0\""
	" xmlns:math=\"http://www.w3.org/1998/Math/MathML\""
	" xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\""
	" xmlns:number=\"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0\""
	" xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\""
	" xmlns:ooo=\"http://openoffice.org/2004/office\""
	" xmlns:oooc=\"http://openoffice.org/2004/calc\""
	" xmlns:ooow=\"http://openoffice.org/2004/writer\""
	" xmlns:script=\"urn:oasis:names:tc:opendocument:xmlns:script:1.0\""
	" xmlns:style=\"urn:oasis:names:tc:opendocument:xmlns:style:1.0\""
	" xmlns:svg=\"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0\""
	" xmlns:table=\"urn:oasis:names:tc:opendocument:xmlns:table:1.0\""
	" xmlns:text=\"urn:oasis:names:tc:opendocument:xmlns:text:1.0\""
	" xmlns:xlink=\"http://www.w3.org/1999/xlink\" />";

static char odf_content_xml_header_template[] = 
	"<?xml version=\"1.0\" ?>"
	"<office:document-content office:version=\"1.0\""
	" xmlns:chart=\"urn:oasis:names:tc:opendocument:xmlns:chart:1.0\""
	" xmlns:dc=\"http://purl.org/dc/elements/1.1/\""
	" xmlns:dom=\"http://www.w3.org/2001/xml-events\""
	" xmlns:dr3d=\"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0\""
	" xmlns:draw=\"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0\""
	" xmlns:fo=\"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0\""
	" xmlns:form=\"urn:oasis:names:tc:opendocument:xmlns:form:1.0\""
	" xmlns:math=\"http://www.w3.org/1998/Math/MathML\""
	" xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\""
	" xmlns:number=\"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0\""
	" xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\""
	" xmlns:ooo=\"http://openoffice.org/2004/office\""
	" xmlns:oooc=\"http://openoffice.org/2004/calc\""
	" xmlns:ooow=\"http://openoffice.org/2004/writer\""
	" xmlns:script=\"urn:oasis:names:tc:opendocument:xmlns:script:1.0\""
	" xmlns:style=\"urn:oasis:names:tc:opendocument:xmlns:style:1.0\""
	" xmlns:svg=\"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0\""
	" xmlns:table=\"urn:oasis:names:tc:opendocument:xmlns:table:1.0\""
	" xmlns:text=\"urn:oasis:names:tc:opendocument:xmlns:text:1.0\""
	" xmlns:xforms=\"http://www.w3.org/2002/xforms\""
	" xmlns:xlink=\"http://www.w3.org/1999/xlink\""
	" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\""
	" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"
	"	<office:automatic-styles>\n"
	"		<style:style style:name=\"table\" style:family=\"table\">\n"
	"			<style:table-properties style:width=\"17cm\" table:align=\"margins\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"table_column\" style:family=\"table-column\">\n"
	"			<style:table-column-properties style:rel-column-width=\"13107*\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"table_first_top_cell\" style:family=\"table-cell\">\n"
	"			<style:table-cell-properties fo:padding=\"0.097cm\" fo:border-left=\"0.05pt solid #000000\" fo:border-right=\"none\" fo:border-top=\"0.05pt solid #000000\" fo:border-bottom=\"0.05pt solid #000000\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"table_last_top_cell\" style:family=\"table-cell\">\n"
	"			<style:table-cell-properties fo:padding=\"0.097cm\" fo:border-left=\"0.05pt solid #000000\" fo:border-right=\"0.05pt solid #000000\" fo:border-top=\"0.05pt solid #000000\" fo:border-bottom=\"0.05pt solid #000000\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"table_first_bottom_cell\" style:family=\"table-cell\">\n"
	"			<style:table-cell-properties fo:padding=\"0.097cm\" fo:border-left=\"0.05pt solid #000000\" fo:border-right=\"none\" fo:border-top=\"none\" fo:border-bottom=\"0.05pt solid #000000\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"table_last_bottom_cell\" style:family=\"table-cell\">\n"
	"			<style:table-cell-properties fo:padding=\"0.097cm\" fo:border-left=\"0.05pt solid #000000\" fo:border-right=\"0.05pt solid #000000\" fo:border-top=\"none\" fo:border-bottom=\"0.05pt solid #000000\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"P1\" style:family=\"paragraph\" style:parent-style-name=\"Standard\">\n"
	"			<style:paragraph-properties fo:text-align=\"center\" style:justify-single-word=\"false\"/>\n"
	"			<style:text-properties fo:font-weight=\"bold\" fo:font-size=\"14pt\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"P2\" style:family=\"paragraph\" style:parent-style-name=\"Standard\">\n"
	"			<style:paragraph-properties fo:text-align=\"center\" style:justify-single-word=\"false\"/>\n"
	"			<style:text-properties fo:font-weight=\"bold\" fo:font-size=\"10pt\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"P3\" style:family=\"paragraph\" style:parent-style-name=\"Standard\">\n"
	"			<style:paragraph-properties style:justify-single-word=\"false\"/>\n"
	"			<style:text-properties fo:font-size=\"10pt\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"P4\" style:family=\"paragraph\" style:parent-style-name=\"Standard\">\n"
	"			<style:paragraph-properties style:justify-single-word=\"false\"/>\n"
	"			<style:text-properties fo:font-weight=\"bold\" style:font-weight-asian=\"bold\" style:font-weight-complex=\"bold\" fo:font-size=\"10pt\"/>\n"
	"		</style:style>\n"
	"		<style:style style:name=\"T1\" style:family=\"text\" style:parent-style-name=\"Standard\">\n"
	"			<style:text-properties fo:font-weight=\"bold\" style:font-weight-asian=\"bold\" style:font-weight-complex=\"bold\" fo:font-size=\"10pt\"/>\n"
	"		</style:style>\n"
	"	</office:automatic-styles>\n"
	"	<office:body>\n"
	"		<office:text>\n";

static char odf_content_xml_footer_template[] = 
	"		</office:text>\n"
	"	</office:body>\n"
	"</office:document-content>";

static char odf_content_xml_heading_template[] = 
	"			<text:h text:outline-level=\"%u\">%s</text:h>\n";

static char odf_content_xml_paragraph_template[] = 
	"			<text:p>%s</text:p>\n";

static char odf_content_xml_link_template[] =
	"			<text:p>\n"
	"				<text:a xlink:type=\"simple\" xlink:href=\"%s\">%s</text:a>\n"
	"			</text:p>\n";

static char odf_content_xml_list_header_template[] = 
	"			<text:list>\n";

static char odf_content_xml_list_entry_template[] = 
	"				<text:list-item>\n"
	"					<text:p>%s</text:p>\n"
	"				</text:list-item>\n";

static char odf_content_xml_list_footer_template[] = 
	"			</text:list>\n";

static char odf_content_xml_table_header_template[] = 
	"			<text:p text:style-name=\"P1\">%s</text:p>\n"
	"			<table:table table:name=\"Table1\" table:style-name=\"table\">\n"
	"				<table:table-column table:style-name=\"table_column\" table:number-columns-repeated=\"%u\"/>\n";

static char odf_content_xml_table_row_start_template[] = 
	"				<table:table-row>\n";

static char odf_content_xml_paragraph_with_caption_template[] =
	"			<text:p text:style-name=\"P3\">%s <text:span text:style-name=\"T1\">%s</text:span></text:p>\n"
	"			<text:p text:style-name=\"P3\">%s</text:p>\n";

static char odf_content_xml_table_row_end_template[] =
	"				</table:table-row>\n";

static char odf_content_xml_table_footer_template[] = 
	"			</table:table>\n";

static char odf_content_xml_table_cell_template[] =
	"					<table:table-cell table:style-name=\"%s\" office:value-type=\"string\">\n"
	"						<text:p text:style-name=\"%s\">%s</text:p>\n"
	"					</table:table-cell>\n";

static char *odf_string_concat(char *srcstring, char *newstring);
static char *odf_string_format(const char *format, ...);
static char *odf_string_concat_format(char *origstr, const char *format, ...);


static char *odf_string_concat(char *srcstring, char *newstring)
{
	char *concstring;
	size_t srclen, newlen;
	
	if ((srcstring == NULL) || (newstring == NULL)) return NULL;
	
	srclen = strlen(srcstring);
	newlen = strlen(newstring);
	
	if ((srclen == 0) && (newlen == 0)) return NULL;
	
	if (srclen == 0) return newstring;
	if (newlen == 0) return srcstring;
	
	concstring = realloc(srcstring, srclen+newlen+1);
	
	if (concstring == NULL) return NULL;
	
	memcpy(concstring+srclen, newstring, newlen);
	
	concstring[srclen+newlen] = '\0';
	
	return concstring;
}

static char *odf_string_format(const char *format, ...)
{
	int symbols, length;
	char *srcstr, *newstr;
	va_list arg;
	
	length = 100;
	
	srcstr = calloc(length, 1);
	
	if (srcstr == NULL) return NULL;
	
	while (1) {
		va_start(arg, format);
		symbols = vsnprintf(srcstr, length, format, arg);
		va_end(arg);
		
		if ((symbols > -1) && (symbols < length)) {
			return srcstr;
		}
		
		if (symbols > -1) {
			length = symbols + 1;
		} else {
			length *= 2;
		}
		
		newstr = realloc(srcstr, length);
		
		if (newstr == NULL) {
			free(srcstr);
			return NULL;
		} else {
			srcstr = newstr;
		}
	}
    
}

static char *odf_string_concat_format(char *origstr, const char *format, ...)
{
	int symbols, length;
	size_t origlen;
	char *srcstr, *newstr, *concstring;
	va_list arg;
	
	length = 100;
	
	srcstr = calloc(length, 1);
	
	if (srcstr == NULL) return NULL;
	
	while (1) {
		va_start(arg, format);
		symbols = vsnprintf(srcstr, length, format, arg);
		va_end(arg);
		
		if ((symbols > -1) && (symbols < length)) {
			break;
		}
		
		if (symbols > -1) {
			length = symbols + 1;
		} else {
			length *= 2;
		}
		
		newstr = realloc(srcstr, length);
		
		if (newstr == NULL) {
			free(srcstr);
			return NULL;
		} else {
			srcstr = newstr;
		}
	}
	
	if (origstr != NULL) {
		origlen = strlen(origstr);
		if (origlen > 0) {
			concstring = realloc(origstr, origlen+symbols+1);
			if (concstring != NULL) {
				memcpy(concstring+origlen, srcstr, symbols);
				concstring[origlen+symbols] = '\0';
				free(srcstr);
				return concstring;
			} else {
				free(srcstr);
				return NULL;
			}
		} else {
			return srcstr;
		}
	} else {
		return srcstr;
	}
}

void odf_add_heading(odf_t instance, int outlinelevel, char *text)
{
	char *newcontent;
	
	if ((instance == NULL) || (text == NULL)) return;
	
	if ((outlinelevel < 1) || (outlinelevel > 2)) outlinelevel = 1;
	
	newcontent = odf_string_concat_format(instance->content, odf_content_xml_heading_template, outlinelevel, text);
	
	if (newcontent != NULL) {
		instance->content = newcontent;
	}
}

void odf_add_paragraph(odf_t instance, char *text)
{
	char *newcontent;
	
	if ((instance == NULL) || (text == NULL)) return;
	
	newcontent = odf_string_concat_format(instance->content, odf_content_xml_paragraph_template, text);
	
	if (newcontent != NULL) {
		instance->content = newcontent;
	}
}

void odf_add_link(odf_t instance, char *link, char *text)
{
	char *newcontent;
	
	if ((instance == NULL) || (link == NULL) || (text == NULL)) return;
	
	newcontent = odf_string_concat_format(instance->content, odf_content_xml_link_template, link, text);
	
	if (newcontent != NULL) {
		instance->content = newcontent;
	}
}

void odf_add_list(odf_t instance, char *entry, ...)
{
	va_list arg;
	char *str, *liststr, *newcontent;
	
	if ((instance == NULL) || (entry == NULL)) return;
	
	liststr = strdup(odf_content_xml_list_header_template);
	
	if (liststr == NULL) return;
	
	va_start(arg, entry);
	
	newcontent = odf_string_concat_format(liststr, odf_content_xml_list_entry_template, entry);
	if (newcontent != NULL) {
		liststr = newcontent;
	}
	
	do {
		str = va_arg(arg, char *);
		if (str != NULL) {
			newcontent = odf_string_concat_format(liststr, odf_content_xml_list_entry_template, str);
			if (newcontent != NULL) {
				liststr = newcontent;
			}
		}
	} while (str != NULL);
	
	va_end(arg);
	
	newcontent = odf_string_concat(liststr, odf_content_xml_list_footer_template);
	
	if (newcontent != NULL) {
		liststr = newcontent;
		newcontent = odf_string_concat(instance->content, liststr);
		if (newcontent != NULL) {
			instance->content = newcontent;
		}
	}
	free(liststr);
}

void odf_add_table_header(odf_t instance, unsigned int columns, char *header, ...)
{
	unsigned int col, paramexists;
	va_list arg;
	char *colname, *cellstyle, *textstyle, *cstring;
	
	if ((instance == NULL) || (header == NULL) || (columns < 2)) return;
	
	instance->table = odf_string_format(odf_content_xml_table_header_template, header, columns, NULL);
	instance->tablecols = columns;
	
	cstring = odf_string_concat(instance->table, odf_content_xml_table_row_start_template);
	if (cstring != NULL) {
		instance->table = cstring;
	}
	
	va_start(arg, header);
	
	textstyle = "P2";
	
	paramexists = 1;
	
	for (col = 0; col<columns; col++) {
		if (paramexists) {
			colname = va_arg(arg, char *);
			if (colname == NULL) {
				paramexists = 0;
				colname = "";
			}
		} else {
			colname = "";
		}
	
		if (col == columns-1) {
			cellstyle = "table_last_top_cell";
		} else {
			cellstyle = "table_first_top_cell";
		}
		
		cstring = odf_string_concat_format(instance->table, odf_content_xml_table_cell_template, cellstyle, textstyle, colname);
		if (cstring != NULL) {
			instance->table = cstring;
		}
	}
	
	va_end(arg);
	
	cstring = odf_string_concat(instance->table, odf_content_xml_table_row_end_template);
	if (cstring != NULL) {
		instance->table = cstring;
	}
}

void odf_add_table_row(odf_t instance, char *text, ...)
{
	unsigned int col, paramexists;
	va_list arg;
	char *colname, *cellstyle, *textstyle, *cstring;
	
	if ((instance == NULL) || (text == NULL)) return;
	
	if (instance->table == NULL) return;
	
	cstring = odf_string_concat(instance->table, odf_content_xml_table_row_start_template);
	if (cstring != NULL) {
		instance->table = cstring;
	}
	
	textstyle = "P3";
	
	cstring = odf_string_concat_format(instance->table, odf_content_xml_table_cell_template, "table_first_bottom_cell", textstyle, text);
	if (cstring != NULL) {
		instance->table = cstring;
	}
	
	va_start(arg, text);
	
	paramexists = 1;
	
	for (col = 1; col<instance->tablecols; col++) {
		if (paramexists) {
			colname = va_arg(arg, char *);
			if (colname == NULL) {
				paramexists = 0;
				colname = "";
			}
		} else {
			colname = "";
		}
	
		if (col == instance->tablecols-1) {
			cellstyle = "table_last_bottom_cell";
		} else {
			cellstyle = "table_first_bottom_cell";
		}
		
		cstring = odf_string_concat_format(instance->table, odf_content_xml_table_cell_template, cellstyle, textstyle, colname);
		if (cstring != NULL) {
			instance->table = cstring;
		}
	}
	
	va_end(arg);
	
	cstring = odf_string_concat(instance->table, odf_content_xml_table_row_end_template);
	if (cstring != NULL) {
		instance->table = cstring;
	}
}

void odf_add_table_footer(odf_t instance)
{
	char *cstring;
	
	if (instance == NULL) return;
	
	if (instance->table == NULL) return;
	
	cstring = odf_string_concat(instance->table, odf_content_xml_table_footer_template);
	
	if (cstring != NULL) {
		instance->table = cstring;
		
		cstring = odf_string_concat(instance->content, instance->table);
		if (cstring != NULL) {
			instance->content = cstring;
		}
	}
	
	free(instance->table);
	instance->table = NULL;
	instance->tablecols = 0;
}

void odf_add_paragraph_with_caption(odf_t instance, char *prefix, char *caption, char *text)
{
	char *newcontent;
	
	if ((instance == NULL) || (prefix == NULL) || (caption == NULL) || (text == NULL)) return;
	
	newcontent = odf_string_concat_format(instance->content, odf_content_xml_paragraph_with_caption_template, prefix, caption, text);
	
	if (newcontent != NULL) {
		instance->content = newcontent;
	}
}

odf_t odf_init(char *filename, char *application)
{
	odf_t instance;
	
	if (filename == NULL) return NULL;
	
	instance = (odf_t)calloc(sizeof(struct _odf), 1);
	
	if (instance == NULL) return NULL;
	
	instance->filename = strdup(filename);
	instance->createtime = time(NULL);
	instance->content = strdup(odf_content_xml_header_template);
	instance->table = NULL;
	instance->tablecols = 0;
	
	if (application == NULL) {
		instance->application = strdup("Unknown");
	} else {
		instance->application = strdup(application);
	}
	
	return instance;
}

void odf_close(odf_t instance)
{
	zipwriter_t zipwriter;
	char *meta_xml;
	char *content_xml;
	char timebuf[64];
	struct tm *btime;
	time_t presenttime;
	char edtimebuf[32];
	char *username;
	struct passwd *passwd;
	
	if (instance == NULL) return;
	
	memset(timebuf, 0, sizeof(timebuf));
	btime = localtime(&instance->createtime);
	if (btime != NULL) {
		if (strftime(timebuf, sizeof(timebuf), "%Y-%m-%dT%H:%M:%S", btime) == 0) {
			sprintf(timebuf, "2012-01-01T00:00:00");
		}
	} else {
		sprintf(timebuf, "2012-01-01T00:00:00");
	}
	
	memset(edtimebuf, 0, sizeof(edtimebuf));
	presenttime = time(NULL);
	sprintf(edtimebuf, "PT%uS", presenttime-instance->createtime);
	
	username = "Unknown";
	passwd = getpwuid(getuid());
	if (passwd != NULL) {
		if (passwd->pw_gecos != NULL) {
			username = passwd->pw_gecos;
		} else if (passwd->pw_name != NULL) {
			username = passwd->pw_name;
		}
	}
		
	meta_xml = odf_string_format(odf_meta_xml_template, username, timebuf, timebuf, username, edtimebuf, instance->application, NULL);
	
	if (meta_xml == NULL) return;
	
	content_xml = odf_string_concat(instance->content, odf_content_xml_footer_template);
	
	if (content_xml != NULL) {
		instance->content = content_xml;
	} else {
		free(meta_xml);
		return;
	}
	
	zipwriter = zipwriter_init(instance->filename);
	
	if (zipwriter != NULL) {
		zipwriter_set_method(zipwriter, ZIPWRITER_METHOD_STORE);
		zipwriter_add_data_file(zipwriter, "mimetype", odf_mimetype_text_template, strlen(odf_mimetype_text_template));
		zipwriter_set_method(zipwriter, ZIPWRITER_METHOD_DEFLATE);
		zipwriter_add_data_file(zipwriter, "content.xml", instance->content, strlen(instance->content));
		zipwriter_add_data_file(zipwriter, "styles.xml", odf_styles_xml_template, strlen(odf_styles_xml_template));
		zipwriter_add_data_file(zipwriter, "settings.xml", odf_settings_xml_template, strlen(odf_settings_xml_template));
		zipwriter_set_method(zipwriter, ZIPWRITER_METHOD_STORE);
		zipwriter_add_data_file(zipwriter, "meta.xml", meta_xml, strlen(meta_xml));
		zipwriter_set_method(zipwriter, ZIPWRITER_METHOD_DEFLATE);
		zipwriter_add_directory(zipwriter, "META-INF");
		zipwriter_add_data_file(zipwriter, "META-INF/manifest.xml", odf_manifest_xml_template, strlen(odf_manifest_xml_template));
		zipwriter_close(zipwriter);
	}
	
	if (meta_xml != NULL) free(meta_xml);
	if (instance->filename != NULL) free(instance->filename);
	if (instance->content != NULL) free(instance->content);
	if (instance->table != NULL) free(instance->table);
	if (instance->application != NULL) free(instance->application);
	free(instance);
}
