/*
 *      communication.c
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


#include <glib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/poll.h>
#include <sys/utsname.h>
#include <sys/stat.h>

#include "gsfcore.h"
//#include "mainwindow.h"
#include "udpserver.h"
#include "settings.h"
#include "common/gost.h"


static gboolean communication_test_socket(gint sockfd);
static void communication_main_loop(gboolean init);

void communication_initialize()
{
	struct _netgate *netgate;
	gchar *password;
	struct sockaddr_in hostaddr;
	struct hostent *lhp;
	struct utsname hostparams;
	gint len; 
	
	netgate = gsfcore_get_netgate();
	
	netgate->resetconf = TRUE;
	
	netgate->serverexpsvclist = settings_read_boolean("local_server_export_services", TRUE);
	password = settings_read_string("local_server_password", "defaultpassword");
	gost_hash_new(password, (guchar *)&netgate->serverhash);
	g_free(password);
	netgate->serverport = (gushort)settings_read_int("local_server_port", 5555);
	
	len = snprintf((char *)&netgate->servernodename, sizeof(netgate->servernodename), "localhost");
	netgate->servernodename[len] = '\0';
	len = snprintf((char *)&netgate->serveripstr, sizeof(netgate->serveripstr), "localhost");
	netgate->serveripstr[len] = '\0';
	
	if (uname(&hostparams) == 0) {
		if ((lhp = gethostbyname(hostparams.nodename)) != NULL) {
			memset(&hostaddr, 0, sizeof(struct sockaddr_in));
			hostaddr.sin_family = AF_INET;
			memcpy((char *)&hostaddr.sin_addr, (char *)lhp->h_addr, lhp->h_length);
			hostaddr.sin_port = htons(5555);
			
			memset(netgate->serveripstr, 0, sizeof(netgate->serveripstr));
			inet_ntop(AF_INET, &(hostaddr.sin_addr), netgate->serveripstr, sizeof(netgate->serveripstr));
		}
		len = snprintf((char *)&netgate->servernodename, sizeof(netgate->servernodename), "%s", hostparams.nodename);
		netgate->servernodename[len] = '\0';
	}
	
	netgate->serversock = udpserver_start(netgate);
	netgate->serverhttpsock = webif_start(netgate, TRUE);
	
	netgate->runstatus = TRUE;
	
	communication_main_loop(TRUE);
}

static gboolean communication_test_socket(gint sockfd)
{
	struct stat statbuf;
	
	if (sockfd == -1) return FALSE;
	
	if (fstat(sockfd, &statbuf) < 0) return FALSE;
	
	if ((statbuf.st_mode & S_IFMT) == S_IFSOCK) {
		return TRUE;
	} else {
		return FALSE;
	}
}

gboolean communication_reinit(gchar *password, gshort port, gboolean expsvclist)
{
	struct _netgate *netgate;
	guchar svrhash[GOST_HASH_SIZE];
	
	
	netgate = gsfcore_get_netgate();
	
	memset(svrhash, 0, sizeof(svrhash));
	gost_hash_new(password, (guchar *)&svrhash);
	
	if (expsvclist != netgate->serverexpsvclist) netgate->serverexpsvclist = expsvclist;
	if (strncmp(svrhash, netgate->serverhash, GOST_HASH_SIZE) != 0) 
		memcpy((gchar *)&netgate->serverhash, (gchar *)&svrhash, GOST_HASH_SIZE);
	if (port != netgate->serverport) {
		netgate->serverport = port;
		netgate->resetconf = TRUE;
	}
	
	return TRUE;
}

gboolean communication_finalize()
{
	struct _netgate *netgate;
		
	netgate = gsfcore_get_netgate();
	
	netgate->runstatus = FALSE;
	
	return TRUE;
}

static void communication_main_loop(gboolean init)
{
	struct _netgate *netgate;
	struct pollfd sockets[2];
	gint readysockets;
	gint numsockets;
	gint sockfd;
	gint socktype;
	gint sockparamlen;
	struct timespec sleeptime = {.tv_sec = 1, .tv_nsec = 0 };
	
	netgate = gsfcore_get_netgate();
	
	while (netgate->runstatus) {
		if (netgate->resetconf) {
			numsockets = 0;
			if (!init) {
				if (communication_test_socket(netgate->serversock)) {
					close(netgate->serversock);
				}
				
				netgate->serversock = udpserver_start(netgate);
				
				if (communication_test_socket(netgate->serverhttpsock)) {
					close(netgate->serverhttpsock);
				}
				
				netgate->serverhttpsock = webif_start(netgate, FALSE);
			}
			
			if (communication_test_socket(netgate->serversock)) {
				sockets[numsockets].fd = netgate->serversock;
				sockets[numsockets].events = POLLIN;
				numsockets++;
			}
			
			if (communication_test_socket(netgate->serverhttpsock)) {
				sockets[numsockets].fd = netgate->serverhttpsock;
				sockets[numsockets].events = POLLIN;
				numsockets++;
			}
			if (init) init = FALSE;
			netgate->resetconf = FALSE;
		}
		
		if ((numsockets > 0) && (!netgate->resetconf)) {
			if ((readysockets = poll(sockets, numsockets, 1000)) > 0) {
				if (sockets[0].revents & POLLIN) {
					sockfd = sockets[0].fd;
				} else if (sockets[1].revents & POLLIN) {
					sockfd = sockets[1].fd;
				}
				
				sockparamlen = sizeof(gint);
				if (getsockopt(sockfd, SOL_SOCKET, SO_TYPE, &socktype, &sockparamlen) != -1) {
					if (socktype == SOCK_DGRAM) {
						udpserver_receive_message(netgate);
					} else if (socktype == SOCK_STREAM) {
						webif_receive_message(netgate);
					}
				}
			}
		} else {
			nanosleep(&sleeptime, NULL);
		}
		
	}
	
}
