/*
 *      dbstorage.h
 *      
 *      Copyright 2009 - 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __DBSTORAGE_H__
#define __DBSTORAGE_H__

#include <glib.h>
#include "gsfcore.h"

typedef struct _service service;
typedef struct _pid pid;
typedef struct _netservice netservice;

struct _service {
	guint32 id;
	gchar   *picture;
	gchar   *name;
	gchar   *node;
	guchar  type;
	gint    pos;
	guint   freq;
	guint   sr;
	guchar  pol;
	guchar  fec;
	guchar  sys;
	guchar  mod;
	guchar  rof;
	guchar  port10;
	guchar  port11;
	guchar  rep11;
	guchar  lnbtype;
	guint   lnblofreq;
	guint   lnbswfreq;
	guint   lnbhifreq;
	guchar  lnb22khz;
	guchar  positioner;
	guchar  possatnum;
	gushort npid;
	gchar   *nip;
	gchar   *nmask;
	gchar   *nmac;
	gchar   *nprog;
	gushort caid;
	gushort sid;
	gchar   *key;
	GSList  *pids;
};

enum {
	SERVICE_TV_RADIO = 0,
	SERVICE_DATA,
	SERVICE_TRANSPONDER,
	SERVICE_NETSHARE,
	SERVICE_RADIO
};


struct _pid {
	guchar  type;
	gushort value;
	gboolean enabled;
};

enum {
	PID_TYPE_VIDEO = 0,
	PID_TYPE_AUDIO,
	PID_TYPE_DATA,
	PID_TYPE_SUBTITLES,
	PID_TYPE_TELETEXT
};

struct _netservice {
	guint32 id;
	gchar name[256];
};

enum {
	LNB_TYPE_AUTO = 0,
	LNB_TYPE_KU_UNIVERSAL,
	LNB_TYPE_KU_WIDEBAND,
	LNB_TYPE_C_STANDARD,
	LNB_TYPE_C_SINGLE_SOLUTION,
	LNB_TYPE_CUSTOM,
	LNB_TYPE_KU_CIRCULAR,
	LNB_TYPE_NULL
};

enum {
	POSITIONER_DIASABLED = 0,
	POSITIONER_POSITION,
	POSITIONER_NUMBER
};

gboolean service_xml_list_rebuild(void);
gchar *service_xml_list_get_fragment(gsize fragmentsize, guint number);
guint service_xml_list_get_length(void);
gchar *service_xml_list_get_string(void);

gboolean service_net_add(guint32 id, gchar *name);
gboolean service_net_clear(void);
guint service_net_get_number(void);
gboolean service_net_read_foreach(gboolean (*add_netservice_cb)(netservice *nsvc));
netservice *service_net_read_id(guint id);
tuneparams_t service_search_position_params(gint position);
//GString *service_get_xml_list(gboolean compression);
gboolean service_storage_init(void);

gboolean service_image_add(guint32 oldid, guint32 newid);
gboolean service_image_exists(guint32 id);
gboolean service_image_remove(guint32 id);

gboolean service_add(service *svc, gboolean replace, gboolean (*add_service_cb)(service *svc));
gboolean service_add_multiple(GSList **serviceslist);
gboolean service_merge(service *svc, gboolean (*update_service_cb)(service *svc));
gboolean service_remove(service *svc, gboolean (*remove_service_cb)(service *svc));
service *service_read(guint id);
gboolean service_read_foreach(gboolean (*add_service_cb)(service *svc), gboolean (*add_netservice_cb)(netservice *nsvc), gboolean (*add_svcxml_cb)(service *svc, gboolean initialize));
gboolean service_write_index(guint32 *indexvec, gsize indexlen);
service *service_new(gchar *picture, gchar *name, guchar type);
service *service_clone(service *svc);

void service_free(service *svc);
void service_pids_free(service *svc);
void service_pids_free_foreach(pid *curpid, gpointer data);


#endif /* __DBSTORAGE_H__ */
