/*
 *      devwindow.c
 *      
 *      Copyright 2011-2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>
#include "gsfcore.h"
//#include "mainwindow.h"
#include "dvbname.h"
#include "dbstorage.h"
#include "dvbstatus.h"
#include "devwindow.h"

enum {
	DEV_ENTRIES_COL_ICON = 0,
	DEV_ENTRIES_COL_ICON_SIZE,
	DEV_ENTRIES_COL_NAME,
	DEV_ENTRIES_COL_ID,
	DEV_ENTRIES_COLUMNS
};

enum {
	DEVLIST_ENABLED = 0,
	DEVLIST_DESCRIPTION,
	DEVLIST_ADAPTER,
	DEVLIST_FRONTEND,
	DEVLIST_COLUMNS
};

struct sdevwindow {
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	GtkWidget *apply;
	GtkWidget *list;
	GtkTreeSelection *selection;
	GtkWidget *panelcaption;
	GtkWidget *panel;
	gint pageid;
	GtkWidget *page;
	gchar *vpid;
	GString *apids;
	gulong changepagesignal;
};
struct sdevwindow devwindow;

gint devwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
void devwindow_filters_foreach(gpointer data, gpointer user_data);

void devwindow_init_list(GtkWidget *list);

GtkWidget **devwindow_device_settings_create(void);
GtkWidget **devwindow_driver_settings_create(void);
GtkWidget **devwindow_frontend_settings_create(void);
GtkWidget **devwindow_filters_settings_create(void);

void devwindow_device_settings_configure(gpointer page);
void devwindow_driver_settings_configure(gpointer page);
void devwindow_frontend_settings_configure(gpointer page);
void devwindow_filters_settings_configure(gpointer page);

void devwindow_device_settings_apply(gpointer page);

typedef GtkWidget **(*dev_page_init_func)(void);
typedef void (*dev_page_configure_func)(gpointer page);
typedef void (*dev_page_apply_func)(gpointer page);

typedef struct {
	GtkWidget **widgets;
	const guint preferenceid;
	const gchar *title;
	const gchar *description;
	const gchar *stock;
	dev_page_init_func init_func;
	dev_page_configure_func configure_func;
	dev_page_apply_func apply_func;
} devpage;

static devpage dev_pages[4] = {
	{ NULL, DEV_PREFERENCE_ID_DEVICE,   N_("Device"),   N_("Hardware part"),  GTK_STOCK_JUMP_TO, devwindow_device_settings_create,   devwindow_device_settings_configure,   devwindow_device_settings_apply},
	{ NULL, DEV_PREFERENCE_ID_DRIVER,   N_("Driver"),   N_("Software part"),  GTK_STOCK_FLOPPY,  devwindow_driver_settings_create,   devwindow_driver_settings_configure,   NULL},
	{ NULL, DEV_PREFERENCE_ID_FRONTEND, N_("Frontend"), N_("Tune params"),    GTK_STOCK_EXECUTE, devwindow_frontend_settings_create, devwindow_frontend_settings_configure, NULL},
	{ NULL, DEV_PREFERENCE_ID_FILTERS,  N_("Filters"),  N_("Data filtering"), GTK_STOCK_CLEAR,   devwindow_filters_settings_create,  devwindow_filters_settings_configure,  NULL},
};


gint devwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gint i;
	
	devwindow.visible = FALSE;
	
	if (g_signal_handler_is_connected(G_OBJECT(devwindow.list), devwindow.changepagesignal)) {
		g_signal_handler_disconnect(G_OBJECT(devwindow.list), devwindow.changepagesignal);
	}
	
	for (i=0; i<(sizeof(dev_pages)/sizeof(devpage)); i++) {
		if (dev_pages[i].widgets != NULL) {
			gtk_widget_destroy(dev_pages[i].widgets[0]);
			g_free(dev_pages[i].widgets);
			dev_pages[i].widgets = NULL;
		}
	}
	
	gtk_widget_destroy(devwindow.win);
		
	return FALSE;
}

/*void devwindow_message(gchar *message, gchar *caption)
{
	GtkWidget *dialog;
	
	dialog = gtk_message_dialog_new(GTK_WINDOW(devwindow.win),
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_INFO,
									GTK_BUTTONS_OK,
									message);
	
	gtk_window_set_title(GTK_WINDOW(dialog), caption);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	
}*/

void devwindow_filters_foreach(gpointer data, gpointer user_data)
{
	struct _pid *currentpid = (struct _pid *)data;
		
	if (currentpid->type == PID_TYPE_VIDEO) {
		devwindow.vpid = g_strdup_printf("%u", currentpid->value);
	} else if (currentpid->type == PID_TYPE_AUDIO) {
		g_string_append_printf(devwindow.apids, "%u ", currentpid->value);
	}
}

/*gint devwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	//devwindow_message("Not enough rights to apply settings", "Information");
	
	gint i;
	
	for (i=0; i<(sizeof(dev_pages)/sizeof(devpage)); i++) {
		if ((dev_pages[i].widgets != NULL) && (dev_pages[i].apply_func != NULL)) {
			dev_pages[i].apply_func(&dev_pages[i]);
		}
	}
	
	return 0;
}*/

static void devwindow_action_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gint i;
	
	if (response_id == GTK_RESPONSE_APPLY) {
		for (i=0; i<(sizeof(dev_pages)/sizeof(devpage)); i++) {
			if ((dev_pages[i].widgets != NULL) && (dev_pages[i].apply_func != NULL)) {
				dev_pages[i].apply_func(&dev_pages[i]);
			}
		}
		settings_flush();
		g_signal_emit_by_name(G_OBJECT(devwindow.win), "destroy", NULL);
	} else if (response_id == GTK_RESPONSE_CLOSE) {
		g_signal_emit_by_name(G_OBJECT(devwindow.win), "destroy", NULL);
	} else if (response_id == GTK_RESPONSE_HELP) {
		
	}
}

void devwindow_init_list(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2;
	GtkListStore       *store;
		
	
	renderer = gtk_cell_renderer_pixbuf_new();
	column1 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column1, NULL);
	gtk_tree_view_column_pack_start(column1, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column1, renderer, "stock-id", DEV_ENTRIES_COL_ICON, "stock-size", DEV_ENTRIES_COL_ICON_SIZE, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, "ellipsize-set", TRUE, NULL);
	column2 = gtk_tree_view_column_new_with_attributes(_("Option"), renderer, "markup", DEV_ENTRIES_COL_NAME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	
	store = gtk_list_store_new(DEV_ENTRIES_COLUMNS, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

gboolean devwindow_fill_entries(void)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar caption[256];
	gint len;
	gint i;
	struct _frontend *frontend;
	struct _service *svc;
	gboolean frontendflag, tuneflag;
	
	frontend = gsfcore_get_frontend();
	svc = gsfcore_get_service();
	
	if (frontend != NULL) {
		frontendflag = dvb_frontend_opened(frontend->frontend_fd);
	} else {
		frontendflag = FALSE;
	}
		
	if ((frontend != NULL) && (svc != NULL) && (svc->type != SERVICE_NETSHARE)) {
		tuneflag = ((frontend->tuned) || (frontend->open_mode_readonly));
	} else {
		tuneflag = FALSE;
	}
	
	if (devwindow.visible) {
		if ((model = gtk_tree_view_get_model(GTK_TREE_VIEW(devwindow.list))) != NULL) {
			g_object_ref(model);
			gtk_tree_view_set_model(GTK_TREE_VIEW(devwindow.list), NULL);
			gtk_list_store_clear(GTK_LIST_STORE(model));
			for (i=0; i<(sizeof(dev_pages)/sizeof(devpage)); i++) {
				if ((dev_pages[i].preferenceid == DEV_PREFERENCE_ID_DRIVER) && (!frontendflag)) continue;
				if ((dev_pages[i].preferenceid == DEV_PREFERENCE_ID_FRONTEND) && (!tuneflag)) continue;
				if ((dev_pages[i].preferenceid == DEV_PREFERENCE_ID_FILTERS) && (!tuneflag)) continue;
				len = snprintf((gchar *)&caption, sizeof(caption), "<small><b>%s</b>\n%s</small>", gettext(dev_pages[i].title), gettext(dev_pages[i].description));
				caption[len] = '\0';
				gtk_list_store_append(GTK_LIST_STORE(model), &iter);
				gtk_list_store_set(GTK_LIST_STORE(model), &iter, DEV_ENTRIES_COL_ICON, dev_pages[i].stock,
																DEV_ENTRIES_COL_ICON_SIZE, GTK_ICON_SIZE_BUTTON,
																DEV_ENTRIES_COL_NAME, caption,
																DEV_ENTRIES_COL_ID, dev_pages[i].preferenceid,
																-1);
			}
			gtk_tree_view_set_model(GTK_TREE_VIEW(devwindow.list), model);
			g_object_unref(model);
		}
	}
	
	return FALSE;
}


void devwindow_entry_selected(GtkTreeView *tree_view, gpointer data)
{
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint i, id;
	gchar caption[256];
	gint len;
		
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, DEV_ENTRIES_COL_ID, &id, -1);
		/*for (i=0; i<(sizeof(dev_pages)/sizeof(devpage)); i++) {
			if (dev_pages[i].preferenceid == id) {
				id = i;
			}
		}*/
		if (((devwindow.pageid == -1) || (devwindow.pageid != id)) && (id < (sizeof(dev_pages)/sizeof(devpage)))) {
			if ((dev_pages[id].widgets == NULL) && (dev_pages[id].init_func != NULL)) {
				//Create new page and configure it
				dev_pages[id].widgets = dev_pages[id].init_func();
				if ((dev_pages[id].widgets != NULL) && (dev_pages[id].configure_func != NULL)) {
					dev_pages[id].configure_func(&dev_pages[id]);
				}
			} else if (dev_pages[id].init_func == NULL) {
				g_warning("No constructor for page %i (%s)", id, dev_pages[id].title);
			}
				
			if (dev_pages[id].widgets != NULL) {
				if (devwindow.page != NULL) {
					if (devwindow.pageid == -1) {
						//After start page selected
						gtk_widget_set_sensitive(devwindow.panel, TRUE);
						gtk_dialog_set_response_sensitive(GTK_DIALOG(devwindow.win), GTK_RESPONSE_APPLY, TRUE);
						//gtk_widget_set_sensitive(devwindow.apply, TRUE);
					} else {
						//Increase refcount to avoid page re-creation
						g_object_ref(devwindow.page);
					}
					//Remove old page
					gtk_container_remove(GTK_CONTAINER(devwindow.panel), devwindow.page);
				}
				//Add new page
				devwindow.page = dev_pages[id].widgets[0];
				gtk_box_pack_end(GTK_BOX(devwindow.panel), devwindow.page, TRUE, TRUE, 10);
				devwindow.pageid = id;
				len = snprintf((gchar *)&caption, sizeof(caption), "<b>%s</b>", gettext(dev_pages[id].title)); 
				caption[len] = '\0';
				gtk_label_set_markup(GTK_LABEL(devwindow.panelcaption), caption);
				gtk_widget_show_all(devwindow.panel);
			}
			
		}
		
	}
}	

gboolean devwindow_devlist_select_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	struct _frontend *frontend;
	gboolean enabled, restart;
	gint adapterid, frontendid;
	
	gtk_tree_model_get(model, iter, DEVLIST_ENABLED, &enabled, DEVLIST_ADAPTER, &adapterid, DEVLIST_FRONTEND, &frontendid, -1);
	
	if (enabled) {
		if ((frontend = gsfcore_get_frontend()) != NULL) {
			restart = GPOINTER_TO_INT(data);
			if ((adapterid != frontend->adapter_nr) || (frontendid != frontend->frontend_nr)) {
				gsfcore_device_search(adapterid, frontendid, FALSE, FALSE);
			} else if (restart) {
				gsfcore_device_search(adapterid, frontendid, FALSE, TRUE);
			}
		}
		return TRUE;
	}
	
	return FALSE;
}

void devwindow_devlist_fill_foreach(gpointer data, gpointer user_data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	struct _device *device;
	gboolean enabled;
	gchar devname[256];
	const gchar *bus;
	struct _frontend *frontend;
	
	frontend = gsfcore_get_frontend();
	
	device = (struct _device *)data;
	model = GTK_TREE_MODEL(user_data);
	
	if ((device->adapter == frontend->adapter_nr) && 
		(device->frontend == frontend->frontend_nr))
			enabled = TRUE; else enabled = FALSE;
	
	if (device->bus == DVBNAME_DEVICE_BUS_USB) {
		bus = "USB";
	} else if (device->bus == DVBNAME_DEVICE_BUS_PCI) {
		bus = "PCI";
	}
	g_snprintf((gchar *)&devname, sizeof(devname), _("<small><b>%s</b>\nDriver:%s Placement:%s:%s</small>"), device->name, device->driver, bus, device->slot);
	
	gtk_list_store_append(GTK_LIST_STORE(model), &iter);
	gtk_list_store_set(GTK_LIST_STORE(model), &iter, DEVLIST_ENABLED, enabled,
													DEVLIST_DESCRIPTION, devname,
													DEVLIST_ADAPTER, device->adapter,
													DEVLIST_FRONTEND, device->frontend,
													-1);
}

gboolean devwindow_devlist_item_unselect_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	gboolean enabled;
	
	gtk_tree_model_get(model, iter, DEVLIST_ENABLED, &enabled, -1);
	if (enabled) {
		gtk_list_store_set(GTK_LIST_STORE(model), iter, DEVLIST_ENABLED, FALSE, -1);
		return TRUE;
	}
	
	return FALSE;
}

void devwindow_devlist_item_selected(GtkCellRendererToggle *cell_renderer, gchar *path, gpointer data)
{
	GtkWidget **widgets;
	GtkTreeIter iter;
	GtkTreeModel *model;
	gboolean enabled;
	struct _frontend *frontend;
	gint adapterid, frontendid;
	
	if (data == NULL) return;
	
	widgets = (GtkWidget **)data;
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widgets[1]));
	if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
		gtk_tree_model_get(model, &iter, DEVLIST_ENABLED, &enabled, -1);
		if (!enabled) {
			gtk_tree_model_foreach(model, devwindow_devlist_item_unselect_foreach, NULL);
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, DEVLIST_ENABLED, TRUE, -1);
			//If device is not opened, enable "Apply" button
			if ((frontend = gsfcore_get_frontend()) != NULL) {
				gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, DEVLIST_ADAPTER, &adapterid, DEVLIST_FRONTEND, &frontendid, -1);
				if ((adapterid != frontend->adapter_nr) || (frontendid != frontend->frontend_nr)) {
					gtk_dialog_set_response_sensitive(GTK_DIALOG(devwindow.win), GTK_RESPONSE_APPLY, TRUE);
					//gtk_widget_set_sensitive(devwindow.apply, TRUE);
				} else {
					if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets[2]))) {
						gtk_dialog_set_response_sensitive(GTK_DIALOG(devwindow.win), GTK_RESPONSE_APPLY, TRUE);
						//gtk_widget_set_sensitive(devwindow.apply, TRUE);
					} else {
						gtk_dialog_set_response_sensitive(GTK_DIALOG(devwindow.win), GTK_RESPONSE_APPLY, FALSE);
						//gtk_widget_set_sensitive(devwindow.apply, FALSE);
					}
				}
			}
			
		}
	}
}


void devwindow_devlist_restart_toggle(GtkToggleButton *togglebutton, gpointer data)
{
	GtkWidget **widgets;
	GtkTreeSelection *selection;
	GtkTreeIter iter;
	GtkTreeModel *model;
	struct _frontend *frontend;
	gint adapterid, frontendid;
	
	if (data == NULL) return;
	
	widgets = (GtkWidget **)data;
	if ((selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widgets[1]))) != NULL) {
		if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
			if ((frontend = gsfcore_get_frontend()) != NULL) {
				gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, DEVLIST_ADAPTER, &adapterid, DEVLIST_FRONTEND, &frontendid, -1);
				if ((adapterid != frontend->adapter_nr) || (frontendid != frontend->frontend_nr)) {
					gtk_dialog_set_response_sensitive(GTK_DIALOG(devwindow.win), GTK_RESPONSE_APPLY, TRUE);
					//gtk_widget_set_sensitive(devwindow.apply, TRUE);
				} else {
					if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets[2]))) {
						gtk_dialog_set_response_sensitive(GTK_DIALOG(devwindow.win), GTK_RESPONSE_APPLY, TRUE);
						//gtk_widget_set_sensitive(devwindow.apply, TRUE);
					} else {
						gtk_dialog_set_response_sensitive(GTK_DIALOG(devwindow.win), GTK_RESPONSE_APPLY, FALSE);
						//gtk_widget_set_sensitive(devwindow.apply, FALSE);
					}
				}
			}
		}
	}
	
	
}

GtkWidget **devwindow_device_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *device_l;
	GtkWidget *listscroll;
	GtkWidget *list;
	GtkCellRenderer *tbrenderer;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkListStore *store;
	GtkTreeIter iter;
	GtkTreeSelection *selection;
	GtkWidget *restart;
	GtkWidget **widgets;
	
	
	table = gtk_table_new(2, 2, FALSE);
	
	device_l = gtk_label_new(_("Available devices"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), device_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 2, 0, 1, GTK_FILL, GTK_SHRINK, 5, 0);
	
	listscroll = gtk_scrolled_window_new(NULL, NULL);
	
	list = gtk_tree_view_new();
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(list), FALSE);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(listscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(listscroll), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(listscroll), list);
	
	tbrenderer = gtk_cell_renderer_toggle_new();
	column = gtk_tree_view_column_new_with_attributes(_("State"), tbrenderer, "active", DEVLIST_ENABLED, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
	gtk_cell_renderer_toggle_set_radio(GTK_CELL_RENDERER_TOGGLE(tbrenderer), TRUE);
		
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(_("Description"), renderer, "markup", DEVLIST_DESCRIPTION, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
	
	store = gtk_list_store_new(DEVLIST_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT);
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
	
	gtk_table_attach(GTK_TABLE(table), listscroll, 0, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 3);
	
	restart = gtk_check_button_new_with_label(_("Allow device restart"));
	align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), restart);
	gtk_table_attach(GTK_TABLE(table), align, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 0);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(3*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = list;
	widgets[2] = restart;
		
	g_signal_connect(G_OBJECT(tbrenderer), "toggled", G_CALLBACK(devwindow_devlist_item_selected), (gpointer)widgets);
	g_signal_connect(G_OBJECT(restart), "toggled", G_CALLBACK(devwindow_devlist_restart_toggle), (gpointer)widgets);
	
	return widgets;
}

void devwindow_device_settings_configure(gpointer page)
{
	devpage *pageptr;
	GSList *devices;
	GtkTreeModel *model;
		
	pageptr = (devpage *)page;
		
	if ((devices = gsfcore_get_devices(TRUE)) != NULL) {
		if ((model = gtk_tree_view_get_model(GTK_TREE_VIEW(pageptr->widgets[1]))) != NULL) {
			g_object_ref(model);
			gtk_tree_view_set_model(GTK_TREE_VIEW(pageptr->widgets[1]), NULL);
			gtk_list_store_clear(GTK_LIST_STORE(model));
			g_slist_foreach(devices, (GFunc)devwindow_devlist_fill_foreach, (gpointer)model);
			gtk_tree_view_set_model(GTK_TREE_VIEW(pageptr->widgets[1]), model);
			g_object_unref(model);
		}
		gtk_widget_set_sensitive(GTK_WIDGET(pageptr->widgets[2]), TRUE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(pageptr->widgets[2]), FALSE);
		gtk_dialog_set_response_sensitive(GTK_DIALOG(devwindow.win), GTK_RESPONSE_APPLY, FALSE);
		//gtk_widget_set_sensitive(GTK_WIDGET(devwindow.apply), FALSE);
	}
}

void devwindow_device_settings_apply(gpointer page)
{
	devpage *pageptr;
	GtkTreeModel *model;
	gboolean restart;
	
	pageptr = (devpage *)page;
	
	restart = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pageptr->widgets[2]));
	
	if (gsfcore_get_devices(FALSE) != NULL) {
		if ((model = gtk_tree_view_get_model(GTK_TREE_VIEW(pageptr->widgets[1]))) != NULL) {
			gtk_tree_model_foreach(model, devwindow_devlist_select_foreach, GINT_TO_POINTER(restart));
		}
	}
		
	//g_signal_emit_by_name(G_OBJECT(devwindow.win), "destroy", NULL);
}

GtkWidget **devwindow_driver_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *frdriver, *frdriver_l;
	GtkWidget *apiver, *apiver_l;
	GtkWidget *freqrange, *freqrange_l;
	GtkWidget *srrange, *srrange_l;
	GtkWidget *systems, *systems_l;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 5, FALSE);
	
	frdriver_l = gtk_label_new(_("Frontend driver"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), frdriver_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	frdriver = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(frdriver), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(frdriver), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(frdriver), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(frdriver), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), frdriver);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
		
	apiver_l = gtk_label_new(_("API version"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), apiver_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	apiver = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(apiver), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(apiver), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(apiver), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(apiver), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), apiver);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	freqrange_l = gtk_label_new(_("Frequencies"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), freqrange_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	freqrange = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(freqrange), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(freqrange), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(freqrange), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(freqrange), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), freqrange);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	srrange_l = gtk_label_new(_("Symbolrates"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), srrange_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	srrange = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(srrange), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(srrange), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(srrange), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(srrange), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), srrange);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	systems_l = gtk_label_new(_("Systems"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), systems_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	systems = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(systems), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(systems), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(systems), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(systems), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), systems);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 4, 5, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(6*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = frdriver;
	widgets[2] = apiver;
	widgets[3] = freqrange;
	widgets[4] = srrange;
	widgets[5] = systems;
	
	return widgets;
}

void devwindow_driver_settings_configure(gpointer page)
{
	devpage *pageptr;
	gchar buffer[32];
	struct _frontend *frontend;
		
	pageptr = (devpage *)page;
		
	frontend = gsfcore_get_frontend();
	
	if ((frontend != NULL) && (dvb_frontend_opened(frontend->frontend_fd))) {
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[1]), frontend->driver);
		g_sprintf(buffer, "%u.%u", ((frontend->api_version & 0xFF00) >> 8), (frontend->api_version & 0x00FF));
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), buffer);
		g_sprintf(buffer, "%u - %u", frontend->frequency_min/1000, frontend->frequency_max/1000);
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), buffer);
		g_sprintf(buffer, "%u - %u", frontend->symbol_rate_min/1000, frontend->symbol_rate_max/1000);
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), buffer);
		g_sprintf(buffer, "%s %s", frontend->dvbs_support ? "DVB-S" : "", frontend->dvbs2_support ? "DVB-S2" : "");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), buffer);
	} else {
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[1]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), "--");
	}
}

GtkWidget **devwindow_frontend_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *frequency, *frequency_l;
	GtkWidget *symbolrate, *symbolrate_l;
	GtkWidget *polarisation, *polarisation_l;
	GtkWidget *fec, *fec_l;
	GtkWidget *system, *system_l;
	GtkWidget *modulation, *modulation_l;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 5, FALSE);
	
	frequency_l = gtk_label_new(_("Frequency"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), frequency_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	frequency = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(frequency), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(frequency), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(frequency), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(frequency), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), frequency);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
		
	symbolrate_l = gtk_label_new(_("Symbolrate"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), symbolrate_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	symbolrate = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(symbolrate), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(symbolrate), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(symbolrate), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(symbolrate), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), symbolrate);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	polarisation_l = gtk_label_new(_("Polarisation"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), polarisation_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	polarisation = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(polarisation), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(polarisation), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(polarisation), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(polarisation), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), polarisation);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	fec_l = gtk_label_new(_("FEC"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), fec_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	fec = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(fec), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(fec), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(fec), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(fec), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), fec);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	system_l = gtk_label_new(_("System"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), system_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	system = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(system), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(system), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(system), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(system), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), system);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 4, 5, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	modulation_l = gtk_label_new(_("Modulation"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), modulation_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 5, 6, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	modulation = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(modulation), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(modulation), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(modulation), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(modulation), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), modulation);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 5, 6, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);

	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(7*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = frequency;
	widgets[2] = symbolrate;
	widgets[3] = polarisation;
	widgets[4] = fec;
	widgets[5] = system;
	widgets[6] = modulation;
	
	return widgets;
}

void devwindow_frontend_settings_configure(gpointer page)
{
	devpage *pageptr;
	tuneparams_t tuneparams;
	gchar buffer[32];
	const gchar *str;
	
	pageptr = (devpage *)page;
	
	tuneparams = gsfcore_get_tuneparams();
	
	if (tuneparams != NULL) {
		g_sprintf(buffer, "%u (IF: %u)", tuneparams->freq, tuneparams->realfreq);
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[1]), buffer);
		g_sprintf(buffer, "%u (SR: %u)", tuneparams->sr, tuneparams->realsr);
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), buffer);
		str = gsfcore_string_definition(DEFINITION_POLARISATION, tuneparams->pol);
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), str);
		str = gsfcore_string_definition(DEFINITION_FEC, tuneparams->fec);
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), str);
		str = gsfcore_string_definition(DEFINITION_SYSTEM, tuneparams->sys);
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), str);
		str = gsfcore_string_definition(DEFINITION_MODULATION, tuneparams->mod);
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[6]), str);
	} else {
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[1]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[6]), "--");
	}
}

GtkWidget **devwindow_filters_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *videopid, *videopid_l;
	GtkWidget *audiopids, *audiopids_l;
	GtkWidget *datapid, *datapid_l;
	GtkWidget *ipaddress, *ipaddress_l;
	GtkWidget *macaddress, *macaddress_l;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 5, FALSE);
	
	videopid_l = gtk_label_new(_("Video PID"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), videopid_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	videopid = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(videopid), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(videopid), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(videopid), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(videopid), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), videopid);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
		
	audiopids_l = gtk_label_new(_("Audio PID"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), audiopids_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	audiopids = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(audiopids), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(audiopids), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(audiopids), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(audiopids), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), audiopids);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	datapid_l = gtk_label_new(_("Data PID"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), datapid_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	datapid = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(datapid), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(datapid), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(datapid), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(datapid), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), datapid);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	ipaddress_l = gtk_label_new(_("IP address"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), ipaddress_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	ipaddress = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(ipaddress), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(ipaddress), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(ipaddress), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(ipaddress), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), ipaddress);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	macaddress_l = gtk_label_new(_("MAC address"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), macaddress_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	macaddress = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(macaddress), TRUE);
	#if GTK_CHECK_VERSION(3, 2, 0)
		gtk_label_set_width_chars(GTK_LABEL(macaddress), 15);
	#else
		gtk_label_set_max_width_chars(GTK_LABEL(macaddress), 25);
	#endif
	gtk_label_set_ellipsize(GTK_LABEL(macaddress), PANGO_ELLIPSIZE_END);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), macaddress);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 4, 5, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(6*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = videopid;
	widgets[2] = audiopids;
	widgets[3] = datapid;
	widgets[4] = ipaddress;
	widgets[5] = macaddress;
		
	return widgets;
}

void devwindow_filters_settings_configure(gpointer page)
{
	devpage *pageptr;
	gchar buffer[32];
	struct _frontend *frontend;
	struct _service *svc;
	gboolean frontendflag, tuneflag;
			
	pageptr = (devpage *)page;
	
	frontend = gsfcore_get_frontend();
	svc = gsfcore_get_service();
	
	if (frontend != NULL) {
		frontendflag = dvb_frontend_opened(frontend->frontend_fd);
	} else {
		frontendflag = FALSE;
	}
		
	if ((frontend != NULL) && (svc != NULL) && (svc->type != SERVICE_NETSHARE)) {
		tuneflag = ((frontend->tuned) || (frontend->open_mode_readonly));
	} else {
		tuneflag = FALSE;
	}
	
	if ((frontendflag) && (tuneflag)) {
		if ((svc->type == SERVICE_TV_RADIO) || (svc->type == SERVICE_RADIO)) {
			if (devwindow.vpid != NULL) {
				g_free(devwindow.vpid);
				devwindow.vpid = NULL;
			}
			if (devwindow.apids != NULL) {
				g_string_free(devwindow.apids, TRUE);
				devwindow.apids = NULL;
			}
			devwindow.apids = g_string_new("");
			g_slist_foreach(svc->pids, devwindow_filters_foreach, NULL);
			
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[1]), (devwindow.vpid != NULL) ? devwindow.vpid : _("Disabled"));
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), ((devwindow.apids->str != NULL) && (devwindow.apids->str[0] != '\0')) ? devwindow.apids->str : _("Disabled"));
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), _("Disabled"));
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), _("Disabled"));
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), _("Disabled"));
			return;
		} else if (svc->type == SERVICE_DATA) {
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[1]), _("Disabled"));
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), _("Disabled"));
			g_sprintf(buffer, "%u", svc->npid);
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), buffer);
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), (svc->npid > 0) ? svc->nip : "--");
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), (svc->npid > 0) ? svc->nmask : "--");
			return;
		}
	}
	
	gtk_label_set_text(GTK_LABEL(pageptr->widgets[1]), "--");
	gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), "--");
	gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), "--");
	gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), "--");
	gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), "--");
}

void devwindow_update_page(gint preference)
{
	gint i;
	
	if (devwindow.visible) {
		if (preference == DEV_PREFERENCE_ID_MENU) {
			devwindow_fill_entries();
		} else {
			for (i=0; i<(sizeof(dev_pages)/sizeof(devpage)); i++) {
				if (dev_pages[i].preferenceid == preference) {
					if ((dev_pages[i].configure_func != NULL) && (dev_pages[i].widgets != NULL)) {
						dev_pages[i].configure_func(&dev_pages[i]);
					} 
				}
			}
		}
	}
}

gboolean devwindow_create(GtkWindow *parentwin)
{
	GtkWidget *vbox, *hbox, *hpaned;
	GtkWidget *align;
	GtkWidget *listscroll, *panelscroll;
	
	if (!devwindow.visible) {
		devwindow.visible = TRUE;
		
		devwindow.vpid = NULL;
		devwindow.apids = NULL;
		
		/*devwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		devwindow.title = _("Device settings");
		gtk_window_set_position(GTK_WINDOW(devwindow.win), GTK_WIN_POS_CENTER);
		gtk_window_set_title(GTK_WINDOW(devwindow.win), devwindow.title);
		gtk_widget_set_size_request(devwindow.win, 500, 300);
		gtk_window_set_default_size(GTK_WINDOW(devwindow.win), 500, 300);
		gtk_window_set_resizable(GTK_WINDOW(devwindow.win), FALSE);
		gtk_window_set_transient_for(GTK_WINDOW(devwindow.win), parentwin);*/
		
		devwindow.win = gtk_dialog_new_with_buttons(_("Device settings"), 
													GTK_WINDOW(parentwin),
													GTK_DIALOG_DESTROY_WITH_PARENT,
													/*GTK_STOCK_HELP, GTK_RESPONSE_HELP,*/
													GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
													GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
													NULL);
		
		gtk_window_set_modal(GTK_WINDOW(devwindow.win), TRUE);
		gtk_widget_set_size_request(GTK_WIDGET(devwindow.win), 500, 300);
		gtk_window_set_default_size(GTK_WINDOW(devwindow.win), 500, 300);
		gtk_window_set_resizable(GTK_WINDOW(devwindow.win), FALSE);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
			hpaned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
		#else
			vbox = gtk_vbox_new(FALSE, 0);
			hpaned = gtk_hpaned_new();
		#endif
		gtk_container_set_border_width(GTK_CONTAINER(hpaned), 5);
		
		listscroll = gtk_scrolled_window_new(NULL, NULL);
		
		devwindow.list = gtk_tree_view_new();
		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(devwindow.list), FALSE);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(listscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(listscroll), GTK_SHADOW_ETCHED_IN);
		
		gtk_container_add(GTK_CONTAINER (listscroll), devwindow.list);
		devwindow.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(devwindow.list));
		gtk_tree_selection_set_mode(devwindow.selection, GTK_SELECTION_BROWSE);
		
		devwindow_init_list(devwindow.list);
		
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(devwindow.list), TRUE);
		
		gtk_paned_pack1(GTK_PANED(hpaned), listscroll, TRUE, FALSE);
		
		panelscroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(panelscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(panelscroll), GTK_SHADOW_NONE);
		
		gtk_widget_set_size_request(listscroll, 150, -1);
		gtk_widget_set_size_request(panelscroll, 300, -1);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			devwindow.panel = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
		#else
			devwindow.panel = gtk_vbox_new(FALSE, 0);
		#endif
		
		devwindow.panelcaption = gtk_label_new(NULL);
		gtk_label_set_markup(GTK_LABEL(devwindow.panelcaption), _("<b>Device settings</b>"));
		gtk_label_set_max_width_chars(GTK_LABEL(devwindow.panelcaption), 30);
		gtk_label_set_ellipsize(GTK_LABEL(devwindow.panelcaption), PANGO_ELLIPSIZE_END);
		gtk_box_pack_start(GTK_BOX(devwindow.panel), devwindow.panelcaption, FALSE, FALSE, 10);
		
		devwindow.page = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_DIALOG);
		gtk_box_pack_end(GTK_BOX(devwindow.panel), devwindow.page, TRUE, FALSE, 10);
		
		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(panelscroll), devwindow.panel);
		
		align = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
		gtk_container_add(GTK_CONTAINER(align), panelscroll);
		
		gtk_paned_pack2(GTK_PANED(hpaned), align, TRUE, FALSE);
		
		gtk_box_pack_start(GTK_BOX(vbox), hpaned, TRUE, TRUE, 0);
		
		/*devwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
		devwindow.apply = gtk_button_new_from_stock(GTK_STOCK_APPLY);
		
		gtk_widget_set_sensitive(devwindow.apply, FALSE);
		gtk_widget_set_sensitive(devwindow.panel, FALSE);
		
		hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), devwindow.apply, FALSE, FALSE, 5);
		gtk_box_pack_end(GTK_BOX(hbox), devwindow.close, FALSE, FALSE, 5);
				
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
		
		g_signal_connect(G_OBJECT(devwindow.win), "destroy", G_CALLBACK(devwindow_close), NULL);
		g_signal_connect(G_OBJECT(devwindow.close), "clicked", G_CALLBACK(devwindow_close), NULL);
		g_signal_connect(G_OBJECT(devwindow.apply), "clicked", G_CALLBACK(devwindow_apply), NULL);
		g_signal_connect(G_OBJECT(devwindow.list), "cursor-changed", G_CALLBACK(devwindow_entry_selected), NULL);
		*/
		devwindow.pageid = -1;
		
		devwindow_fill_entries();
		
		//gtk_container_add(GTK_CONTAINER(devwindow.win),vbox);
		//gtk_widget_show_all(devwindow.win);
		
		//gtk_window_set_focus(GTK_WINDOW(devwindow.win), devwindow.list);
		
		gtk_dialog_set_response_sensitive(GTK_DIALOG(devwindow.win), GTK_RESPONSE_APPLY, FALSE);
		
		GtkWidget *dialogarea = gtk_dialog_get_content_area(GTK_DIALOG(devwindow.win));
		
		gtk_box_pack_start(GTK_BOX(dialogarea), vbox, TRUE, TRUE, 0);
		gtk_widget_show_all(devwindow.win);
		
		g_signal_connect(G_OBJECT(devwindow.win), "destroy", G_CALLBACK(devwindow_close), NULL);
		g_signal_connect(G_OBJECT(devwindow.win), "response", G_CALLBACK(devwindow_action_response), NULL);
		devwindow.changepagesignal = g_signal_connect(G_OBJECT(devwindow.list), "cursor-changed", G_CALLBACK(devwindow_entry_selected), NULL);
		
	} else {
		gtk_window_present(GTK_WINDOW(devwindow.win));
	}
	
	return TRUE;
}
