/*
 *      dialogs.c
 *      
 *      Copyright 2009 - 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include "resources.h"
#include "udpclient.h"


gboolean message_error_nonmodal(gpointer data)
{
	GtkWidget *dialog;
	GError *error;
	gchar *message;
	
	if ((error = (GError *)data) == NULL) return FALSE;
		
	switch (error->code) {
		case GSF_UDPCLIENT_ERROR: {
			message = g_strdup_printf(_("<b>Network connection status</b>\n\n%s"), error->message, NULL);
			break;
		}
		
		default: {
			message = g_strdup_printf(_("<b>Undefined error</b>\n\n%s"), error->message, NULL);
			break;
		}
	}
			
	dialog = gtk_message_dialog_new_with_markup(NULL,
                                                GTK_DIALOG_DESTROY_WITH_PARENT,
                                                GTK_MESSAGE_ERROR,
                                                GTK_BUTTONS_CLOSE,
                                                message,
									            NULL);
	gtk_widget_show_all(dialog);
	g_signal_connect_swapped (dialog, "response", G_CALLBACK (gtk_widget_destroy), dialog);
	g_free(message);
	if (error != NULL) printf("Error: %s\n", error->message);
	
	return FALSE;
}

gboolean dialog_error_message(gchar *text, GtkWidget *win)
{
	GtkWidget *dialog;
	dialog = gtk_message_dialog_new(GTK_WINDOW(win),
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_OK,
									text, "Error");
	gtk_window_set_title(GTK_WINDOW(dialog), "Error");
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	
	return FALSE;
}

gint dialog_about(GtkWindow *parentwin)
{
	GtkWidget *dialog;
	GdkPixbuf *logo;
	GError *error = NULL;
	
	const gchar *authors[] = {
		"Alex <alex@linuxonly.ru>",
		"Testing:",
		"Konstantin <http://vivatv.net.ru>",
		NULL
	};
	
	const gchar *documenters[] = {
		"Alex <alex@linuxonly.ru>",
		NULL
	};
	
	const gchar *artists[] = {
		"Alexandre Moore <http://sa-ki.deviantart.com>",
		"Aungkarn Sugcharoun <http://www.hawalum.com/>",
		NULL
	};
	
	const gchar *license = 
		"This program is free software: you can redistribute it and/or modify\n"
		"it under the terms of the GNU General Public License as published by\n"
		"the Free Software Foundation, either version 3 of the License, or\n"
		"(at your option) any later version.\n"
		"\n"
		"This program is distributed in the hope that it will be useful,\n"
		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
		"GNU General Public License for more details.\n"
		"\n"
		"You should have received a copy of the GNU General Public License\n"
		"along with this program.  If not, see <http://www.gnu.org/licenses/>.";
		
	dialog = gtk_about_dialog_new();
	logo = gdk_pixbuf_new_from_file (RESOURCE_MAINWINDOW_MAINICON, &error);
	if (error == NULL) {
		gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(dialog), logo);
	} else {
		if (error->domain == GDK_PIXBUF_ERROR) {
			g_print("GdkPixbufError: %s\n", error->message);
		} else if (error->domain == G_FILE_ERROR) {
			g_print ("GFileError: %s\n", error->message);
		} else {
			g_print ("An error in the domain: %d has occurred!\n", error->domain);
			g_error_free (error);
		}
	}
	
	#if GTK_CHECK_VERSION(2, 12, 0)
		gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(dialog), "GSF");
	#else
		gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(dialog), "GSF");
	#endif
	
	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog), "0.2.3");
	gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG(dialog), "(C) 2009-2012 Alex");
	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(dialog), "Software satfinder for GNU/Linux");
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		gtk_about_dialog_set_license_type(GTK_ABOUT_DIALOG(dialog), GTK_LICENSE_GPL_3_0);
	#endif
	gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(dialog), license);
		
	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(dialog), "http://linuxonly.ru");
	gtk_about_dialog_set_website_label(GTK_ABOUT_DIALOG(dialog), _("Homepage"));
	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(dialog), authors);
	gtk_about_dialog_set_artists(GTK_ABOUT_DIALOG(dialog), artists);
	gtk_about_dialog_set_documenters(GTK_ABOUT_DIALOG(dialog), documenters);
	gtk_about_dialog_set_translator_credits(GTK_ABOUT_DIALOG(dialog), _("No translations yet"));
	
	gtk_window_set_transient_for(GTK_WINDOW(dialog), parentwin);
	
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	
	return FALSE;
}
