/*
 *      dvbname.c
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <dirent.h>
#include "gsfcore.h"
#include "dvbname.h"


static char *dvbname_get_sysfs_string(char *str, char *param, char *buffer, ssize_t bufsize);
static int dvbname_get_devfs_device_num(char *fname);
static void dvbname_free_devices_foreach(gpointer data, gpointer user_data);


static char *dvbname_get_sysfs_string(char *str, char *param, char *buffer, ssize_t bufsize)
{
	char *first, *second;
	ssize_t linklen;
	
	if ((str == NULL) || (buffer == NULL)) return NULL;
	
	if ((param != NULL) && ((first = strstr(str, param)) != NULL)) {
		first += strlen(param);
	} else {
		if ((str[0] == '0') && (str[1] == 'x')) first = str + 2; else first = str;
	}
	
	if ((second = strstr(first+1, "\n")) != NULL) {
		if ((linklen = second - first) > 0) {
			if (linklen > bufsize) linklen = bufsize;
			strncpy(buffer, first, linklen);
			buffer[linklen] = '\0';
			
			return buffer;
		}
	}
	
	return NULL;
}

static int dvbname_get_devfs_device_num(char *fname)
{
	char numbuf[16];
	int i = 0, numptr = 0;
	
	while ((fname[i] != '\0') && (numptr < 15)) {
		if (isdigit(fname[i])) {
			numbuf[numptr] = fname[i];
			numptr++;
		}
		i++;
	}
	numbuf[numptr] = '\0';
	
	return atoi(numbuf);
}

static void dvbname_free_devices_foreach(gpointer data, gpointer user_data)
{
	struct _device *device = (struct _device *)data;
	
	g_free(device);
}

gboolean dvbname_get_device_info(guint index, struct _device *device)
{
	struct stat statbuf;
	char link[256], path[256];
	ssize_t linklen;
	char *first, *second;
	int numsym;
	int usbdevice = 0;
	char deviceid[16], vendorid[16], value1[16], value2[16], driver[256], slot[256];
	char *pcisfiles[6][3] = {{"device", NULL, (char *)&deviceid}, {"vendor", NULL, (char *)&vendorid}, {"subsystem_device", NULL, (char *)&value1}, {"subsystem_vendor", NULL, (char *)&value2}, {"uevent", "DRIVER=", (char *)&driver}, {"uevent", "PCI_SLOT_NAME=", (char *)&slot}};
	char *usbsfiles[6][3] = {{"idProduct", NULL, (char *)&deviceid}, {"idVendor", NULL, (char *)&vendorid}, {"busnum", NULL, (char *)&value1}, {"devnum", NULL, (char *)&value2}, {"uevent", "DRIVER=", (char *)&driver}, {NULL, NULL, (char *)&slot}};
	int sfilefd;
	int i;
	char sfilebuf[256];
	char *pcidbfiles[2] = {"/usr/share/hwdata/pci.ids", "/usr/share/misc/pci.ids"};
	char *usbdbfiles[2] = {"/usr/share/hwdata/usb.ids", "/usr/share/misc/usb.ids"};
	int dbfound = 0;
	FILE *dbfile;
	char dbentry[4096];
	int vendorfound = 0;
	int devicefound = 0;
	char vendorname[256], devicename[256];
	gboolean result = FALSE;
	
	
	memset((char *)&path, 0, sizeof(path));
	snprintf((char *)&path, sizeof(path), "/dev/dvb/adapter%u/frontend%u", index, 0);
	
	if (stat((char *)&path, &statbuf) >= 0) {
		snprintf((char *)&link, sizeof(link), "/sys/dev/char/%i:%i", major(statbuf.st_rdev), minor(statbuf.st_rdev));
		linklen = readlink((char *)&link, (char *)&link, sizeof(link));
		link[linklen] = '\0';
		
		if ((first = strstr((char *)&link, "devices")) != NULL) {
			if ((second = strstr(first+1, "dvb")) != NULL) {
				if ((linklen = second - first - 1) > 0) {
					strncpy((char *)&link, first, linklen);
					link[linklen] = '\0';
					
					if (strstr((char *)&link, "usb") != NULL) {
						if ((first = strrchr((char *)&link, '/')) != NULL) {
							linklen -= strlen(first);
							link[linklen] = '\0';
						}
						usbdevice = 1;
					}
					
					memset((char *)&path, 0, sizeof(path));
					snprintf((char *)&path, sizeof(path), "/sys/%s", link);
					
					if (usbdevice) {
						for (i=0; i<=4; i++) {
							memset((char *)&link, 0, sizeof(link));
							snprintf((char *)&link, sizeof(link), "%s/%s", path, usbsfiles[i][0]);
							if ((sfilefd = open(link, O_RDONLY)) >= 0) {
								memset(&sfilebuf, 0, sizeof(sfilebuf));
								read(sfilefd, &sfilebuf, sizeof(sfilebuf));
								close(sfilefd);
								dvbname_get_sysfs_string(sfilebuf, usbsfiles[i][1], usbsfiles[i][2], SYSFS_GET_BUFSIZE(i));
							}
						}
						snprintf(usbsfiles[5][2], SYSFS_GET_BUFSIZE(5), "%s:%s", value1, value2);
					} else {
						for (i=0; i<=5; i++) {
							memset((char *)&link, 0, sizeof(link));
							snprintf((char *)&link, sizeof(link), "%s/%s", path, pcisfiles[i][0]);
							if ((sfilefd = open(link, O_RDONLY)) >= 0) {
								memset(&sfilebuf, 0, sizeof(sfilebuf));
								read(sfilefd, &sfilebuf, sizeof(sfilebuf));
								close(sfilefd);
								dvbname_get_sysfs_string(sfilebuf, pcisfiles[i][1], pcisfiles[i][2], SYSFS_GET_BUFSIZE(i));
							}
						}
					}
					
					dbfound = 0;
					
					for (i=0; i<=1; i++) {
						memset((char *)&path, 0, sizeof(path));
						if (usbdevice) {
							strncpy((char *)&path, usbdbfiles[i], sizeof(path)); 
						} else {
							strncpy((char *)&path, pcidbfiles[i], sizeof(path)); 
						}
						if (stat((char *)&path, &statbuf) >= 0) {
							while (S_ISLNK(statbuf.st_mode)) {
								linklen = readlink((char *)&path, (char *)&path, sizeof(path));
								path[linklen] = '\0';
								memset(&statbuf, 0, sizeof(statbuf));
								if (stat((char *)&path, &statbuf) < 0) break;
							}
							if (S_ISREG(statbuf.st_mode)) {
								dbfound = 1;
								break;
							}
						}					
					}
					
					memset((char *)&vendorname, 0, sizeof(vendorname));
					memset((char *)&devicename, 0, sizeof(devicename));
					
					if (dbfound) {
						if ((dbfile = fopen(path, "r")) != NULL) {
							while (fgets(dbentry, 4096, dbfile) != NULL) {
								if (strncmp((char *)&dbentry, (char *)&vendorid, 4) == 0) {
									strncpy((char *)&vendorname, (char *)&dbentry + SYSFS_DB_LEVEL1_OFFSET, strlen(dbentry) - SYSFS_DB_LEVEL1_OFFSET - 1); 
									vendorfound = 1;
									break;
								}
							}
							
							if (vendorfound) {
								while (fgets(dbentry, 4096, dbfile) != NULL) {
									if (dbentry[0] != '\t') break;
									if (strncmp((char *)&dbentry+1, (char *)&deviceid, 4) == 0) {
										strncpy((char *)&devicename, (char *)&dbentry + SYSFS_DB_LEVEL2_OFFSET, strlen(dbentry) - SYSFS_DB_LEVEL2_OFFSET - 1);
										devicefound = 1;
										break;
									}
								}
							}
							
							fclose(dbfile);
						}
					}
					
					if (!devicefound) snprintf((char *)&vendorname, sizeof(vendorname), "Unknown 0x%s", vendorid);
					if (!vendorfound) snprintf((char *)&devicename, sizeof(devicename), "Unknown 0x%s", deviceid);
						
					strncpy((char *)&device->name, (char *)&devicename, sizeof(device->name));
					strncpy((char *)&device->vendor, (char *)&vendorname, sizeof(device->vendor));
					strncpy((char *)&device->driver, (char *)&driver, sizeof(device->driver));
					strncpy((char *)&device->slot, (char *)&slot, sizeof(device->slot));
					if (usbdevice) device->bus = DVBNAME_DEVICE_BUS_USB; else device->bus = DVBNAME_DEVICE_BUS_PCI;
					sscanf(vendorid, "%X", &device->vendorid);
					sscanf(deviceid, "%X", &device->deviceid);
					sscanf(value1, "%X", &device->value1);
					sscanf(value2, "%X", &device->value2);
					device->adapter = index;
					device->frontend = 0;
					
					result = TRUE;
				}
			}
		}
	}
	
	return result; 
}

GSList *dvbname_scan_devices(void)
{
	GSList *devices = NULL;
	DIR *adaptersdir, *devicesdir;
	struct dirent *adapterd, *deviced;
	gchar adapterpath[PATH_MAX], frontendpath[PATH_MAX];
	struct _device devinfo;
	struct _device *device;
	
	if ((adaptersdir = opendir("/dev/dvb")) != NULL) {
		while (adapterd = readdir(adaptersdir)) {
			if (strstr(adapterd->d_name, "adapter") != NULL) {
				if (dvbname_get_device_info(dvbname_get_devfs_device_num(adapterd->d_name), &devinfo)) {
					printf("[+] DVB device: %s (%s) [0x%x:0x%x]\n", devinfo.name, devinfo.vendor, devinfo.deviceid, devinfo.vendorid);
					memset((char *)&adapterpath, 0, sizeof(adapterpath));
					snprintf((char *)&adapterpath, sizeof(adapterpath), "/dev/dvb/%s", adapterd->d_name);
					if ((devicesdir = opendir(adapterpath)) != NULL) {
						while (deviced = readdir(devicesdir)) {
							if (strstr(deviced->d_name, "frontend") != NULL) {
								memset((char *)&frontendpath, 0, sizeof(frontendpath));
								snprintf(frontendpath, sizeof(frontendpath), "%s/%s", adapterpath, deviced->d_name);
								printf("[+] Frontend ID: %i\n", dvbname_get_devfs_device_num(deviced->d_name));
								device = g_malloc0(sizeof(struct _device));
								memcpy(device, &devinfo, sizeof(struct _device));
								device->frontend = dvbname_get_devfs_device_num(deviced->d_name);
								devices = g_slist_prepend(devices, device);
							}
						}
						closedir(devicesdir);
					}
				}
			}
		}
	}
	
	closedir(adaptersdir);
	
	devices = g_slist_reverse(devices);
	
	return devices;	
}

void dvbname_free_devices(GSList **devices)
{
	GSList *devlist = (GSList *)*devices;
	
	if (devlist == NULL) return;
	
	g_slist_foreach((GSList *)devlist, (GFunc)dvbname_free_devices_foreach, NULL);		
}

gchar *dvbname_get_frontend_filename(gint adapter, gint frontend)
{
	gchar fname[256];
	gchar *res;
	
	sprintf(fname,FRONTEND_TEMPLATE,adapter,frontend);
	res = strdup(fname);
	
	return res;
}
