/*
 *      dvbscan.h
 *      
 *      Copyright 2009 - 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __DVBSCAN_H__
#define __DVBSCAN_H__

#include <glib.h>

#define DVBSCAN_SECTION_HEADER_LENGTH  8
#define DVBSCAN_SECTION_CRC_LENGTH     4

#define DVBSCAN_PAT_READ_TIMEOUT       20
#define DVBSCAN_PMT_READ_TIMEOUT       5
#define DVBSCAN_SDT_READ_TIMEOUT       20
#define DVBSCAN_NIT_READ_TIMEOUT       25
#define DVBSCAN_TDT_READ_TIMEOUT       35

#define DVBSCAN_SERVICE_LANG_ID        16

#define DVBSCAN_EIT_READ_TIMEOUT       15

enum {
	DVBSCAN_STATUS_PAT = 0,
	DVBSCAN_STATUS_PMT,
	DVBSCAN_STATUS_SDT,
	DVBSCAN_STATUS_NIT,
	DVBSCAN_STATUS_TDT,
	DVBSCAN_STATUS_NULL
};

enum {
	DVBSCAN_ENTRY_SERVICE = 0,
	DVBSCAN_ENTRY_TRANSPONDER,
	DVBSCAN_ENTRY_UNKNOWN
};

enum {
	DVBSCAN_EPG_STATUS_CURRENT = 0,
	DVBSCAN_EPG_STATUS_FULL,
	DVBSCAN_EPG_STATUS_NULL
};

enum {
	DVBSCAN_TYPE_TRANSPONDER = 0,
	DVBSCAN_TYPE_EPG
};

struct _srcservice {
	gint     sid;
	gushort  pmt;
	gushort  pcr;
	gushort  vpid;
	gushort  svclang;
	gushort  apid[16];
	gushort  lang[16];
	gushort  caid[16];
	gushort  dpid[16];
	gushort  ttpid;
	gushort  stpid;
	gushort  ac3pid;
	guchar   type;
	guchar   status;
	gchar    *name;
	gchar    *provider;
};

struct _netentry {
	gint     position;
	guint    frequency;
	guint    sr;
	guint8   polarisation;
	guint8   modulation;
	guint8   system;
	guint8   rolloff;
	guint8   fec;
};

union _uscanentry {
	struct _srcservice service;
	struct _netentry transponder;
};

struct _scanentry {
	guint8   type;
	gulong   id;
	gboolean sticky;
	union _uscanentry values;
};

struct _scanevent {
	guint16  id;
	gboolean current;
	time_t   start;
	gdouble  duration;
	gchar    language[4];
	gchar    *name;
	gchar    *description;
};

typedef void (*dvbscan_status_callback)(gint status);

struct _dvbscan {
	gint filterfd;
	gint ctlfd[2];
	guint adapter;
	guint demux;
	guint sid;
	guint type;
	guint status;
	guint command;
	GSList *scanlist;
	GSList *eventlist;
	gchar *networkname;
	struct tm *timestamp;
	dvbscan_status_callback callback;
	GThread *thread;
};

typedef struct _dvbscan * dvbscan_t;


gpointer dvbscan_search_entry(dvbscan_t instance, guint id);
gpointer dvbscan_search_entry_sid(dvbscan_t instance, guint sid);
guint dvbscan_count_enabled(dvbscan_t instance);
GSList *dvbscan_get_list(dvbscan_t instance);
gchar *dvbscan_get_network_name(dvbscan_t instance);
struct tm *dvbscan_get_tptime(dvbscan_t instance);
gchar *dvbscan_get_cas_name(guint caid);
dvbscan_t dvbscan_new(guint datasrc[3], gint type, dvbscan_status_callback callback);
gboolean dvbscan_free(dvbscan_t instance);
GSList *dvbscan_get_event_list(dvbscan_t instance);

#endif /* __DVBSCAN_H__ */
