/*
 *      dvbstatus.c
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include "gsfcore.h"
#include "dbstorage.h"
#include "resources.h"

#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <string.h>
#include <math.h>

#include <sys/ipc.h>
#include <sys/shm.h>
#include "gsfprivileged/gsfshm.h"

#include "satscanwindow.h"
#include "dvbstatus.h"
#include "gsfcore.h"

#include <linux/dvb/frontend.h>
#include <linux/dvb/dmx.h>

#define FRONTENDDEVICE "/dev/dvb/adapter%d/frontend%d\0"
#define DEMUXERDEVICE "/dev/dvb/adapter%d/demux%d\0"


static fec_val[] = {FEC_AUTO, FEC_1_2, FEC_2_3, FEC_3_4, FEC_3_5, FEC_4_5, FEC_5_6, FEC_6_7, FEC_7_8, FEC_8_9, FEC_9_10};
static modulation_val[] = {QPSK, PSK_8};
static system_val[] = {SYS_DVBS, SYS_DVBS2};
static rolloff_val[] = {ROLLOFF_AUTO, ROLLOFF_35, ROLLOFF_25, ROLLOFF_20};


//static gboolean dvb_calculate_if(struct _frontend *frontend, struct _service *svc);
static gdouble dvb_send_positioner_command(struct _frontend *frontend, struct _service *svc);
static void dvb_send_diseqc_command(struct _frontend *frontend, struct _service *svc);

void dvb_mediastreams_free_foreach(struct _mediastream *se, gpointer data);


void dvb_mediastreams_free_foreach(struct _mediastream *se, gpointer data)
{
	g_free(se);
}

gboolean dvb_netfilter_open(struct _frontend *frontend, struct _demuxer *demuxer, struct _service *svc)
{
	key_t shmkey;
	int shmid;
	struct srequest *shmblock;
	gchar **clientargv;
	gint client_status;
	GError *client_error = NULL;
	struct stat statinfo;
	gchar **processargv;
	GError *process_error = NULL;
	gchar *workdir;
	
	if ((svc->npid > 0) && (demuxer->netif_nr == -1)) {
		shmkey = ftok(PRIVILEGED_TOKEN_FILE, PRIVILEGED_PROJID);
		shmid = shmget(shmkey, sizeof(struct srequest), 0666 | IPC_CREAT);
		shmblock = (struct srequest *)shmat(shmid, 0, 0);
		
		shmblock->action = PRIVILEGED_ACTION_START_NET;
		shmblock->adapter = frontend->adapter_nr;
		shmblock->frontend = frontend->frontend_nr;
		shmblock->netpid = svc->npid;
		if (svc->nip != NULL)	strncpy(shmblock->ipaddress, svc->nip, (strlen(svc->nip) >= 16) ? 16 : strlen(svc->nip));
		if (svc->nmask != NULL) strncpy(shmblock->netmask, svc->nmask, (strlen(svc->nmask) >= 16) ? 16 : strlen(svc->nmask));
		if (svc->nmac != NULL) strncpy(shmblock->macaddress, svc->nmac, (strlen(svc->nmac) >= 18) ? 18 : strlen(svc->nmac));
		shmblock->interface = 0;
		
		clientargv = (gchar **)g_malloc(sizeof(gchar *));
				
		clientargv[0] = g_strdup(RESOURCE_BINARY_PRIVILEGED); 
		clientargv[1] = NULL;
		if (!g_spawn_sync(NULL, clientargv, NULL, G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL | G_SPAWN_FILE_AND_ARGV_ZERO, NULL, NULL, NULL, NULL, &client_status, &client_error)) {
			printf("[-] Error while executing client process: %s\n", client_error->message);
			g_error_free(client_error);
			g_strfreev(clientargv);
			demuxer->netif_nr = -1;
			shmdt(shmblock);
			shmctl(shmid, IPC_RMID, 0);
			return FALSE;
		}
		g_strfreev(clientargv);
		
		client_status = WEXITSTATUS(client_status);
		switch(client_status) {
			case 0: demuxer->netif_nr = shmblock->interface; printf("[+] Set network interface number %i\n", shmblock->interface); break;
			case 1: demuxer->netif_nr = -1; printf("[-] Client error status: %i\n", client_status); break;
			default: printf("[-] Client error status: %i\n", client_status); break;
		}
		
		shmdt(shmblock);
		shmctl(shmid, IPC_RMID, 0);
	}
	
	if ((svc->nprog != NULL) && (g_file_test(svc->nprog, G_FILE_TEST_IS_EXECUTABLE))) {
		if (g_stat(svc->nprog, &statinfo) == 0) {
			if (((statinfo.st_mode & S_IXOTH) == 0) && ((statinfo.st_mode & S_IXGRP) == 0) && ((statinfo.st_mode & S_IXUSR) == 0)) return FALSE; 
			processargv = (gchar **)g_malloc(sizeof(gchar *));
			workdir = g_strndup(svc->nprog, g_strrstr(svc->nprog, "/") - svc->nprog);
			
			processargv[0] = g_strdup(svc->nprog);
			processargv[1] = NULL;
			if (!g_spawn_async(workdir, processargv, NULL, G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL, NULL, NULL, &demuxer->nprocesspid, &process_error)) {
				printf("[-] Error while running process: %s\n", process_error->message);
				g_error_free(process_error);
				g_strfreev(processargv);
				g_free(workdir);
				demuxer->nprocesspid = -1;
				return FALSE;
			}
			g_strfreev(processargv);
			g_free(workdir);
			
			demuxer->nprocessfn = g_strdup(svc->nprog);
			
			printf("[+] Started user - defined process %s, pid %i\n", svc->nprog, demuxer->nprocesspid);
		}
	}
	
	return TRUE;
}


gboolean dvb_netfilter_close(struct _frontend *frontend, struct _demuxer *demuxer, struct _service *svc)
{
	key_t shmkey;
	int shmid;
	struct srequest *shmblock;
	gchar **clientargv;
	gint client_status;
	GError *client_error = NULL;
	gchar buf[PATH_MAX];
	GDir *procdir;
	const gchar *procname;
	gchar *fullpath;
		
	
	if ((demuxer->netif_nr) >= 0) {
		shmkey = ftok(PRIVILEGED_TOKEN_FILE, PRIVILEGED_PROJID);
		shmid = shmget(shmkey, sizeof(struct srequest), 0666 | IPC_CREAT);
		shmblock = (struct srequest *)shmat(shmid, 0, 0);
		
		shmblock->action = PRIVILEGED_ACTION_STOP_NET;
		shmblock->adapter = frontend->adapter_nr;
		shmblock->frontend = frontend->frontend_nr;
		shmblock->interface = demuxer->netif_nr;
		shmblock->netpid = -1;
		memset(shmblock->ipaddress, 0, sizeof(shmblock->ipaddress));
		memset(shmblock->netmask, 0, sizeof(shmblock->netmask));
		memset(shmblock->macaddress, 0, sizeof(shmblock->macaddress));
		
		clientargv = (gchar **)g_malloc(sizeof(gchar *));
		
		clientargv[0] = g_strdup(RESOURCE_BINARY_PRIVILEGED); 
		clientargv[1] = NULL;
		if (!g_spawn_sync(NULL, clientargv, NULL, G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL | G_SPAWN_FILE_AND_ARGV_ZERO, NULL, NULL, NULL, NULL, &client_status, &client_error)) {
			printf("[-] Error while executing client process: %s\n", client_error->message);
			g_error_free(client_error);
			g_strfreev(clientargv);
			demuxer->netif_nr = -1;
			shmdt(shmblock);
			shmctl(shmid, IPC_RMID, 0);
			return FALSE;
		}
		g_strfreev(clientargv);
		
		client_status = WEXITSTATUS(client_status);
		switch(client_status) {
			case 0: demuxer->netif_nr = -1; printf("[+] Removed network interface number %i\n", shmblock->interface); break;
			case 1:  printf("[-] Client error status: %i\n", client_status); break;
			default: printf("[-] Client error status: %i\n", client_status); break;
		}
		
		shmdt(shmblock);
		shmctl(shmid, IPC_RMID, 0);
	}
	
	if ((demuxer->nprocessfn != NULL) && (g_file_test(demuxer->nprocessfn, G_FILE_TEST_IS_EXECUTABLE))) {
		if (demuxer->nprocesspid > 0) {
			memset(buf, 0, sizeof(buf));
			g_sprintf(buf, "/proc/%i", demuxer->nprocesspid);
			if (access(buf, F_OK) ==  0) {
				kill(demuxer->nprocesspid, SIGKILL);
				printf("[+] Stopped user - defined process %s, pid %i\n", demuxer->nprocessfn, demuxer->nprocesspid);
			}
		}
		
		if ((procdir = g_dir_open("/proc", 0, NULL)) != NULL) {
			while ((procname = g_dir_read_name(procdir)) != NULL) {
				fullpath = g_strconcat("/proc/", procname, "/exe", NULL);
				if ((g_file_test(fullpath, G_FILE_TEST_EXISTS)) && (g_str_equal(demuxer->nprocessfn, g_file_read_link(fullpath, NULL)))) {
					kill(atoi(procname), SIGKILL);
					printf("[+] Stopped user - defined process %s, pid %i\n", demuxer->nprocessfn, atoi(procname));
				}
				g_free(fullpath);
			}
			g_dir_close(procdir);
			g_free(demuxer->nprocessfn);
		}
			
	}
	
	return TRUE;
}

void dvb_demux_close_foreach(gpointer data, gpointer user_data)
{
	struct _mediastream *ms = (struct _mediastream *)data;
	
	if ((ms->enabled) && (ms->demuxerpid >= 0)) {
		if (ioctl(ms->demuxerpid, DMX_STOP) < 0) {
			fprintf(stderr, "DMX_STOP failed (PID = 0x%04x): %d %m\n", ms->value, errno);
		}
		close(ms->demuxerpid);
		ms->demuxerpid = -1;
	}
}

gboolean dvb_demux_close(struct _frontend *frontend, struct _demuxer *demuxer, struct _service *svc)
{
	gchar dev_fn[256];
	
	if ((frontend->open_mode_readonly) || (svc == NULL)) return TRUE;
	
	g_sprintf(dev_fn, DEMUXERDEVICE, frontend->adapter_nr, frontend->frontend_nr);
	
	if (g_access(dev_fn, W_OK) != 0) {
		printf("[-] No wrie access to demux device %s\n", dev_fn);
		return FALSE;
	}
	
	if (svc->type == SERVICE_TV_RADIO) {
		if (demuxer->mediastreams != NULL) {
			g_slist_foreach(demuxer->mediastreams, (GFunc)dvb_demux_close_foreach, &dev_fn);
			g_slist_foreach(demuxer->mediastreams, (GFunc)dvb_mediastreams_free_foreach, NULL);
			g_slist_free(demuxer->mediastreams);
			demuxer->mediastreams = NULL;
		}		
	} else if (svc->type == SERVICE_DATA) {
		dvb_netfilter_close(frontend, demuxer, svc);
	}
	
	return TRUE;
}

void dvb_demux_open_foreach(gpointer data, gpointer user_data)
{
	struct _mediastream *ms = (struct _mediastream *)data;
	gchar *dev_fn = (gchar *)user_data;
	struct dmx_pes_filter_params pesfilter;
	gint abuffersize = 64 * 1024;
	gint vbuffersize = 64 * 1024;
	
	if (!ms->enabled) return;
	
	if ((ms->demuxerpid = open(dev_fn, O_RDWR | O_NONBLOCK)) >= 0) {
		
		switch (ms->type) {
			case PID_TYPE_VIDEO: {
				if (ioctl(ms->demuxerpid, DMX_SET_BUFFER_SIZE, abuffersize) == -1) {
					perror("DMX_SET_BUFFER_SIZE failed");
				}
				
				memset(&pesfilter, 0, sizeof(pesfilter));
				pesfilter.pid = ms->value;
				pesfilter.input = DMX_IN_FRONTEND;
				pesfilter.output = DMX_OUT_TS_TAP;
				pesfilter.pes_type = DMX_PES_OTHER;
				pesfilter.flags = DMX_IMMEDIATE_START;
				
				if (ioctl(ms->demuxerpid, DMX_SET_PES_FILTER, &pesfilter) < 0) {
					fprintf(stderr, "DMX_SET_PES_FILTER failed (PID = 0x%04x): %d %m\n", ms->value, errno);
					close(ms->demuxerpid);
					ms->demuxerpid = -1;
				}
				
				break;
			}
			
			case PID_TYPE_AUDIO: {
				if (ioctl(ms->demuxerpid, DMX_SET_BUFFER_SIZE, vbuffersize) == -1) {
					perror("DMX_SET_BUFFER_SIZE failed");
				}
				
				memset(&pesfilter, 0, sizeof(pesfilter));
				pesfilter.pid = ms->value;
				pesfilter.input = DMX_IN_FRONTEND;
				pesfilter.output = DMX_OUT_TS_TAP;
				pesfilter.pes_type = DMX_PES_OTHER;
				pesfilter.flags = DMX_IMMEDIATE_START;
				
				if (ioctl(ms->demuxerpid, DMX_SET_PES_FILTER, &pesfilter) < 0) {
					fprintf(stderr, "DMX_SET_PES_FILTER failed (PID = 0x%04x): %d %m\n", ms->value, errno);
					close(ms->demuxerpid);
					ms->demuxerpid = -1;
				}
								
				break;
			}
			
			default: {
				close(ms->demuxerpid);
				ms->demuxerpid = -1;
				break;
			}
			
		}
		
	} else {
		printf("[-] Failed to open demux device %s\n", dev_fn);
	}
}

gboolean dvb_demux_open(struct _frontend *frontend, struct _demuxer *demuxer, struct _service *svc)
{
	gchar dev_fn[256];
	guint i, len = 0;
	struct _pid *oldpid;
	struct _mediastream *ms;
	
	if ((frontend->open_mode_readonly) || (svc == NULL)) return FALSE;
	
	g_sprintf(dev_fn, DEMUXERDEVICE, frontend->adapter_nr, frontend->frontend_nr);
	
	if (g_access(dev_fn, W_OK) != 0) {
		printf("[-] No wrie access to demux device %s\n", dev_fn);
		return FALSE;
	}
		
	if (svc->type == SERVICE_TV_RADIO) {
		if (svc->pids != NULL) {
			len = g_slist_length(svc->pids);
			for (i=0; i<len; i++) {
				oldpid = (struct _pid *)g_slist_nth_data(svc->pids, i);
				if ((oldpid->type == PID_TYPE_AUDIO) || (oldpid->type == PID_TYPE_VIDEO)) {
					ms = (struct _mediastream *)g_malloc0(sizeof(struct _mediastream));
					ms->type = oldpid->type;
					ms->value = oldpid->value;
					ms->demuxerpid = -1;
					ms->enabled = oldpid->enabled;
					demuxer->mediastreams = g_slist_prepend(demuxer->mediastreams, (gpointer)ms);
				}			
			}
			demuxer->mediastreams = g_slist_reverse(demuxer->mediastreams);
			g_slist_foreach(demuxer->mediastreams, (GFunc)dvb_demux_open_foreach, &dev_fn);
		}
	} else if (svc->type == SERVICE_DATA) {
		dvb_netfilter_open(frontend, demuxer, svc);
	}
		
	
	return TRUE;
}

gint dvb_calculate_values(struct _frontend *frontend, struct _service *svc, gboolean test)
{
	gint ifreq = 0;
	gboolean needtone = FALSE;
	
	if ((frontend == NULL) || (svc == NULL)) return DVBSTATUS_VALUES_ALL_WRONG;
	
	if (!test) {
		frontend->ifreq = 0;
		frontend->conttone = FALSE;
	}
	
	if ((svc->sr < (frontend->symbol_rate_min / 1000)) || (svc->sr > (frontend->symbol_rate_max / 1000))) {
		//Symbol rate out of demodulator supported range
		return DVBSTATUS_VALUES_SR_WRONG;
	}
	
	if (svc->lnbtype == LNB_TYPE_AUTO) {
		if (svc->freq > 4200) {
			if (svc->freq >= 11700) {
				ifreq = svc->freq - 10600;
				needtone = TRUE;
			} else if (svc->freq < 9750) {
				ifreq = 9750 - svc->freq;
			} else {
				ifreq = svc->freq - 9750;
			}
		} else {
			ifreq = 5150 - svc->freq;
		}
	} else if (svc->lnbtype == LNB_TYPE_KU_UNIVERSAL) {
		if (svc->freq >= 11700) {
			ifreq = svc->freq - 10600;
			needtone = TRUE;
		} else if (svc->freq < 9750) {
			ifreq = 9750 - svc->freq;
		} else {
			ifreq = svc->freq - 9750;
		}
	} else if (svc->lnbtype == LNB_TYPE_KU_WIDEBAND) {
		if (svc->freq >= 11700) {
			ifreq = svc->freq - 10750;
			needtone = TRUE;
		} else if (svc->freq < 9750) {
			ifreq = 9750 - svc->freq;
		} else {
			ifreq = svc->freq - 9750;
		}
	} else if (svc->lnbtype == LNB_TYPE_KU_CIRCULAR) {
		ifreq = svc->freq - 10750;
		needtone = TRUE;
	} else if (svc->lnbtype == LNB_TYPE_C_STANDARD) {
		ifreq = 5150 - svc->freq;
	} else if (svc->lnbtype == LNB_TYPE_C_SINGLE_SOLUTION) {
		ifreq = 5750 - svc->freq;
	} else if (svc->lnbtype == LNB_TYPE_CUSTOM) {
		if (svc->freq >= svc->lnbswfreq) {
			ifreq = svc->freq - svc->lnbhifreq;
			needtone = TRUE;
		} else if (svc->freq < svc->lnblofreq) {
			ifreq = svc->lnblofreq - svc->freq;
		} else {
			ifreq = svc->freq - svc->lnblofreq;
		}
	}
	
	if ((ifreq < frontend->frequency_min/1000) || (ifreq > frontend->frequency_max/1000)) {
		//IF out of tuner supported range
		return DVBSTATUS_VALUES_IF_WRONG;
	}
	
	if (!test) {
		frontend->ifreq = ifreq;
		frontend->conttone = needtone;
	}
	
	return DVBSTATUS_VALUES_ALL_GOOD;
}

static gdouble dvb_send_positioner_command(struct _frontend *frontend, struct _service *svc)
{
	gdouble longitude, latitude, azimuth, elevation, hourangle, waittime;
	guint command;
	gint movetable[10] = {0x00, 0x02, 0x03, 0x05, 0x06, 0x08, 0x0A, 0x0B, 0x0D, 0x0E};
	struct dvb_diseqc_master_cmd pos_command_position = {{0xe0, 0x10, 0x6e, 0x00, 0x00, 0x00}, 5};
	struct dvb_diseqc_master_cmd pos_command_number   = {{0xe0, 0x31, 0x6b, 0x00, 0x00, 0x00}, 4};
	
	waittime = (fabs((svc->pos/10.0)-(frontend->lastposition/10.0)))/10.0;
	if (waittime < 0.0) {
		waittime = 0.0;
	} else if (waittime > 60.0) {
		waittime = 60.0;
	}
		
	if (svc->positioner == POSITIONER_POSITION) {
		gsfcore_get_location(&longitude, &latitude);
		azimuth = calculations_azimuth(longitude, latitude, svc->pos/10.0, 200.0);
		elevation = calculations_elevation(longitude, latitude, svc->pos/10.0, 200.0);
		hourangle = calculations_hourangle(latitude, azimuth, elevation);
		if (latitude >= 0) {
			command = (guint)round(fabs(180.0-hourangle)*10.0);
			command = (command/10)*16 + movetable[command%10];
			if (hourangle < 180) {
				command |= 0xe000;
			} else {
				command |= 0xd000;
			}
		} else {
			if (hourangle < 180) {
				command = (guint)round(fabs(hourangle)*10.0);
				command = (command/10)*16 + movetable[command%10];
				command |= 0xd000;
			} else {
				command = (guint)round(fabs(360.0-hourangle)*10.0);
				command = (command/10)*16 + movetable[command%10];
				command |= 0xe000;
			}
		}
		if (ioctl(frontend->frontend_fd, FE_SET_VOLTAGE, SEC_VOLTAGE_18) == -1) {
			perror("[-] High voltage for positioner setting failed");
		}
		pos_command_position.msg[3] = (command & 0xff00) >> 8;
		pos_command_position.msg[4] = (command & 0x00ff);
		if (ioctl(frontend->frontend_fd, FE_DISEQC_SEND_MASTER_CMD, &pos_command_position) == -1) {
			perror("[-] Positiner Goto x.x command send failed");
			return 0.0;
		}
	} else if (svc->positioner == POSITIONER_NUMBER) {
		if (ioctl(frontend->frontend_fd, FE_SET_VOLTAGE, SEC_VOLTAGE_18) == -1) {
			perror("[-] High voltage for positioner setting failed");
		}
		pos_command_number.msg[3] = svc->possatnum;
		if (ioctl(frontend->frontend_fd, FE_DISEQC_SEND_MASTER_CMD, &pos_command_number) == -1) {
			perror("[-] Positiner Goto nn command send failed");
			return 0.0;
		}
	}
	
	return waittime;
}

static void dvb_send_diseqc_command(struct _frontend *frontend, struct _service *svc)
{
	gint i;
	struct dvb_diseqc_master_cmd diseqc_command11 = {{0xe0, 0x10, 0x39, 0xf0, 0x00, 0x00}, 4};
	struct dvb_diseqc_master_cmd diseqc_command10 = {{0xe0, 0x10, 0x38, 0xf0, 0x00, 0x00}, 4};
	fe_sec_mini_cmd_t tone;
	gboolean hpolarisation;
	
	if (svc->port11 != 0) {
		for (i=0; i<svc->rep11; i++) {
			diseqc_command11.msg[3] = 0xf0 | (((svc->port11 - 1) * 4) & 0x0f);
			if (ioctl(frontend->frontend_fd, FE_DISEQC_SEND_MASTER_CMD, &diseqc_command11) == -1)
				perror("FE_DISEQC_SEND_MASTER_CMD failed");
			usleep(100 * 1000);
		}
	}
	
	if (svc->port10 != 0) {
		if ((svc->pol == 1) || (svc->pol == 3)) hpolarisation = TRUE; else hpolarisation = FALSE;
		diseqc_command10.msg[3] = 0xf0 | (((svc->port10 - 1) * 4) & 0x0f) | (hpolarisation ? 2 : 0) | (frontend->conttone ? 1 : 0);
		if (ioctl(frontend->frontend_fd, FE_DISEQC_SEND_MASTER_CMD, &diseqc_command10) == -1)
			perror("FE_DISEQC_SEND_MASTER_CMD failed");
		usleep(100 * 1000);
		tone = ((svc->port10 - 1) / 4) % 2 ? SEC_MINI_B : SEC_MINI_A;
		if (ioctl(frontend->frontend_fd, FE_DISEQC_SEND_BURST, tone) == -1)
			perror("FE_DISEQC_SEND_BURST failed");
		usleep(15 * 1000);
	}
	
}

gboolean dvb_tune(struct _frontend *frontend, struct _service *svc)
{
	gdouble waittime;
	
	if (frontend->open_mode_readonly) return FALSE;
	
	printf("[+] Started tuning process\n");
	
	if (frontend->api_version > 512) {
		struct dtv_property clear_property[] = {
			{ .cmd = DTV_CLEAR },
		};
		
		struct dtv_properties clear_cmd = {
			.num = 1,
			.props = clear_property
		};
		
		if ((ioctl(frontend->frontend_fd, FE_SET_PROPERTY, &clear_cmd)) == -1) {
			perror("[-] FE_SET_PROPERTY DTV_CLEAR failed");
		}
	}
	
	if (dvb_calculate_values(frontend, svc, FALSE) != DVBSTATUS_VALUES_ALL_GOOD) {
		printf("[-] Wrong LNB type selected - unable to tune transponder\n");
		return FALSE;
	}
		
	if (ioctl(frontend->frontend_fd, FE_SET_TONE, SEC_TONE_OFF) == -1) {
		perror("[-] FE_SET_TONE failed");
		return FALSE;
	}
	
	if ((svc->positioner != POSITIONER_DIASABLED) && (svc->pos != frontend->lastposition)) {
		waittime = dvb_send_positioner_command(frontend, svc);
		if (waittime != 0.0) {
			printf("[+] Waiting %.2f sec for positioner movement\n", waittime);
			while (waittime > 0.0) {
				usleep(100000);
				waittime -= 0.1;
			}
			printf("[+] Positioner movement time passed\n");
			frontend->lastposition = svc->pos;
			usleep(20 * 1000);
		}
	}
			
	if ((svc->pol == 0) || (svc->pol == 2)) {
		if (ioctl(frontend->frontend_fd, FE_SET_VOLTAGE, SEC_VOLTAGE_13) == -1)
		perror("[-] FE_SET_VOLTAGE failed");
	} else {
		if (ioctl(frontend->frontend_fd, FE_SET_VOLTAGE, SEC_VOLTAGE_18) == -1)
		perror("[-] FE_SET_VOLTAGE failed");
	}
	
	usleep(15 * 1000);
	
	dvb_send_diseqc_command(frontend, svc);
	
	if (((frontend->conttone) && (svc->lnb22khz != 2)) || (svc->lnb22khz == 1)) {
		if (ioctl(frontend->frontend_fd, FE_SET_TONE, SEC_TONE_ON) == -1)
			perror("[-] FE_SET_TONE failed");
	}
		
	struct dvb_frontend_event ev;
	
	while (TRUE) {
		if (ioctl(frontend->frontend_fd, FE_GET_EVENT, &ev) == -1)
		break;
	}
	
	if (frontend->api_version > 512) {
		struct dtv_property p[] = {
			{ .cmd = DTV_CLEAR },
			{ .cmd = DTV_DELIVERY_SYSTEM, .u.data = system_val[svc->sys] },
			{ .cmd = DTV_FREQUENCY,		  .u.data = frontend->ifreq*1000 },
			{ .cmd = DTV_MODULATION,	  .u.data = modulation_val[svc->mod] },
			{ .cmd = DTV_SYMBOL_RATE,	  .u.data = svc->sr*1000 },
			{ .cmd = DTV_INNER_FEC,		  .u.data = fec_val[svc->fec] },
			{ .cmd = DTV_INVERSION,		  .u.data = INVERSION_AUTO },
			{ .cmd = DTV_ROLLOFF,		  .u.data = rolloff_val[svc->rof] },
			{ .cmd = DTV_PILOT,		      .u.data = PILOT_AUTO },
			{ .cmd = DTV_TUNE },
		};
		
		struct dtv_properties cmdseq = {
			.num = 10,
			.props = p
		};
		
		if ((ioctl(frontend->frontend_fd, FE_SET_PROPERTY, &cmdseq)) == -1) {
			perror("[-] Failed to tune transponder");
			return FALSE;
		}
	} else {
		struct dvb_frontend_parameters frontendparams;
		
		frontendparams.frequency = frontend->ifreq*1000;
		frontendparams.inversion = INVERSION_AUTO;
		frontendparams.u.qpsk.symbol_rate = svc->sr*1000;
		frontendparams.u.qpsk.fec_inner = fec_val[svc->fec];
		
		if ((ioctl(frontend->frontend_fd, FE_SET_FRONTEND, &frontendparams)) == -1) {
			perror("[-] Failed to tune transponder");
			return FALSE;
		}
	}
	
	frontend->locked = FALSE;
	frontend->timestamp = time(NULL);
	frontend->lockattempts = 0;
	
	printf("[+] Tuned to transponder\n");
	
	return TRUE;
}

gboolean dvb_parameters(struct _frontend *frontend, struct _service *svc)
{
	int fec_tr[] = {11, 1, 2, 3, 5, 6, 7, 8, 9, 0, 4, 10}; //translate to tuner structure format
	
	if (frontend->api_version > 512) {
		
		struct dtv_property p[] = {
			{ .cmd = DTV_FREQUENCY },
			{ .cmd = DTV_SYMBOL_RATE },
			{ .cmd = DTV_VOLTAGE },
			{ .cmd = DTV_INNER_FEC },
			{ .cmd = DTV_DELIVERY_SYSTEM },
			{ .cmd = DTV_MODULATION },
			{ .cmd = DTV_ROLLOFF },
		};
		
		struct dtv_properties p_cmd = {
			.num = 7,
			.props = p
		};
		
		if (ioctl(frontend->frontend_fd, FE_GET_PROPERTY, &p_cmd) != -1) {
			svc->freq = p[0].u.data/1000;
			svc->sr = p[1].u.data/1000;
			
			switch(p[2].u.data) {
				case SEC_VOLTAGE_13: svc->pol = 0; break;
				case SEC_VOLTAGE_18: svc->pol = 1; break;
				case SEC_VOLTAGE_OFF: svc->pol = 0; break;
				default: svc->pol = 0; break;
			}
			
			svc->fec = fec_tr[p[3].u.data];
			
			switch(p[4].u.data) {
				case SYS_DVBS: svc->sys = 0; break;
				case SYS_DVBS2: svc->sys = 1; break;
				case SYS_UNDEFINED: svc->sys = 2; break;
				default: svc->sys = 2; break;
			}
			
			switch(p[5].u.data) {
				case QPSK: svc->mod = 0; break;
				case PSK_8: svc->mod = 1; break;
				default: svc->mod = 0; break;
			}
			
			switch(p[6].u.data) {
				case ROLLOFF_AUTO: svc->rof = 0; break;
				case ROLLOFF_35: svc->rof = 1; break;
				case ROLLOFF_25: svc->rof = 2; break;
				case ROLLOFF_20: svc->rof = 3; break;
				default: svc->rof = 0; break;
			}
			
			return TRUE;
		} else return FALSE;
	} else {
		
		struct dvb_frontend_parameters frontendparams;
		
		if ((ioctl(frontend->frontend_fd, FE_GET_FRONTEND, &frontendparams)) != -1) {
			svc->freq = frontendparams.frequency/1000;
			svc->sr = frontendparams.u.qpsk.symbol_rate/1000;
			svc->pol = 0;
			svc->fec = fec_tr[frontendparams.u.qpsk.fec_inner];
			svc->sys = 0;
			svc->mod = 0;
			svc->rof = 0;
			return TRUE;
		} else return FALSE;
	}
}

gboolean dvb_status(struct _frontend *frontend, struct _service *svc, gint (*event_cb)(gint status, uint32_t freq, uint32_t sr, gint fec))
{
	fe_status_t status;
	struct dvb_frontend_event ev;
	time_t idletime;
	gdouble idleperiod;
	
	while (event_cb != NULL) {
		if (ioctl(frontend->frontend_fd, FE_GET_EVENT, &ev) != -1) {
			event_cb(ev.status, ev.parameters.frequency, ev.parameters.u.qpsk.symbol_rate, ev.parameters.u.qpsk.fec_inner); 
		} else break;
	}
	
	if (frontend->open_mode_readonly) {
		if (!frontend->tuned) frontend->tuned = TRUE;
	} else {
		if (!frontend->locked) {
			idletime = time(NULL);
			idleperiod = difftime(idletime, frontend->timestamp);
			if ((frontend->lockattempts == 0) && (idleperiod > 15)) {
				//printf("1 tune attempt\n");
				dvb_send_diseqc_command(frontend, svc);
				frontend->lockattempts++;
			} else if ((frontend->lockattempts == 1) && (idleperiod > 20)) {
				//printf("2 tune attempt\n");
				dvb_send_diseqc_command(frontend, svc);
				frontend->lockattempts++;
			} else if ((frontend->lockattempts == 2) && (idleperiod > 25)) {
				//printf("3 tune attempt\n");
				dvb_send_diseqc_command(frontend, svc);
				frontend->lockattempts++;
			}
		}
	}
	
	if (ioctl(frontend->frontend_fd, FE_READ_STATUS, &status) < 0) status = FE_HAS_VITERBI;
	if (ioctl(frontend->frontend_fd, FE_READ_SIGNAL_STRENGTH, &frontend->signal) < 0) frontend->signal = 0;
	if (ioctl(frontend->frontend_fd, FE_READ_SNR, &frontend->snr) < 0) frontend->snr = 0;
	if (ioctl(frontend->frontend_fd, FE_READ_BER, &frontend->ber) < 0) frontend->ber = 0;
	
	printf ("[+] SNR: %d | AGC: %d | BER: %d ", frontend->snr, frontend->signal, frontend->ber);
	if (status & FE_HAS_LOCK) {frontend->lock = TRUE; frontend->locked = TRUE; printf("[LOCK]");} else frontend->lock = FALSE;
	if (status & FE_HAS_CARRIER) {frontend->carrier = TRUE; printf("[CARRIER]");} else frontend->carrier = FALSE;
	printf("\n");
	
	return TRUE;
}

gboolean dvb_frontend_opened(gint fd)
{
	struct stat st;
	
	if (fstat(fd, &st) == -1) return FALSE;
	
	if (!S_ISCHR(st.st_mode)) return FALSE;
	
	return TRUE;
}

gint dvb_open(struct _frontend *frontend)
{
	gint fefd;
	struct dvb_frontend_info fe_info;
	
	if ((fefd = open(frontend->frontend_fn, O_RDWR | O_NONBLOCK)) > 0) {
		frontend->open_mode_readonly = FALSE;
		/*dvb_demux_open(tuner);*/
	} else if ((fefd = open(frontend->frontend_fn, O_RDONLY | O_NONBLOCK)) > 0) {
		frontend->open_mode_readonly = TRUE;
	} else {
		perror("[-] Opening frontend failed");
		return 0;
	}
	
	struct dtv_property p[] = {
		{ .cmd = DTV_API_VERSION },
	} ;
	
	struct dtv_properties p_cmd = {
		.num = 1,
		.props = p
	} ;
	
	if (ioctl(fefd, FE_GET_PROPERTY, &p_cmd) == 0) {
		printf("[+] LinuxTV API ver. %i.%i\n", ((p[0].u.data & 0xFF00)>>8),(p[0].u.data & 0x00FF));
		frontend->api_version = p[0].u.data;
	} else {
		frontend->api_version = 768;
		printf("[-] LinuxTV API ver. %i.%i (deprecated)\n", (frontend->api_version >> 8), (frontend->api_version & 0x00FF));
		//return 0;
	}
	
	
	if (ioctl(fefd, FE_GET_INFO, &fe_info) < 0) {
		perror("[-] Unable to get frontend information");
		close(fefd);
		return 0;
	}
	
	if (fe_info.type != FE_QPSK) {
		printf("[-] DVB-S(2) is not supported by this frontend\n");
		close(fefd);
		return 0;
	}
	
	printf("[+] Opened frontend %s ", frontend->frontend_fn);
	printf((frontend->open_mode_readonly) ? "(readonly mode)\n" : "\n");
	printf(" Driver: %s\n", fe_info.name);
	strncpy(frontend->driver,fe_info.name,128);
	printf(" Frequencies supported: %i - %i\n", fe_info.frequency_min,fe_info.frequency_max);
	frontend->frequency_min = fe_info.frequency_min;
	frontend->frequency_max = fe_info.frequency_max;
	printf(" Frequency stepsize: %i\n", fe_info.frequency_stepsize);
	frontend->frequency_stepsize = fe_info.frequency_stepsize;
	printf(" Symbolrates supported: %i - %i\n", fe_info.symbol_rate_min,fe_info.symbol_rate_max);
	frontend->symbol_rate_min = fe_info.symbol_rate_min;
	frontend->symbol_rate_max = fe_info.symbol_rate_max;
	printf(" Systems supported:");
	if (fe_info.caps & FE_CAN_QPSK) {
		frontend->dvbs_support = TRUE;
		printf(" DVB-S");
	} else frontend->dvbs_support = FALSE;
	if ((frontend->api_version > 512) && (fe_info.caps & FE_CAN_2G_MODULATION)) {
		frontend->dvbs2_support = TRUE;
		printf(" DVB-S2");
	} else frontend->dvbs2_support = FALSE;
	printf("\n");
	
	/*if (ioctl(fefd, FE_SET_FRONTEND_TUNE_MODE, FE_TUNE_MODE_ONESHOT) < 0) {
		perror("[-] Unable to get frontend tune mode");
		close(fefd);
		return 0;
	}*/
	
	return fefd;
}

gboolean dvb_close(struct _frontend *frontend)
{
	if (dvb_frontend_opened(frontend->frontend_fd)) {
		if (!frontend->open_mode_readonly) {
			struct dtv_property p[] = {
				{ .cmd = DTV_CLEAR }
			};
			
			struct dtv_properties cmdseq = {
				.num = 1,
				.props = p
			};
			
			struct dvb_frontend_event ev;
			while (TRUE) {
				if (ioctl(frontend->frontend_fd, FE_GET_EVENT, &ev) == -1)	break;
			}
			
			if (frontend->api_version > 512) {
				if ((ioctl(frontend->frontend_fd, FE_SET_PROPERTY, &cmdseq)) == -1) {
					perror("[-] Failed to clear frontend info");
				}
			}
		}
		
		close(frontend->frontend_fd);
		printf("[+] Closed frontend %s\n",frontend->frontend_fn);
		/*dvb_demux_stop(tuner);
		dvb_demux_close(tuner);*/
		
		return TRUE;
	}
	
	return FALSE;
}


gboolean dvb_satscan_init(struct _frontend *frontend, struct _transponders *transponders)
{
	guint i;
	
	if (frontend->open_mode_readonly) return FALSE;
	
	g_message("Started satscan init process");
	
	if (ioctl(frontend->frontend_fd, FE_SET_TONE, SEC_TONE_OFF) == -1) {
		g_warning("FE_SET_TONE failed");
		return FALSE;
	}
	
	usleep(15 * 1000);
	
	if (transponders->port11 != 0) {
		for (i=0; i<transponders->repeats11; i++) {
			struct dvb_diseqc_master_cmd diseqc_command11 = {{0xe0, 0x10, 0x39, 0xf0, 0x00, 0x00}, 4};
			diseqc_command11.msg[3] =	0xf0 | (((transponders->port11 - 1) * 4) & 0x0f);
			
			if (ioctl(frontend->frontend_fd, FE_DISEQC_SEND_MASTER_CMD, &diseqc_command11) == -1)
				g_warning("FE_DISEQC_SEND_MASTER_CMD failed");
			usleep(500000);
		}
	}
	
	if (transponders->port10 != 0) {
		struct dvb_diseqc_master_cmd diseqc_command10 = {{0xe0, 0x10, 0x38, 0xf0, 0x00, 0x00}, 4};
		diseqc_command10.msg[3] =	0xf0 | (((transponders->port10 - 1) * 4) & 0x0f);
		
		if (ioctl(frontend->frontend_fd, FE_DISEQC_SEND_MASTER_CMD, &diseqc_command10) == -1)
			g_warning("FE_DISEQC_SEND_MASTER_CMD failed");
		usleep(15 * 1000);
		
		fe_sec_mini_cmd_t tone = ((transponders->port10 - 1) / 4) % 2 ? SEC_MINI_B : SEC_MINI_A;
		if (ioctl(frontend->frontend_fd, FE_DISEQC_SEND_BURST, tone) == -1)
			g_warning("FE_DISEQC_SEND_BURST failed");
		usleep(15 * 1000);
	}
	
	struct dvb_frontend_event ev;
	
	while (TRUE) {
		if (ioctl(frontend->frontend_fd, FE_GET_EVENT, &ev) == -1)
		break;
	}
	
	g_message("Satscan initialization completed");
	
	return TRUE;
}


void dvb_satscan_tune_callback(struct _transponder *transponder, struct _frontend *frontend)
{
	gint ifreq = 0;
	gboolean needtone = FALSE;
	struct dvb_frontend_event ev;
	
	fd_set evset;
	struct timeval timeout;
	gint ready;
	
	guint16 vsignal[10];
	guint i;
	gdouble avgsignal;
	gboolean waitlock = TRUE;
	
	
	if (ioctl(frontend->frontend_fd, FE_SET_TONE, SEC_TONE_OFF) == -1)
		g_warning("FE_SET_TONE failed");
	
	if ((transponder->pol == 0) || (transponder->pol == 2)) {
		if (ioctl(frontend->frontend_fd, FE_SET_VOLTAGE, SEC_VOLTAGE_13) == -1)
			g_warning("FE_SET_VOLTAGE failed");
	} else {
		if (ioctl(frontend->frontend_fd, FE_SET_VOLTAGE, SEC_VOLTAGE_18) == -1)
			g_warning("FE_SET_VOLTAGE failed");
	}
	
	usleep(15 * 1000);
	
	if (transponder->freq > 4200) {
		if (transponder->freq >= 11700) {
			ifreq = transponder->freq - 10600;
			needtone = TRUE;
		} else if (transponder->freq < 9750) {
			ifreq = 9750 - transponder->freq;
		} else {
			ifreq = transponder->freq - 9750;
		}
	} else {
		ifreq = 5150 - transponder->freq;
	}
		
	if (needtone) {
		if (ioctl(frontend->frontend_fd, FE_SET_TONE, SEC_TONE_ON) == -1)
			g_warning("FE_SET_TONE failed");
	}
	
	while (TRUE) {
		if (ioctl(frontend->frontend_fd, FE_GET_EVENT, &ev) == -1)
		break;
	}
	
	if (frontend->api_version > 512) {
		
		struct dtv_property p[] = {
			{ .cmd = DTV_CLEAR },
			{ .cmd = DTV_DELIVERY_SYSTEM, .u.data = system_val[transponder->sys] },
			{ .cmd = DTV_FREQUENCY,		  .u.data = ifreq*1000 },
			{ .cmd = DTV_MODULATION,	  .u.data = modulation_val[transponder->mod] },
			{ .cmd = DTV_SYMBOL_RATE,	  .u.data = transponder->sr*1000 },
			{ .cmd = DTV_INNER_FEC,		  .u.data = fec_val[transponder->fec] },
			{ .cmd = DTV_INVERSION,		  .u.data = INVERSION_AUTO },
			{ .cmd = DTV_ROLLOFF,		  .u.data = rolloff_val[transponder->rof] },
			{ .cmd = DTV_PILOT,		      .u.data = PILOT_AUTO },
			{ .cmd = DTV_TUNE },
		};
		
		struct dtv_properties cmdseq = {
			.num = 10,
			.props = p
		};
		
		if ((ioctl(frontend->frontend_fd, FE_SET_PROPERTY, &cmdseq)) == -1) {
			g_warning("Failed to tune transponder");
			return;
		}
		
	} else {
		
		struct dvb_frontend_parameters frontendparams;
		frontendparams.frequency = ifreq*1000;
		frontendparams.inversion = INVERSION_AUTO;
		frontendparams.u.qpsk.symbol_rate = transponder->sr*1000;
		frontendparams.u.qpsk.fec_inner = fec_val[transponder->fec];
		
		if ((ioctl(frontend->frontend_fd, FE_SET_FRONTEND, &frontendparams)) == -1) {
			g_warning("Failed to tune transponder");
			return;
		}
	}
	
	do {
		FD_ZERO(&evset);
		FD_SET(frontend->frontend_fd, &evset);
		timeout.tv_sec = 5;
		timeout.tv_usec = 0;
		if ((ready = select(frontend->frontend_fd+1, &evset, NULL, NULL, &timeout)) != -1) {
			if (FD_ISSET(frontend->frontend_fd, &evset)) {
				if (ioctl(frontend->frontend_fd, FE_GET_EVENT, &ev) != -1) {
					if (ev.status > 0) {
						if (ioctl(frontend->frontend_fd, FE_READ_SNR, &vsignal[0]) < 0) 
							if (vsignal[0] > 100) 
								usleep(1000000);
						
						for (i=0; i<10; i++) {
							if (ioctl(frontend->frontend_fd, FE_READ_SNR, &vsignal[i]) < 0) 
								vsignal[i] = 0;
							avgsignal += vsignal[i];
							usleep(1000000);
						}
						avgsignal /= 10;
						if (avgsignal > 100.0) avgsignal = 100.0;
						transponder->signal = (guint16)avgsignal;
						transponder->tuned = TRUE;
						transponder->sticky = TRUE;
						waitlock = FALSE;
						g_message("Signal from transponder %u locked, SNR = %u", transponder->freq, transponder->signal);
					} else if (ev.status == 0) continue;				
				}
			}
			
			if (ready == 0) {
				waitlock = FALSE;
				transponder->tuned = FALSE;
				transponder->sticky = FALSE;
				transponder->signal = 0;
				g_message("No signal from transponder %u locked, timeout", transponder->freq);
			}
		}
	} while (waitlock);
			
	
}
