/*
 *      epgwindow.c
 *      
 *      Copyright 2009 - 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>
#include "dvbscan.h"
#include "resources.h"
#include "dbstorage.h"
#include "gsfcore.h"
//#include "mainwindow.h"
#include "common/iso3166.h"
#include "common/odf.h"

enum {
	EPGWIN_ICON = 0,
	EPGWIN_TIME,
	EPGWIN_EVENT,
	EPGWIN_ID,
	EPGWIN_COLUMNS
};

struct sepgwindow {
	//epg event scan window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *toolbar;
	GtkToolItem *read_tb, *save_tb, *sep_tb, *close_tb;
	GtkWidget *statusbar, *progressbar;
	GtkWidget *list;
	GtkTreeSelection *selection;
	guint timeout;
	guint datasrc[3];
	dvbscan_t instance;
};
struct sepgwindow epgwindow;

struct _epgprivatedata {
	GtkTreeModel *model;
	gint currentid;
};

static void epgwindow_onclose(GtkWidget *object, gpointer data);
static gint epgwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
static void epgwindow_init_list(GtkWidget *list);
static gboolean epgwindow_update_progress(gpointer data);
static void epgwindow_rescan_response(GtkDialog *dialog, gint response_id, gpointer data);
static gboolean epgwindow_services_select_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
static void epgwindow_services_add_foreach(gpointer data, gpointer user_data);
static gboolean epgwindow_thread_finished(gpointer data);
static gboolean epgwindow_set_status(gpointer data);
static void epgwindow_status_callback(gint status);
static void epgwindow_read(GtkWidget *widget, gpointer data);
static void epgwindow_save_foreach(gpointer data, gpointer user_data);
static void epgwindow_add(GtkWidget *widget, gpointer data);
static void epgwindow_device_error_response(GtkDialog *dialog, gint response_id, gpointer data);


static void epgwindow_onclose(GtkWidget *object, gpointer data)
{
	
	dvbscan_free(epgwindow.instance);
	if (epgwindow.timeout > 0) {
		g_source_remove(epgwindow.timeout);
		epgwindow.timeout = 0;
	}
	
	epgwindow.visible = FALSE;
	gtk_widget_destroy(epgwindow.win);
}

static gint epgwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	g_signal_emit_by_name(epgwindow.win, "destroy", NULL);
}

static void epgwindow_init_list(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2, *column3, *column4, *column5;
	GtkListStore       *store;
	GtkTreeIter        iter;
		
	column1 = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(column1, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column1, renderer, "stock-id", EPGWIN_ICON, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	
	renderer = gtk_cell_renderer_text_new();
	column2 = gtk_tree_view_column_new_with_attributes(_("Time"), renderer, "markup", EPGWIN_TIME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, "ellipsize-set", TRUE, NULL);
	column3 = gtk_tree_view_column_new_with_attributes(_("Event"), renderer, "markup", EPGWIN_EVENT, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column3);
	
	store = gtk_list_store_new(EPGWIN_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

static gboolean epgwindow_update_progress(gpointer data)
{
	if (epgwindow.visible) {
		gtk_progress_bar_pulse(GTK_PROGRESS_BAR(epgwindow.progressbar));
	}
	
	return TRUE;
}

static void epgwindow_rescan_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog));
	
	if (response_id == GTK_RESPONSE_YES) {
		g_signal_emit_by_name(G_OBJECT(epgwindow.read_tb), "clicked", NULL);
	} else if (response_id == GTK_RESPONSE_NO) {
		g_signal_emit_by_name(epgwindow.win, "destroy", NULL);
	}
}

static gboolean epgwindow_services_select_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	struct _epgprivatedata *privatedata;
	guint id;
	
	privatedata = (struct _epgprivatedata *)data;
	
	gtk_tree_model_get(model, iter, EPGWIN_ID, &id, -1);
	
	if (id == privatedata->currentid) {
		gtk_tree_view_set_cursor(GTK_TREE_VIEW(epgwindow.list), path, NULL, 0);
		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(epgwindow.list), path, 0, TRUE, 0.5, 0.0);
		gtk_tree_selection_select_iter(GTK_TREE_SELECTION(epgwindow.selection), iter);
		return TRUE;
	} else {
		return FALSE;
	}
}

static void epgwindow_services_add_foreach(gpointer data, gpointer user_data)
{
	struct _epgprivatedata *privatedata;
	struct _scanevent *event;
	GtkTreeIter iter;
	struct tm *timestamp;
	time_t endtime;
	gchar timebuf[2][256];
	gchar *timemarkup, *eventmarkup;
	gchar *stockicon;
	
	privatedata = (struct _epgprivatedata *)user_data;
	event = (struct _scanevent *)data;
	
	sprintf(timebuf[0], "--");
	sprintf(timebuf[1], "--");
	
	timestamp = localtime(&(event->start));
	if (timestamp != NULL) {
		strftime(timebuf[0], sizeof(timebuf[0]), "%d.%m %H:%M", timestamp);
	}
	
	endtime = event->start + (time_t)event->duration;
	timestamp = localtime(&endtime);
	if (timestamp != NULL) {
		strftime(timebuf[1], sizeof(timebuf[1]), "%d.%m %H:%M", timestamp);
	}
	
	timemarkup = g_strdup_printf("<small>%s\n%s</small>", timebuf[0], timebuf[1]);
	
	if (event->description == NULL) {
		eventmarkup = g_strdup_printf("<small><b>%s</b></small>", event->name);
	} else {
		eventmarkup = g_strdup_printf("<small><b>%s</b>\n%s</small>", event->name, event->description);
	}
	
	if (event->current) {
		if (privatedata->currentid == -1) {
			privatedata->currentid = event->id;
		}
		stockicon = GTK_STOCK_FULLSCREEN;
	} else {
		stockicon = GTK_STOCK_LEAVE_FULLSCREEN;
	}
	
	gtk_list_store_append(GTK_LIST_STORE(privatedata->model), &iter);
	gtk_list_store_set(GTK_LIST_STORE(privatedata->model), &iter, EPGWIN_ICON, stockicon, EPGWIN_TIME, timemarkup, EPGWIN_EVENT, eventmarkup, EPGWIN_ID, event->id, -1);
	
	g_free(timemarkup);
	g_free(eventmarkup);
}

static gboolean epgwindow_thread_finished(gpointer data)
{
	GSList *eventlist;
	struct _epgprivatedata privatedata;
	GtkWidget *dialog;
	gint res;
	
	if (epgwindow.visible) {
		eventlist = dvbscan_get_event_list(epgwindow.instance);
		if (eventlist != NULL) {
			privatedata.currentid = -1;
			privatedata.model = gtk_tree_view_get_model(GTK_TREE_VIEW(epgwindow.list));
			
			g_object_ref(privatedata.model);
			gtk_tree_view_set_model(GTK_TREE_VIEW(epgwindow.list), NULL);
					
			g_slist_foreach(eventlist, epgwindow_services_add_foreach, &privatedata);
			
			gtk_tree_view_set_model(GTK_TREE_VIEW(epgwindow.list), privatedata.model);
			g_object_unref(privatedata.model);
			
			if (privatedata.currentid != -1) {
				gtk_tree_model_foreach(privatedata.model, epgwindow_services_select_foreach, &privatedata);
			}
			
			gtk_widget_set_sensitive(GTK_WIDGET(epgwindow.save_tb), TRUE);
		} else {
			dialog = gtk_message_dialog_new(GTK_WINDOW(epgwindow.win),
											GTK_DIALOG_MODAL,
											GTK_MESSAGE_QUESTION,
											GTK_BUTTONS_YES_NO,
											_("No events read. Do you want to read events once again?"),
											NULL);
			g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(epgwindow_rescan_response), NULL);
			gtk_widget_show(dialog);
		}
		
		gtk_widget_set_sensitive(GTK_WIDGET(epgwindow.read_tb), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(epgwindow.close_tb), TRUE);
	}
	
	return FALSE;
}

static gboolean epgwindow_set_status(gpointer data)
{
	gint events;
	gchar statusstr[16];
	gint sl;
	
	events = GPOINTER_TO_INT(data);
	
	switch (events) {
		case DVBSCAN_EPG_STATUS_CURRENT:
			sl = sprintf((gchar *)&statusstr, "CURRENT LIST");
			break;
		case DVBSCAN_EPG_STATUS_FULL:
			sl = sprintf((gchar *)&statusstr, "FULL LIST");
			break;
		case DVBSCAN_EPG_STATUS_NULL:
			sl = sprintf((gchar *)&statusstr, " ");
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(epgwindow.progressbar), 0.0);
			if (epgwindow.timeout > 0) {
				g_source_remove(epgwindow.timeout);
				epgwindow.timeout = 0;
			}
			break;
		default:
			sl = sprintf((gchar *)&statusstr, " ");
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(epgwindow.progressbar), 0.0);
			if (epgwindow.timeout > 0) {
				g_source_remove(epgwindow.timeout);
				epgwindow.timeout = 0;
			}
			break;
	}
	
	statusstr[sl] = '\0';
	
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(epgwindow.progressbar), (gchar *)&statusstr);
	
	return FALSE;
}

static void epgwindow_status_callback(gint status)
{
	g_idle_add(epgwindow_set_status, GINT_TO_POINTER(status));
	
	if (status == DVBSCAN_EPG_STATUS_NULL) {
		g_idle_add(epgwindow_thread_finished, NULL);
	}
}

static void epgwindow_read(GtkWidget *widget, gpointer data)
{
	GtkTreeModel *model;
		
	gtk_widget_set_sensitive(GTK_WIDGET(epgwindow.read_tb), FALSE);	
	gtk_widget_set_sensitive(GTK_WIDGET(epgwindow.save_tb), FALSE);
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(epgwindow.list));
	gtk_list_store_clear(GTK_LIST_STORE(model));
	
	dvbscan_free(epgwindow.instance);
	
	epgwindow.instance = dvbscan_new(epgwindow.datasrc, DVBSCAN_TYPE_EPG, epgwindow_status_callback);
	
	epgwindow.timeout = g_timeout_add(100, epgwindow_update_progress, NULL);
}

static void epgwindow_save_foreach(gpointer data, gpointer user_data)
{
	struct _scanevent *event;
	odf_t odf;
	struct tm *timestamp;
	time_t endtime;
	gchar timebuf[256];
	gsize chars;
		
	event = (struct _scanevent *)data;
	odf = (odf_t)user_data;
	
	sprintf(timebuf, "--");
	chars = 0;
	
	timestamp = localtime(&(event->start));
	if (timestamp != NULL) {
		chars = strftime(timebuf+chars, sizeof(timebuf)-chars, "%d.%m %H:%M", timestamp);
	}
	
	chars += sprintf(timebuf+chars, " - ");
	
	endtime = event->start + (time_t)event->duration;
	timestamp = localtime(&endtime);
	if (timestamp != NULL) {
		chars += strftime(timebuf+chars, sizeof(timebuf)-chars, "%d.%m %H:%M", timestamp);
	}
		
	odf_add_paragraph_with_caption(odf, timebuf, event->name, event->description != NULL ? event->description : "");
}

static void epgwindow_save(GtkWidget *widget, gpointer data)
{
	GSList *eventlist;
	GtkWidget *dialog;
	gint res;
	gchar *filename;
	service_t svc;
	odf_t odf;
	gchar *message;
	
	eventlist = dvbscan_get_event_list(epgwindow.instance);
	
	if (eventlist != NULL) {
		dialog = gtk_file_chooser_dialog_new(_("Save File"),
											GTK_WINDOW(epgwindow.win),
											GTK_FILE_CHOOSER_ACTION_SAVE,
											GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
											GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
											NULL);
		
		gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
		
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), "events.odt");
		
		res = gtk_dialog_run(GTK_DIALOG(dialog));
		
		if (res == GTK_RESPONSE_ACCEPT) {
			filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
			
			gtk_widget_destroy(dialog);
			
			odf = odf_init(filename, "GSF");
			svc = gsfcore_get_service();
			
			if (odf != NULL) {
				if (svc != NULL) {
					odf_add_heading(odf, 1, svc->name);
				} else {
					odf_add_heading(odf, 1, _("EPG"));
				}
				g_slist_foreach(eventlist, epgwindow_save_foreach, odf);
												
				odf_close(odf);
			} else  {
				message = g_strdup_printf(_("Error saving file '%s'"), filename);
				dialog = gtk_message_dialog_new(GTK_WINDOW(epgwindow.win),
											GTK_DIALOG_DESTROY_WITH_PARENT,
											GTK_MESSAGE_ERROR,
											GTK_BUTTONS_CLOSE,
											message);
				gtk_dialog_run(GTK_DIALOG(dialog));
				gtk_widget_destroy(dialog);
				
				g_free(message);
			}
			g_free(filename);
			
			g_signal_emit_by_name(epgwindow.win, "destroy", NULL);
		} else {
			gtk_widget_destroy(dialog);
		}
	}
}


static void epgwindow_device_error_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog));
	
	if (response_id == GTK_RESPONSE_CLOSE) {
		g_signal_emit_by_name(epgwindow.win, "destroy", NULL);
	}
}

gboolean epgwindow_create(GtkWindow *rootwindow)
{
    GtkWidget *vbox, *hbox;
	GtkWidget *sw;
	GtkWidget *image;
	GtkWidget *dialog;
	frontend_t frontend;
	service_t  svc;
	
	
	if (!epgwindow.visible) {
		epgwindow.visible = TRUE;
		epgwindow.instance = NULL;
		
		epgwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_transient_for(GTK_WINDOW(epgwindow.win), GTK_WINDOW(rootwindow));
		gtk_window_set_position(GTK_WINDOW(epgwindow.win), GTK_WIN_POS_CENTER_ON_PARENT);
		epgwindow.title = _("EPG events");
		gtk_window_set_title(GTK_WINDOW(epgwindow.win), epgwindow.title);
		gtk_widget_set_size_request(epgwindow.win, 570, 300);
		gtk_window_set_default_size(GTK_WINDOW(epgwindow.win), 570, 300);
		
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
		#else
			vbox = gtk_vbox_new(TRUE, 0);
		#endif
		
		epgwindow.toolbar = gtk_toolbar_new();
		
		gtk_toolbar_set_show_arrow(GTK_TOOLBAR(epgwindow.toolbar), TRUE);
		gtk_toolbar_set_style(GTK_TOOLBAR(epgwindow.toolbar), GTK_TOOLBAR_BOTH_HORIZ);
				
		image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_BUTTON);
		epgwindow.read_tb = gtk_tool_button_new(image, _("Read EPG"));
		gtk_tool_item_set_is_important(GTK_TOOL_ITEM(epgwindow.read_tb), TRUE);
		
		image = gtk_image_new_from_stock(GTK_STOCK_SAVE_AS, GTK_ICON_SIZE_BUTTON);
		epgwindow.save_tb = gtk_tool_button_new(image, _("Save to file"));
		gtk_tool_item_set_is_important(GTK_TOOL_ITEM(epgwindow.save_tb), TRUE);
		
		epgwindow.sep_tb = gtk_separator_tool_item_new();
		
		image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
		epgwindow.close_tb = gtk_tool_button_new(image, _("Close"));
		gtk_tool_item_set_is_important(GTK_TOOL_ITEM(epgwindow.close_tb), TRUE);
		
		gtk_toolbar_insert(GTK_TOOLBAR(epgwindow.toolbar), epgwindow.read_tb, 0);
		gtk_toolbar_insert(GTK_TOOLBAR(epgwindow.toolbar), epgwindow.save_tb, 1);
		gtk_toolbar_insert(GTK_TOOLBAR(epgwindow.toolbar), epgwindow.sep_tb, 2);
		gtk_toolbar_insert(GTK_TOOLBAR(epgwindow.toolbar), epgwindow.close_tb, 3);
		
		gtk_box_pack_start(GTK_BOX(vbox), epgwindow.toolbar, FALSE, FALSE, 0);
		
		sw = gtk_scrolled_window_new(NULL, NULL);
		gtk_container_set_border_width(GTK_CONTAINER(sw), 5);
		
		epgwindow.list = gtk_tree_view_new();
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
		
		epgwindow.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(epgwindow.list));
		gtk_tree_selection_set_mode(GTK_TREE_SELECTION(epgwindow.selection), GTK_SELECTION_SINGLE);
		
		gtk_container_add(GTK_CONTAINER(sw), epgwindow.list);
		epgwindow_init_list(epgwindow.list);
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(epgwindow.list), TRUE);
		
		gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
		#else
			hbox = gtk_hbox_new(FALSE, 0);
		#endif
		
		epgwindow.progressbar = gtk_progress_bar_new();
		
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(epgwindow.progressbar), 0.0);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(epgwindow.progressbar), " ");
			gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(epgwindow.progressbar), TRUE);
		#endif
		
		gtk_box_pack_start(GTK_BOX(hbox), epgwindow.progressbar, FALSE, FALSE, 5);
		
		epgwindow.statusbar = gtk_statusbar_new();
		
		gtk_box_pack_start(GTK_BOX(hbox), epgwindow.statusbar, TRUE, TRUE, 5);
		
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
		
		g_signal_connect(G_OBJECT(epgwindow.win), "destroy", G_CALLBACK(epgwindow_onclose), NULL);
		g_signal_connect(G_OBJECT(epgwindow.close_tb), "clicked", G_CALLBACK(epgwindow_close), NULL);
		g_signal_connect(G_OBJECT(epgwindow.read_tb), "clicked", G_CALLBACK(epgwindow_read), NULL);
		g_signal_connect(G_OBJECT(epgwindow.save_tb), "clicked", G_CALLBACK(epgwindow_save), NULL);
		
		gtk_container_add(GTK_CONTAINER(epgwindow.win), vbox);
		gtk_widget_show_all(epgwindow.win);
		
		gtk_widget_set_sensitive(GTK_WIDGET(epgwindow.save_tb), FALSE);
		
		frontend = gsfcore_get_frontend();
		svc = gsfcore_get_service();
		
		if ((frontend != NULL) || (svc != NULL)) {
			epgwindow.datasrc[0] = frontend->adapter_nr;
			epgwindow.datasrc[1] = frontend->frontend_nr;
			epgwindow.datasrc[2] = svc->sid;
		} else {
			dialog = gtk_message_dialog_new(GTK_WINDOW(epgwindow.win),
											GTK_DIALOG_MODAL,
											GTK_MESSAGE_ERROR,
											GTK_BUTTONS_CLOSE,
											_("Error: Unable to open device and determine service ID"),
											NULL);
			
			g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(epgwindow_device_error_response), NULL);
			gtk_widget_show(dialog);
		}
	} else {
		gtk_window_present(GTK_WINDOW(epgwindow.win));
	}
	
	return TRUE;
}
