/*
 *      gsfcore.h
 *      
 *      Copyright 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __GSFCORE_H__
#define __GSFCORE_H__

#include <glib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "streamconfig.h"

enum {
	DEFINITION_POLARISATION = 0,
	DEFINITION_FEC,
	DEFINITION_SYSTEM,
	DEFINITION_MODULATION,
	DEFINITION_ROLLOFF,
	DEFINITION_PORT10,
	DEFINITION_PORT11
};

enum {
	STATUS_MESSAGE_SVC_NETSARE = 0,
	STATUS_MESSAGE_SVC,
	STATUS_MESSAGE_FR_OPENING,
	STATUS_MESSAGE_FR_OPEN_FAIL,
	STATUS_MESSAGE_TP_SELECTION,
	STATUS_MESSAGE_FR_READONLY_MODE,
	STATUS_MESSAGE_FR_NOT_FOUND,
	STATUS_MESSAGE_TP_TUNING,
	STATUS_MESSAGE_TP_TUNING_FAIL,
	STATUS_MESSAGE_DEV_NOT_FOUND
};

enum {
	GSF_EVENT_UPDATE_SIGNAL = 0,
	GSF_EVENT_TRANSPONDER_SELECTED,
	GSF_EVENT_TRANSPONDER_TUNED,
	GSF_EVENT_UPDATE_TUNE_PARAMS,
	GSF_EVENT_UPDATE_DEVICE_PARAMS,
	GSF_EVENT_STATUS_MESSAGE,
	GSF_EVENT_ERROR_MESSAGE,
	GSF_EVENT_TUNE_ERROR_MESSAGE,
	GSF_EVENT_LOCK_TUNE_BUTTONS,
	GSF_EVENT_LOCK_SCAN_BUTTONS,
	GSF_EVENT_LOCK_SOUND_BUTTONS,
	GSF_EVENT_SATSCAN_STATUS
};

struct _device {
	//devices GSList entry
	gchar name[256];
	gchar vendor[256];
	gchar driver[256];
	gchar slot[256];
	guint bus;
	guint vendorid;
	guint deviceid;
	gint value1;
	gint value2;
	gint adapter;
	gint frontend;
};

struct _frontend {
	//frontend params
	gchar *frontend_fn;
	gint frontend_fd;
	gint adapter_nr;
	gint frontend_nr;
	gchar driver[128];
	uint32_t frequency_min;
	uint32_t frequency_max;
	uint32_t frequency_stepsize;
	uint32_t symbol_rate_min;
	uint32_t symbol_rate_max;
	gboolean dvbs_support;
	gboolean dvbs2_support;
	gboolean open_mode_readonly;
	uint16_t api_version;
	//signal status
	uint16_t snr;
	uint16_t signal;
	uint32_t ber;
	gboolean lock;
	gboolean carrier;
	gboolean tuned;
	//fix for diseqc fail
	gboolean locked;
	time_t   timestamp;
	guint    lockattempts;
	//fix for multiple tunes
	gboolean reset;
	uint16_t oldsnr;
	gboolean frozen;
	//calculated values
	guint32  ifreq;
	gboolean conttone;
	gint lastposition;
	//corrected parametes
	guint32 realfreq;
	guint32 realsr;
	guint8  realfec;
	//generic
	GError  *error;
};

typedef struct _frontend *frontend_t;

struct _demuxer {
	GSList *mediastreams;
	gint netif_nr;
	gchar *nprocess;
	gchar *nprocessfn;
	gint nprocesspid;
};


struct _mediastream {
	guchar  type;
	gushort value;
	gint demuxerpid;
	gboolean enabled;
};

struct _netgate {
	//client variables
	gint clientsock;
	gchar clientipstr[INET_ADDRSTRLEN];
	struct sockaddr_in server;
	gchar *serveraddrstr;
	guchar clienthash[32];
	gushort serveraddrport;
	gboolean clientconnected;
	//server variables
	gint serversock;
	gint serverhttpsock;
	gchar serveripstr[INET_ADDRSTRLEN];
	gchar servernodename[256];
	guchar serverhash[32];
	gushort serverport;
	gboolean serverexpsvclist;
	gboolean resetconf;
	gboolean runstatus;
	//generic
	GError *error;
	//custom
	guint32 netsvcid;
};

struct _tuneparams {
	gint    pos;
	guint   freq;
	guint   sr;
	guint   realfreq;
	guint   realsr;
	guchar  tuned;
	guchar  pol;
	guchar  fec;
	guchar  sys;
	guchar  mod;
	guchar  rof;
	guchar  port10;
	guchar  port11;
	guchar  rep11;
	guchar  lnbtype;
	guchar  lnb22khz;
	guchar  positioner;
	guchar  possatnum;
};

typedef struct _tuneparams *tuneparams_t;

struct _sigstatus {
	gchar svcname[256*4];
	uint16_t snr;
	uint16_t agc;
	uint32_t ber;
	gboolean lock;
	gboolean carrier;
};

struct _location {
	gdouble longitude;
	gdouble latitude;
};

typedef struct _location *location_t;

typedef struct _netgate *netgate_t;

typedef struct _service *service_t;

typedef void (*gsfcore_event_callback)(gint event, gpointer data);

gchar *gsfcore_string_definition(guint table, guint id);
void gsfcore_sound_set_enabled(gboolean enabled);
gboolean gsfcore_sound_get_enabled(void);
gchar *gsfcore_get_netstatus_client_address(void);
netgate_t gsfcore_get_netgate(void);
frontend_t gsfcore_get_frontend(void);
service_t gsfcore_get_service(void);
void gsfcore_network_servicelist_refresh(void);
tuneparams_t gsfcore_get_tuneparams(void);
void gsfcore_get_signal_status(struct _sigstatus *sigstatus);
gboolean gsfcore_tune_start(guint32 id, guchar type);
gboolean gsfcore_network_start(gchar *address, gushort port, gchar *password);
void gsfcore_satscan_start(gboolean resume);
gboolean gsfcore_service_has_pid_type(guint type, gushort *pidvalue);
GSList *gsfcore_get_devices(gboolean rescan);
gint gsfcore_device_search(guint adapterid, guint frontendid, gboolean rescan, gboolean restart);
void gsfcore_get_location(gdouble *longitude, gdouble *latitude);
void gsfcore_set_location(gdouble longitude, gdouble latitude);
streamconfig_t gsfcore_get_streamconfig(void);
gboolean gsfcore_restart_streamserver(void);
pid_t gsfcore_get_streamserver_pid(void);
gboolean gsfcore_get_streamserver_control(void);
gboolean gsfcore_handle_streamserver_stop_signal(pid_t sender);
gboolean gsfcore_set_streamserver_record_status(gboolean status);
gboolean gsfcore_set_streamserver_udpbc_status(gboolean status);
gboolean gsfcore_set_streamserver_udpmc_status(gboolean status);
gboolean gsfcore_set_streamserver_tcpuc_status(gboolean status);
gboolean gsfcore_get_streamserver_available(void);
gboolean gsfcore_get_streamserver_services(gboolean *record, gboolean *udpbc, gboolean *udpmc, gboolean *tcpuc);
gboolean gsfcore_get_streaming_disabled(void);
gboolean gsfcore_get_network_disabled(void);
gboolean gsfcore_init(gboolean disablenetwork, gboolean disablestreaming, gsfcore_event_callback callback);
gboolean gsfcore_stop(void);

#endif /* __GSFCORE_H__ */
