/*
 *      gsfprivileged.c
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <net/route.h>
#include <fcntl.h>
#include <sys/param.h>

#include <linux/dvb/net.h>

#include <sys/ipc.h>
#include <sys/shm.h>

#include "gsfshm.h"

int ifcreate(int adapter, int frontend, int netpid);
int ifremove(int adapter, int frontend, int interface);
int ifup(int adapter, int interface, char *ipaddress, char *netmask, char *macaddress);
int ifdown(int adapter, int interface);
int ifstatus(int adapter, int interface);
int add_multicast_route(char *intname);
int remove_multicast_route(char *intname);
int main(int argc, char** argv);


int ifcreate(int adapter, int frontend, int netpid)
{
	char dev_fn[PATH_MAX];
	int fd;
	int interface;
	
	memset(dev_fn, 0, sizeof(dev_fn));
	sprintf(dev_fn, "/dev/dvb/adapter%i/net%i", adapter, frontend);
	
	if (access(dev_fn, W_OK) != 0) {
		printf("[-] No network device %s\n", dev_fn);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	if ((fd = open(dev_fn, O_RDWR | O_NONBLOCK)) <= 0) {
		printf("[-] Failed to open network device %s\n", dev_fn);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	struct dvb_net_if netfilter;
	
	if ((netpid > 0) && (netpid <= 0x1FFF)) {
		memset(&netfilter, 0, sizeof(netfilter));
		netfilter.pid = netpid;
		netfilter.feedtype = DVB_NET_FEEDTYPE_MPE;
		
		if (ioctl(fd, NET_ADD_IF, &netfilter) < 0) {
			perror("[-] Failed to add filter for pid");
			close(fd);
			exit(PRIVILEGED_STATUS_NETIF_ERROR);
		}
		
		interface = netfilter.if_num;
	}
	
	close(fd);
	
	printf("[+] Network interface dvb%i_%i created for pid %i\n", adapter, interface, netpid);
	
	return interface;
}


int ifremove(int adapter, int frontend, int interface)
{
	char dev_fn[PATH_MAX];
	int fd;
	
	memset(dev_fn, 0, sizeof(dev_fn));
	sprintf(dev_fn, "/dev/dvb/adapter%i/net%i", adapter, frontend);
	
	if (access(dev_fn, W_OK) != 0) {
		printf("[-] No network device %s\n", dev_fn);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	if ((fd = open(dev_fn, O_RDWR | O_NONBLOCK)) <= 0) {
		printf("[-] Failed to open network device %s\n", dev_fn);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	if ((ioctl(fd, NET_REMOVE_IF, interface) < 0)) {
		perror("[-] Failed to remove filter for pid");
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	printf("[+] Network interface dvb%i_%i removed\n", adapter, interface);
	
	return 1;
}


int ifup(int adapter, int interface, char *ipaddress, char *netmask, char *macaddress)
{
	int socket_fd;
	char ifname[256];
	struct ifreq ifr;
	char smacbyte[3];
	int macbyte, i;
	
	if (ifstatus(adapter, interface) == -1) {
		printf("[-] Interface not exists\n");
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	if ((socket_fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("[-] Error while opening socket");
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	memset (&ifr, 0, sizeof(ifr));
	
	((struct sockaddr_in *)&ifr.ifr_addr)->sin_family = AF_INET;
	((struct sockaddr_in *)&ifr.ifr_dstaddr)->sin_family = AF_INET;
	
	sprintf(ifname, "dvb%i_%i", adapter, interface);
	strncpy(ifr.ifr_name, ifname, strlen(ifname));
	
	((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr = inet_addr(ipaddress);
	if (ioctl(socket_fd, SIOCSIFADDR, &ifr) < 0) {
		perror("[-] Error while setting IP address");
		close(socket_fd);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	((struct sockaddr_in *)&ifr.ifr_netmask)->sin_family = AF_INET;
	((struct sockaddr_in *)&ifr.ifr_netmask)->sin_addr.s_addr = inet_addr(netmask);
	if (ioctl(socket_fd, SIOCSIFNETMASK, &ifr) < 0) {
		perror("[-] Error while setting gateway IP address");
		close(socket_fd);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	((struct sockaddr_in *)&ifr.ifr_hwaddr)->sin_family = AF_UNIX;
	memset(smacbyte, 0, 3);
	for (i=0; i<6; i++) {
		strncpy(smacbyte, &(macaddress[i*3]), 2);
		sscanf(smacbyte,"%x", &(ifr.ifr_hwaddr.sa_data[i]));
	}
	
	if (ioctl(socket_fd, SIOCSIFHWADDR, &ifr) < 0) {
		perror("[-] Error while setting MAC address");
		close(socket_fd);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	if (ioctl(socket_fd, SIOCGIFFLAGS, &ifr) < 0) {
		perror("[-] Error while getting IF flags");
		close(socket_fd);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	ifr.ifr_flags |= IFF_UP;
	ifr.ifr_flags |= IFF_RUNNING;
	ifr.ifr_flags |= IFF_PROMISC;
	
	if (ioctl(socket_fd, SIOCSIFFLAGS, &ifr) < 0) {
		perror("[-] Error while setting IF flags");
		close(socket_fd);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	close(socket_fd);
	
	printf("[+] Network interface dvb%i_%i brought up (IP: %s, GW: %s, MAC: %s)\n", adapter, interface, ipaddress, netmask, macaddress);

    return 0;
}


int ifdown(int adapter, int interface)
{
	int socket_fd;
	char ifname[256];
	struct ifreq ifr;
	
	if (ifstatus(adapter, interface) == -1) {
		printf("[-] Interface not exists\n");
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	if ((socket_fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("[-] Error while opening socket");
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	memset (&ifr, 0, sizeof(ifr));
	
	sprintf(ifname, "dvb%i_%i", adapter, interface);
	strncpy(ifr.ifr_name, ifname, strlen(ifname));
	
	if (ioctl(socket_fd, SIOCGIFFLAGS, &ifr) < 0) {
		perror("[-] Error while getting IF flags");
		close(socket_fd);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	ifr.ifr_flags &= !IFF_UP;
	ifr.ifr_flags &= !IFF_RUNNING;
	ifr.ifr_flags &= !IFF_PROMISC;
	
	if (ioctl(socket_fd, SIOCSIFFLAGS, &ifr) < 0) {
		perror("[-] Error while setting IF flags");
		close(socket_fd);
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	close(socket_fd);
	
	printf("[+] Network interface dvb%i_%i brought down\n", adapter, interface);

    return 0;
}


int ifstatus(int adapter, int interface)
{
	int socket_fd;
	char ifname[256];
	struct ifreq ifr;
	int status;
	
	if ((socket_fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("[-] Error while opening socket");
		exit(PRIVILEGED_STATUS_NETIF_ERROR);
	}
	
	memset (&ifr, 0, sizeof(ifr));
	
	sprintf(ifname, "dvb%i_%i", adapter, interface);
	strncpy(ifr.ifr_name, ifname, strlen(ifname));
	
	if (ioctl(socket_fd, SIOCGIFFLAGS, &ifr) < 0) {
		status = -1;
	} else {
		status = ifr.ifr_flags & IFF_UP;
	}
	
	close(socket_fd);
	
	return status;
}

int add_multicast_route(char *intname)
{
	int socketfd;
	struct rtentry route;
	struct sockaddr_in *address;
	
	if ((socketfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("Socket creation error");
		exit(PRIVILEGED_STATUS_ROUTE_ERROR);
	}
	
	memset(&route, 0, sizeof(struct rtentry));
	
	address = (struct sockaddr_in*)&route.rt_dst;
	address->sin_family = AF_INET;
	address->sin_addr.s_addr = inet_addr("224.0.0.0");
	
	address = (struct sockaddr_in*)&route.rt_genmask;
	address->sin_family = AF_INET;
	address->sin_addr.s_addr = inet_addr("240.0.0.0");
	
	route.rt_dev = intname;
	
	route.rt_flags = RTF_UP;
	
	if (ioctl(socketfd, SIOCADDRT, &route) != 0) {
		close(socketfd);
		perror("SIOCADDRT failed");
		exit(PRIVILEGED_STATUS_ROUTE_ERROR);
	}
	
	close(socketfd);
	
	return 0;
}

int remove_multicast_route(char *intname)
{
	int socketfd;
	struct rtentry route;
	struct sockaddr_in *address;
	
	if ((socketfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("Socket creation error");
		exit(PRIVILEGED_STATUS_ROUTE_ERROR);
	}
	
	memset(&route, 0, sizeof(struct rtentry));
	
	address = (struct sockaddr_in*)&route.rt_dst;
	address->sin_family = AF_INET;
	address->sin_addr.s_addr = inet_addr("224.0.0.0");
	
	address = (struct sockaddr_in*)&route.rt_genmask;
	address->sin_family = AF_INET;
	address->sin_addr.s_addr = inet_addr("240.0.0.0");
	
	route.rt_dev = intname;
	
	route.rt_flags = 0;
	
	if (ioctl(socketfd, SIOCDELRT, &route) != 0) {
		close(socketfd);
		perror("SIOCDELRT failed");
		exit(PRIVILEGED_STATUS_ROUTE_ERROR);
	}
	
	close(socketfd);
	
	return 0;
}

int main(int argc, char** argv)
{
	key_t shmkey;
	int shmid;
	struct srequest *shmblock;
	
	/*if (geteuid() != 0) {
		printf("[-] Root privileges needed\n");
		exit(1);
	}*/
	
	shmkey = ftok(PRIVILEGED_TOKEN_FILE, PRIVILEGED_PROJID);
	shmid = shmget(shmkey, sizeof(struct srequest), 0666);
	if (shmid == -1) {
		printf("[-] GSF is not running\n");
		exit(PRIVILEGED_STATUS_SHM_ERROR);
	}
	shmblock = (struct srequest *)shmat(shmid, 0, 0);
	
	switch(shmblock->action) {
		case PRIVILEGED_ACTION_START_NET: {
			shmblock->interface = ifcreate(shmblock->adapter, shmblock->frontend, shmblock->netpid);
			ifup(shmblock->adapter, shmblock->interface, shmblock->ipaddress, shmblock->netmask, shmblock->macaddress);
			break;
		}
		case PRIVILEGED_ACTION_STOP_NET: {
			ifdown(shmblock->adapter, shmblock->interface);
			ifremove(shmblock->adapter, shmblock->frontend, shmblock->interface);
			break;
		}
		case PRIVILEGED_ACTION_ADD_MULTICAST_ROUTE: {
			if (shmblock->remintname[0] != '\0') 
				remove_multicast_route((char *)&shmblock->remintname);
			add_multicast_route((char *)&shmblock->intname);
			break;
		}
		case PRIVILEGED_ACTION_REMOVE_MULTICAST_ROUTE: {
			remove_multicast_route((char *)&shmblock->intname);
			break;
		}
		default: break;
	}

	return PRIVILEGED_STATUS_OK;
}
