/*
 *      csadec.c
 *      
 *      Copyright 2009-2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#include <stdio.h>
#include <stdlib.h>

unsigned char SBox1[32] = {2,0,1,1,2,3,3,0,3,2,2,0,1,1,0,3,0,3,3,0,2,2,1,1,2,2,0,3,1,1,3,0};
unsigned char SBox2[32] = {3,1,0,2,2,3,3,0,1,3,2,1,0,0,1,2,3,1,0,3,3,2,0,2,0,0,1,2,2,1,3,1};
unsigned char SBox3[32] = {2,0,1,2,2,3,3,1,1,1,0,3,3,0,2,0,1,3,0,1,3,0,2,2,2,0,1,2,0,3,3,1};
unsigned char SBox4[32] = {3,1,2,3,0,2,1,2,1,2,0,1,3,0,0,3,1,0,3,1,2,3,0,3,0,3,2,0,1,2,2,1};
unsigned char SBox5[32] = {2,0,0,1,3,2,3,2,0,1,3,3,1,0,2,1,2,3,2,0,0,3,1,1,1,0,3,2,3,1,0,2};
unsigned char SBox6[32] = {0,1,2,3,1,2,2,0,0,1,3,0,2,3,1,3,2,3,0,2,3,0,1,1,2,1,1,2,0,3,3,0};
unsigned char SBox7[32] = {0,3,2,2,3,0,0,1,3,0,1,3,1,2,2,1,1,0,3,3,0,1,1,2,2,3,1,0,2,3,0,2};

unsigned char x, y, z, d, e, f, p, q, r;
unsigned char A[11], B[11];

void sc_clear(unsigned char *key)
{
	x = 0;
	y = 0;
	z = 0;
	d = 0;
	e = 0;
	f = 0;
	p = 0;
	q = 0;
	r = 0;
	A[1] = (key[0] >> 4) & 0xf;
	A[2] = (key[0] >> 0) & 0xf;
	A[3] = (key[1] >> 4) & 0xf;
	A[4] = (key[1] >> 0) & 0xf;
	A[5] = (key[2] >> 4) & 0xf;
	A[6] = (key[2] >> 0) & 0xf;
	A[7] = (key[3] >> 4) & 0xf;
	A[8] = (key[3] >> 0) & 0xf;
	A[9] = 0;
	A[10] = 0;
	B[1] = (key[4] >> 4) & 0xf;
	B[2] = (key[4] >> 0) & 0xf;
	B[3] = (key[5] >> 4) & 0xf;
	B[4] = (key[5] >> 0) & 0xf;
	B[5] = (key[6] >> 4) & 0xf;
	B[6] = (key[6] >> 0) & 0xf;
	B[7] = (key[7] >> 4) & 0xf;
	B[8] = (key[7] >> 0) & 0xf;
	B[9] = 0;
	B[10] = 0;

}

unsigned char sc_initialize(unsigned int step, unsigned char *block)
{
	unsigned char MNibble, LNibble;
	int i;
	unsigned char Dold, Buffer, SB1, SB2, SB3, SB4, SB5, SB6, SB7, res;
	
	i = (step - 1) / 4;
	MNibble = (block[i] >> 4) & 0x0F;
	LNibble = (block[i] >> 0) & 0x0F;
	
	// T3
	// 'Buffer' used for Nibble derived from B
	Buffer =           (((B[9] & 4) >> 2) ^ ((B[6] & 8) >> 3) ^ ((B[3] & 2) >> 1) ^ (B[8] & 1));
	Buffer = (Buffer | (((B[5] & 8) >> 2) ^ ((B[8] & 4) >> 1) ^ ((B[4] & 1) << 1) ^ (B[5] & 2)));
	Buffer = (Buffer | (((B[6] & 1) << 2) ^ ((B[8] & 2) << 1) ^ ((B[3] & 8) >> 1) ^ (B[4] & 4)));
	Buffer = (Buffer | (((B[3] & 1) << 3) ^ ((B[6] & 2) << 2) ^ ((B[7] & 4) << 1) ^ (B[9] & 8)));
	Dold = d;
	d = (Buffer ^ e ^ z);
	res = d ^ (d >> 1);
	
	// T4
	Buffer = e;
	e = f;
	if (q == 0) {
		f = Buffer;
	} else {
		f = (Buffer + z + r);
		r = ((f & 16) >> 4);
		f = (f & 15);
	}
	
	//SBoxes have to be read out BEFORE shifting!!! *
	SB1 = SBox1[(((A[4] & 1) << 4) | ((A[1] & 4) << 1) | ((A[6] & 2) << 1) | ((A[7] & 8) >> 2) | (A[9] & 1))];
	SB2 = SBox2[(((A[2] & 2) << 3) | ((A[3] & 4) << 1) | ((A[6] & 8) >> 1) | ((A[7] & 1) << 1) | ((A[9] & 2) >> 1))];
	SB3 = SBox3[(((A[1] & 8) << 1) | ((A[2] & 1) << 3) | ((A[5] & 2) << 1) | ((A[5] & 8) >> 2) | ((A[6] & 4) >> 2))];
	SB4 = SBox4[(((A[3] & 8) << 1) | ((A[1] & 2) << 2) | ((A[2] & 8) >> 1) | ((A[4] & 4) >> 1) | (A[8] & 1))];
	SB5 = SBox5[(((A[5] & 4) << 2) |  (A[4] & 8)       | ((A[6] & 1) << 2) | (A[8] & 2)        | ((A[9] & 4) >> 2))];
	SB6 = SBox6[(((A[3] & 2) << 3) | ((A[4] & 2) << 2) | ((A[5] & 1) << 2) | ((A[7] & 4) >> 1) | ((A[9] & 8) >> 3))];
	SB7 = SBox7[(((A[2] & 4) << 2) | ((A[3] & 1) << 3) | ((A[7] & 2) << 1) | ((A[8] & 4) >> 1) | ((A[8] & 8) >> 3))];
	
	// T1
	if ((step & 1) == 1) {
		Buffer = (A[10] ^ x ^ Dold ^ MNibble);
	} else {
		Buffer = (A[10] ^ x ^ Dold ^ LNibble);
	}
	
	for (i=10; i>=2; i--) {
		A[i] = A[i-1];
	}
	A[1] = Buffer;
	
	// T2
	if ((step & 1) == 1) {
		Buffer = (LNibble ^ y ^ B[7] ^ B[10]);
	} else {
		Buffer = (MNibble ^ y ^ B[7] ^ B[10]);
	}
	
	if (p == 1) {
		Buffer = (((Buffer & 7) << 1) | ((Buffer & 8) >> 3));
	}
	
	for (i=10; i>=2; i--) {
		B[i] = B[i-1];
	}
	B[1] = Buffer;
	
	// * but Registers have to be filled later
	p = ((SB7 & 2) >> 1);
	q = (SB7 & 1);
	x = (((SB4 & 1) << 3) | ((SB3 & 1) << 2) | (SB2 & 2) | ((SB1 & 2) >> 1));
	y = (((SB6 & 1) << 3) | ((SB5 & 1) << 2) | (SB4 & 2) | ((SB3 & 2) >> 1));
	z = (((SB2 & 1) << 3) | ((SB1 & 1) << 2) | (SB6 & 2) | ((SB5 & 2) >> 1));
	
	return res;
}


unsigned char sc_generate(unsigned char oldres)
{
	int i;
	unsigned char Buffer, SB1, SB2, SB3, SB4, SB5, SB6, SB7, res;
	
	// T3
	// 'Buffer' used for Nibble derived from B
	Buffer =           (((B[9] & 4) >> 2) ^ ((B[6] & 8) >> 3) ^ ((B[3] & 2) >> 1) ^  (B[8] & 1));
	Buffer = (Buffer | (((B[5] & 8) >> 2) ^ ((B[8] & 4) >> 1) ^ ((B[4] & 1) << 1) ^  (B[5] & 2)));
	Buffer = (Buffer | (((B[6] & 1) << 2) ^ ((B[8] & 2) << 1) ^ ((B[3] & 8) >> 1) ^  (B[4] & 4)));
	Buffer = (Buffer | (((B[3] & 1) << 3) ^ ((B[6] & 2) << 2) ^ ((B[7] & 4) << 1) ^  (B[9] & 8)));
	
	d = (Buffer ^ e ^ z);
	Buffer = (((d  ^ (d >> 1)) & 1) | ((((d >> 1) ^ d) >> 1) & 2));
	res = (Buffer | (oldres << 2));
	
	// T4
	Buffer = e;
	e = f;
	if (q == 0) {
		f = Buffer;
	} else {
		f = (Buffer + z + r);
		r = ((f & 16) >> 4);
		f = (f & 15);
	}
		
	//SBoxes have to be read out BEFORE shifting!!! *
	SB1 = SBox1[(((A[4] & 1) << 4) | ((A[1] & 4) << 1) | ((A[6] & 2) << 1) | ((A[7] & 8) >> 2) | (A[9] & 1))];
	SB2 = SBox2[(((A[2] & 2) << 3) | ((A[3] & 4) << 1) | ((A[6] & 8) >> 1) | ((A[7] & 1) << 1) | ((A[9] & 2) >> 1))];
	SB3 = SBox3[(((A[1] & 8) << 1) | ((A[2] & 1) << 3) | ((A[5] & 2) << 1) | ((A[5] & 8) >> 2) | ((A[6] & 4) >> 2))];
	SB4 = SBox4[(((A[3] & 8) << 1) | ((A[1] & 2) << 2) | ((A[2] & 8) >> 1) | ((A[4] & 4) >> 1) | (A[8] & 1))];
	SB5 = SBox5[(((A[5] & 4) << 2) |  (A[4] & 8)       | ((A[6] & 1) << 2) | (A[8] & 2)        | ((A[9] & 4) >> 2))];
	SB6 = SBox6[(((A[3] & 2) << 3) | ((A[4] & 2) << 2) | ((A[5] & 1) << 2) | ((A[7] & 4) >> 1) | ((A[9] & 8) >> 3))];
	SB7 = SBox7[(((A[2] & 4) << 2) | ((A[3] & 1) << 3) | ((A[7] & 2) << 1) | ((A[8] & 4) >> 1) | ((A[8] & 8) >> 3))];
	
	// T1
	Buffer = (A[10] ^ x);
	for (i=10; i>=2; i--) {
		A[i] = A[i-1];
	}
	A[1] = Buffer;
	
	// T2
	Buffer = (y ^ B[7] ^ B[10]);
	if (p == 1) {
		Buffer = (((Buffer & 7) << 1) | ((Buffer & 8) >> 3));
	}
	
	for (i=10; i>=2; i--) {
		B[i] = B[i-1];
	}
	B[1] = Buffer;
	
	// * but Registers have to be filled later
	p = ((SB7 & 2) >> 1);
	q = (SB7 & 1);
	x = (((SB4 & 1) << 3) | ((SB3 & 1) << 2) | (SB2 & 2) | ((SB1 & 2) >> 1));
	y = (((SB6 & 1) << 3) | ((SB5 & 1) << 2) | (SB4 & 2) | ((SB3 & 2) >> 1));
	z = (((SB2 & 1) << 3) | ((SB1 & 1) << 2) | (SB6 & 2) | ((SB5 & 2) >> 1));
	
	return res;
}

unsigned char CKbPerm[64] = {18,36,9,7,42,49,29,21,28,54,62,50,19,33,59,64,24,20,37,39,2,53,27,1,34,4,13,14,57,40,26,41,51,35,52,12,
                             22,48,30,58,45,31,8,25,23,47,61,17,60,5,56,43,11,6,10,44,32,63,46,15,3,38,16,55};
							 
unsigned char BCPerm[256] = {0x00,0x02,0x80,0x82,0x20,0x22,0xA0,0xA2,0x10,0x12,0x90,0x92,0x30,0x32,0xB0,0xB2,0x04,0x06,0x84,0x86,0x24,
                             0x26,0xA4,0xA6,0x14,0x16,0x94,0x96,0x34,0x36,0xB4,0xB6,0x40,0x42,0xC0,0xC2,0x60,0x62,0xE0,0xE2,0x50,0x52,
							 0xD0,0xD2,0x70,0x72,0xF0,0xF2,0x44,0x46,0xC4,0xC6,0x64,0x66,0xE4,0xE6,0x54,0x56,0xD4,0xD6,0x74,0x76,0xF4,
							 0xF6,0x01,0x03,0x81,0x83,0x21,0x23,0xA1,0xA3,0x11,0x13,0x91,0x93,0x31,0x33,0xB1,0xB3,0x05,0x07,0x85,0x87,
							 0x25,0x27,0xA5,0xA7,0x15,0x17,0x95,0x97,0x35,0x37,0xB5,0xB7,0x41,0x43,0xC1,0xC3,0x61,0x63,0xE1,0xE3,0x51,
							 0x53,0xD1,0xD3,0x71,0x73,0xF1,0xF3,0x45,0x47,0xC5,0xC7,0x65,0x67,0xE5,0xE7,0x55,0x57,0xD5,0xD7,0x75,0x77,
							 0xF5,0xF7,0x08,0x0A,0x88,0x8A,0x28,0x2A,0xA8,0xAA,0x18,0x1A,0x98,0x9A,0x38,0x3A,0xB8,0xBA,0x0C,0x0E,0x8C,
							 0x8E,0x2C,0x2E,0xAC,0xAE,0x1C,0x1E,0x9C,0x9E,0x3C,0x3E,0xBC,0xBE,0x48,0x4A,0xC8,0xCA,0x68,0x6A,0xE8,0xEA,
							 0x58,0x5A,0xD8,0xDA,0x78,0x7A,0xF8,0xFA,0x4C,0x4E,0xCC,0xCE,0x6C,0x6E,0xEC,0xEE,0x5C,0x5E,0xDC,0xDE,0x7C,
							 0x7E,0xFC,0xFE,0x09,0x0B,0x89,0x8B,0x29,0x2B,0xA9,0xAB,0x19,0x1B,0x99,0x9B,0x39,0x3B,0xB9,0xBB,0x0D,0x0F,
							 0x8D,0x8F,0x2D,0x2F,0xAD,0xAF,0x1D,0x1F,0x9D,0x9F,0x3D,0x3F,0xBD,0xBF,0x49,0x4B,0xC9,0xCB,0x69,0x6B,0xE9,
							 0xEB,0x59,0x5B,0xD9,0xDB,0x79,0x7B,0xF9,0xFB,0x4D,0x4F,0xCD,0xCF,0x6D,0x6F,0xED,0xEF,0x5D,0x5F,0xDD,0xDF,
							 0x7D,0x7F,0xFD,0xFF};
							 
unsigned char BCSbox[256] = {0x3A,0xEA,0x68,0xFE,0x33,0xE9,0x88,0x1A,0x83,0xCF,0xE1,0x7F,0xBA,0xE2,0x38,0x12,0xE8,0x27,0x61,0x95,0x0C,
                             0x36,0xE5,0x70,0xA2,0x06,0x82,0x7C,0x17,0xA3,0x26,0x49,0xBE,0x7A,0x6D,0x47,0xC1,0x51,0x8F,0xF3,0xCC,0x5B,
                             0x67,0xBD,0xCD,0x18,0x08,0xC9,0xFF,0x69,0xEF,0x03,0x4E,0x48,0x4A,0x84,0x3F,0xB4,0x10,0x04,0xDC,0xF5,0x5C,
                             0xC6,0x16,0xAB,0xAC,0x4C,0xF1,0x6A,0x2F,0x3C,0x3B,0xD4,0xD5,0x94,0xD0,0xC4,0x63,0x62,0x71,0xA1,0xF9,0x4F,
                             0x2E,0xAA,0xC5,0x56,0xE3,0x39,0x93,0xCE,0x65,0x64,0xE4,0x58,0x6C,0x19,0x42,0x79,0xDD,0xEE,0x96,0xF6,0x8A,
                             0xEC,0x1E,0x85,0x53,0x45,0xDE,0xBB,0x7E,0x0A,0x9A,0x13,0x2A,0x9D,0xC2,0x5E,0x5A,0x1F,0x32,0x35,0x9C,0xA8,
                             0x73,0x30,0x29,0x3D,0xE7,0x92,0x87,0x1B,0x2B,0x4B,0xA5,0x57,0x97,0x40,0x15,0xE6,0xBC,0x0E,0xEB,0xC3,0x34,
                             0x2D,0xB8,0x44,0x25,0xA4,0x1C,0xC7,0x23,0xED,0x90,0x6E,0x50,0x00,0x99,0x9E,0x4D,0xD9,0xDA,0x8D,0x6F,0x5F,
                             0x3E,0xD7,0x21,0x74,0x86,0xDF,0x6B,0x05,0x8E,0x5D,0x37,0x11,0xD2,0x28,0x75,0xD6,0xA7,0x77,0x24,0xBF,0xF0,
                             0xB0,0x02,0xB7,0xF8,0xFC,0x81,0x09,0xB1,0x01,0x76,0x91,0x7D,0x0F,0xC8,0xA0,0xF2,0xCB,0x78,0x60,0xD1,0xF7,
                             0xE0,0xB5,0x98,0x22,0xB3,0x20,0x1D,0xA6,0xDB,0x7B,0x59,0x9F,0xAE,0x31,0xFB,0xD3,0xB6,0xCA,0x43,0x72,0x07,
                             0xF4,0xD8,0x41,0x14,0x55,0x0D,0x54,0x8B,0xB9,0xAD,0x46,0x0B,0xAF,0x80,0x52,0x2C,0xFA,0x8C,0x89,0x66,0xFD,
                             0xB2,0xA9,0x9B,0xC0};
							 
unsigned char BitValueTable[8] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};

unsigned char KB[57];

void bc_permute_ck(unsigned char *ckbuffer)
{
	unsigned char i,j,permbit;
	unsigned char resbuffer[9];
	
	for (i=0; i<9; i++) {
		resbuffer[i] = 0;
	}
		
	for (i=0; i<=63; i++) {
		j = (i & 7);
		permbit = CKbPerm[i];
		permbit--;
		j = BitValueTable[j];
		if ((ckbuffer[((i >> 3)+1)] & j) == j) {
			j = (permbit >> 3);
			permbit = (permbit & 7);
			permbit = BitValueTable[permbit];
			resbuffer[j+1] = (resbuffer[j+1] | permbit);
		}
	}
	
	for (i=0; i<9; i++) {
		ckbuffer[i] = resbuffer[i];
	}
}


void bc_key_scheduler(unsigned char *key)
{
	int i,j;
	unsigned char ckbuffer[9];
	
	for (i=0; i<8; i++) {
		ckbuffer[i+1] = key[i];
	}

	for (j=6; j>=0; j--) {
		for (i=1; i<=8; i++) {
			KB[j*8+i] = ckbuffer[i];
		}
		//permute CK for next round
		bc_permute_ck((unsigned char *)&ckbuffer); 
	}
}


unsigned char *bc_generate(unsigned char *block)
{
	unsigned char r8buf,SBout,SBin;
	int i;
		
	for (i=55; i>=0; i--) {
		SBin = (KB[i+1] ^ (i >> 3) ^ block[6]);
		r8buf = (BCSbox[SBin] ^ block[7]);
		SBout = BCSbox[SBin];
		block[7] = block[6];
		block[6] = (block[5] ^ BCPerm[SBout]);
		block[5] = block[4];
		block[4] = (block[3] ^ r8buf);
		block[3] = (block[2] ^ r8buf);
		block[2] = (block[1] ^ r8buf);
		block[1] = block[0];
		block[0] = r8buf;
	}
	return block;
}

void csa_decrypt(unsigned char *packet, unsigned char *key)
{
	unsigned char block[8], ib[8], scout[8];
	int i, j, sc_step, sc_res;
	int numblocks, restbytes, offset = 4;

	if ((packet[3] & 0x20) == 0x20) {
		offset += (packet[4] + 1);
	}
	packet[3] = packet[3] & 0x3F;
	
	numblocks = (188 - offset) / 8;
    restbytes = (188 - offset) % 8;
	
	bc_key_scheduler(key);
	sc_clear(key);
	
	for (sc_step=1; sc_step<=32; sc_step++) {
		sc_initialize(sc_step, (unsigned char *)&packet[offset]);
	}
	
	for (i=0; i<8; i++) {
		ib[i] = packet[offset+i];
	}
	
	for (j=1; j<=numblocks; j++) {
		for (i=0; i<8; i++) {
			block[i] = ib[i];
		}
		bc_generate((unsigned char*)&block);
		if (j != numblocks) {
			for (i=0; i<8; i++) {
				sc_res = 0;
				for (sc_step=0; sc_step<=3; sc_step++) {
					sc_res = sc_generate(sc_res);
				}
				scout[i] = (sc_res & 0xFF);
			}
			for (i=0; i<8; i++) {
				ib[i] = (packet[offset+8*j+i] ^ scout[i]);
			}
		} else {
			for (i=0; i<8; i++) {
				ib[i] = 0;
			}
		}
		for (i=0; i<8; i++) {
			packet[offset+8*(j-1)+i] = (ib[i] ^ block[i]);
		}
	}
	
	if (restbytes > 0) {
		for (i=0; i<restbytes; i++) {
			sc_res = 0;
			for (sc_step=0; sc_step<=3; sc_step++) {
				sc_res = sc_generate(sc_res);
			}
			scout[i] = (sc_res & 0xFF);
		}
		for (i=0; i<restbytes; i++) {
			packet[188-restbytes+i] = (packet[188-restbytes+i] ^ scout[i]);
		}
	}
	
}

/*
int main(int argc, char** argv)
{
	unsigned char inpack[188] = { 0x47, 0x00, 0x00, 0xd0,
      0x33, 0x6F, 0x6F, 0xEE, 0x74, 0x8D, 0x42, 0x76, 0x51, 0x07, 0x5D, 0xC6, 0xBB, 0x1F, 0xA0,
	  0x57, 0x9B, 0xC7, 0xC5, 0xED, 0x22, 0xEB, 0x2A, 0xC3, 0xC9, 0xAA, 0x77, 0x4B, 0xFC, 0xF6,
	  0x46, 0xB0, 0x33, 0x70, 0x24, 0x03, 0x95, 0x7B, 0xF9, 0x2E, 0x6B, 0x79, 0x57, 0xCF, 0xEF,
	  0xE0, 0xD1, 0x93, 0xCA, 0xF3, 0xC2, 0x3C, 0x36, 0x8E, 0xF2, 0x07, 0x66, 0xAC, 0xC2, 0xD8,
	  0x5F, 0x84, 0xC8, 0xF8, 0x02, 0x90, 0xBA, 0x99, 0xEF, 0x98, 0x44, 0x9C, 0xBC, 0x16, 0xF5,
	  0x65, 0xA8, 0xE9, 0xC0, 0x9D, 0x68, 0x75, 0x4D, 0xB2, 0xF9, 0xFB, 0xCE, 0x04, 0xE0, 0x99,
	  0x0C, 0x1C, 0xF1, 0xD6, 0x12, 0x5B, 0xAC, 0x7B, 0xA4, 0x5F, 0x4A, 0x7E, 0x59, 0x64, 0x68,
	  0xD5, 0xA6, 0x87, 0xB6, 0xE7, 0x4E, 0x91, 0x23, 0x2F, 0xF3, 0x37, 0x8E, 0xA3, 0xB2, 0x5B,
	  0x33, 0x89, 0x11, 0x58, 0x81, 0x9C, 0xBF, 0xAC, 0x55, 0x08, 0x6E, 0xED, 0x8F, 0xA4, 0xEF,
	  0x7D, 0x00, 0x69, 0x8F, 0x55, 0x24, 0x1D, 0x9F, 0xA6, 0xBB, 0x00, 0xEA, 0x71, 0xD5, 0xE4,
	  0x1F, 0x7F, 0x13, 0xC3, 0xF6, 0x56, 0xD8, 0x8F, 0xCC, 0x5E, 0x78, 0xD3, 0xED, 0x91, 0x88,
	  0xB4, 0xD1, 0xED, 0x27, 0xFB, 0x36, 0xBD, 0xD2, 0x9E, 0xD9, 0x3E, 0x40, 0x0A, 0xDB, 0xE1,
	  0x8F, 0x69, 0x6A, 0xEB };
	
	unsigned char key[8] = { 0xDE, 0xBE, 0x67, 0x03, 0xE6, 0xEC, 0x3B, 0x0D }; 
	
	csa_decrypt((unsigned char *)&inpack, (unsigned char *)&key);
	
	int i;
	
	for (i=0; i<=187; i++) {
		printf("%x ", inpack[i]);
	}
	printf("\n");
	
	
	return 0;
}*/

