/*
 *      streamserver.h
 *      
 *      Copyright 2010 alex <alex@laptop.org>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __STREAMSERVER_H__
#define __STREAMSERVER_H__

#include <net/if.h>
#include <netinet/in.h>
#include <sys/utsname.h>

#include "transportstream.h"

//Prototypes
int streamserver_client_operation_lock(int clisemid);
int streamserver_client_operation_unlock(int clisemid);
unsigned int streamserver_get_free_tcp_client_slots(void);
void streamserver_stop_client(unsigned int clientid);
void streamserver_init_tcp_listen_process(void);
int streamserver_stop_tcp_listen_process(void);
int streamserver_init_tcp_unicast(void);
int streamserver_init_udp_broadcast(struct sockaddr_in *bcaddr);
int streamserver_stop_udp_broadcast(void);
int streamserver_init_udp_multicast(struct sockaddr_in *udpmgroup);
int streamserver_stop_udp_multicast(void);
int streamserver_init_record(void);
int streamserver_stop_record(void);
int streamserver_init_csa_decoder(int caid, char *key);
int streamserver_stop_csa_decoder(void);
int streamserver_open_device(int adapter, int frontend);
int streamserver_change_service(char *svcname, unsigned short caid, char *cakey);
int streamserver_send_log_message(int flags, char *format, ...);

//Constants
#define HTTPREQUEST             "GET /"
#define HTTPANSWER200           "HTTP/1.1 200 OK\r\nServer: GSF streaming server 0.2.3\r\nContent-Type: %s \r\n\r\n"
#define HTTPANSWER404           "HTTP/1.1 404 NOT FOUND\r\nServer: GSF streaming server 0.2.3 \r\n\r\n"
#define HTTPANSWER503           "HTTP/1.1 503 SERVICE UNAVAILABLE\r\nServer: GSF streaming server 0.2.3 \r\n\r\n"

#define HTTPANSWERICY           "ICY 200 OK\r\nContent-Type:%s\r\nicy-name:%s\r\nicy-genre:Live\r\nicy-url:%s:%i/%i\r\nicy-pub:1\r\nicy-br:%i \r\n\r\n"
#define HTTPANSWERICECAST       "HTTP/1.1 200 OK\r\nServer: GSF streaming server 0.2.3\r\nContent-Type: %s\r\nx-audiocast-name: %s\r\nx-audiocast-genre: Live\r\nx-audiocast-server-url: %s:%i/%i\r\nx-audiocast-description: Live\r\nx-audiocast-public: 1\r\nx-audiocast-bitrate: %i \r\n\r\n"

#define CONTENT_TYPE_AUDIO_MPEG "audio/mpeg"
#define CONTENT_TYPE_AUDIO_AAC  "audio/aac"
#define CONTENT_TYPE_AUDIO_AC3  "audio/ac3"
#define CONTENT_TYPE_AUDIO_DTS  "audio/dts"
#define CONTENT_TYPE_VIDEO_MPEG "video/mpeg"
#define CONTENT_TYPE_TS         "video/mp2t"
#define CONTENT_TYPE_OTHER      "application/octet-stream"

#define FTOK_FILE               "./gsfstrsvr"
#define MAIN_PROJECT_ID         1
#define SHED_PROJECT_ID         2

#define BUFSIZE                 1880*15
#define SENDSIZE                1880
#define PACKETSIZE              188
#define MAXCLIENTS              100

#define ERROR_COUNT_LIMIT       100
#define IDLE_COUNT_LIMIT        25

#define READ_PROCESS_INDEX      0
#define LISTEN_PROCESS_INDEX    1
#define UDP_BC_PROCESS_INDEX    2
#define UDP_MC_PROCESS_INDEX    3
#define TCP_FIRST_CLIENT        4

enum {
	SHEDULER_SIGNAL = 0,
	SHEDULER_RWLOCK
};

enum {
	CLIENT_TYPE_READ_HELPER = 0,
	CLIENT_TYPE_TCP_LISTEN_HELPER,
	CLIENT_TYPE_UDP_BROADCAST,
	CLIENT_TYPE_UDP_MULTICAST,
	CLIENT_TYPE_TCP_UNICAST,
};

enum {
	CLIENT_FAMILY_V4_TRANSPORT = 0,
	CLIENT_FAMILY_V6_TRANSPORT,
	CLIENT_FAMILY_HELPER,
	CLIENT_FAMILY_STOPPED
};

enum {
	STREAM_PID_TYPE_VIDEO = 0,
	STREAM_PID_TYPE_AUDIO,
	STREAM_PID_TYPE_DATA,
	STREAM_PID_TYPE_SUBTITLES,
	STREAM_PID_TYPE_TELETEXT
};

struct sclient {
	char ready;
	char family;
	char type;
	pid_t pid;
	int fdtransport;
	int socketfd;
	time_t starttime;
	unsigned short pidfilter;
	struct sockaddr_in address;
	struct sockaddr_in6 addressv6;
	char ifname[IFNAMSIZ];
	char useragent[64];
	unsigned long long transmittedbytes;
};

struct streampid {
	char type;
	unsigned short value;
	char enabled;
};

struct memblock {
	unsigned char buffer[BUFSIZE];
	struct sclient clients[MAXCLIENTS];
	struct _tsinfo tsinfo;
	unsigned int numclients;
	unsigned int readyclients;
	unsigned int numtcpclients;
	char tcpqueue;
	char tcpclients;
	char udpmcloop;
	char udpmcttl;
	char servicename[256];
	struct streampid streampids[256];
	int recording;
	char encrypted;
	char csakey[8];
	char adapter;
	char dvr;
	int sheduler;
	pid_t hostpid;
	char nodename[_UTSNAME_NODENAME_LENGTH];
	char osname[_UTSNAME_SYSNAME_LENGTH];
	char osrelease[_UTSNAME_RELEASE_LENGTH];
	unsigned long fsblocksize;
};


#define STREAMCONFIG_LOCK_PATH                "%s/.config/gsf/streamserver.pid"
#define STREAMCONFIG_CONFIG_PATH              "%s/.config/gsf/streamserver.conf"
#define STREAMCONFIG_CONFDIR_PATH             ".config/gsf"
#define STREAMCONFIG_DIR_PERMISSIONS          0744
#define STREAMCONFIG_CONFFILE_PATH            ".config/gsf/streamserver.conf"
#define STREAMCONFIG_FILE_SECTION             "settings"
#define STREAMCONFIG_SERVER_SOCKET            "/tmp/streamserver.%s"
#define STREAMCONFIG_CLIENT_SOCKET            "/tmp/streamclient.%s"

#define STREAMCONFIG_SET_RECORD_STATUS        0x01
#define STREAMCONFIG_SET_UDP_BC_STATUS        0x02
#define STREAMCONFIG_SET_UDP_MC_STATUS        0x03
#define STREAMCONFIG_SET_TCP_UC_STATUS        0x04
#define STREAMCONFIG_SET_STREAM_DATA          0x05
#define STREAMCONFIG_SET_STREAM_CONTENT       0x25
#define STREAMCONFIG_SET_CONNECTION_DETAILS   0x06
#define STREAMCONFIG_SET_DEVICE               0x07
#define STREAMCONFIG_SET_FILECONFIG           0x08
#define STREAMCONFIG_STOP_CLIENT              0x09

#define STREAMCONFIG_GET_CLIENTS              0x10
#define STREAMCONFIG_GET_SERVICE_FLAGS        0x11
#define STREAMCONFIG_GET_CONNECTION_DETAILS   0x12
#define STREAMCONFIG_GET_STREAM_STRUCTURE     0x13
#define STREAMCONFIG_GET_SERVER_STATISTICS    0x14

#define STREAMCONFIG_RES_FAILURE              0x00
#define STREAMCONFIG_RES_SUCCESS              0x01
#define STREAMCONFIG_RES_UNDEFINED            0x02

#define SET_RECORD_STATUS_REQ                 "%b%b"
/*b - request, b - recordstatus*/
#define SET_UDPBC_STATUS_REQ                  "%b%b"
/*b - request, b - udpbcstatus*/
#define SET_UDPMC_STATUS_REQ                  "%b%b"
/*b - request, b - udpmcstatus*/
#define SET_TCPUC_STATUS_REQ                  "%b%b"
/*b - request, b - tcpucstatus*/
#define SET_STREAM_DATA_REQ                   "%b%s256%s256%h"
/*b - request, s - servicename, s - cakey, h - caid ... */
#define SET_STREAM_CONTENT_ARRAY_REQ          "%b%h%b"
/* < b - mediatype, h - mediastream > */
#define SET_CONNECTION_DETAILS_REQ            "%b%s16%h%s16%s16%h%b%b%h%b%b"
/*b - request, s - udpbcint, h - udpbcport, s - udpmcint, s - udpmcgroup, h - udpmcport,
  b - udpmcloop, b - udpmcttl, h - tcpucport, b - tcpucclients, b - tcpucqueue */
#define SET_DEVICE_REQ                        "%b%b%b"
/*b - request, b - adapter, b - dvr */
#define STOP_CLIENT_REQ                       "%b%i"
/*b - request, i - clientid*/

#define GET_SIMPLE_REQ                        "%b"
/*b - request */
#define GET_ANSWER_REQ                        "%b%b"
/*b - request, b - answer*/
#define GET_SERVICE_FLAGS_REQ                 "%b%b%b%b%b%b"
/*b - request, b - answer, b - recordstatus, b - udpbcstatus, b - udpmcstatus, b - tcpucstatus */
#define GET_CONNECTION_DETAILS_REQ            "%b%b%s32%h%s32%s32%h%b%b%h%b%b"
/*b - request, b - answer, s - udpbcint, h - udpbcport, s - udpmcint, s - udpmcgroup, h - udpmcport,
  b - udpmcloop, b - udpmcttl, h - tcpucport, b - tcpucclients, b - tcpucqueue */
#define GET_CLIENTS_ARRAY_REQ                 "%b%b%i%i%h%s55%s64%l%l"
/* < b - family, b - type, i - pid, i - seq, h - pidfilter, l - transmittedbytes, l - starttime, s - address, s - useragent > */
#define GET_STREAM_STRUCTURE_ARRAY_REQ        "%b%b%b%b%h%i%i%i%i%i%i%s64"
/* < b - type, b - format, b - value, b - percentage, h - pid, i - bandwidth, i - revision, i - param1, i - param2, i - param3, i - param4, s - description> */
#define GET_SERVER_STATISTICS_REQ             "%b%b%b%b%i%i%l"
/*b - request, b - answer, b - udpbcclients, b - udpmcclients, i - tcpucclients, i - tcpucclilimit, l - lasttunetime */

#endif /* __STREAMSERVER_H__ */
