/*
 *      transportstream.h
 *      
 *      Copyright 2009-2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TRANSPORTSTREAM_H__
#define __TRANSPORTSTREAM_H__

#include <time.h>

#define TS_NUMBER_PIDS             8192
#define TS_ANALYZE_PACKETS_LIMIT   100
#define TS_STREAM_SWITCH_TIMEOUT   3.0
#define TS_STREAM_PID_LIFETIME     15.0

enum {
	TS_STREAM_UNKNOWN = 0,
	TS_STREAM_PES_PROGRAM_STREAM_MAP,
	TS_STREAM_PES_PRIVATE_STREAM_1,
	TS_STREAM_PES_PADDING_STREAM,
	TS_STREAM_PES_PRIVATE_STREAM_2,
	TS_STREAM_PES_AUDIO_STREAM,
	TS_STREAM_PES_VIDEO_STREAM,
	TS_STREAM_PES_ECM_STREAM,
	TS_STREAM_PES_EMM_STREAM,
	TS_STREAM_PES_DSMCC_STREAM,
	TS_STREAM_PES_IEC_13522_STREAM,
	TS_STREAM_PES_H_222_1_TYPE_A,
	TS_STREAM_PES_H_222_1_TYPE_B,
	TS_STREAM_PES_H_222_1_TYPE_C,
	TS_STREAM_PES_H_222_1_TYPE_D,
	TS_STREAM_PES_H_222_1_TYPE_E,
	TS_STREAM_PES_ANCILLARY_STREAM,
	TS_STREAM_PES_SL_PACKETIZED_STREAM,
	TS_STREAM_PES_FLEXMUX_STREAM,
	TS_STREAM_PES_PROGRAM_STREAM_DIRECTORY,
	TS_STREAM_PES_RESERVED_DATA_STREAM,
	TS_STREAM_TABLE_PAT,
	TS_STREAM_TABLE_CAT,
	TS_STREAM_TABLE_TSDT,
	TS_STREAM_TABLE_RESERVED,
	TS_STREAM_TABLE_NIT,
	TS_STREAM_TABLE_SDT_BAT,
	TS_STREAM_TABLE_EIT,
	TS_STREAM_TABLE_RST,
	TS_STREAM_TABLE_TDT_TOT,
	TS_STREAM_TABLE_NETWORK_SYNCHRONIZATION,
	TS_STREAM_TABLE_INBAND_SIGNALLING,
	TS_STREAM_TABLE_MEASUREMENT,
	TS_STREAM_TABLE_DIT,
	TS_STREAM_TABLE_SIT
};

enum {
	TS_STREAM_FORMAT_UNKNOWN = 0,
	TS_STREAM_FORMAT_SPECIAL,
	TS_STREAM_FORMAT_TABLE,
	TS_STREAM_FORMAT_ENCRYPTED,
	TS_STREAM_FORMAT_VIDEO_H262,
	TS_STREAM_FORMAT_VIDEO_H264,
	TS_STREAM_FORMAT_AUDIO_MPEG,
	TS_STREAM_FORMAT_AUDIO_AC3,
	TS_STREAM_FORMAT_AUDIO_DTS,
	TS_STREAM_FORMAT_AUDIO_AAC
};

enum {
	TS_STREAM_AUDIO_MODE_UNKNOWN = 0,
	TS_STREAM_AUDIO_MODE_MONO,
	TS_STREAM_AUDIO_MODE_STEREO,
	TS_STREAM_AUDIO_MODE_C,
	TS_STREAM_AUDIO_MODE_L_R,
	TS_STREAM_AUDIO_MODE_L_C_R,
	TS_STREAM_AUDIO_MODE_L_R_S,
	TS_STREAM_AUDIO_MODE_L_C_R_S,
	TS_STREAM_AUDIO_MODE_L_R_SL_SR,
	TS_STREAM_AUDIO_MODE_L_C_R_SL_SR
};

enum {
	TS_STREAM_AUDIO_PROFILE_UNKNOWN = 0,
	TS_STREAM_AUDIO_PROFILE_DEFAULT,
	TS_STREAM_AUDIO_PROFILE_AAC_MAIN,
	TS_STREAM_AUDIO_PROFILE_AAC_LOW_COMPLEXITY,
	TS_STREAM_AUDIO_PROFILE_AAC_SCALABLE_SAMPLE_RATE,
	TS_STREAM_AUDIO_PROFILE_AAC_RESERVED,
	TS_STREAM_AUDIO_PROFILE_AAC_LTP
};

struct _tsheader {
	unsigned int transport_error_indicator;
	unsigned int payload_unit_start_indicator;
	unsigned int transport_priority;
	unsigned int pid;
	unsigned int transport_scrambling_control;
	unsigned int adaptation_field_control;
	unsigned int continuity_counter;
};

struct _audioparams {
	unsigned short int version;
	unsigned short int revision;
	unsigned short int profile;
	unsigned short int mode;
	unsigned int bitrate;
	unsigned int samplerate;
};

struct _videoparams {
	unsigned int picwidth;
	unsigned int picheight;
	unsigned int aspect;
	unsigned int framerate;
};

union _streamparams {
	struct _audioparams audio;
	struct _videoparams video;
};

struct _tspid {
	unsigned int stream_type;
	unsigned int stream_format;
	unsigned int stream_value;
	unsigned int stream_revision;
	unsigned int bandwidth;
	unsigned int percentage;
	unsigned int packets;
	time_t lasttime;
	union _streamparams params;
};

struct _tsinfo {
	unsigned int timeframe;
	unsigned int packets;
	time_t starttime;
	struct _tspid pids[TS_NUMBER_PIDS];
};

int transportstream_decode_header(unsigned char *packet, struct _tsheader *header);
int transportstream_print_stream_params(struct _tspid *tspid, char *buffer, size_t bufsize);
int transportstream_parse_buffer(unsigned char *buffer, size_t bufsize, struct _tsinfo *tsinfo);
void transportstream_clear_statistics(struct _tsinfo *tsinfo);
unsigned int transportstream_demux_pes(unsigned char *tsbuf, size_t tsbufsize, unsigned char *dmxbuf, size_t dmxbufsize, unsigned short pid);

#endif /* __TRANSPORTSTREAM_H__ */
