/*
 *      prefwindow.c
 *      
 *      Copyright 2011-2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>
#include <glib/gi18n.h>
#include "settings.h"
#include "gsfcore.h"
#include "sound.h"
#include "fswindow.h"
#include "communication.h"
#include "resources.h"

enum {
	PREF_LANGUAGE_ICON = 0,
	PREF_LANGUAGE_NAME,
	PREF_LANGUAGE_CODE,
	PREF_LANGUAGE_COLUMNS
};

enum {
	ENTRIES_COL_ICON = 0,
	ENTRIES_COL_ICON_SIZE,
	ENTRIES_COL_NAME,
	ENTRIES_COL_ID,
	ENTRIES_COLUMNS
};

enum {
	PREFERENCE_ID_SOUNDS = 0,
	PREFERENCE_ID_SCALES,
	PREFERENCE_ID_COLOURS,
	PREFERENCE_ID_NETWORK,
	PREFERENCE_ID_PLACE,
	PREFERENCE_ID_OTHER
};
	

struct sprefwindow {
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	GtkWidget *apply;
	GtkWidget *list;
	GtkTreeSelection *selection;
	GtkWidget *panelcaption;
	GtkWidget *panel;
	gint pageid;
	GtkWidget *page;
	gulong changepagesignal;
};
struct sprefwindow prefwindow;

gint prefwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
gint prefwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data);
void prefwindow_init_list(GtkWidget *list);
gboolean prefwindow_fill_entries(void);
void prefwindow_entry_selected(GtkTreeView *tree_view, gpointer data);


GtkWidget **prefwindow_sound_settings_create(void);
GtkWidget **prefwindow_scales_settings_create(void);
GtkWidget **prefwindow_graphs_settings_create(void);
GtkWidget **prefwindow_network_settings_create(void);
GtkWidget **prefwindow_place_settings_create(void);
GtkWidget **prefwindow_other_settings_create(void);

void prefwindow_sound_settings_configure(gpointer page);
void prefwindow_scales_settings_configure(gpointer page);
void prefwindow_graphs_settings_configure(gpointer page);
void prefwindow_network_settings_configure(gpointer page);
void prefwindow_place_settings_configure(gpointer page);
void prefwindow_other_settings_configure(gpointer page);

void prefwindow_sound_settings_apply(gpointer page);
void prefwindow_scales_settings_apply(gpointer page);
void prefwindow_graphs_settings_apply(gpointer page);
void prefwindow_network_settings_apply(gpointer page);
void prefwindow_place_settings_apply(gpointer page);
void prefwindow_other_settings_apply(gpointer page);

typedef GtkWidget **(*pref_page_init_func)(void);
typedef void (*pref_page_configure_func)(gpointer page);
typedef void (*pref_page_apply_func)(gpointer page);

typedef struct {
	GtkWidget **widgets;
	const guint preferenceid;
	const gchar *title;
	const gchar *description;
	const gchar *stock;
	pref_page_init_func init_func;
	pref_page_configure_func configure_func;
	pref_page_apply_func apply_func;
} prefpage;

static prefpage pref_pages[6] = {
	{ NULL, PREFERENCE_ID_SOUNDS,  N_("Sounds"),  N_("Driver, language"),   GTK_STOCK_MEDIA_PLAY,   prefwindow_sound_settings_create,   prefwindow_sound_settings_configure,   prefwindow_sound_settings_apply},
	{ NULL, PREFERENCE_ID_SCALES,  N_("Scales"),  N_("Measurement ranges"), GTK_STOCK_PAGE_SETUP,   prefwindow_scales_settings_create,  prefwindow_scales_settings_configure,  prefwindow_scales_settings_apply},
	{ NULL, PREFERENCE_ID_COLOURS, N_("Colours"), N_("Graphs appearance"),  GTK_STOCK_SELECT_COLOR, prefwindow_graphs_settings_create,  prefwindow_graphs_settings_configure,  prefwindow_graphs_settings_apply},
	{ NULL, PREFERENCE_ID_NETWORK, N_("Network"), N_("Server settings"),    GTK_STOCK_NETWORK,      prefwindow_network_settings_create, prefwindow_network_settings_configure, prefwindow_network_settings_apply},
	{ NULL, PREFERENCE_ID_PLACE,   N_("Place"),   N_("Site cordinates"),    GTK_STOCK_HOME,         prefwindow_place_settings_create,   prefwindow_place_settings_configure,   prefwindow_place_settings_apply},
	{ NULL, PREFERENCE_ID_OTHER,   N_("Other"),   N_("Specific settings"),  GTK_STOCK_PROPERTIES,   prefwindow_other_settings_create,   prefwindow_other_settings_configure,   prefwindow_other_settings_apply},
};

struct _prefwidget {
	GtkWidget *widget;
	gchar *str;
};

gint prefwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gint i;
	
	prefwindow.visible = FALSE;
	
	if (g_signal_handler_is_connected(G_OBJECT(prefwindow.list), prefwindow.changepagesignal)) {
		g_signal_handler_disconnect(G_OBJECT(prefwindow.list), prefwindow.changepagesignal);
	}
	
	for (i=0; i<(sizeof(pref_pages)/sizeof(prefpage)); i++) {
		if (pref_pages[i].widgets != NULL) {
			gtk_widget_destroy(pref_pages[i].widgets[0]);
			g_free(pref_pages[i].widgets);
			pref_pages[i].widgets = NULL;
		}
	}
	
	gtk_widget_destroy(prefwindow.win);
	
	return FALSE;
}

/*void prefwindow_message(gchar *message, gchar *caption)
{
	GtkWidget *dialog;
	
	dialog = gtk_message_dialog_new(GTK_WINDOW(prefwindow.win),
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_INFO,
									GTK_BUTTONS_OK,
									message);
	
	gtk_window_set_title(GTK_WINDOW(dialog), caption);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	
}*/

/*gint prefwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	//prefwindow_message("Not enough rights to apply settings", "Information");
	
	gint i;
	
	for (i=0; i<(sizeof(pref_pages)/sizeof(prefpage)); i++) {
		if ((pref_pages[i].widgets != NULL) && (pref_pages[i].apply_func != NULL)) {
			pref_pages[i].apply_func(&pref_pages[i]);
		}
	}
	
	settings_flush();
	
	g_signal_emit_by_name(G_OBJECT(prefwindow.win), "destroy", NULL);
	
	return 0;
}*/

static void prefwindow_action_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gint i;
	
	if (response_id == GTK_RESPONSE_APPLY) {
		for (i=0; i<(sizeof(pref_pages)/sizeof(prefpage)); i++) {
			if ((pref_pages[i].widgets != NULL) && (pref_pages[i].apply_func != NULL)) {
				pref_pages[i].apply_func(&pref_pages[i]);
			}
		}
		settings_flush();
		g_signal_emit_by_name(G_OBJECT(prefwindow.win), "destroy", NULL);
	} else if (response_id == GTK_RESPONSE_CLOSE) {
		g_signal_emit_by_name(G_OBJECT(prefwindow.win), "destroy", NULL);
	} else if (response_id == GTK_RESPONSE_HELP) {
		
	}
}


void prefwindow_init_list(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2;
	GtkListStore       *store;
		
	
	renderer = gtk_cell_renderer_pixbuf_new();
	column1 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column1, NULL);
	gtk_tree_view_column_pack_start(column1, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column1, renderer, "stock-id", ENTRIES_COL_ICON, "stock-size", ENTRIES_COL_ICON_SIZE, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, "ellipsize-set", TRUE, NULL);
	column2 = gtk_tree_view_column_new_with_attributes(_("Option"), renderer, "markup", ENTRIES_COL_NAME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	
	store = gtk_list_store_new(ENTRIES_COLUMNS, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

gboolean prefwindow_fill_entries(void)
{
	//globalsettings_t globalsettings;
	GtkTreeModel *model;
	GtkTreeIter iter;
	//GdkPixbuf *icon;
	gchar caption[256];
	gint len;
	gint i;
		
	
	prefwindow.pageid = -1;
	//globalsettings = mainwindow_get_globalsettings();
	
	if (prefwindow.visible) {
		if ((model = gtk_tree_view_get_model(GTK_TREE_VIEW(prefwindow.list))) != NULL) {
			g_object_ref(model);
			gtk_tree_view_set_model(GTK_TREE_VIEW(prefwindow.list), NULL);
			gtk_list_store_clear(GTK_LIST_STORE(model));
			//icon = gdk_pixbuf_new_from_file_at_size("resources/media-optical.svg", 48, 48, NULL);
			for (i=0; i<(sizeof(pref_pages)/sizeof(prefpage)); i++) {
				if (pref_pages[i].widgets != NULL) continue;
				//pref_pages[i].widgets = NULL;
				if ((/*globalsettings->nonetwork*/gsfcore_get_network_disabled()) && (pref_pages[i].preferenceid == PREFERENCE_ID_NETWORK)) continue;
				len = snprintf((gchar *)&caption, sizeof(caption), "<small><b>%s</b>\n%s</small>", gettext(pref_pages[i].title), gettext(pref_pages[i].description));
				caption[len] = '\0';
				gtk_list_store_append(GTK_LIST_STORE(model), &iter);
				gtk_list_store_set(GTK_LIST_STORE(model), &iter, ENTRIES_COL_ICON, pref_pages[i].stock,
																ENTRIES_COL_ICON_SIZE, GTK_ICON_SIZE_BUTTON,
																ENTRIES_COL_NAME, caption,
																ENTRIES_COL_ID, pref_pages[i].preferenceid,
																-1);
			}
			//gdk_pixbuf_unref(icon);
			gtk_tree_view_set_model(GTK_TREE_VIEW(prefwindow.list), model);
			g_object_unref(model);
		}
	}
	
	return FALSE;
}


void prefwindow_entry_selected(GtkTreeView *tree_view, gpointer data)
{
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint id;
	gchar caption[256];
	gint len;
		
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, ENTRIES_COL_ID, &id, -1);
		
		if (((prefwindow.pageid == -1) || (prefwindow.pageid != id)) && (id < (sizeof(pref_pages)/sizeof(prefpage)))) {
			if ((pref_pages[id].widgets == NULL) && (pref_pages[id].init_func != NULL)) {
				//Create new page and configure it
				pref_pages[id].widgets = pref_pages[id].init_func();
				if ((pref_pages[id].widgets != NULL) && (pref_pages[id].configure_func != NULL)) {
					pref_pages[id].configure_func(&pref_pages[id]);
				}
			} else if (pref_pages[id].init_func == NULL) {
				g_warning("No constructor for page %i (%s)", id, pref_pages[id].title);
			}
				
			if (pref_pages[id].widgets != NULL) {
				if (prefwindow.page != NULL) {
					if (prefwindow.pageid == -1) {
						//After start page selected
						gtk_widget_set_sensitive(prefwindow.panel, TRUE);
						gtk_dialog_set_response_sensitive(GTK_DIALOG(prefwindow.win), GTK_RESPONSE_APPLY, TRUE);
						//gtk_widget_set_sensitive(prefwindow.apply, TRUE);
					} else {
						//Increase refcount to avoid page re-creation
						g_object_ref(prefwindow.page);
					}
					//Remove old page
					gtk_container_remove(GTK_CONTAINER(prefwindow.panel), prefwindow.page);
				}
				//Add new page
				prefwindow.page = pref_pages[id].widgets[0];
				gtk_box_pack_end(GTK_BOX(prefwindow.panel), prefwindow.page, TRUE, FALSE, 10);
				prefwindow.pageid = id;
				len = snprintf((gchar *)&caption, sizeof(caption), "<b>%s</b>", gettext(pref_pages[id].title));
				caption[len] = '\0';
				gtk_label_set_markup(GTK_LABEL(prefwindow.panelcaption), caption);
				gtk_widget_show_all(prefwindow.panel);
			}
			
		}
		
	}
}	

GtkWidget **prefwindow_sound_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *driver, *driver_l;
	GtkWidget *locale, *locale_l;
	GtkWidget *parameter, *parameter_l;
	GtkWidget *limit, *limit_l;
	GtkListStore *localestore;
	GtkCellRenderer *renderer;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 4, FALSE);
	
	driver_l = gtk_label_new(_("Driver"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), driver_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 5, 5);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		driver = gtk_combo_box_text_new();
	#else
		driver = gtk_combo_box_new_text();
	#endif
	gtk_combo_box_set_active(GTK_COMBO_BOX(driver), 0);
	gtk_table_attach(GTK_TABLE(table), driver, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
		
	locale_l = gtk_label_new(_("Language"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), locale_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 5, 5);
		
	locale = gtk_combo_box_new();
	
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(locale), renderer, FALSE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(locale), renderer, "pixbuf", PREF_LANGUAGE_ICON, NULL);
	
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(locale), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(locale), renderer, "markup", PREF_LANGUAGE_NAME, NULL);
	
	localestore = gtk_list_store_new(PREF_LANGUAGE_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
	
	gtk_combo_box_set_model(GTK_COMBO_BOX(locale), GTK_TREE_MODEL(localestore));
	g_object_unref(localestore);
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(locale), 0);
	gtk_table_attach(GTK_TABLE(table), locale, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	parameter_l = gtk_label_new(_("Parameter"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), parameter_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 2, 3, GTK_FILL, GTK_SHRINK, 5, 5);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		parameter = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(parameter), _("SNR"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(parameter), _("AGC"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(parameter), _("BER"));
	#else
		parameter = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(parameter), _("SNR"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(parameter), _("AGC"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(parameter), _("BER"));
	#endif
	gtk_combo_box_set_active(GTK_COMBO_BOX(parameter), 0);
	gtk_table_attach(GTK_TABLE(table), parameter, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	limit_l = gtk_label_new(_("Low limit"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), limit_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 3, 4, GTK_FILL, GTK_SHRINK, 5, 5);
	
	limit = gtk_spin_button_new_with_range(0.0, 100.0, 1.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(limit), 0.0);
	gtk_table_attach(GTK_TABLE(table), limit, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(5*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = driver;
	widgets[2] = locale;
	widgets[3] = parameter;
	widgets[4] = limit;
	
	return widgets;
}

void prefwindow_sound_settings_modules_foreach(gpointer data, gpointer user_data)
{
	struct _soundmodule *module;
	GtkWidget *widget;
	
	module = (struct _soundmodule *)data;
	widget = (GtkWidget *)user_data;
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widget), module->name);
	#else
		gtk_combo_box_append_text(GTK_COMBO_BOX(widget), module->name);
	#endif
}

void prefwindow_sound_settings_languages_foreach(gpointer data, gpointer user_data)
{
	struct _soundlanguage *language;
	GtkWidget *widget;
	GtkTreeModel *model;
	GtkTreeIter iter;
	GdkPixbuf *pixbuf;
	GError *error;
	gchar strbuf[256];
	gchar *flagpath;
		
	language = (struct _soundlanguage *)data;
	widget = (GtkWidget *)user_data;
		
	error = NULL;
	
	if ((model = gtk_combo_box_get_model(GTK_COMBO_BOX(widget))) != NULL) {
		if (language->flagexists) {
			flagpath = sound_languages_flag_path(language->code);
			pixbuf = gdk_pixbuf_new_from_file(flagpath, &error);
			g_free(flagpath);
		} else {
			pixbuf = gdk_pixbuf_new_from_file(RESOURCE_SOUND_FLAG_UNKNOWN, &error);
		}
		
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		if (language->islocale) {
			g_snprintf(strbuf, sizeof(strbuf), _("%s (default)"), language->name);
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, PREF_LANGUAGE_ICON, pixbuf, PREF_LANGUAGE_NAME, strbuf, PREF_LANGUAGE_CODE, language->code, -1);
		} else {
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, PREF_LANGUAGE_ICON, pixbuf, PREF_LANGUAGE_NAME, language->name, PREF_LANGUAGE_CODE, language->code, -1);
		}
		
		if (pixbuf != NULL) g_object_unref(pixbuf);
	}
	
}

void prefwindow_sound_settings_configure(gpointer page)
{
	prefpage *pageptr;
	gint driver, langindex, parameter, limit;
	
	GSList *soundmodules;
	GSList *languages;
	
	pageptr = (prefpage *)page;
	
	if ((soundmodules = sound_modules_get(&driver)) != NULL) {
		g_slist_foreach(soundmodules, prefwindow_sound_settings_modules_foreach, pageptr->widgets[1]);
		gtk_combo_box_set_active(GTK_COMBO_BOX(pageptr->widgets[1]), driver);
	} else {
		#if GTK_CHECK_VERSION(2, 24, 0)
			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(pageptr->widgets[1]), _("Not found"));
		#else
			gtk_combo_box_append_text(GTK_COMBO_BOX(pageptr->widgets[1]), _("Not found"));
		#endif
		gtk_combo_box_set_active(GTK_COMBO_BOX(pageptr->widgets[1]), 0);
		gtk_widget_set_sensitive(GTK_WIDGET(pageptr->widgets[1]), FALSE);
	}
	
	if ((languages = sound_languages_get(&langindex)) != NULL) {
		g_slist_foreach(languages, prefwindow_sound_settings_languages_foreach, pageptr->widgets[2]);
	}
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(pageptr->widgets[2]), langindex);
	
	parameter = sound_parameter_get();
	gtk_combo_box_set_active(GTK_COMBO_BOX(pageptr->widgets[3]), parameter);
	limit = sound_limit_get();
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[4]), (gdouble)limit);
}

void prefwindow_sound_settings_apply(gpointer page)
{
	prefpage *pageptr;
	gint driver, parameter, limit;
	GSList *soundmodules;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar *language;
	
	struct _soundmodule *currentmodule;
		
	pageptr = (prefpage *)page;
	driver = gtk_combo_box_get_active(GTK_COMBO_BOX(pageptr->widgets[1]));
	parameter = gtk_combo_box_get_active(GTK_COMBO_BOX(pageptr->widgets[3]));
	limit = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pageptr->widgets[4]));
	
	if ((soundmodules = sound_modules_get(NULL)) != NULL) {
		currentmodule = g_slist_nth_data(soundmodules, driver);
		sound_modules_set_module(currentmodule->id);
	}
	
	if ((model = gtk_combo_box_get_model(GTK_COMBO_BOX(pageptr->widgets[2]))) != NULL) {
		gtk_combo_box_get_active_iter(GTK_COMBO_BOX(pageptr->widgets[2]), &iter);
		gtk_tree_model_get(model, &iter, PREF_LANGUAGE_CODE, &language, -1);
		sound_languages_set_language(language);
		g_free(language);
	}
	
	sound_parameter_set((guchar)parameter);
	sound_limit_set((guchar)limit);
		
	//printf("Values from page 1: %i,%i,%i,%i\n", driver, locale, parameter, limit);
}

GtkWidget **prefwindow_scales_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *scsnr, *scsnr_l;
	GtkWidget *scagc, *scagc_l;
	GtkWidget *scber, *scber_l;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 3, FALSE);
	
	scsnr_l = gtk_label_new(_("SNR scale"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), scsnr_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 5, 5);
	
	scsnr = gtk_spin_button_new_with_range(1.0, 65535.0, 1.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(scsnr), 50.0);
	gtk_table_attach(GTK_TABLE(table), scsnr, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
		
	scagc_l = gtk_label_new(_("AGC scale"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), scagc_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 5, 5);
	
	scagc = gtk_spin_button_new_with_range(1.0, 65535.0, 1.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(scagc), 100.0);
	gtk_table_attach(GTK_TABLE(table), scagc, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	scber_l = gtk_label_new(_("BER scale"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), scber_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 2, 3, GTK_FILL, GTK_SHRINK, 5, 5);
	
	scber = gtk_spin_button_new_with_range(1.0, 4294967296.0, 1.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(scber), 200.0);
	gtk_table_attach(GTK_TABLE(table), scber, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(4*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = scsnr;
	widgets[2] = scagc;
	widgets[3] = scber;
		
	return widgets;
}

void prefwindow_scales_settings_configure(gpointer page)
{
	prefpage *pageptr;
	gfloat snrsc, agcsc, bersc;
	
	snrsc = settings_read_float("scale_snr", 100.0);
	agcsc = settings_read_float("scale_agc", 1000.0);
	bersc = settings_read_float("scale_ber", 4294967296.0);
	
	pageptr = (prefpage *)page;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[1]), (gdouble)snrsc);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[2]), (gdouble)agcsc);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[3]), (gdouble)bersc);
}

void prefwindow_scales_settings_apply(gpointer page)
{
	prefpage *pageptr;
	gdouble snrsc, agcsc, bersc;
	
	pageptr = (prefpage *)page;
	snrsc = gtk_spin_button_get_value(GTK_SPIN_BUTTON(pageptr->widgets[1]));
	agcsc = gtk_spin_button_get_value(GTK_SPIN_BUTTON(pageptr->widgets[2]));
	bersc = gtk_spin_button_get_value(GTK_SPIN_BUTTON(pageptr->widgets[3]));
	
	settings_append_float("scale_snr", (gfloat)snrsc);
	settings_append_float("scale_agc", (gfloat)agcsc);
	settings_append_float("scale_ber", (gfloat)bersc);
	
	//printf("Values from page 2: %8.0f,%8.0f,%8.0f\n", snrsc, agcsc, bersc);
}

GtkWidget **prefwindow_graphs_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *sigclow, *sigchigh, *sigcolor_l;
	GtkWidget *fsclow, *fscmiddle, *fschigh, *fscolor_l;
	GtkWidget *fsbg, *fsfg, *fsgcolor_l;
	GtkWidget **widgets;
	
	table = gtk_table_new(4, 3, FALSE);
	
	sigcolor_l = gtk_label_new(_("Signal graph"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), sigcolor_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 5, 5);
	
	sigclow = gtk_color_button_new();
	gtk_widget_set_tooltip_text(sigclow, _("Low values in signal status window"));
	gtk_table_attach(GTK_TABLE(table), sigclow, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	sigchigh = gtk_color_button_new();
	gtk_widget_set_tooltip_text(sigchigh, _("High values in signal status window"));
	gtk_table_attach(GTK_TABLE(table), sigchigh, 3, 4, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	fscolor_l = gtk_label_new(_("Fullscreen graph"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), fscolor_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 5, 5);
	
	fsclow = gtk_color_button_new();
	gtk_widget_set_tooltip_text(fsclow, _("Low values for bars in fullscreen mode"));
	gtk_table_attach(GTK_TABLE(table), fsclow, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	fscmiddle = gtk_color_button_new();
	gtk_widget_set_tooltip_text(fscmiddle, _("Middle values for bars in fullscreen mode"));
	gtk_table_attach(GTK_TABLE(table), fscmiddle, 2, 3, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	fschigh = gtk_color_button_new();
	gtk_widget_set_tooltip_text(fschigh, _("High values for bars in fullscreen mode"));
	gtk_table_attach(GTK_TABLE(table), fschigh, 3, 4, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	fsgcolor_l = gtk_label_new(_("Fullscreen text"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), fsgcolor_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 2, 3, GTK_FILL, GTK_SHRINK, 5, 5);
	
	fsbg = gtk_color_button_new();
	gtk_widget_set_tooltip_text(fsbg, _("Background in fullscreen mode"));
	gtk_table_attach(GTK_TABLE(table), fsbg, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	fsfg = gtk_color_button_new();
	gtk_widget_set_tooltip_text(fsfg, _("Text in fullscreen mode"));
	gtk_table_attach(GTK_TABLE(table), fsfg, 3, 4, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(8*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = sigclow;
	widgets[2] = sigchigh;
	widgets[3] = fsclow;
	widgets[4] = fscmiddle;
	widgets[5] = fschigh;
	widgets[6] = fsbg;
	widgets[7] = fsfg;
	
	return widgets;
}

void prefwindow_graphs_settings_configure(gpointer page)
{
	prefpage *pageptr;
	GdkColor color;
	gchar *tmpstr;
	gdouble r, g, b;
	
	pageptr = (prefpage *)page;
	tmpstr = settings_read_string("graph_low_color", "white");
	gdk_color_parse(tmpstr, &color);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(pageptr->widgets[1]), &color);
	g_free(tmpstr);
	
	tmpstr = settings_read_string("graph_high_color", "green3");
	gdk_color_parse(tmpstr, &color);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(pageptr->widgets[2]), &color);
	g_free(tmpstr);
	
	fswindow_get_color(FSWIN_COLOR_LOWLEVEL, &r, &g, &b);
	color.red = (guint16)(r*65535);
	color.green = (guint16)(g*65535);
	color.blue = (guint16)(b*65535);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(pageptr->widgets[3]), &color);
	
	fswindow_get_color(FSWIN_COLOR_MIDLEVEL, &r, &g, &b);
	color.red = (guint16)(r*65535);
	color.green = (guint16)(g*65535);
	color.blue = (guint16)(b*65535);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(pageptr->widgets[4]), &color);
	
	fswindow_get_color(FSWIN_COLOR_HILEVEL, &r, &g, &b);
	color.red = (guint16)(r*65535);
	color.green = (guint16)(g*65535);
	color.blue = (guint16)(b*65535);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(pageptr->widgets[5]), &color);
	
	fswindow_get_color(FSWIN_COLOR_BACKGROUND, &r, &g, &b);
	color.red = (guint16)(r*65535);
	color.green = (guint16)(g*65535);
	color.blue = (guint16)(b*65535);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(pageptr->widgets[6]), &color);
	
	fswindow_get_color(FSWIN_COLOR_TEXT, &r, &g, &b);
	color.red = (guint16)(r*65535);
	color.green = (guint16)(g*65535);
	color.blue = (guint16)(b*65535);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(pageptr->widgets[7]), &color);
}

void prefwindow_graphs_settings_apply(gpointer page)
{
	prefpage *pageptr;
	GdkColor color, color2;
	gchar *fsclow, *fscmiddle, *fschigh, *fsbg, *fsfg;
	gdouble r, g, b;
	
	pageptr = (prefpage *)page;
	gtk_color_button_get_color(GTK_COLOR_BUTTON(pageptr->widgets[1]), &color);
	gtk_color_button_get_color(GTK_COLOR_BUTTON(pageptr->widgets[2]), &color2);
	statuswindow_graph_set_colors(&color, &color2);
	
	gtk_color_button_get_color(GTK_COLOR_BUTTON(pageptr->widgets[3]), &color);
	r = (gdouble)color.red/65535.0;
	g = (gdouble)color.green/65535.0;
	b = (gdouble)color.blue/65535.0;
	fswindow_set_color(FSWIN_COLOR_LOWLEVEL, r, g, b);
	fsclow = gdk_color_to_string(&color);
	settings_append_string("fullscreen_low_color", fsclow);
	
	gtk_color_button_get_color(GTK_COLOR_BUTTON(pageptr->widgets[4]), &color);
	r = (gdouble)color.red/65535.0;
	g = (gdouble)color.green/65535.0;
	b = (gdouble)color.blue/65535.0;
	fswindow_set_color(FSWIN_COLOR_MIDLEVEL, r, g, b);
	fscmiddle = gdk_color_to_string(&color);
	settings_append_string("fullscreen_middle_color", fscmiddle);
	
	gtk_color_button_get_color(GTK_COLOR_BUTTON(pageptr->widgets[5]), &color);
	r = (gdouble)color.red/65535.0;
	g = (gdouble)color.green/65535.0;
	b = (gdouble)color.blue/65535.0;
	fswindow_set_color(FSWIN_COLOR_HILEVEL, r, g, b);
	fschigh = gdk_color_to_string(&color);
	settings_append_string("fullscreen_high_color", fschigh);
	
	gtk_color_button_get_color(GTK_COLOR_BUTTON(pageptr->widgets[6]), &color);
	r = (gdouble)color.red/65535.0;
	g = (gdouble)color.green/65535.0;
	b = (gdouble)color.blue/65535.0;
	fswindow_set_color(FSWIN_COLOR_BACKGROUND, r, g, b);
	fsbg = gdk_color_to_string(&color);
	settings_append_string("fullscreen_background_color", fsbg);
	
	gtk_color_button_get_color(GTK_COLOR_BUTTON(pageptr->widgets[7]), &color);
	r = (gdouble)color.red/65535.0;
	g = (gdouble)color.green/65535.0;
	b = (gdouble)color.blue/65535.0;
	fswindow_set_color(FSWIN_COLOR_TEXT, r, g, b);
	fsfg = gdk_color_to_string(&color);
	settings_append_string("fullscreen_text_color", fsfg);
	
	//printf("Values from page 3: %s,%s,%s,%s,%s\n", fsclow, fscmiddle, fschigh, fsbg, fsfg);
	
	g_free(fsclow); g_free(fscmiddle); g_free(fschigh); g_free(fsbg); g_free(fsfg);
}

GtkWidget **prefwindow_network_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *passwd, *passwd_l;
	GtkWidget *port, *port_l;
	GtkWidget *export;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 3, FALSE);
	
	passwd_l = gtk_label_new(_("Server password"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), passwd_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 5, 5);
	
	passwd = gtk_entry_new();
	gtk_entry_set_visibility(GTK_ENTRY(passwd), FALSE);
	gtk_table_attach(GTK_TABLE(table), passwd, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
		
	port_l = gtk_label_new(_("Server port"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), port_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 5, 5);
	
	port = gtk_spin_button_new_with_range(1024.0, 65535.0, 1.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(port), 5555.0);
	gtk_table_attach(GTK_TABLE(table), port, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	export = gtk_check_button_new_with_label(_("Export services list"));
	gtk_table_attach(GTK_TABLE(table), export, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(4*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = passwd;
	widgets[2] = port;
	widgets[3] = export;
		
	return widgets;
}

void prefwindow_network_settings_configure(gpointer page)
{
	prefpage *pageptr;
	gchar *passwd;
	gint port;
	gboolean export;
	
	passwd = settings_read_string("local_server_password", "defaultpassword");
	port = settings_read_int("local_server_port", 5555);
	export = settings_read_boolean("local_server_export_services", TRUE);
	
	pageptr = (prefpage *)page;
	gtk_entry_set_text(GTK_ENTRY(pageptr->widgets[1]), passwd);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[2]), (gdouble)port);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pageptr->widgets[3]), export);
	
	g_free(passwd);
}

void prefwindow_network_settings_apply(gpointer page)
{
	prefpage *pageptr;
	gchar *passwd;
	gint port;
	gboolean export;
	//globalsettings_t globalsettings;
	
	pageptr = (prefpage *)page;
	passwd = g_strdup(gtk_entry_get_text(GTK_ENTRY(pageptr->widgets[1])));
	port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pageptr->widgets[2]));
	export = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pageptr->widgets[3]));
	
	//globalsettings = mainwindow_get_globalsettings();
	if (!(gsfcore_get_network_disabled())/*globalsettings->nonetwork*/) {
		if (communication_reinit(passwd, port, export)) {
			settings_append_string("local_server_password", passwd);
			settings_append_int("local_server_port", port);
			settings_append_boolean("local_server_export_services", export);
		}
	}
	
	//printf("Values from page 4: %s,%i,%i\n", passwd, port, export);
	
	g_free(passwd);
}

GtkWidget **prefwindow_place_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *longitude, *longitude_l;
	GtkWidget *latitude, *latitude_l;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 2, FALSE);
	
	longitude_l = gtk_label_new(_("Longitude"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), longitude_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 5, 5);
	
	longitude = gtk_spin_button_new_with_range(-180.0, 180.0, 0.1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(longitude), 0.0);
	gtk_table_attach(GTK_TABLE(table), longitude, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
		
	latitude_l = gtk_label_new(_("Latitude"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), latitude_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 5, 5);
	
	latitude = gtk_spin_button_new_with_range(-90.0, 90.0, 0.1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(latitude), 0.0);
	gtk_table_attach(GTK_TABLE(table), latitude, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(3*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = longitude;
	widgets[2] = latitude;
			
	return widgets;
}

void prefwindow_place_settings_configure(gpointer page)
{
	prefpage *pageptr;
	gdouble longitude, latitude;
	
	/*longitude = settings_read_float("place_longitude", 37.70);
	latitude = settings_read_float("place_latitude", 55.75);*/
	
	gsfcore_get_location(&longitude, &latitude);
	
	pageptr = (prefpage *)page;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[1]), longitude);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[2]), latitude);
}

void prefwindow_place_settings_apply(gpointer page)
{
	prefpage *pageptr;
	gdouble longitude, latitude;
	
	pageptr = (prefpage *)page;
	longitude = gtk_spin_button_get_value(GTK_SPIN_BUTTON(pageptr->widgets[1]));
	latitude = gtk_spin_button_get_value(GTK_SPIN_BUTTON(pageptr->widgets[2]));
	
	/*settings_append_float("place_longitude", longitude);
	settings_append_float("place_latitude", latitude);*/
	
	gsfcore_set_location(longitude, latitude);	
	
	//printf("Values from page 5: %i,%i\n", hidetotray, exitonclose);
}

GtkWidget **prefwindow_other_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *hidetotray;
	GtkWidget *exitonclose;
	/*#if GTK_CHECK_VERSION(3, 4, 0)
		GtkWidget *hidecaption;
		GtkWidget *blacktheme;
	#endif*/
	GtkWidget **widgets;
	
	#if GTK_CHECK_VERSION(3, 4, 0)
		table = gtk_table_new(1, 4, FALSE);
	#else
		table = gtk_table_new(1, 2, FALSE);
	#endif
	
	hidetotray = gtk_check_button_new_with_label(_("Start in tray"));
	gtk_table_attach(GTK_TABLE(table), hidetotray, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 5, 5);
	
	exitonclose = gtk_check_button_new_with_label(_("Exit on close"));
	gtk_table_attach(GTK_TABLE(table), exitonclose, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 5, 5);
	
	/*#if GTK_CHECK_VERSION(3, 4, 0)
		hidecaption = gtk_check_button_new_with_label(_("Hide caption on maximize"));
		gtk_table_attach(GTK_TABLE(table), hidecaption, 0, 1, 2, 3, GTK_FILL, GTK_SHRINK, 5, 5);
		
		blacktheme = gtk_check_button_new_with_label(_("Use black theme"));
		gtk_table_attach(GTK_TABLE(table), blacktheme, 0, 1, 3, 4, GTK_FILL, GTK_SHRINK, 5, 5);
	#endif*/
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	#if GTK_CHECK_VERSION(3, 4, 0)
		widgets = (GtkWidget **)g_malloc0(5*sizeof(GtkWidget *));
	#else
		widgets = (GtkWidget **)g_malloc0(3*sizeof(GtkWidget *));
	#endif
	widgets[0] = align;
	widgets[1] = hidetotray;
	widgets[2] = exitonclose;
	/*#if GTK_CHECK_VERSION(3, 4, 0)
		widgets[3] = hidecaption;
		widgets[4] = blacktheme;
	#endif*/
			
	return widgets;
}

void prefwindow_other_settings_configure(gpointer page)
{
	prefpage *pageptr;
	gboolean hidetotray, exitonclose;
	/*#if GTK_CHECK_VERSION(3, 4, 0)
		gboolean hidecaption, blacktheme;
	#endif*/
	
	hidetotray = settings_read_boolean("program_start_in_tray", FALSE);
	exitonclose = settings_read_boolean("program_exit_on_close", FALSE);
	/*#if GTK_CHECK_VERSION(3, 4, 0)
		hidecaption = settings_read_boolean("program_hide_caption", FALSE);
		blacktheme = settings_read_boolean("program_use_black_theme", FALSE);
	#endif*/
	
	pageptr = (prefpage *)page;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pageptr->widgets[1]), hidetotray);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pageptr->widgets[2]), exitonclose);
	/*#if GTK_CHECK_VERSION(3, 4, 0)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pageptr->widgets[3]), hidecaption);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pageptr->widgets[4]), blacktheme);
	#endif*/
}

void prefwindow_other_settings_apply(gpointer page)
{
	prefpage *pageptr;
	gboolean hidetotray, exitonclose;
	/*#if GTK_CHECK_VERSION(3, 4, 0)
		gboolean hidecaption, blacktheme;
	#endif*/
	
	pageptr = (prefpage *)page;
	hidetotray = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pageptr->widgets[1]));
	exitonclose = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pageptr->widgets[2]));
	/*#if GTK_CHECK_VERSION(3, 4, 0)
		hidecaption = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pageptr->widgets[3]));
		blacktheme = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pageptr->widgets[4]));
	#endif*/
	
	settings_append_boolean("program_start_in_tray", hidetotray);
	settings_append_boolean("program_exit_on_close", exitonclose);
	/*settings_append_boolean("program_hide_caption", hidecaption);
	settings_append_boolean("program_use_black_theme", blacktheme);*/
	
	//printf("Values from page 5: %i,%i\n", hidetotray, exitonclose);
}

gboolean prefwindow_create(GtkWindow *parentwin)
{
	GtkWidget *vbox, *hbox, *hpaned;
	GtkWidget *align;
	GtkWidget *listscroll, *panelscroll;
	
	if (!prefwindow.visible) {
		prefwindow.visible = TRUE;
		
		/*prefwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		prefwindow.title = _("GSF Preferences");
		gtk_window_set_position(GTK_WINDOW(prefwindow.win), GTK_WIN_POS_CENTER);
		gtk_window_set_title(GTK_WINDOW(prefwindow.win), prefwindow.title);
		gtk_widget_set_size_request(prefwindow.win, 500, 300);
		gtk_window_set_default_size(GTK_WINDOW(prefwindow.win), 500, 300);
		gtk_window_set_resizable(GTK_WINDOW(prefwindow.win), FALSE);
		gtk_window_set_transient_for(GTK_WINDOW(prefwindow.win), parentwin);*/
		
		prefwindow.win = gtk_dialog_new_with_buttons(_("GSF Preferences"), 
													GTK_WINDOW(parentwin),
													GTK_DIALOG_DESTROY_WITH_PARENT,
													/*GTK_STOCK_HELP, GTK_RESPONSE_HELP,*/
													GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
													GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
													NULL);
		
		gtk_window_set_modal(GTK_WINDOW(prefwindow.win), TRUE);
		gtk_widget_set_size_request(GTK_WIDGET(prefwindow.win), 500, 300);
		gtk_window_set_default_size(GTK_WINDOW(prefwindow.win), 500, 300);
		gtk_window_set_resizable(GTK_WINDOW(prefwindow.win), FALSE);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
			hpaned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
		#else
			vbox = gtk_vbox_new(FALSE, 0);
			hpaned = gtk_hpaned_new();
		#endif
		gtk_container_set_border_width(GTK_CONTAINER(hpaned), 5);
		
		listscroll = gtk_scrolled_window_new(NULL, NULL);
		
		prefwindow.list = gtk_tree_view_new();
		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(prefwindow.list), FALSE);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(listscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(listscroll), GTK_SHADOW_ETCHED_IN);
		
		gtk_container_add(GTK_CONTAINER (listscroll), prefwindow.list);
		prefwindow.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(prefwindow.list));
		gtk_tree_selection_set_mode(prefwindow.selection, GTK_SELECTION_BROWSE);
		
		prefwindow_init_list(prefwindow.list);
		
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(prefwindow.list), TRUE);
		
		gtk_paned_pack1(GTK_PANED(hpaned), listscroll, TRUE, FALSE);
		
		panelscroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(panelscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(panelscroll), GTK_SHADOW_NONE);
		
		gtk_widget_set_size_request(listscroll, 150, -1);
		gtk_widget_set_size_request(panelscroll, 300, -1);
		
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			prefwindow.panel = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
		#else
			prefwindow.panel = gtk_vbox_new(FALSE, 0);
		#endif
		
		prefwindow.panelcaption = gtk_label_new(NULL);
		gtk_label_set_markup(GTK_LABEL(prefwindow.panelcaption), _("<b>GSF Preferences</b>"));
		gtk_label_set_max_width_chars(GTK_LABEL(prefwindow.panelcaption), 30);
		gtk_label_set_ellipsize(GTK_LABEL(prefwindow.panelcaption), PANGO_ELLIPSIZE_END);
		gtk_box_pack_start(GTK_BOX(prefwindow.panel), prefwindow.panelcaption, FALSE, FALSE, 10);
		
		prefwindow.page = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_DIALOG);
		gtk_box_pack_end(GTK_BOX(prefwindow.panel), prefwindow.page, TRUE, FALSE, 10);
		
		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(panelscroll), prefwindow.panel);
		
		align = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
		gtk_container_add(GTK_CONTAINER(align), panelscroll);
		
		gtk_paned_pack2(GTK_PANED(hpaned), align, TRUE, FALSE);
		
		gtk_box_pack_start(GTK_BOX(vbox), hpaned, TRUE, TRUE, 0);
		
		/*prefwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
		prefwindow.apply = gtk_button_new_from_stock(GTK_STOCK_APPLY);
		
		gtk_widget_set_sensitive(prefwindow.panel, FALSE);
		gtk_widget_set_sensitive(prefwindow.apply, FALSE);
		
		hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), prefwindow.apply, FALSE, FALSE, 5);
		gtk_box_pack_end(GTK_BOX(hbox), prefwindow.close, FALSE, FALSE, 5);
				
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);*/
		
		//g_signal_connect(G_OBJECT(prefwindow.win), "destroy", G_CALLBACK(prefwindow_close), NULL);
		/*g_signal_connect(G_OBJECT(prefwindow.close), "clicked", G_CALLBACK(prefwindow_close), NULL);
		g_signal_connect(G_OBJECT(prefwindow.apply), "clicked", G_CALLBACK(prefwindow_apply), NULL);*/
		//g_signal_connect(G_OBJECT(prefwindow.list), "cursor-changed", G_CALLBACK(prefwindow_entry_selected), NULL);
		
		prefwindow_fill_entries();
		
		/*gtk_container_add(GTK_CONTAINER(prefwindow.win),vbox);
		gtk_widget_show_all(prefwindow.win);*/
		
		GtkWidget *dialogarea = gtk_dialog_get_content_area(GTK_DIALOG(prefwindow.win));
		
		gtk_box_pack_start(GTK_BOX(dialogarea), vbox, TRUE, TRUE, 0);
		gtk_widget_show_all(prefwindow.win);
		
		gtk_dialog_set_response_sensitive(GTK_DIALOG(prefwindow.win), GTK_RESPONSE_APPLY, FALSE);
		
		//gtk_window_set_focus(GTK_WINDOW(prefwindow.win), prefwindow.list);
		
		g_signal_connect(G_OBJECT(prefwindow.win), "destroy", G_CALLBACK(prefwindow_close), NULL);
		g_signal_connect(G_OBJECT(prefwindow.win), "response", G_CALLBACK(prefwindow_action_response), NULL);
		prefwindow.changepagesignal = g_signal_connect(G_OBJECT(prefwindow.list), "cursor-changed", G_CALLBACK(prefwindow_entry_selected), NULL);
	
	} else {
		gtk_window_present(GTK_WINDOW(prefwindow.win));
	}
	
	return TRUE;
}

