/*
 *      satscanwindow.c
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <glib/gstdio.h>
#include <string.h>
#include <stdlib.h>
#include "satscanwindow.h"
#include "dbstorage.h"
#include "resources.h"
#include "gsfcore.h"

static void button_clicked(GtkButton *button, GtkAssistant *assistant);
static void assistant_cancel (GtkAssistant *assistant, gpointer data);
static void assistant_close(GtkAssistant *assistant, gpointer data);

enum {
	TPS_SELECTED = 0,
	TPS_STATUS,
	TPS_NAME,
	TPS_SIGNAL,
	TPS_ID,
	TPS_COLUMNS
};

typedef struct {
	GtkWidget *widget;
	gint index;
	const gchar *title;
	GtkAssistantPageType type;
	gboolean complete;
} satscanpages;

satscanpages satscanpage[5] = {
	{ NULL, -1, N_("Introduction"),           GTK_ASSISTANT_PAGE_INTRO,    FALSE},
	{ NULL, -1, N_("Preferences"),            GTK_ASSISTANT_PAGE_CONTENT,  TRUE},
	{ NULL, -1, N_("Scan progress"),          GTK_ASSISTANT_PAGE_PROGRESS, FALSE},
	{ NULL, -1, N_("Scan status"),            GTK_ASSISTANT_PAGE_CONTENT,  TRUE},
	{ NULL, -1, N_("Confirmation"),           GTK_ASSISTANT_PAGE_CONFIRM,  TRUE},
};

struct _satscanwindow {
	GtkWidget *assistant;
	GtkWidget *satname;
	GtkWidget *satfile;
	GtkWidget *port10;
	GtkWidget *port11;
	GtkWidget *repeats11;
	GtkWidget *cbandrb, *kubandrb;
	GtkWidget *progress;
	GtkWidget *scanbutton;
	GtkWidget *tplist;
	GtkWidget *addtps;
} satscanwindow;

struct _transponders transponders;

struct _satscanmodel {
	guchar band;
	GtkTreeModel *model;
};

struct _satscanid {
	guint id;
	struct _transponder *transponder;
};

void satscan_window_free_transponders_foreach(gpointer data, gpointer user_data)
{
	g_free(data);
}

gboolean satscan_window_parse_fsf_transponders(gchar *filecontent, gsize filelength)
{
	GKeyFile *inifile;
	GError *error = NULL;
	gsize sections;
	gint i;
	gchar strkey[16];
	gchar *tpstring;
	gchar **tpstringv;
	gint tpstrpos;
	struct _transponder *transponder;
	guint currentid = 0;
	
	inifile = g_key_file_new();
	if (g_key_file_load_from_data(inifile, filecontent, filelength, G_KEY_FILE_NONE, &error)) {
		if ((sections = g_key_file_get_integer(inifile, "DVB", "0", &error)) > 0) {
			if (transponders.list != NULL) {
				g_slist_foreach(transponders.list, satscan_window_free_transponders_foreach, NULL);
				g_slist_free(transponders.list);
				transponders.list = NULL;
				transponders.cbandnum = 0;
				transponders.kubandnum = 0;
			}
	
			for (i=1; i<=sections; i++) {
			g_sprintf(strkey, "%i", i);
			if ((tpstring = g_key_file_get_string(inifile, "DVB", strkey, &error)) != NULL) {
				tpstring = g_strdelimit(tpstring, ";", ',');
				tpstringv = g_strsplit((const gchar *)tpstring, ",", 6);
				tpstrpos = 0;
				transponder = (struct _transponder *)g_malloc0(sizeof(struct _transponder));
				
				while (tpstringv[tpstrpos] != NULL) {
					switch(tpstrpos) {
						case 0: {
							transponder->freq = strtoul(tpstringv[tpstrpos], (gchar **)NULL, 10);
							break;
						} 
						
						case 1: {
							if (tpstringv[tpstrpos][0] == 'V') 
								transponder->pol = 0;
							else if (tpstringv[tpstrpos][0] == 'H')
								transponder->pol = 1;
							break;
						}
						
						case 2: {
							transponder->sr = strtoul(tpstringv[tpstrpos], (gchar **)NULL, 10);
							break;
						}
						
						case 3: {
							if (tpstringv[tpstrpos][0] == '1') 
								transponder->fec = 1;
							else if (tpstringv[tpstrpos][0] == '2')
								transponder->fec = 2;
							else if ((tpstringv[tpstrpos][0] == '3') && (tpstringv[tpstrpos][1] == '4'))
								transponder->fec = 3;
							else if ((tpstringv[tpstrpos][0] == '3') && (tpstringv[tpstrpos][1] == '5'))
								transponder->fec = 4;
							else if (tpstringv[tpstrpos][0] == '4')
								transponder->fec = 5;
							else if (tpstringv[tpstrpos][0] == '5')
								transponder->fec = 6;
							else if (tpstringv[tpstrpos][0] == '6')
								transponder->fec = 7;
							else if (tpstringv[tpstrpos][0] == '7')
								transponder->fec = 8;
							else if (tpstringv[tpstrpos][0] == '8')
								transponder->fec = 9;
							else if (tpstringv[tpstrpos][0] == '9')
								transponder->fec = 10;
							else
								transponder->fec = 0;
							break;
						}
						
						case 4: {
							if (g_str_equal(tpstringv[tpstrpos], "S2")) {
								transponder->sys = 1;
							} else {
								transponder->sys = 2;
							}
							break;
						}
						
						case 5: {
							if (g_str_equal(tpstringv[tpstrpos], "QPSK")) {
								transponder->mod = 0;
							} else if (g_str_equal(tpstringv[tpstrpos], "8PSK")) {
								transponder->mod = 1;
							} else {
								transponder->mod = 2;
							}
							break;
						}
						
						default: break;
					}
					
					tpstrpos++;							
				}
				
				if ((transponder->mod == 2) || (transponder->sys == 2) || (transponder->sr < 1000) || (transponder->sr > 45000) ||
				     (transponder->freq < 3400) || (transponder->freq > 12750) || ((transponder->sys == 0) && ((transponder->mod != 0) || (transponder->rof != 0))) ) {
					g_free(transponder);
				} else {
					transponder->id = currentid;
					transponders.list = g_slist_prepend(transponders.list, transponder);
					currentid++;
					if ((transponder->freq > 3400) && (transponder->freq < 4200))
						transponders.cbandnum++;
					if ((transponder->freq > 10700) && (transponder->freq < 12750))
						transponders.kubandnum++;
					//printf("%u %u %u %u %u %u\n", transponder->freq, transponder->pol, transponder->sr, transponder->fec, transponder->sys, transponder->mod);
				}
				
				g_strfreev(tpstringv);
				g_free(tpstring);
			}
		}
		
		transponders.list = g_slist_reverse(transponders.list);
		
		if (g_slist_length(transponders.list) > 0) {
			return TRUE;
		} else {
			g_warning("Empty transponders list");
			return FALSE;
		}
		
		}  else {
			g_warning("Empty transponders file");
			
			return FALSE;
		}
	} else {
		g_warning("Error: %s", error->message);
		g_error_free(error);
		
		return FALSE;
	}
	
}
	

gboolean satscan_window_parse_scans2_transponders(const gchar *filename)
{
	FILE *file;
	gchar buf[1024];
	gchar *tpstring;
	gchar **tpstringv;
	gint tpstrpos;
	gsize len;
	guint chr;
	struct _transponder *transponder;
	guint currentid = 0;
	
    if ((file = g_fopen(filename, "r")) != NULL) {
		if (transponders.list != NULL) {
			g_slist_foreach(transponders.list, satscan_window_free_transponders_foreach, NULL);
			g_slist_free(transponders.list);
			transponders.list = NULL;
			transponders.cbandnum = 0;
			transponders.kubandnum = 0;
		}
		
		while ((tpstring = fgets((gchar *)&buf, sizeof(buf), file)) != NULL) {
			len = strlen(tpstring);
			chr = 0;
			while ((chr <= len-1) && (tpstring[chr] == ' ')) chr++;
			if (tpstring[chr] == '#') continue;
			if (len <= chr) continue;
			
			tpstringv = g_strsplit((const gchar *)tpstring, " ", 7);
			tpstrpos = 0;
			transponder = (struct _transponder *)g_malloc0(sizeof(struct _transponder));
			
			while (tpstringv[tpstrpos] != NULL) {
					switch(tpstrpos) {
						case 0: {
							if (g_str_equal(tpstringv[tpstrpos], "S1")) {
								transponder->sys = 0;
							} else if (g_str_equal(tpstringv[tpstrpos], "S2")) {
								transponder->sys = 1;
							} else if (g_str_equal(tpstringv[tpstrpos], "S")) {
								transponder->sys = 0;
							} else {
								transponder->sys = 2;
							}
							break;
						}
						
						case 1: {
							transponder->freq = strtoul(tpstringv[tpstrpos], (gchar **)NULL, 10);
							transponder->freq /= 1000;
							break;
						} 
						
						case 2: {
							if (tpstringv[tpstrpos][0] == 'V') 
								transponder->pol = 0;
							else if (tpstringv[tpstrpos][0] == 'H')
								transponder->pol = 1;
							break;
						}
						
						case 3: {
							transponder->sr = strtoul(tpstringv[tpstrpos], (gchar **)NULL, 10);
							transponder->sr /= 1000;
							break;
						}
						
						case 4: {
							if (tpstringv[tpstrpos][0] == '1') 
								transponder->fec = 1;
							else if (tpstringv[tpstrpos][0] == '2')
								transponder->fec = 2;
							else if ((tpstringv[tpstrpos][0] == '3') && (tpstringv[tpstrpos][2] == '4'))
								transponder->fec = 3;
							else if ((tpstringv[tpstrpos][0] == '3') && (tpstringv[tpstrpos][2] == '5'))
								transponder->fec = 4;
							else if (tpstringv[tpstrpos][0] == '4')
								transponder->fec = 5;
							else if (tpstringv[tpstrpos][0] == '5')
								transponder->fec = 6;
							else if (tpstringv[tpstrpos][0] == '6')
								transponder->fec = 7;
							else if (tpstringv[tpstrpos][0] == '7')
								transponder->fec = 8;
							else if (tpstringv[tpstrpos][0] == '8')
								transponder->fec = 9;
							else if (tpstringv[tpstrpos][0] == '9')
								transponder->fec = 10;
							else
								transponder->fec = 0;
							break;
						}
						
						case 5: {
							if (g_str_equal(tpstringv[tpstrpos], "AUTO")) {
								transponder->rof = 0;
							} else if (g_str_equal(tpstringv[tpstrpos], "0.35")) {
								transponder->rof = 1;
							} else if (g_str_equal(tpstringv[tpstrpos], "0.25")) {
								transponder->rof = 2;
							} else if (g_str_equal(tpstringv[tpstrpos], "0.20")) {
								transponder->rof = 3;
							} else {
								transponder->rof = 0;
							}
							break;
						}
						
						case 6: {
							if (g_str_equal(tpstringv[tpstrpos], "QPSK\n")) {
								transponder->mod = 0;
							} else if (g_str_equal(tpstringv[tpstrpos], "8PSK\n")) {
								transponder->mod = 1;
							} else {
								transponder->mod = 2;
							}
							break;
						}
						
						default: break;
					}
					
					tpstrpos++;							
				}
				
				if ((transponder->mod == 2) || (transponder->sys == 2) || (transponder->sr < 1000) || (transponder->sr > 45000) ||
				     (transponder->freq < 3400) || (transponder->freq > 12750) || ((transponder->sys == 0) && ((transponder->mod != 0) || (transponder->rof != 0))) ) {
					g_free(transponder);
				} else {
					transponder->id = currentid;
					transponders.list = g_slist_prepend(transponders.list, transponder);
					currentid++;
					if ((transponder->freq > 3400) && (transponder->freq < 4200))
						transponders.cbandnum++;
					if ((transponder->freq > 10700) && (transponder->freq < 12750))
						transponders.kubandnum++;
					//printf("%u %u %u %u %u %u\n", transponder->freq, transponder->pol, transponder->sr, transponder->fec, transponder->sys, transponder->mod);
				}
				
				g_strfreev(tpstringv);
		}
				
		transponders.list = g_slist_reverse(transponders.list);
		
		fclose(file);
		
		if (g_slist_length(transponders.list) > 0) {
			return TRUE;
		} else {
			g_warning("Empty transponders list");
			return FALSE;
		}
	} else {
		g_warning("Empty transponders file");
		return FALSE;
	}
}


void satscan_window_file_page_select(GtkFileChooserButton *widget, gpointer data)
{
	gchar *filename;
	gchar *filecontent;
	gsize filelength;
	GError *error = NULL;
	GtkAssistant *assistant;
	gint num;
	GtkWidget *page;
	gboolean result = FALSE;
		
	if ((filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(satscanwindow.satfile))) == NULL) return;
	
	if (g_file_get_contents((const gchar *)filename, &filecontent, &filelength, &error)) {
		assistant = GTK_ASSISTANT(data);
		num = gtk_assistant_get_current_page(assistant);
		page = gtk_assistant_get_nth_page(assistant, num);
		if ((filecontent[0] == 0x0D) && (filecontent[1] == 0x0A) && (filecontent[2] == 0x5B) && (filecontent[3] == 0x55)) {
			//FastSatFinder .ini file
			result = satscan_window_parse_fsf_transponders(filecontent, filelength);
		} else {
			//Scan-S2 .ini file
			result = satscan_window_parse_scans2_transponders(filename);
		}
		
		if (result) {
			gtk_assistant_set_page_complete(assistant, page, TRUE);
			if ((transponders.cbandnum == 0) || (transponders.kubandnum == 0)) {
				if (transponders.cbandnum == 0) {
					gtk_widget_set_sensitive(satscanwindow.cbandrb, FALSE);
					gtk_widget_set_sensitive(satscanwindow.kubandrb, TRUE);
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(satscanwindow.kubandrb), TRUE);
				} else if (transponders.kubandnum == 0) {
					gtk_widget_set_sensitive(satscanwindow.cbandrb, TRUE);
					gtk_widget_set_sensitive(satscanwindow.kubandrb, FALSE);
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(satscanwindow.cbandrb), TRUE);
				}
			}
		} else {
			gtk_assistant_set_page_complete(assistant, page, FALSE);
		}
	} else {
		g_error("Error: %s\n", error->message);
		g_error_free(error);
	}
    
    g_free(filecontent);
    g_free(filename);
}

GtkWidget *satscan_window_file_page_new(GtkAssistant *assistant)
{
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *satname_l, *satfile_l, *intro_l;
	GtkFileFilter *filter;
	
	gentable = gtk_table_new(3, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	intro_l = gtk_label_new(_("This wizard will guide you through satellite scan process.\nYou need transponders file in FSF or SCAN-S(2) format."));
	gtk_table_attach_defaults(GTK_TABLE(gentable),intro_l,0,2,0,1);
	
	satname_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(satname_l),_("<b>Transponders file:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), satname_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,1,2);
	
	satscanwindow.satfile = gtk_file_chooser_button_new(_("Choose transponders file"), GTK_FILE_CHOOSER_ACTION_OPEN);
	g_signal_connect(G_OBJECT(satscanwindow.satfile), "file-set", G_CALLBACK(satscan_window_file_page_select), (gpointer)assistant);
	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.ini");
	gtk_file_filter_set_name(filter, _("Transponders INI file"));
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(satscanwindow.satfile), filter);

	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), satscanwindow.satfile);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,1,2);
	
	satfile_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(satfile_l),_("<b>Satellite name:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), satfile_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,2,3);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		satscanwindow.satname = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.satname),_("Not selected"));
	#else
		satscanwindow.satname = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.satname), _("Not selected"));
	#endif
	gtk_widget_set_sensitive(satscanwindow.satname, FALSE);
	gtk_combo_box_set_active(GTK_COMBO_BOX(satscanwindow.satname), 0);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), satscanwindow.satname);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,2,3);
	
	return gentable;
}

GtkWidget *satscan_window_params_page_new(GtkAssistant *assistant)
{
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *port11_l, *repeats11_l, *port10_l, *band_l;
	GtkWidget *cellview;
	GtkTreeIter iter;
	GtkListStore *store;
	GtkWidget *box;
		
	gentable = gtk_table_new(6, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	port10_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(port10_l),_("<b>Diseqc 1.0 port:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), port10_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,0,1);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		satscanwindow.port10 = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port10),_("Unused"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port10),_("Port A"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port10),_("Port B"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port10),_("Port C"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port10),_("Port D"));
	#else
		satscanwindow.port10 = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port10), _("Unused"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port10), _("Port A"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port10), _("Port B"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port10), _("Port C"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port10), _("Port D"));
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(satscanwindow.port10), settings_read_int("satscan_port10", 0));
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), satscanwindow.port10);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,0,1);
		
	port11_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(port11_l),_("<b>Diseqc 1.1 port:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), port11_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,1,2);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		satscanwindow.port11 = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port11),_("Unused"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port11),_("Port 1"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port11),_("Port 2"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port11),_("Port 3"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port11),_("Port 4"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port11),_("Port 5"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port11),_("Port 6"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port11),_("Port 7"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.port11),_("Port 8"));
	#else
		satscanwindow.port11 = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port11), _("Unused"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port11), _("Port 1"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port11), _("Port 2"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port11), _("Port 3"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port11), _("Port 4"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port11), _("Port 5"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port11), _("Port 6"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port11), _("Port 7"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.port11), _("Port 8"));
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(satscanwindow.port11), settings_read_int("satscan_port11", 0));
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), satscanwindow.port11);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,1,2);
	
	repeats11_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(repeats11_l),_("<b>Diseqc 1.1 repeats:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), repeats11_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,2,3);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		satscanwindow.repeats11 = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.repeats11),_("0"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.repeats11),_("1"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(satscanwindow.repeats11),_("2"));
	#else
		satscanwindow.repeats11 = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.repeats11), _("0"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.repeats11), _("1"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(satscanwindow.repeats11), _("2"));
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(satscanwindow.repeats11), settings_read_int("satscan_repeats11", 1));
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), satscanwindow.repeats11);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,2,3);
		
	band_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(band_l),_("<b>Band to scan:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), band_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,3,4);
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
	#else
		box = gtk_vbox_new(TRUE, 2);
	#endif
	
	satscanwindow.cbandrb = gtk_radio_button_new_with_label(NULL, _("C Band"));
	satscanwindow.kubandrb = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(satscanwindow.cbandrb), _("Ku Band"));
	
	gtk_box_pack_start(GTK_BOX(box), satscanwindow.cbandrb, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(box), satscanwindow.kubandrb, TRUE, TRUE, 2);
	
	gtk_table_attach_defaults(GTK_TABLE(gentable), box, 1, 2, 3, 4);

	
	return gentable;
}

GtkWidget *satscan_window_progress_page_new(GtkAssistant *assistant)
{
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *intro_l, *vbox, *hbox;
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
	#else
		vbox = gtk_vbox_new(FALSE, 5);
	#endif
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
		
	#if GTK_CHECK_VERSION(3, 0, 0)
		hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 3);
	#else
		hbox = gtk_vbox_new(FALSE, 3);
	#endif
	satscanwindow.progress = gtk_progress_bar_new();
	gtk_box_pack_start(GTK_BOX(hbox), satscanwindow.progress, TRUE, TRUE, 0);
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(satscanwindow.progress), " ");
		gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(satscanwindow.progress), TRUE);
	#endif
	
	satscanwindow.scanbutton = gtk_button_new_with_label(_("Start scan"));
	g_signal_connect(G_OBJECT(satscanwindow.scanbutton), "clicked", G_CALLBACK(button_clicked), (gpointer)assistant);
	gtk_box_pack_start(GTK_BOX(hbox), satscanwindow.scanbutton, FALSE, FALSE, 0);
	
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);
	
	return vbox;
}

void satscan_window_id_foreach(gpointer data, gpointer user_data)
{
	struct _transponder *transponder;
	struct _satscanid *satscanid;
		
	transponder = (struct _transponder *)data;
	satscanid = (struct _satscanid *)user_data;
	
	if (transponder->id == satscanid->id) {
		satscanid->transponder = transponder;
	}	
}

void satscan_window_transponders_toggle(GtkCellRendererToggle *renderer, gchar *path, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	struct _transponder *transponder;
	struct _satscanid satscanid;
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(satscanwindow.tplist));
	if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
		gtk_tree_model_get(model, &iter, TPS_ID, &satscanid.id, -1);
		satscanid.transponder = NULL;
		g_slist_foreach(transponders.list, satscan_window_id_foreach, &satscanid);
		if (satscanid.transponder != NULL) {
			if (satscanid.transponder->sticky) satscanid.transponder->sticky = FALSE; else satscanid.transponder->sticky = TRUE;
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, TPS_SELECTED, satscanid.transponder->sticky, -1);
		}
		
	}
}

GtkWidget *satscan_window_transponders_page_new(GtkAssistant *assistant)
{
	GtkWidget          *gentable;
	GtkWidget          *sw;
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2, *column3, *column4;
	GtkListStore       *store;
	GtkTreeIter        iter;
	 
	gentable = gtk_table_new(3, 3, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	sw = gtk_scrolled_window_new(NULL, NULL);
	satscanwindow.tplist = gtk_tree_view_new();
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
	
	gtk_container_add(GTK_CONTAINER (sw), satscanwindow.tplist);
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(satscanwindow.tplist), TRUE);
			
	renderer = gtk_cell_renderer_toggle_new();
	g_object_set(G_OBJECT(renderer), "activatable", TRUE, NULL);
	g_signal_connect(G_OBJECT(renderer), "toggled", G_CALLBACK(satscan_window_transponders_toggle), NULL);
	column1 = gtk_tree_view_column_new_with_attributes("", renderer, "active", TPS_SELECTED, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(satscanwindow.tplist), column1);
	column2 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column2, "");
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(column2, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column2, renderer, "stock-id", TPS_STATUS, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(satscanwindow.tplist), column2);
	renderer = gtk_cell_renderer_text_new();
	column3 = gtk_tree_view_column_new_with_attributes(_("Name"), renderer, "text", TPS_NAME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(satscanwindow.tplist), column3);
	renderer = gtk_cell_renderer_progress_new();
	column4 = gtk_tree_view_column_new_with_attributes(_("Signal"), renderer, "value", TPS_SIGNAL, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(satscanwindow.tplist), column4);
	
	store = gtk_list_store_new(TPS_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_UINT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(satscanwindow.tplist), GTK_TREE_MODEL(store));
	g_object_unref(store);
	
	gtk_table_attach_defaults(GTK_TABLE(gentable), sw, 0, 3, 0, 1);
	
	
	return gentable;
}

GtkWidget *satscan_window_confirmation_page_new(GtkAssistant *assistant)
{
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *satname_l, *satfile_l, *intro_l;
	GtkFileFilter *filter;
	
	gentable = gtk_table_new(2, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	intro_l = gtk_label_new(_("Satellite scan successfully finished.\nIf you are done, then it is time to leave."));
	gtk_table_attach_defaults(GTK_TABLE(gentable), intro_l, 0, 2, 0, 1);
	
	satscanwindow.addtps = gtk_check_button_new_with_label(_("Save selected transponders to database"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(satscanwindow.addtps), settings_read_boolean("satscan_savetps", FALSE));
	gtk_table_attach(GTK_TABLE(gentable), satscanwindow.addtps, 0, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
	
	return gentable;
}

gint satscan_window_page_change(gint current_page, gpointer data)
{
	if (current_page == 1) {
		transponders.port10 = gtk_combo_box_get_active(GTK_COMBO_BOX(satscanwindow.port10));
		transponders.port11 = gtk_combo_box_get_active(GTK_COMBO_BOX(satscanwindow.port11));
		transponders.repeats11 = gtk_combo_box_get_active(GTK_COMBO_BOX(satscanwindow.repeats11));
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(satscanwindow.cbandrb))) {
			transponders.band = SATSCAN_BAND_C;
		} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(satscanwindow.kubandrb))) {
			transponders.band = SATSCAN_BAND_KU;
		}
		settings_append_int("satscan_port10", transponders.port10);
		settings_append_int("satscan_port11", transponders.port11);
		settings_append_int("satscan_repeats11", transponders.repeats11);
	}
	
	return current_page + 1;
}

gboolean satscan_window_create(GtkWindow *rootwindow)
{
	GtkWidget *label;
	guint i;
	#if !GTK_CHECK_VERSION(3, 2, 0)
		GdkPixbuf *pixbuf;
	#endif
	
	satscanwindow.assistant = gtk_assistant_new();
	gtk_widget_set_size_request(satscanwindow.assistant, 450, 300);
	gtk_window_set_title(GTK_WINDOW(satscanwindow.assistant), _("Satellite scaner"));
	gtk_window_set_modal(GTK_WINDOW(satscanwindow.assistant), TRUE);
	gtk_window_set_transient_for(GTK_WINDOW(satscanwindow.assistant), GTK_WINDOW(rootwindow));
	gtk_window_set_position(GTK_WINDOW(satscanwindow.assistant), GTK_WIN_POS_CENTER_ON_PARENT);
	gtk_assistant_set_forward_page_func(GTK_ASSISTANT(satscanwindow.assistant), satscan_window_page_change, NULL, NULL);
	satscanpage[0].widget = satscan_window_file_page_new(GTK_ASSISTANT(satscanwindow.assistant));
	satscanpage[1].widget = satscan_window_params_page_new(GTK_ASSISTANT(satscanwindow.assistant));
	satscanpage[2].widget = satscan_window_progress_page_new(GTK_ASSISTANT(satscanwindow.assistant));
	satscanpage[3].widget = satscan_window_transponders_page_new(GTK_ASSISTANT(satscanwindow.assistant));
	satscanpage[4].widget = satscan_window_confirmation_page_new(GTK_ASSISTANT(satscanwindow.assistant));
	
	#if !GTK_CHECK_VERSION(3, 2, 0)
		pixbuf = gdk_pixbuf_new_from_file(RESOURCE_SATSCAN_WIZARD, NULL);
	#endif
	
	for (i = 0; i < 5; i++) {
		satscanpage[i].index = gtk_assistant_append_page (GTK_ASSISTANT (satscanwindow.assistant), satscanpage[i].widget);
		gtk_assistant_set_page_title (GTK_ASSISTANT (satscanwindow.assistant), satscanpage[i].widget, gettext(satscanpage[i].title));
		#if !GTK_CHECK_VERSION(3, 2, 0)
			gtk_assistant_set_page_header_image(GTK_ASSISTANT (satscanwindow.assistant), satscanpage[i].widget, pixbuf);
		#endif
		gtk_assistant_set_page_type (GTK_ASSISTANT (satscanwindow.assistant), satscanpage[i].widget, satscanpage[i].type);
		gtk_assistant_set_page_complete (GTK_ASSISTANT (satscanwindow.assistant), satscanpage[i].widget, satscanpage[i].complete);
	}
	
	#if !GTK_CHECK_VERSION(3, 2, 0)
		gdk_pixbuf_unref(pixbuf);
	#endif
	
	g_signal_connect (G_OBJECT (satscanwindow.assistant), "cancel", G_CALLBACK (assistant_cancel), NULL);
	g_signal_connect (G_OBJECT (satscanwindow.assistant), "close", G_CALLBACK (assistant_close), NULL);
	gtk_widget_show_all (satscanwindow.assistant);
	
	return TRUE;
}

gboolean satscan_window_scan_status_callback(gpointer data)
{
	struct _satscanprogress *fprogress;
	gchar *progressmsg;
	
	if ((fprogress = (struct _satscanprogress *)data) != NULL) {
		progressmsg = g_strdup_printf(_("%.0f%% (%s)"), fprogress->full * 100.0, fprogress->tpstring);
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(satscanwindow.progress), fprogress->full);
	} else {
		progressmsg = g_strdup_printf(_("100%%"));
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(satscanwindow.progress), 1.0);
	}
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(satscanwindow.progress), progressmsg);
	g_free(progressmsg);
		
	return FALSE;
}

void satscan_window_scan_foreach(gpointer data, gpointer user_data)
{
	struct _satscanprogress *fprogress;
	struct _transponder *transponder;
	
	fprogress = (struct _satscanprogress *)user_data;
	transponder = (struct _transponder *)data;
	
	if (((transponder->freq > 3400) && (transponder->freq < 4200)) && (fprogress->band == SATSCAN_BAND_KU)) return;
	if (((transponder->freq > 10700) && (transponder->freq < 12750)) && (fprogress->band == SATSCAN_BAND_C)) return;
	if ((transponder->sr < (fprogress->frontend->symbol_rate_min/1000)) || (transponder->sr > (fprogress->frontend->symbol_rate_max/1000))) goto finalize; 
		
	if (fprogress->status_callback != NULL) {
		g_sprintf(fprogress->tpstring, "%u %s %u %s", transponder->freq, gsfcore_string_definition(DEFINITION_POLARISATION, transponder->pol), transponder->sr, gsfcore_string_definition(DEFINITION_FEC, transponder->fec));
		g_idle_add(fprogress->status_callback, fprogress);
	}
	
	dvb_satscan_tune_callback(transponder, fprogress->frontend);
	
	finalize:
	
	if (fprogress->status_callback != NULL) {
		fprogress->full += fprogress->step;
		if (fprogress->full > 1.0) fprogress->full = 1.0;
		if (fprogress->full < 0.0) fprogress->full = 0.0;
	}
}

void satscan_window_fill_foreach(gpointer data, gpointer user_data)
{
	struct _satscanmodel *fmodel;
	struct _transponder *transponder;
	GtkTreeIter iter;
	gchar *tpdescr;
	
	fmodel = (struct _satscanmodel *)user_data;
	transponder = (struct _transponder *)data;
	
	if (((transponder->freq > 3400) && (transponder->freq < 4200)) && (fmodel->band == SATSCAN_BAND_KU)) return;
	if (((transponder->freq > 10700) && (transponder->freq < 12750)) && (fmodel->band == SATSCAN_BAND_C)) return;
		
	gtk_list_store_append(GTK_LIST_STORE(fmodel->model), &iter);
	tpdescr = g_strdup_printf("%u %s %u %s", transponder->freq, gsfcore_string_definition(DEFINITION_POLARISATION, transponder->pol), transponder->sr, gsfcore_string_definition(DEFINITION_FEC, transponder->fec));
	gtk_list_store_set(GTK_LIST_STORE(fmodel->model), &iter, TPS_SELECTED, transponder->sticky, TPS_STATUS, transponder->tuned ? GTK_STOCK_YES : GTK_STOCK_NO, TPS_NAME, tpdescr, TPS_SIGNAL, transponder->signal, TPS_ID, /*fmodel->index*/transponder->id, -1);
	g_free(tpdescr);
}

gboolean satscan_window_scan_status_fill(gpointer data)
{
	struct _satscanmodel fmodel;
	GtkWidget *page;
	GtkTreeModel *model;
	
	satscan_window_scan_status_callback(NULL);
	
	page = gtk_assistant_get_nth_page(GTK_ASSISTANT(satscanwindow.assistant), 2);
	gtk_widget_set_sensitive(GTK_WIDGET(satscanwindow.scanbutton), TRUE);
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(satscanwindow.tplist));
	gtk_list_store_clear(GTK_LIST_STORE(model));
	
	fmodel.model = gtk_tree_view_get_model(GTK_TREE_VIEW(satscanwindow.tplist));
	fmodel.band = transponders.band;
	
	g_slist_foreach(transponders.list, satscan_window_fill_foreach, &fmodel);
	
	gtk_assistant_set_page_complete(GTK_ASSISTANT(satscanwindow.assistant), page, TRUE);
	
	return FALSE;
}

gboolean satscan_window_scan(struct _frontend *frontend)
{
	gsize tplength;
	struct _satscanprogress fprogress;
	GtkTreeModel *model;
			
	
	if (transponders.band == SATSCAN_BAND_C) {
		tplength = transponders.cbandnum;
	} else if (transponders.band == SATSCAN_BAND_KU) {
		tplength = transponders.kubandnum;
	}
	
	if ((tplength > 0) && (frontend != NULL)) {
		
		fprogress.full = 0.0;
		fprogress.step = 1.0 / (gfloat)tplength;
		fprogress.band = transponders.band;
		fprogress.status_callback = satscan_window_scan_status_callback;
		fprogress.frontend = frontend;
		
		g_slist_foreach(transponders.list, satscan_window_scan_foreach, &fprogress); 
		
	}
		
}

static void button_clicked(GtkButton *button, GtkAssistant *assistant)
{
	GtkWidget *page;
	gsize tplength;
	struct _satscanprogress fprogress;
	GtkTreeModel *model;
			
	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
	page = gtk_assistant_get_nth_page(assistant, 2);
	
	gsfcore_satscan_start(FALSE);
}

transponders_t satscan_window_get_transponders(void)
{
	return &transponders;
}

static void assistant_cancel (GtkAssistant *assistant, gpointer data)
{
	gsfcore_satscan_start(TRUE);
	gtk_widget_destroy(GTK_WIDGET(assistant));
}

void satscan_window_save_foreach(gpointer data, gpointer user_data)
{
	struct _transponder *transponder;
	struct _transponders *transponders;
	struct _service *svc;
	
	transponder = (struct _transponder *)data;
	transponders = (struct _transponders *)user_data;
	
	if (transponder != NULL) {
		if (transponder->sticky) {
			svc = g_malloc0(sizeof(struct _service));
			svc->name = g_strdup_printf("TP %u", transponder->freq);
			svc->freq = transponder->freq;
			svc->sr = transponder->sr;
			svc->pol = transponder->pol;
			svc->fec = transponder->fec;
			svc->sys = transponder->sys;
			svc->mod = transponder->mod;
			svc->rof = transponder->rof;
			svc->type = SERVICE_TRANSPONDER;
			svc->caid = 0;
			svc->sid = 0;
			svc->key = NULL;
			svc->id = 0;
			svc->nip = NULL;
			svc->nmask = NULL;
			svc->nmac = NULL;
			svc->nprog = NULL;
			svc->npid = 0;
			svc->picture = NULL;
			svc->pids = NULL;
			svc->port10 = transponders->port10;
			svc->port11 = transponders->port11;
			svc->rep11 = transponders->repeats11;
			svc->lnb22khz = 0;
			svc->lnbtype = 0;
			transponders->svclist = g_slist_prepend(transponders->svclist, svc);
			//printf("ADD SVC: %s\n", svc->name);
		}
	}
}

void satscan_window_svclist_free_foreach(gpointer data, gpointer user_data)
{
	struct _service *svc;
	
	svc = (struct _service *)data;
	
	if (svc != NULL) g_free(svc->name);
}

void satscan_window_list_free_foreach(gpointer data, gpointer user_data)
{
	struct _transponder *tp;
	
	tp = (struct _transponder *)data;
	
	if (tp != NULL) g_free(tp);
}

static void assistant_close(GtkAssistant *assistant, gpointer data)
{
	gboolean savetps;
	
	savetps = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(satscanwindow.addtps));
	settings_append_boolean("satscan_savetps", savetps);
	
	if (savetps) {
		transponders.svclist = NULL;
		g_slist_foreach(transponders.list, satscan_window_save_foreach, &transponders);
		transponders.svclist = g_slist_reverse(transponders.svclist);
		
		//printf("Satscan taransponders to save: %u\n", g_slist_length(transponders.svclist));
		
		service_add_multiple(&transponders.svclist);
		
		//g_slist_foreach(transponders.svclist, satscan_window_svclist_free_foreach, NULL);
		g_slist_free(transponders.svclist);
		transponders.svclist = NULL;
	}
	
	g_slist_foreach(transponders.list, satscan_window_list_free_foreach, NULL);
	g_slist_free(transponders.list);
	transponders.list = NULL;
	
	gsfcore_satscan_start(TRUE);
	gtk_widget_destroy(GTK_WIDGET(assistant));
}
