/*
 *      scanwindow.c
 *      
 *      Copyright 2009 - 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>
#include "dvbscan.h"
#include "resources.h"
#include "dbstorage.h"
#include "gsfcore.h"
//#include "mainwindow.h"
#include "common/iso3166.h"
#include "common/odf.h"

#define SCANWINDOW_STATUSBAR_CONTEXT "scanstatus"

enum {
	SVC_SELECTED = 0,
	SVC_TYPE,
	SVC_NAME,
	SVC_PROVIDER,
	SVC_SID,
	SVC_PMT,
	SVC_VPID,
	SVC_APID,
	SVC_DPID,
	SVC_CAID,
	SVC_ID,
	SVC_COLUMNS
};

enum {
	SCANWINDOW_TP_SELECTED = 0,
	SCANWINDOW_TP_TYPE,
	SCANWINDOW_TP_POSITION,
	SCANWINDOW_TP_FREQUENCY,
	SCANWINDOW_TP_SR,
	SCANWINDOW_TP_FEC,
	SCANWINDOW_TP_POLARISATION,
	SCANWINDOW_TP_SYSTEM,
	SCANWINDOW_TP_MODULATION,
	SCANWINDOW_TP_ROLLOFF,
	SCANWINDOW_TP_ID,
	SCANWINDOW_TP_COLUMNS
};

struct sscanwindow {
	//transponder scan window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *svcpage;
	GtkWidget *tppage;
	GtkWidget *toolbar;
	GtkToolItem *scan_tb, *add_tb, *save_tb, *sep_tb, *close_tb;
	GtkWidget *statusbar, *progressbar;
	GtkWidget *list;
	GtkWidget *tplist;
	guint timeout;
	guint datasrc[3];
	dvbscan_t instance;
};
struct sscanwindow scanwindow;

struct _scanprivatedata {
	guint counter;
	GtkTreeModel *svcmodel, *tpmodel;
	GdkPixbuf *ptv, *pradio, *pdata, *pother;
	struct _tuneparams *tuneparams;
	gushort llcode;
	GSList *svclist;
};


static void scanwindow_onclose(GtkWidget *object, gpointer data);
static gint scanwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
static void scanwindow_list_toggle(GtkCellRendererToggle *renderer, gchar *path, gpointer data);
static void scanwindow_tplist_toggle(GtkCellRendererToggle *renderer, gchar *path, gpointer data);
static void scanwindow_list_textedit(GtkCellRendererText *renderer, gchar *path, gchar *new_text, gpointer data);
static void scanwindow_init_list(GtkWidget *list);
static void scanwindow_init_tplist(GtkWidget *list);
static gboolean scanwindow_update_progress(gpointer data);
static gchar **scanwindow_markup_string(struct _srcservice *service, gboolean pangomarkup);
static void scanwindow_status_refresh(gchar *networkname, struct tm *networktime);
static void scanwindow_rescan_response(GtkDialog *dialog, gint response_id, gpointer data);
static void scanwindow_services_add_foreach(gpointer data, gpointer user_data);
static void scanwindow_transponders_add_foreach(gpointer data, gpointer user_data);
static gboolean scanwindow_thread_finished(gpointer data);
static gboolean scanwindow_set_status(gpointer data);
static void scanwindow_status_callback(gint status);
static gpointer scanwindow_thread(gpointer data);
static void scanwindow_scan(GtkWidget *widget, gpointer data);
static void scanwindow_save_foreach(gpointer data, gpointer user_data);
static void scanwindow_add(GtkWidget *widget, gpointer data);
static void scanwindow_device_error_response(GtkDialog *dialog, gint response_id, gpointer data);


static void scanwindow_onclose(GtkWidget *object, gpointer data)
{
	dvbscan_free(scanwindow.instance);
	if (scanwindow.timeout > 0) {
		g_source_remove(scanwindow.timeout);
		scanwindow.timeout = 0;
	}
	
	scanwindow.visible = FALSE;
	gtk_widget_destroy(scanwindow.win);
}

static gint scanwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	g_signal_emit_by_name(scanwindow.win, "destroy", NULL);
}

static void scanwindow_list_toggle(GtkCellRendererToggle *renderer, gchar *path, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint id;
	struct _scanentry *entry;
	
				
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.list));
	if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
		gtk_tree_model_get(model, &iter, SVC_ID, &id, -1);
		
		entry = (struct _scanentry *)dvbscan_search_entry(scanwindow.instance, id);
		if (entry != NULL) {
			entry->sticky = !entry->sticky;
			
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, SVC_SELECTED, entry->sticky, -1);
			
			if (dvbscan_count_enabled(scanwindow.instance) > 0) {
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.add_tb), TRUE);
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.save_tb), TRUE);
			} else {
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.add_tb), FALSE);
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.save_tb), FALSE);
			}
		}
	}
}

static void scanwindow_tplist_toggle(GtkCellRendererToggle *renderer, gchar *path, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint id;
	struct _scanentry *entry;
	
		
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.tplist));
	if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
		gtk_tree_model_get(model, &iter, SCANWINDOW_TP_ID, &id, -1);
		
		entry = (struct _scanentry *)dvbscan_search_entry(scanwindow.instance, id);
		if (entry != NULL) {
			entry->sticky = !entry->sticky;
			
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, SCANWINDOW_TP_SELECTED, entry->sticky, -1);
			
			if (dvbscan_count_enabled(scanwindow.instance) > 0) {
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.add_tb), TRUE);
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.save_tb), TRUE);
			} else {
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.add_tb), FALSE);
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.save_tb), FALSE);
			}
		}
	}
}

static void scanwindow_list_textedit(GtkCellRendererText *renderer, gchar *path, gchar *new_text, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	gint option;
	struct _scanentry *entry;
	gint id;
	
	if (g_ascii_strcasecmp(new_text, "") != 0) {
		model = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.list));
		option = GPOINTER_TO_INT(data);
		if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
			gtk_tree_model_get(model, &iter, SVC_ID, &id, -1);
			entry = (struct _scanentry *)dvbscan_search_entry(scanwindow.instance, id);
			if (entry != NULL) {
				switch (option) {
					case 0: {
						g_free(entry->values.service.name);
						entry->values.service.name = g_strdup(new_text);
						gtk_list_store_set(GTK_LIST_STORE(model), &iter, SVC_NAME, new_text, -1);
						break;
					}
					case 1: {
						g_free(entry->values.service.provider);
						entry->values.service.provider = g_strdup(new_text);
						gtk_list_store_set(GTK_LIST_STORE(model), &iter, SVC_PROVIDER, new_text, -1);
						break;
					}
					default: break;
				}
			}
		}
	}
}


static void scanwindow_init_list(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2, *column3, *column4, *column5, *column6, *column7, *column8, *column9, *column10, *column11;
	GtkListStore       *store;
	GtkTreeIter        iter;
		
	renderer = gtk_cell_renderer_toggle_new();
	g_object_set(G_OBJECT(renderer), "activatable", TRUE, NULL);
	g_signal_connect(G_OBJECT(renderer), "toggled", G_CALLBACK(scanwindow_list_toggle), NULL);
	column1 = gtk_tree_view_column_new_with_attributes("", renderer, "active", SVC_SELECTED, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	column2 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column2, "");
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(column2, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column2, renderer, "pixbuf", SVC_TYPE, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "editable", TRUE, "editable-set", TRUE, NULL);
	g_signal_connect(G_OBJECT(renderer), "edited", G_CALLBACK(scanwindow_list_textedit), GINT_TO_POINTER(0));
	column3 = gtk_tree_view_column_new_with_attributes(_("Name"), renderer, "text", SVC_NAME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column3);
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "editable", TRUE, "editable-set", TRUE, NULL);
	g_signal_connect(G_OBJECT(renderer), "edited", G_CALLBACK(scanwindow_list_textedit), GINT_TO_POINTER(1));
	column4 = gtk_tree_view_column_new_with_attributes(_("Provider"), renderer, "text", SVC_PROVIDER, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column4);
	renderer = gtk_cell_renderer_text_new();
	column5 = gtk_tree_view_column_new_with_attributes(_("SID"), renderer, "text", SVC_SID, NULL);  
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column5);
	renderer = gtk_cell_renderer_text_new();
	column6 = gtk_tree_view_column_new_with_attributes(_("PMT"), renderer, "text", SVC_PMT, NULL);  
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column6);
	renderer = gtk_cell_renderer_text_new();
	column7 = gtk_tree_view_column_new_with_attributes(_("VPID"), renderer, "text", SVC_VPID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column7);
	renderer = gtk_cell_renderer_text_new();
	column8 = gtk_tree_view_column_new_with_attributes(_("APID"), renderer, "markup", SVC_APID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column8);
	renderer = gtk_cell_renderer_text_new();
	column9 = gtk_tree_view_column_new_with_attributes(_("DPID"), renderer, "markup", SVC_DPID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column9);
	renderer = gtk_cell_renderer_text_new();
	column10 = gtk_tree_view_column_new_with_attributes(_("CAID"), renderer, "markup", SVC_CAID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column10);
	
	store = gtk_list_store_new(SVC_COLUMNS, G_TYPE_BOOLEAN, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, 
											G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING,
											G_TYPE_STRING, G_TYPE_INT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

static void scanwindow_init_tplist(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2, *column3, *column4, *column5, *column6, *column7, *column8, *column9, *column10;
	GtkListStore       *store;
	GtkTreeIter        iter;
		
	
	renderer = gtk_cell_renderer_toggle_new();
	g_object_set(G_OBJECT(renderer), "activatable", TRUE, NULL);
	g_signal_connect(G_OBJECT(renderer), "toggled", G_CALLBACK(scanwindow_tplist_toggle), NULL);
	column1 = gtk_tree_view_column_new_with_attributes("", renderer, "active", SCANWINDOW_TP_SELECTED, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	column2 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column2, "");
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(column2, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column2, renderer, "pixbuf", SCANWINDOW_TP_TYPE, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	renderer = gtk_cell_renderer_text_new();
	column3 = gtk_tree_view_column_new_with_attributes(_("Position"), renderer, "text", SCANWINDOW_TP_POSITION, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column3);
	renderer = gtk_cell_renderer_text_new();
	column4 = gtk_tree_view_column_new_with_attributes(_("Frequency"), renderer, "text", SCANWINDOW_TP_FREQUENCY, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column4);
	renderer = gtk_cell_renderer_text_new();
	column5 = gtk_tree_view_column_new_with_attributes(_("SR"), renderer, "text", SCANWINDOW_TP_SR, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column5);
	renderer = gtk_cell_renderer_text_new();
	column6 = gtk_tree_view_column_new_with_attributes(_("FEC"), renderer, "text", SCANWINDOW_TP_FEC, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column6);
	renderer = gtk_cell_renderer_text_new();
	column7 = gtk_tree_view_column_new_with_attributes(_("Pol"), renderer, "text", SCANWINDOW_TP_POLARISATION, NULL);  
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column7);
	renderer = gtk_cell_renderer_text_new();
	column8 = gtk_tree_view_column_new_with_attributes(_("System"), renderer, "text", SCANWINDOW_TP_SYSTEM, NULL);  
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column8);
	renderer = gtk_cell_renderer_text_new();
	column9 = gtk_tree_view_column_new_with_attributes(_("Mod"), renderer, "text", SCANWINDOW_TP_MODULATION, NULL);  
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column9);
	renderer = gtk_cell_renderer_text_new();
	column10 = gtk_tree_view_column_new_with_attributes(_("Rolloff"), renderer, "text", SCANWINDOW_TP_ROLLOFF, NULL);  
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column10);
	
	store = gtk_list_store_new(SCANWINDOW_TP_COLUMNS, G_TYPE_BOOLEAN, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT,
														G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

static gboolean scanwindow_update_progress(gpointer data)
{
	if (scanwindow.visible) {
		gtk_progress_bar_pulse(GTK_PROGRESS_BAR(scanwindow.progressbar));
	}
	
	return TRUE;
}

static gchar **scanwindow_markup_string(struct _srcservice *service, gboolean pangomarkup)
{
	gint i = 1;
	gchar **strv;
	gchar *res1, *res2, *res3;
	gboolean neednl = TRUE;
	gchar a3lang[4];
	
	
	strv = (gchar **)g_malloc(19*sizeof(gchar *));
	if (pangomarkup) {
		strv[0] = g_strdup("<small>");
	} else {
		strv[0] = g_strdup("");
	}
	while ((service->apid[i-1] != 0) && (i-1 < 16)) {
		if ((i-1==16) || ((i-1<16) && (service->apid[i] == 0))) neednl = FALSE;
		if (service->lang[i-1] != 0) {
			if (pangomarkup) {
				strv[i] = g_strdup_printf(neednl ? "%i (%s)\n" : "%i (%s)", service->apid[i-1], iso3166_get_a3_from_code_static(service->lang[i-1], (char *)&a3lang), NULL);
			} else {
				strv[i] = g_strdup_printf(neednl ? "%i (%s)<text:line-break />" : "%i (%s)", service->apid[i-1], iso3166_get_a3_from_code_static(service->lang[i-1], (char *)&a3lang), NULL);
			}
		} else {
			if (pangomarkup) {
				strv[i] = g_strdup_printf(neednl ? "%i\n" : "%i", service->apid[i-1], NULL);
			} else {
				strv[i] = g_strdup_printf(neednl ? "%i<text:line-break />" : "%i", service->apid[i-1], NULL);
			}
		}
		i++;
	}
	if (i == 1) {
		strv[i] = g_strdup("--");
		i++;
	}
	if (pangomarkup) {
		strv[i] = g_strdup("</small>");
	} else {
		strv[i] = g_strdup("");
	}
	strv[i+1] = NULL;
	
	res1 = g_strjoinv("", strv);
	g_strfreev(strv);
	
	i = 1;
	neednl = TRUE;
	strv = (gchar **)g_malloc(19*sizeof(gchar *));
	if (pangomarkup) {
		strv[0] = g_strdup("<small>");
	} else {
		strv[0] = g_strdup("");
	}
	while ((service->dpid[i-1] != 0) && (i-1 < 16)) {
		if ((i-1==16) || ((i-1<16) && (service->dpid[i] == 0))) neednl = FALSE;
		if (pangomarkup) {
			strv[i] = g_strdup_printf(neednl ? "%i\n" : "%i", service->dpid[i-1], NULL);
		} else {
			strv[i] = g_strdup_printf(neednl ? "%i<text:line-break />" : "%i", service->dpid[i-1], NULL);
		}
		i++;
	}
	if (i == 1) {
		strv[i] = g_strdup("--");
		i++;
	}
	if (pangomarkup) {
		strv[i] = g_strdup("</small>");
	} else {
		strv[i] = g_strdup("");
	}
	strv[i+1] = NULL;
	
	res2 = g_strjoinv("", strv);
	g_strfreev(strv);
	
	i = 1;
	neednl = TRUE;
	strv = (gchar **)g_malloc(19*sizeof(gchar *));
	if (pangomarkup) {
		strv[0] = g_strdup("<small>");
	} else {
		strv[0] = g_strdup("");
	}
	while ((service->caid[i-1] != 0) && (i-1 < 16)) {
		if ((i-1==16) || ((i-1<16) && (service->caid[i] == 0))) neednl = FALSE;
		gchar *tmp = dvbscan_get_cas_name(service->caid[i-1]);
		if (pangomarkup) {
			strv[i] = g_strdup_printf(neednl ? "%s\n" : "%s", tmp, NULL);
		} else {
			strv[i] = g_strdup_printf(neednl ? "%s<text:line-break />" : "%s", tmp, NULL);
		}
		g_free(tmp);
		i++;
	}
	if (i == 1) {
		strv[i] = g_strdup("FTA");
		i++;
	}
	if (pangomarkup) {
		strv[i] = g_strdup("</small>");
	} else {
		strv[i] = g_strdup("");
	}
	strv[i+1] = NULL;
	
	res3 = g_strjoinv("", strv);
	g_strfreev(strv);
	
	strv = (gchar **)g_malloc(4*sizeof(gchar *));
	strv[0] = res1;
	strv[1] = res2;
	strv[2] = res3;
	strv[3] = NULL;
	
	return strv;
}

static void scanwindow_status_refresh(gchar *networkname, struct tm *networktime)
{
	gchar timebuf[64];
	gchar *status;
	guint context_id;
	
	if ((networkname == NULL) && (networktime == NULL)) return;
	
	memset(timebuf, 0, sizeof(timebuf));
	
	if (networktime != NULL) {
		strftime(timebuf, sizeof(timebuf), "%T %d-%m-%Y", networktime);
	}
	
	status = g_strdup_printf(_("Network: %s, UTC time: %s"), networkname == NULL ? _("Unknown") : networkname, (timebuf[0] == '\0') ? _("Unknown") : timebuf);
	
	if (status != NULL) {
		context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(scanwindow.statusbar), SCANWINDOW_STATUSBAR_CONTEXT);
		gtk_statusbar_pop(GTK_STATUSBAR(scanwindow.statusbar), context_id);
		gtk_statusbar_push(GTK_STATUSBAR(scanwindow.statusbar), context_id, status);
		g_free(status);
	}
}

static void scanwindow_rescan_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog));
	
	if (response_id == GTK_RESPONSE_YES) {
		g_signal_emit_by_name(G_OBJECT(scanwindow.scan_tb), "clicked", NULL);
	} else if (response_id == GTK_RESPONSE_NO) {
		g_signal_emit_by_name(scanwindow.win, "destroy", NULL);
	}
}

static void scanwindow_services_add_foreach(gpointer data, gpointer user_data)
{
	struct _scanprivatedata *privatedata;
	struct _scanentry *entry;
	GtkTreeIter iter;
	gchar **markup;
	GdkPixbuf *pixbuf;
	
	privatedata = (struct _scanprivatedata *)user_data;
	entry = (struct _scanentry *)data;
	
	if (entry->type != DVBSCAN_ENTRY_SERVICE) return;
	
	markup = scanwindow_markup_string(&(entry->values.service), TRUE);
	
	switch (entry->values.service.type) {
		case 0x01: pixbuf = privatedata->ptv; break;
		case 0x02: pixbuf = privatedata->pradio; break;
		case 0x0c: pixbuf = privatedata->pdata; break;
		default:   pixbuf = privatedata->pother; break;
	}
	
	if (entry->sticky) privatedata->counter++;
	
	gtk_list_store_append(GTK_LIST_STORE(privatedata->svcmodel), &iter);
	gtk_list_store_set(GTK_LIST_STORE(privatedata->svcmodel), &iter, SVC_SELECTED, entry->sticky, SVC_TYPE, pixbuf, SVC_NAME, entry->values.service.name, SVC_PROVIDER, entry->values.service.provider,
							SVC_SID, entry->values.service.sid, SVC_PMT, entry->values.service.pmt, SVC_VPID, entry->values.service.vpid, SVC_APID, markup[0], SVC_DPID, markup[1],
							SVC_CAID, markup[2], SVC_ID, entry->id, -1);
	g_strfreev(markup);
}

static void scanwindow_transponders_add_foreach(gpointer data, gpointer user_data)
{
	struct _scanprivatedata *privatedata;
	struct _scanentry *entry;
	GtkTreeIter iter;
	gchar posbuf[16];
		
	privatedata = (struct _scanprivatedata *)user_data;
	entry = (struct _scanentry *)data;
	
	if (entry->type != DVBSCAN_ENTRY_TRANSPONDER) return;
	
	if (entry->values.transponder.position < 0) {
		g_snprintf(posbuf, sizeof(posbuf), "%3.1f W", abs(entry->values.transponder.position)/10.0);
	} else {
		g_snprintf(posbuf, sizeof(posbuf), "%3.1f E", abs(entry->values.transponder.position)/10.0);
	}
	
	if (entry->sticky) privatedata->counter++;
	
	gtk_list_store_append(GTK_LIST_STORE(privatedata->tpmodel), &iter);
	gtk_list_store_set(GTK_LIST_STORE(privatedata->tpmodel), &iter, SCANWINDOW_TP_SELECTED, entry->sticky, SCANWINDOW_TP_TYPE, privatedata->pother, SCANWINDOW_TP_POSITION, posbuf,
													SCANWINDOW_TP_FREQUENCY, entry->values.transponder.frequency, SCANWINDOW_TP_SR, entry->values.transponder.sr,
													SCANWINDOW_TP_FEC, gsfcore_string_definition(DEFINITION_FEC, entry->values.transponder.fec),
													SCANWINDOW_TP_POLARISATION, gsfcore_string_definition(DEFINITION_POLARISATION, entry->values.transponder.polarisation),
													SCANWINDOW_TP_SYSTEM, gsfcore_string_definition(DEFINITION_SYSTEM, entry->values.transponder.system),
													SCANWINDOW_TP_MODULATION, gsfcore_string_definition(DEFINITION_MODULATION, entry->values.transponder.modulation),
													SCANWINDOW_TP_ROLLOFF, gsfcore_string_definition(DEFINITION_ROLLOFF, entry->values.transponder.rolloff),
													SCANWINDOW_TP_ID, entry->id, -1);
}

static gboolean scanwindow_thread_finished(gpointer data)
{
	GSList *scanlist;
	struct _scanprivatedata privatedata;
	GtkWidget *dialog;
	gint res;
	
	if (scanwindow.visible) {
		scanlist = dvbscan_get_list(scanwindow.instance);
		if (scanlist != NULL) {
			privatedata.svcmodel = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.list));
			privatedata.tpmodel = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.tplist));
			privatedata.ptv = gdk_pixbuf_new_from_file(RESOURCE_SCANWINDOW_TV, NULL);
			privatedata.pradio = gdk_pixbuf_new_from_file(RESOURCE_SCANWINDOW_RADIO, NULL);
			privatedata.pdata = gdk_pixbuf_new_from_file(RESOURCE_SCANWINDOW_DATA, NULL);
			privatedata.pother = gdk_pixbuf_new_from_file(RESOURCE_SCANWINDOW_OTHER, NULL);
			privatedata.counter = 0;
			
			g_slist_foreach(scanlist, scanwindow_services_add_foreach, &privatedata);
			
			g_slist_foreach(scanlist, scanwindow_transponders_add_foreach, &privatedata);
			
			if (privatedata.counter > 0) {
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.add_tb), TRUE);
				gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.save_tb), TRUE);
			}
			
			#if GTK_CHECK_VERSION(3, 4, 0)
				if (privatedata.ptv != NULL) g_object_unref(G_OBJECT(privatedata.ptv));
				if (privatedata.pradio != NULL) g_object_unref(G_OBJECT(privatedata.pradio));
				if (privatedata.pdata != NULL) g_object_unref(G_OBJECT(privatedata.pdata));
				if (privatedata.pother != NULL) g_object_unref(G_OBJECT(privatedata.pother));
			#else
				if (privatedata.ptv != NULL) gdk_pixbuf_unref(privatedata.ptv);
				if (privatedata.pradio != NULL) gdk_pixbuf_unref(privatedata.pradio);
				if (privatedata.pdata != NULL) gdk_pixbuf_unref(privatedata.pdata);
				if (privatedata.pother != NULL) gdk_pixbuf_unref(privatedata.pother);
			#endif
		} else {
			dialog = gtk_message_dialog_new(GTK_WINDOW(scanwindow.win),
											GTK_DIALOG_MODAL,
											GTK_MESSAGE_QUESTION,
											GTK_BUTTONS_YES_NO,
											_("No services found. Do you want to scan services once again?"),
											NULL);
			g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(scanwindow_rescan_response), NULL);
			gtk_widget_show(dialog);
		}
		
		scanwindow_status_refresh(dvbscan_get_network_name(scanwindow.instance), dvbscan_get_tptime(scanwindow.instance));
		
		gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.scan_tb), TRUE);
	}
	
	return FALSE;
}

static gboolean scanwindow_set_status(gpointer data)
{
	gint status;
	gchar statusstr[16];
	gint sl;
	
	status = GPOINTER_TO_INT(data);
	
	switch(status) {
		case DVBSCAN_STATUS_PAT:
			sl = sprintf((gchar *)&statusstr, "PAT");
			break;
		case DVBSCAN_STATUS_PMT:
			sl = sprintf((gchar *)&statusstr, "PMT");
			break;
		case DVBSCAN_STATUS_SDT:
			sl = sprintf((gchar *)&statusstr, "SDT");
			break;
		case DVBSCAN_STATUS_NIT:
			sl = sprintf((gchar *)&statusstr, "NIT");
			break;
		case DVBSCAN_STATUS_TDT:
			sl = sprintf((gchar *)&statusstr, "TDT");
			break;
		case DVBSCAN_STATUS_NULL:
			sl = sprintf((gchar *)&statusstr, " ");
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(scanwindow.progressbar), 0.0);
			if (scanwindow.timeout > 0) {
				g_source_remove(scanwindow.timeout);
				scanwindow.timeout = 0;
			}
			break;
		default:
			sl = sprintf((gchar *)&statusstr, " ");
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(scanwindow.progressbar), 0.0);
			if (scanwindow.timeout > 0) {
				g_source_remove(scanwindow.timeout);
				scanwindow.timeout = 0;
			}
			break;
	}
	
	statusstr[sl] = '\0';
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(scanwindow.progressbar), (gchar *)&statusstr);
	
	return FALSE;
}

static void scanwindow_status_callback(gint status)
{
	g_idle_add(scanwindow_set_status, GINT_TO_POINTER(status));
	
	if (status == DVBSCAN_STATUS_NULL) {
		g_idle_add(scanwindow_thread_finished, NULL);
	}
}

static void scanwindow_scan(GtkWidget *widget, gpointer data)
{
	GtkTreeModel *model;
	guint context_id;
		
	gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.scan_tb), FALSE);	
	//gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.close_tb), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.add_tb), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.save_tb), FALSE);
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.list));
	gtk_list_store_clear(GTK_LIST_STORE(model));
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.tplist));
	gtk_list_store_clear(GTK_LIST_STORE(model));
	
	/*gtk_label_set_label(GTK_LABEL(scanwindow.netname), " ");
	gtk_label_set_label(GTK_LABEL(scanwindow.utctime), " ");*/
	
	context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(scanwindow.statusbar), SCANWINDOW_STATUSBAR_CONTEXT);
	gtk_statusbar_remove_all(GTK_STATUSBAR(scanwindow.statusbar), context_id);
	
	dvbscan_free(scanwindow.instance);
	
	scanwindow.instance = dvbscan_new(scanwindow.datasrc, DVBSCAN_TYPE_TRANSPONDER, scanwindow_status_callback);
	
	scanwindow.timeout = g_timeout_add(100, scanwindow_update_progress, NULL);
}

static void scanwindow_save_foreach(gpointer data, gpointer user_data)
{
	struct _scanprivatedata *privatedata;
	struct _scanentry *entry;
	struct _service *svc;
	struct _pid *curpid;
	struct _tuneparams *satparams;
	guint firstapid, i;
	gboolean audioset;
	
	privatedata = (struct _scanprivatedata *)user_data;
	entry = (struct _scanentry *)data;
	
	if (!entry->sticky) return;
	
	if (entry->type == DVBSCAN_ENTRY_SERVICE) {
		svc = g_malloc0(sizeof(struct _service));
		svc->id = 0;
		svc->picture = NULL;
		svc->name = g_strdup(entry->values.service.name);
		svc->pos = privatedata->tuneparams->pos;
		svc->positioner = privatedata->tuneparams->positioner;
		svc->possatnum = privatedata->tuneparams->possatnum;
		svc->freq = privatedata->tuneparams->freq;
		svc->sr = privatedata->tuneparams->sr;
		svc->pol = privatedata->tuneparams->pol;
		svc->fec = privatedata->tuneparams->fec;
		svc->sys = privatedata->tuneparams->sys;
		svc->mod = privatedata->tuneparams->mod;
		svc->rof = privatedata->tuneparams->rof;
		svc->port10 = privatedata->tuneparams->port10;
		svc->port11 = privatedata->tuneparams->port11;
		svc->rep11 = privatedata->tuneparams->rep11;
		svc->lnb22khz = privatedata->tuneparams->lnb22khz;
		svc->lnbtype = privatedata->tuneparams->lnbtype;
		svc->caid = entry->values.service.caid[0];
		svc->key = NULL;
		
		switch (entry->values.service.type) {
			case 0x01 ... 0x02: {
				if (entry->values.service.type == 0x01) {
					svc->type = SERVICE_TV_RADIO;
				} else {
					svc->type = SERVICE_RADIO;
				}
				svc->nip = NULL;
				svc->nmask = NULL;
				svc->nmac = NULL;
				svc->nprog = NULL;
				svc->npid = 0;
				svc->sid = entry->values.service.sid;
				
				if (entry->values.service.vpid != 0) {
					curpid = (pid *)g_malloc0(sizeof(pid));
					curpid->type = PID_TYPE_VIDEO;
					curpid->value = entry->values.service.vpid;
					curpid->enabled = TRUE;
					svc->pids = g_slist_prepend(svc->pids, curpid);
				}
				
				firstapid = 0;
								
				if (privatedata->llcode != 0) {
					for (i=0; i<16; i++) {
						if (entry->values.service.apid[i] != 0) {
							if (entry->values.service.lang[i] == privatedata->llcode) {
								firstapid = i;
								break;
							}
						} else break;
					}
				}
				
				audioset = FALSE;
				
				for (i=0; i<16; i++) {
					if (entry->values.service.apid[i] != 0) {
						curpid = (pid *)g_malloc0(sizeof(pid));
						curpid->type = PID_TYPE_AUDIO;
						curpid->value = entry->values.service.apid[i];
						if (i == firstapid) {
							curpid->enabled = TRUE;
							audioset = TRUE;
						} else {
							curpid->enabled = FALSE;
						}
						svc->pids = g_slist_prepend(svc->pids, curpid);
					} else break;
				}
				
				if (entry->values.service.ac3pid != 0) {
					curpid = (pid *)g_malloc0(sizeof(pid));
					curpid->type = PID_TYPE_AUDIO;
					curpid->value = entry->values.service.ac3pid;
					if (!audioset) {
						curpid->enabled = TRUE;
					} else {
						curpid->enabled = FALSE;
					}
					svc->pids = g_slist_prepend(svc->pids, curpid);
				}
				
				if (entry->values.service.ttpid != 0) {
					curpid = (pid *)g_malloc0(sizeof(pid));
					curpid->type = PID_TYPE_TELETEXT;
					curpid->value = entry->values.service.ttpid;
					curpid->enabled = TRUE;
					svc->pids = g_slist_prepend(svc->pids, curpid);
				}
				
				if (entry->values.service.stpid != 0) {
					curpid = (pid *)g_malloc0(sizeof(pid));
					curpid->type = PID_TYPE_SUBTITLES;
					curpid->value = entry->values.service.stpid;
					curpid->enabled = TRUE;
					svc->pids = g_slist_prepend(svc->pids, curpid);
				}
				
				svc->pids = g_slist_reverse(svc->pids);
					
				break;
			}
			
			case 0x0c: {
				svc->type = SERVICE_DATA;
				svc->nip = g_strdup("192.168.0.1");
				svc->nmask = g_strdup("255.255.255.0");
				svc->nmac = g_strdup("00:01:02:03:04:05");
				svc->nprog = NULL;
				svc->npid = entry->values.service.dpid[0];
				svc->sid = entry->values.service.sid;
				svc->pids = NULL;
				break;
			}
				
			default: {
				svc->type = SERVICE_TRANSPONDER;
				svc->nip = NULL;
				svc->nmask = NULL;
				svc->nmac = NULL;
				svc->nprog = NULL;
				svc->npid = 0;
				svc->sid = 0;
				svc->pids = NULL;
				break;
			}
		}
	} else if (entry->type == DVBSCAN_ENTRY_TRANSPONDER) {
		svc = g_malloc0(sizeof(struct _service));
		svc->id = 0;
		svc->picture = NULL;
		svc->name = g_strdup_printf("TP %i", entry->values.transponder.frequency);
		svc->pos = entry->values.transponder.position;
		svc->freq = entry->values.transponder.frequency;
		svc->sr = entry->values.transponder.sr;
		svc->pol = entry->values.transponder.polarisation;
		svc->fec = entry->values.transponder.fec;
		svc->sys = entry->values.transponder.system;
		svc->mod = entry->values.transponder.modulation;
		svc->rof = entry->values.transponder.rolloff;
		satparams = service_search_position_params(svc->pos);
		if (satparams != NULL) {
			svc->port10 = satparams->port10;
			svc->port11 = satparams->port11;
			svc->rep11 = satparams->rep11;
			svc->positioner = satparams->positioner;
			svc->possatnum = satparams->possatnum;
			g_free(satparams);
		} else {
			svc->port10 = privatedata->tuneparams->port10;
			svc->port11 = privatedata->tuneparams->port11;
			svc->rep11 = privatedata->tuneparams->rep11;
			svc->positioner = POSITIONER_DIASABLED;
			svc->possatnum = 0;
		}
		svc->lnb22khz = 0;
		svc->lnbtype = LNB_TYPE_AUTO;
		svc->caid = 0;
		svc->key = NULL;
		svc->type = SERVICE_TRANSPONDER;
		svc->nip = NULL;
		svc->nmask = NULL;
		svc->nmac = NULL;
		svc->nprog = NULL;
		svc->npid = 0;
		svc->sid = 0;
		svc->pids = NULL;
	} 
	
	
	printf("[+] Added service after scan %s %u\n", svc->name, svc->freq);
	privatedata->svclist = g_slist_prepend(privatedata->svclist, svc);
}

static void scanwindow_add(GtkWidget *widget, gpointer data)
{
	GSList *scanlist;
	struct _scanprivatedata privatedata;
	
	
	privatedata.tuneparams = gsfcore_get_tuneparams();
	privatedata.llcode = iso3166_get_code_from_locale_env();
	privatedata.svclist = NULL;
	
	if (privatedata.tuneparams == NULL) {
		g_warning("Failed to get tuner parameters");
		return;
	}
	
	scanlist = dvbscan_get_list(scanwindow.instance);
	if (scanlist != NULL) {
		g_slist_foreach(scanlist, scanwindow_save_foreach, &privatedata);
		if (privatedata.svclist != NULL) {
			privatedata.svclist = g_slist_reverse(privatedata.svclist);
			service_add_multiple(&privatedata.svclist);
			g_slist_free(privatedata.svclist);
			privatedata.svclist = NULL;
		}
	}
	
	g_signal_emit_by_name(scanwindow.win, "destroy", NULL);
}

static void scanwindow_to_file_services_foreach(gpointer data, gpointer user_data)
{
	struct _scanentry *entry;
	odf_t odf;
	gchar *svctype;
	gchar numvalues[3][16];
	gchar **markup;
	
	entry = (struct _scanentry *)data;
	odf = (odf_t)user_data;
	
	if (entry->type != DVBSCAN_ENTRY_SERVICE) return;
	
	markup = scanwindow_markup_string(&(entry->values.service), FALSE);
	
	switch (entry->values.service.type) {
		case 0x01: svctype = "TV"; break;
		case 0x02: svctype = "Radio"; break;
		case 0x0c: svctype = "Data"; break;
		default:   svctype = "Unknown"; break;
	}
	
	memset(numvalues, 0, sizeof(numvalues));
	snprintf(numvalues[0], sizeof(numvalues[0]), "%u", entry->values.service.sid);
	snprintf(numvalues[1], sizeof(numvalues[1]), "%u", entry->values.service.pmt);
	snprintf(numvalues[2], sizeof(numvalues[2]), "%u", entry->values.service.vpid);
	
	odf_add_table_row(odf, entry->values.service.name, entry->values.service.provider, svctype, numvalues[0], numvalues[1], numvalues[2], markup[0], markup[1], markup[2], NULL);
	
	g_strfreev(markup);
}

static void scanwindow_to_file_transpnders_foreach(gpointer data, gpointer user_data)
{
	struct _scanentry *entry;
	odf_t odf;
	gchar numvalues[3][16];
	
	entry = (struct _scanentry *)data;
	odf = (odf_t)user_data;
	
	if (entry->type != DVBSCAN_ENTRY_TRANSPONDER) return;
	
	memset(numvalues, 0, sizeof(numvalues));
		
	if (entry->values.transponder.position < 0) {
		g_snprintf(numvalues[0], sizeof(numvalues[0]), "%3.1f W", abs(entry->values.transponder.position)/10.0);
	} else {
		g_snprintf(numvalues[0], sizeof(numvalues[0]), "%3.1f E", abs(entry->values.transponder.position)/10.0);
	}
	
	snprintf(numvalues[1], sizeof(numvalues[1]), "%u", entry->values.transponder.frequency);
	snprintf(numvalues[2], sizeof(numvalues[2]), "%u", entry->values.transponder.sr);
	
	odf_add_table_row(odf, numvalues[0], numvalues[1], numvalues[2], gsfcore_string_definition(DEFINITION_FEC, entry->values.transponder.fec),
						gsfcore_string_definition(DEFINITION_SYSTEM, entry->values.transponder.system),
						gsfcore_string_definition(DEFINITION_POLARISATION, entry->values.transponder.polarisation),
						gsfcore_string_definition(DEFINITION_MODULATION, entry->values.transponder.modulation),
						gsfcore_string_definition(DEFINITION_ROLLOFF, entry->values.transponder.rolloff), NULL);
}

static void scanwindow_to_file(GtkWidget *widget, gpointer data)
{
	GSList *scanlist;
	GtkWidget *dialog;
	gint res;
	gchar *filename;
	odf_t odf;
	gchar *message;
	
	scanlist = dvbscan_get_list(scanwindow.instance);
	
	if (scanlist != NULL) {
		dialog = gtk_file_chooser_dialog_new(_("Save File"),
											GTK_WINDOW(scanwindow.win),
											GTK_FILE_CHOOSER_ACTION_SAVE,
											GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
											GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
											NULL);
		
		gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
		
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), "services.odt");
		
		res = gtk_dialog_run(GTK_DIALOG(dialog));
		
		if (res == GTK_RESPONSE_ACCEPT) {
			filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
			
			gtk_widget_destroy(dialog);
			
			odf = odf_init(filename, "GSF");
			
			if (odf != NULL) {
				odf_add_table_header(odf, 9, _("Services"), _("Name"), _("Provider"), _("Type"), _("SID"), _("PMT"), _("VPID"), _("APID"), _("DPID"), _("CAID"),  NULL);
				g_slist_foreach(scanlist, scanwindow_to_file_services_foreach, odf);
				odf_add_table_footer(odf);
				
				odf_add_table_header(odf, 8, _("Transponders"), _("Position"), _("Frequency"), _("SR"), _("FEC"), _("Standard"), _("Pol"), _("Modulation"), _("Rolloff"),  NULL);
				g_slist_foreach(scanlist, scanwindow_to_file_transpnders_foreach, odf);
				odf_add_table_footer(odf);
				
				odf_close(odf);
			} else  {
				message = g_strdup_printf(_("Error saving file '%s'"), filename);
				dialog = gtk_message_dialog_new(GTK_WINDOW(scanwindow.win),
											GTK_DIALOG_DESTROY_WITH_PARENT,
											GTK_MESSAGE_ERROR,
											GTK_BUTTONS_CLOSE,
											message);
				gtk_dialog_run(GTK_DIALOG(dialog));
				gtk_widget_destroy(dialog);
				
				g_free(message);
			}
			g_free(filename);
			
			g_signal_emit_by_name(scanwindow.win, "destroy", NULL);
		} else {
			gtk_widget_destroy(dialog);
		}
	}
}

static void scanwindow_device_error_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog));
	
	if (response_id == GTK_RESPONSE_CLOSE) {
		g_signal_emit_by_name(scanwindow.win, "destroy", NULL);
	}
}

gboolean scanwindow_create(GtkWindow *rootwindow)
{
	GtkWidget *vbox, *hbox;
	GtkWidget *notebook;
	GtkWidget *sw;
	GtkWidget *image;
	GtkWidget *dialog;
	frontend_t frontend;
	
	
	if (!scanwindow.visible) {
		scanwindow.instance = NULL;
		scanwindow.visible = TRUE;
		
		scanwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_transient_for(GTK_WINDOW(scanwindow.win), GTK_WINDOW(rootwindow));
		gtk_window_set_position(GTK_WINDOW(scanwindow.win), GTK_WIN_POS_CENTER_ON_PARENT);
		scanwindow.title = _("Transponder structure");
		gtk_window_set_title(GTK_WINDOW(scanwindow.win), scanwindow.title);
		gtk_widget_set_size_request(scanwindow.win, 570, 350);
		//gtk_window_set_default_size(GTK_WINDOW(scanwindow.win), 570, 350);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
		#else
			vbox = gtk_vbox_new(TRUE, 0);
		#endif
		
		scanwindow.toolbar = gtk_toolbar_new();
		
		gtk_toolbar_set_show_arrow(GTK_TOOLBAR(scanwindow.toolbar), TRUE);
		gtk_toolbar_set_style(GTK_TOOLBAR(scanwindow.toolbar), GTK_TOOLBAR_BOTH_HORIZ);
				
		image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_BUTTON);
		scanwindow.scan_tb = gtk_tool_button_new(image, _("Start scan"));
		gtk_tool_item_set_is_important(GTK_TOOL_ITEM(scanwindow.scan_tb), TRUE);
		
		image = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
		scanwindow.add_tb = gtk_tool_button_new(image, _("Add selected"));
		gtk_tool_item_set_is_important(GTK_TOOL_ITEM(scanwindow.add_tb), TRUE);
		
		image = gtk_image_new_from_stock(GTK_STOCK_SAVE_AS, GTK_ICON_SIZE_BUTTON);
		scanwindow.save_tb = gtk_tool_button_new(image, _("Save to file"));
		gtk_tool_item_set_is_important(GTK_TOOL_ITEM(scanwindow.save_tb), TRUE);
		
		scanwindow.sep_tb = gtk_separator_tool_item_new();
		
		image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
		scanwindow.close_tb = gtk_tool_button_new(image, _("Close"));
		gtk_tool_item_set_is_important(GTK_TOOL_ITEM(scanwindow.close_tb), TRUE);
		
		gtk_toolbar_insert(GTK_TOOLBAR(scanwindow.toolbar), scanwindow.scan_tb, 0);
		gtk_toolbar_insert(GTK_TOOLBAR(scanwindow.toolbar), scanwindow.add_tb, 1);
		gtk_toolbar_insert(GTK_TOOLBAR(scanwindow.toolbar), scanwindow.save_tb, 2);
		gtk_toolbar_insert(GTK_TOOLBAR(scanwindow.toolbar), scanwindow.sep_tb, 3);
		gtk_toolbar_insert(GTK_TOOLBAR(scanwindow.toolbar), scanwindow.close_tb, 4);
		
		gtk_box_pack_start(GTK_BOX(vbox), scanwindow.toolbar, FALSE, FALSE, 0);
		
		notebook = gtk_notebook_new();
		
		sw = gtk_scrolled_window_new(NULL, NULL);
		gtk_container_set_border_width(GTK_CONTAINER(sw), 5);
		scanwindow.list = gtk_tree_view_new();
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
		
		gtk_container_add(GTK_CONTAINER(sw), scanwindow.list);
		scanwindow_init_list(scanwindow.list);
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(scanwindow.list), TRUE);
		
		scanwindow.svcpage = gtk_label_new(_("Services"));
		
		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), sw, scanwindow.svcpage);
		
		sw = gtk_scrolled_window_new(NULL, NULL);
		gtk_container_set_border_width(GTK_CONTAINER(sw), 5);
		scanwindow.tplist = gtk_tree_view_new();
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
		
		gtk_container_add(GTK_CONTAINER(sw), scanwindow.tplist);
		scanwindow_init_tplist(scanwindow.tplist);
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(scanwindow.tplist), TRUE);
		
		scanwindow.tppage = gtk_label_new(_("Transponders"));
		
		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), sw, scanwindow.tppage);
		
		gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
		#else
			hbox = gtk_hbox_new(FALSE, 0);
		#endif
		
		scanwindow.progressbar = gtk_progress_bar_new();
		
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(scanwindow.progressbar), 0.0);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(scanwindow.progressbar), " ");
			gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(scanwindow.progressbar), TRUE);
		#endif
		
		gtk_box_pack_start(GTK_BOX(hbox), scanwindow.progressbar, FALSE, FALSE, 5);
		
		scanwindow.statusbar = gtk_statusbar_new();
		
		gtk_box_pack_start(GTK_BOX(hbox), scanwindow.statusbar, TRUE, TRUE, 5);
		
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
		
		g_signal_connect(G_OBJECT(scanwindow.win), "destroy", G_CALLBACK(scanwindow_onclose), NULL);
		g_signal_connect(G_OBJECT(scanwindow.close_tb), "clicked", G_CALLBACK(scanwindow_close), NULL);
		g_signal_connect(G_OBJECT(scanwindow.scan_tb), "clicked", G_CALLBACK(scanwindow_scan), NULL);
		g_signal_connect(G_OBJECT(scanwindow.add_tb), "clicked", G_CALLBACK(scanwindow_add), NULL);
		g_signal_connect(G_OBJECT(scanwindow.save_tb), "clicked", G_CALLBACK(scanwindow_to_file), NULL);
		
		gtk_container_add(GTK_CONTAINER(scanwindow.win), vbox);
		gtk_widget_show_all(scanwindow.win);
		
		gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.add_tb), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(scanwindow.save_tb), FALSE);
		
		frontend = gsfcore_get_frontend();
		
		if (frontend != NULL) {
			scanwindow.datasrc[0] = frontend->adapter_nr;
			scanwindow.datasrc[1] = frontend->frontend_nr;
			scanwindow.datasrc[2] = 0;
		} else {
			dialog = gtk_message_dialog_new(GTK_WINDOW(scanwindow.win),
											GTK_DIALOG_MODAL,
											GTK_MESSAGE_ERROR,
											GTK_BUTTONS_CLOSE,
											_("Error: Device for scan not opened"),
											NULL);
			g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(scanwindow_device_error_response), NULL);
			gtk_widget_show(dialog);
		}
	} else {
		gtk_window_present(GTK_WINDOW(scanwindow.win));
	}
	
	return TRUE;
}
