/*
 *      servicelist.c
 *      
 *      Copyright 2009 - 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gi18n.h>
#include <string.h>
#include "dbstorage.h"
#include "svcparams.h"
#include "servicelist.h"
//#include "udpserver.h"
#include "udpclient.h"
#include "gsfcore.h"
//#include "mainwindow.h"
#include "dvbstatus.h"
#include "resources.h"


struct _servicelist {
	gboolean visible;
	GtkWidget *win;
	GtkWidget *toolbar;
	GtkWidget *serch_tb;
	GtkToolItem *tb_tune, *tb_properties, *tb_separator1, *tb_new, *tb_edit, *tb_remove, *tb_netreload, *tb_netclear, *tb_separator2;
	GtkWidget *icon_view; /*GtkWidget *tree_view;*/ GtkListStore *store;
	GtkWidget *pm_tune, *pm_sep1, *pm_menu, *pm_new, *pm_edit, *pm_delete, *pm_sep2, *pm_properties;
	GtkAccelGroup *pm_group;
} servicelist;

static guint32 current_service = 0;
static guchar current_service_type = SERVICE_TRANSPONDER;
static gboolean rebuild_index;

enum {
	SERVICELIST_COL_TEXT = 0,
	SERVICELIST_COL_PIXBUF,
	SERVICELIST_COL_ID,
	SERVICELIST_COL_TYPE,
	SERVICELIST_COL_FULLNAME,
	SERVICELIST_NUM_COLS
};

/*enum {
	SERVICELIST_LIST_COL_PIXBUF = 0,
	SERVICELIST_LIST_COL_NAME,
	SERVICELIST_LIST_COL_POS,
	SERVICELIST_LIST_COL_FREQ,
	SERVICELIST_LIST_COL_SR,
	SERVICELIST_LIST_COL_POL,
	SERVICELIST_LIST_COL_FEC,
	SERVICELIST_LIST_COL_SYS,
	SERVICELIST_LIST_COL_MOD,
	SERVICELIST_LIST_COL_ID,
	SERVICELIST_LIST_COL_TYPE,
	SERVICELIST_LIST_NUM_COLS
};*/

enum {
	SERVICELIST_PROP_DESCR = 0,
	SERVICELIST_PROP_VALUE,
	SERVICELIST_PROP_COLS
};


struct _sindex {
	guint32 *vec;
	gsize count;
	guint32 ptr;
};

struct _ssearch {
	guint16 length;
	gchar *request;
};

static void servicelist_tune_service_id(gint id, gint type);
static gchar *servicelist_service_short_name(gchar *name);
static void servicelist_row_deleted(GtkTreeModel *tree_model, GtkTreePath *path, gpointer user_data);
static void servicelist_row_inserted(GtkTreeModel *tree_model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data);
static gboolean servicelist_preselect_service_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
static gboolean servicelist_count_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, guint32 *indexcount);
static gboolean servicelist_save_index_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct _sindex *index);
static gboolean servicelist_close_window(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean servicelist_remove_service_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, service *svc);
static gboolean servicelist_update_service_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, service *svc);
static void servicelist_item_activated(GtkIconView *icon_view, GtkTreePath *tree_path, gpointer data);
static void servicelist_tune_service(GtkMenuItem *menuitem, gpointer data);
static void servicelist_properties_service(GtkMenuItem *menuitem, gpointer data);
static void servicelist_new_service(GtkMenuItem *menuitem, gpointer data);
static void servicelist_edit_service(GtkMenuItem *menuitem, gpointer data);
static void servicelist_remove_service(GtkMenuItem *menuitem, gpointer data);
static void servicelist_reload_net_services(GtkMenuItem *menuitem, gpointer data);
static void servicelist_clear_net_services(GtkMenuItem *menuitem, gpointer data);
static gboolean servicelist_remove_net_services_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GList **rowref_list);
static gboolean servicelist_remove_net_services(void);
static gboolean servicelist_get_selected_service(gboolean netservices, guint *id, guint *type);
static void servicelist_popup_menu_show(GtkWidget *widget, GdkEventButton *event);
static gboolean servicelist_button_press(GtkWidget *widget, GdkEventButton *event);
static gboolean servicelist_popup_menu(GtkWidget *widget);
static void servicelist_selection_changed(GtkIconView *iconview, gpointer data);


static void servicelist_tune_service_id(gint id, gint type)
{
	current_service = id;
	current_service_type = type;
	
	settings_append_int("servicelist_service_id", (gint)id);
	settings_append_int("servicelist_service_type", (gint)type);
	
	gsfcore_tune_start(id, type);
}

static gchar *servicelist_service_short_name(gchar *name)
{
	gulong length;
	gchar buffer[256*4];
	gchar *shortname;
	
	if (name == NULL) return NULL;
	
	length = g_utf8_strlen((const gchar *)name, 256*4);
	if (length > 16) {
		memset(buffer, 0, sizeof(buffer));
		g_utf8_strncpy((gchar *)&buffer, (const gchar *)name, 16);
		shortname = g_strconcat((const gchar *)&buffer, "...", NULL);
	} else {
		shortname = g_strdup(name);
	}
	
	return shortname;
}

static gboolean servicelist_search_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	struct _ssearch *search;
	gchar *currentname, *currentucname;
	glong currentlen;
	
	search = (struct _ssearch *)data;
	
	gtk_tree_model_get(model, iter, SERVICELIST_COL_FULLNAME, &currentname, -1);
	
	if ((currentname != NULL) && (search->request != NULL)) {
		currentlen = g_utf8_strlen(currentname, search->length);
		currentucname = g_utf8_casefold(currentname, search->length);
		if (currentucname != NULL) {
			if (g_utf8_collate(search->request, currentucname) == 0) {
				gtk_icon_view_select_path(GTK_ICON_VIEW(servicelist.icon_view), path);
				gtk_icon_view_scroll_to_path(GTK_ICON_VIEW(servicelist.icon_view), path, TRUE, 0.5, 0.5);
				return TRUE;
			}
			g_free(currentucname);
		}
	}
	
	return FALSE;
}

static void servicelist_search(GtkEntry *entry, gpointer data)
{
	guint16 searchlen;
	const gchar *searchtext;
	struct _ssearch search;
	
	searchlen = gtk_entry_get_text_length(entry);
	searchtext = gtk_entry_get_text(entry);
	
	if ((searchlen == 0) || (searchtext[0] == '\0')) return;
	
	search.length = searchlen;
	search.request = g_utf8_casefold(searchtext, searchlen);
	
	if (search.request != NULL) {
		gtk_tree_model_foreach(GTK_TREE_MODEL(servicelist.store), servicelist_search_foreach, &search);
		g_free(search.request);
	}
}

static void servicelist_search_icon(GtkEntry *entry, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer data)
{
	g_signal_emit_by_name(G_OBJECT(entry), "activate", NULL);
} 

static void servicelist_row_deleted(GtkTreeModel *tree_model, GtkTreePath *path, gpointer user_data)
{
	rebuild_index = TRUE;
}

static void servicelist_row_inserted(GtkTreeModel *tree_model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data)
{
	rebuild_index = TRUE;
}

static gboolean servicelist_preselect_service_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	guint32 id;
	guchar type;
	
	gtk_tree_model_get(model, iter, SERVICELIST_COL_ID, &id, SERVICELIST_COL_TYPE, &type, -1);
	
	if ((id == current_service) && (type == current_service_type)) {
		gtk_icon_view_select_path(GTK_ICON_VIEW(servicelist.icon_view), path);
		gtk_icon_view_scroll_to_path(GTK_ICON_VIEW(servicelist.icon_view), path, TRUE, 0.5, 0.5);
		return TRUE;
	}
	
	return FALSE;
}

static gboolean servicelist_count_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, guint32 *indexcount)
{
	guchar type;
	
	gtk_tree_model_get(model, iter, SERVICELIST_COL_TYPE, &type, -1);
	
	if (type != SERVICE_NETSHARE) *indexcount += 1;
	
	return FALSE;
}

static gboolean servicelist_save_index_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct _sindex *index)
{
	guchar type;
	guint32 id;
	
	gtk_tree_model_get(model, iter, SERVICELIST_COL_ID, &id, SERVICELIST_COL_TYPE, &type, -1);
	
	if (type != SERVICE_NETSHARE) {
		index->vec[index->ptr] = id;
		index->ptr += 1;
	}
	
	return FALSE;
}

static gboolean servicelist_close_window(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	struct _sindex index;
		
	if (rebuild_index) {
		g_message("Writing service database index");
		index.count = 0;
		gtk_tree_model_foreach(GTK_TREE_MODEL(servicelist.store), (GtkTreeModelForeachFunc)servicelist_count_foreach, (gpointer)&index.count);
		
		index.vec = (guint32 *)g_malloc0(1*sizeof(guint32));
		
		if (index.count > 0) {
			index.vec = (guint32 *)g_realloc(index.vec, (index.count+1)*sizeof(guint32));
			index.ptr = 0;
			gtk_tree_model_foreach(GTK_TREE_MODEL(servicelist.store), (GtkTreeModelForeachFunc)servicelist_save_index_foreach, (gpointer)&index);
		}
		index.vec[index.count] = 0;
		index.count += 1;
		
		service_write_index(index.vec, index.count);
		
		g_free(index.vec);
	}
	
	if (servicelist.visible) {
		gtk_widget_destroy(servicelist.win);
		servicelist.visible = FALSE;
		udpclient_set_status_callback(NULL);
	}
	
	return FALSE;
}

static gboolean servicelist_remove_service_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, service *svc)
{
	guint id;
	
	gtk_tree_model_get(model, iter, SERVICELIST_COL_ID, &id, -1);
	
	if (id == svc->id) {
		gtk_list_store_remove(GTK_LIST_STORE(model), iter);
		
		return TRUE;
	} 
	
	return FALSE;
}

gboolean servicelist_remove_service_cb(service *svc)
{
	gtk_tree_model_foreach(GTK_TREE_MODEL(servicelist.store), (GtkTreeModelForeachFunc)servicelist_remove_service_foreach, (gpointer)svc);
	
	return TRUE;
}

static gboolean servicelist_update_service_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, service *svc)
{
	gchar *name;
	guint id;
	GdkPixbuf *image;
	gchar strid[15];
	gchar *imgfn, *tmpfn;
	const gchar *homedir;
	gchar *shortname;
	
	gtk_tree_model_get(model, iter, SERVICELIST_COL_ID, &id, -1);
	
	if (id == svc->id) {
		switch (svc->type) {
			case SERVICE_TV_RADIO: 
				imgfn = g_strdup(RESOURCE_SVCLIST_TVRADIO);
				break;
			case SERVICE_DATA: 
				imgfn = g_strdup(RESOURCE_SVCLIST_INTERNET);
				break;
			case SERVICE_TRANSPONDER: 
				imgfn = g_strdup(RESOURCE_SVCLIST_TRANSPONDER);
				break;
			case SERVICE_RADIO: 
				imgfn = g_strdup(RESOURCE_SVCLIST_RADIO);
				break;
			default:
				imgfn = g_strdup(RESOURCE_SVCLIST_TRANSPONDER);
				break;
		}
		
		if ((svc->picture != NULL) && (strlen(svc->picture) > 0)) {
			if ((homedir = g_getenv("HOME")) == NULL) homedir = g_get_home_dir();
			memset(strid, 0, sizeof(strid));
			g_sprintf(strid, "%u.png", svc->id);
			tmpfn = g_build_filename(homedir, strid, NULL);
			if (g_file_test(tmpfn, G_FILE_TEST_EXISTS)) imgfn = g_strdup(tmpfn);
			g_free(tmpfn);
		}
		
		image = gdk_pixbuf_new_from_file(imgfn, NULL);
		g_free(imgfn);
		
		shortname = servicelist_service_short_name(svc->name);
		name = g_markup_printf_escaped("<b>%s</b>\n<small>%i %s %s %i</small>", shortname, svc->freq, gsfcore_string_definition(DEFINITION_POLARISATION, svc->pol), gsfcore_string_definition(DEFINITION_FEC, svc->fec), svc->sr);
		gtk_list_store_set(GTK_LIST_STORE(model), iter, SERVICELIST_COL_TEXT, name, SERVICELIST_COL_PIXBUF, image, SERVICELIST_COL_TYPE, svc->type, -1);
		
		if (image != NULL) g_object_unref(image);
		g_free(name);
		g_free(shortname);
		
		return TRUE;
	} 
	
	return FALSE;
}

gboolean servicelist_update_service_cb(service *svc)
{
	gtk_tree_model_foreach(GTK_TREE_MODEL(servicelist.store), (GtkTreeModelForeachFunc)servicelist_update_service_foreach, (gpointer)svc);
	
	return TRUE;
}

gboolean servicelist_add_service_cb(service *svc)
{
	GtkTreeIter iter;
	gchar *name;
	GdkPixbuf *image;
	gchar strid[15];
	gchar *imgfn, *tmpfn;
	const gchar *homedir;
	gchar *shortname;
	frontend_t frontend;
	gchar posbuf[16];
		
	switch (svc->type) {
		case SERVICE_TV_RADIO: 
			imgfn = g_strdup(RESOURCE_SVCLIST_TVRADIO);
			break;
		case SERVICE_DATA: 
			imgfn = g_strdup(RESOURCE_SVCLIST_INTERNET);
			break;
		case SERVICE_TRANSPONDER: 
			imgfn = g_strdup(RESOURCE_SVCLIST_TRANSPONDER);
			break;
		case SERVICE_RADIO: 
			imgfn = g_strdup(RESOURCE_SVCLIST_RADIO);
			break;
		default:
			imgfn = g_strdup(RESOURCE_SVCLIST_TRANSPONDER);
			break;
	}
	
	if ((svc->picture != NULL) && (strlen(svc->picture) > 0)) {
		if ((homedir = g_getenv("HOME")) == NULL) homedir = g_get_home_dir();
		memset(strid, 0, sizeof(strid));
		g_sprintf(strid, "%u.png", svc->id);
		tmpfn = g_build_filename(homedir, strid, NULL);
		if (g_file_test(tmpfn, G_FILE_TEST_EXISTS)) imgfn = g_strdup(tmpfn);
		g_free(tmpfn);
	}
	
	image = gdk_pixbuf_new_from_file(imgfn, NULL);
	g_free(imgfn);
	
	frontend = gsfcore_get_frontend();
	
	shortname = servicelist_service_short_name(svc->name);	
	
	if (svc->pos < 0) {
		g_snprintf(posbuf, sizeof(posbuf), "%3.1f W", abs(svc->pos)/10.0);
	} else {
		g_snprintf(posbuf, sizeof(posbuf), "%3.1f E", abs(svc->pos)/10.0);
	}
	
	if (dvb_calculate_values(frontend, svc, TRUE) == DVBSTATUS_VALUES_ALL_GOOD) {
		name = g_markup_printf_escaped("<b>%s</b>\n<small>%s %i %s %s %i</small>", shortname, posbuf, svc->freq, gsfcore_string_definition(DEFINITION_POLARISATION, svc->pol), gsfcore_string_definition(DEFINITION_FEC, svc->fec), svc->sr);
	} else {
		name = g_markup_printf_escaped("<span foreground=\"grey\"><b>%s</b>\n<small>%s %i %s %s %i</small></span>", shortname, posbuf, svc->freq, gsfcore_string_definition(DEFINITION_POLARISATION, svc->pol), gsfcore_string_definition(DEFINITION_FEC, svc->fec), svc->sr);
	}
		
	gtk_list_store_append(servicelist.store, &iter);
	gtk_list_store_set(servicelist.store, &iter, SERVICELIST_COL_TEXT, name, SERVICELIST_COL_PIXBUF, image, SERVICELIST_COL_ID, svc->id, SERVICELIST_COL_TYPE, svc->type, SERVICELIST_COL_FULLNAME, svc->name, -1);
	
	if (image != NULL) g_object_unref(image);
	g_free(name);
	g_free(shortname);
	
	return TRUE;
}

gboolean servicelist_add_netservice_cb(netservice *nsvc)
{
	GtkTreeIter iter;
	gchar *name;
	GdkPixbuf *image;
	gchar *imgfn;
	gchar *shortname;
	gchar *address;
	
	imgfn = g_strdup(RESOURCE_SVCLIST_NETSHARE);
	
	image = gdk_pixbuf_new_from_file(imgfn, NULL);
	g_free(imgfn);
	
	shortname = servicelist_service_short_name(nsvc->name);	
	address = gsfcore_get_netstatus_client_address();
	name = g_markup_printf_escaped("<b>%s</b>\n<small>%s</small>", shortname, address); 
	gtk_list_store_append(servicelist.store, &iter);
	gtk_list_store_set(servicelist.store, &iter, SERVICELIST_COL_TEXT, name, SERVICELIST_COL_PIXBUF, image, SERVICELIST_COL_ID, nsvc->id, SERVICELIST_COL_TYPE, SERVICE_NETSHARE, SERVICELIST_COL_FULLNAME, nsvc->name, -1);
	
	if (image != NULL) g_object_unref(image);
	g_free(name);
	g_free(address);
	g_free(shortname);
	
	return TRUE;
}


static void servicelist_item_activated(GtkIconView *icon_view, GtkTreePath *tree_path, gpointer data)
{
	guint id, type;
	
	if (servicelist_get_selected_service(TRUE, &id, &type)) {
		servicelist_tune_service_id(id, type);
	}
		
	g_signal_emit_by_name(G_OBJECT(servicelist.win), "destroy", NULL);
}

static void servicelist_tune_service(GtkMenuItem *menuitem, gpointer data)
{
	guint id, type;
	
	if (servicelist_get_selected_service(TRUE, &id, &type)) {
		servicelist_tune_service_id(id, type);
	}
		
	g_signal_emit_by_name(G_OBJECT(servicelist.win), "destroy", NULL);
}

static void servicelist_properties_service(GtkMenuItem *menuitem, gpointer data)
{
	GtkWidget *dialog;
	GtkWidget *dialogarea;
	GtkWidget *proptree, *propswin, *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeStore *propstore;
	GtkTreeIter propiter, propchild;
	gchar propstr[256];
	gint res;
	guint id, type;
	service *svc;
	netservice *netsvc;
	guint numstreams, i;
	GSList *iterator;
	struct _pid *pidstr;
		
	if (servicelist_get_selected_service(TRUE, &id, &type)) {
		dialog = gtk_dialog_new_with_buttons(_("Service properties"), 
											GTK_WINDOW(servicelist.win),
											GTK_DIALOG_DESTROY_WITH_PARENT,
											GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
											NULL);
		
		gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
		gtk_widget_set_size_request(dialog, 400, 350);
		
		proptree = gtk_tree_view_new();
		
		renderer = gtk_cell_renderer_text_new();
		column = gtk_tree_view_column_new_with_attributes("Property", renderer, "text", SERVICELIST_PROP_DESCR, NULL);
		gtk_tree_view_append_column(GTK_TREE_VIEW(proptree), column);
		
		renderer = gtk_cell_renderer_text_new();
		column = gtk_tree_view_column_new_with_attributes("Value", renderer, "text", SERVICELIST_PROP_VALUE, NULL);
		gtk_tree_view_append_column(GTK_TREE_VIEW(proptree), column);
		
		propstore = gtk_tree_store_new(SERVICELIST_PROP_COLS, G_TYPE_STRING, G_TYPE_STRING);
		
		if (type == SERVICE_NETSHARE) {
			netsvc = service_net_read_id(id);
			if (netsvc != NULL) {
				gtk_tree_store_append(propstore, &propiter, NULL);
				gtk_tree_store_set(propstore, &propiter, SERVICELIST_PROP_DESCR, _("Service"), SERVICELIST_PROP_VALUE, netsvc->name, -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Type"), SERVICELIST_PROP_VALUE, _("Network shared service"), -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Host"), SERVICELIST_PROP_VALUE, gsfcore_get_netstatus_client_address(), -1);
			}
		} else {
			svc = service_read(id);
			if (svc != NULL) {
				gtk_tree_store_append(propstore, &propiter, NULL);
				gtk_tree_store_set(propstore, &propiter, SERVICELIST_PROP_DESCR, _("Service"), SERVICELIST_PROP_VALUE, svc->name, -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				if (svc->type == SERVICE_TV_RADIO) {
					gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Type"), SERVICELIST_PROP_VALUE, _("TV/Radio service"), -1);
				} else if (svc->type == SERVICE_RADIO) {
					gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Type"), SERVICELIST_PROP_VALUE, _("Radio service"), -1);
				} else if (svc->type == SERVICE_DATA) {
					gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Type"), SERVICELIST_PROP_VALUE, _("Data service"), -1);
				} else if (svc->type == SERVICE_TRANSPONDER) {
					gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Type"), SERVICELIST_PROP_VALUE, _("Empty transponder"), -1);
				} else {
					gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Type"), SERVICELIST_PROP_VALUE, _("Unknown"), -1);
				}
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Host"), SERVICELIST_PROP_VALUE, _("Local service"), -1);
				
				memset(propstr, 0, sizeof(propstr));
				if (svc->pos < 0) {
					g_snprintf(propstr, sizeof(propstr), "%3.1f W", abs(svc->pos)/10.0);
				} else {
					g_snprintf(propstr, sizeof(propstr), "%3.1f E", abs(svc->pos)/10.0);
				}
				
				gtk_tree_store_append(propstore, &propiter, NULL);
				gtk_tree_store_set(propstore, &propiter, SERVICELIST_PROP_DESCR, _("Satellite"), SERVICELIST_PROP_VALUE, propstr, -1);
				
				memset(propstr, 0, sizeof(propstr));
				g_snprintf(propstr, sizeof(propstr), "%u MHz", svc->freq);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Frequency"), SERVICELIST_PROP_VALUE, propstr, -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Polarisation"), SERVICELIST_PROP_VALUE, gsfcore_string_definition(DEFINITION_POLARISATION, svc->pol), -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("FEC"), SERVICELIST_PROP_VALUE, gsfcore_string_definition(DEFINITION_FEC, svc->fec), -1);
				
				memset(propstr, 0, sizeof(propstr));
				g_snprintf(propstr, sizeof(propstr), "%u", svc->sr);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Symbol rate"), SERVICELIST_PROP_VALUE, propstr, -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("System"), SERVICELIST_PROP_VALUE, gsfcore_string_definition(DEFINITION_SYSTEM, svc->sys), -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Modulation"), SERVICELIST_PROP_VALUE, gsfcore_string_definition(DEFINITION_MODULATION, svc->mod), -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Rolloff"), SERVICELIST_PROP_VALUE, gsfcore_string_definition(DEFINITION_ROLLOFF, svc->rof), -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("DiSEqC 1.0"), SERVICELIST_PROP_VALUE, gsfcore_string_definition(DEFINITION_PORT10, svc->port10), -1);
				
				gtk_tree_store_append(propstore, &propchild, &propiter);
				gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("DiSEqC 1.1"), SERVICELIST_PROP_VALUE, gsfcore_string_definition(DEFINITION_PORT11, svc->port11), -1);
				
				if (svc->type != SERVICE_TRANSPONDER) {
					if (svc->type == SERVICE_DATA) {
						memset(propstr, 0, sizeof(propstr));
						g_snprintf(propstr, sizeof(propstr), "0x%x", svc->npid);
						
						gtk_tree_store_append(propstore, &propiter, NULL);
						gtk_tree_store_set(propstore, &propiter, SERVICELIST_PROP_DESCR, _("Data PID"), SERVICELIST_PROP_VALUE, propstr, -1);
						
						gtk_tree_store_append(propstore, &propchild, &propiter);
						gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("IP"), SERVICELIST_PROP_VALUE, (svc->nip == NULL) ? _("Unknown") : svc->nip, -1);
						
						gtk_tree_store_append(propstore, &propchild, &propiter);
						gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("MAC"), SERVICELIST_PROP_VALUE, (svc->nmac == NULL) ? _("Unknown") : svc->nmac, -1);
					} else {
						numstreams = g_slist_length(svc->pids);
						memset(propstr, 0, sizeof(propstr));
						g_snprintf(propstr, sizeof(propstr), "%u", numstreams);
						
						gtk_tree_store_append(propstore, &propiter, NULL);
						gtk_tree_store_set(propstore, &propiter, SERVICELIST_PROP_DESCR, _("Data streams"), SERVICELIST_PROP_VALUE, propstr, -1);
						
						memset(propstr, 0, sizeof(propstr));
						g_snprintf(propstr, sizeof(propstr), "0x%x", svc->sid);
						
						gtk_tree_store_append(propstore, &propchild, &propiter);
						gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("SID"), SERVICELIST_PROP_VALUE, propstr, -1);
						
						memset(propstr, 0, sizeof(propstr));
						g_snprintf(propstr, sizeof(propstr), "0x%x", svc->caid);
						
						gtk_tree_store_append(propstore, &propchild, &propiter);
						gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("CAID"), SERVICELIST_PROP_VALUE, propstr, -1);
						
						for (iterator = svc->pids; iterator != NULL; iterator = iterator->next) {
							pidstr = iterator->data;
							if (pidstr != NULL) {
								memset(propstr, 0, sizeof(propstr));
								g_snprintf(propstr, sizeof(propstr), "%c 0x%x", pidstr->enabled ? '+' : '-', pidstr->value);
								
								gtk_tree_store_append(propstore, &propchild, &propiter);
								
								if (pidstr->type == PID_TYPE_VIDEO) {
									gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Video PID"), SERVICELIST_PROP_VALUE, propstr, -1);
								} else if (pidstr->type == PID_TYPE_AUDIO) {
									gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Audio PID"), SERVICELIST_PROP_VALUE, propstr, -1);
								} else if (pidstr->type == PID_TYPE_DATA) {
									gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Data PID"), SERVICELIST_PROP_VALUE, propstr, -1);
								} else if (pidstr->type == PID_TYPE_SUBTITLES) {
									gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Subtitles PID"), SERVICELIST_PROP_VALUE, propstr, -1);
								} else if (pidstr->type == PID_TYPE_TELETEXT) {
									gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Teletext PID"), SERVICELIST_PROP_VALUE, propstr, -1);
								} else {
									gtk_tree_store_set(propstore, &propchild, SERVICELIST_PROP_DESCR, _("Unknown PID"), SERVICELIST_PROP_VALUE, propstr, -1);
								}
							}
						}
						
					}
				}
				
			}
		}
		
		gtk_tree_view_set_model(GTK_TREE_VIEW(proptree), GTK_TREE_MODEL(propstore));
		gtk_tree_view_expand_all(GTK_TREE_VIEW(proptree));
		g_object_unref(propstore);
		
		propswin = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(propswin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(propswin), GTK_SHADOW_IN);
		gtk_container_set_border_width(GTK_CONTAINER(propswin), 5);
		
		gtk_container_add(GTK_CONTAINER(propswin), proptree);
		
		dialogarea = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
		
		gtk_box_pack_start(GTK_BOX(dialogarea), propswin, TRUE, TRUE, 5);
		
		gtk_widget_show_all(dialog);
		
		res = gtk_dialog_run(GTK_DIALOG(dialog));
		
		gtk_widget_destroy(dialog);
	}
}

static void servicelist_new_service(GtkMenuItem *menuitem, gpointer data)
{
	service_assistant_new(NULL, GTK_WINDOW(servicelist.win));
}

static void servicelist_edit_service(GtkMenuItem *menuitem, gpointer data)
{
	guint id;
	service *svc;
	
	if (servicelist_get_selected_service(FALSE, &id, NULL)) {
		if ((svc = service_read(id)) != NULL) {
			service_assistant_new(svc, GTK_WINDOW(servicelist.win));
		}
	}
	
}

static void servicelist_remove_service(GtkMenuItem *menuitem, gpointer data)
{
	GtkWidget *dialog;
	guint id;
	gint qres;
	service *svc;
	
	if (servicelist_get_selected_service(FALSE, &id, NULL)) {
		if ((svc = service_read(id)) != NULL) {
			dialog = gtk_message_dialog_new(GTK_WINDOW(servicelist.win),
											GTK_DIALOG_DESTROY_WITH_PARENT,
											GTK_MESSAGE_WARNING,
											GTK_BUTTONS_YES_NO,
											_("Removing service \"%s\". Are you sure?"),
											svc->name, 
											NULL);
			qres = gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
			
			if (qres == GTK_RESPONSE_YES) {
				service_remove(svc, servicelist_remove_service_cb);		
			}
		}
	}
	
}

static void servicelist_reload_net_services(GtkMenuItem *menuitem, gpointer data)
{
	struct _netgate *netgate;
			
	netgate = gsfcore_get_netgate();
	
	if (netgate->clientconnected) {
		gsfcore_network_servicelist_refresh();
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_netreload), FALSE);
	}
}

static void servicelist_clear_net_services(GtkMenuItem *menuitem, gpointer data)
{
	struct _netgate *netgate;
			
	netgate = gsfcore_get_netgate();
	
	if (netgate->clientconnected) {
		servicelist_remove_net_services();
		service_net_clear();
		if (service_net_get_number() == 0)
			gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_netclear), FALSE);
	}
}

static gboolean servicelist_remove_net_services_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GList **rowref_list)
{
	guchar type;
	GtkTreeRowReference *rowref;
	
	gtk_tree_model_get(model, iter, SERVICELIST_COL_TYPE, &type, -1);
	
	if (type == SERVICE_NETSHARE) {
		rowref = gtk_tree_row_reference_new(model, path);
		*rowref_list = g_list_append(*rowref_list, rowref);
	}
	
	return FALSE;
}

static gboolean servicelist_remove_net_services(void)
{
	GtkTreeModel *model;
	GList *rr_list = NULL;
	GList *node;
	GtkTreePath *path;
	GtkTreeIter iter;
	
	model = gtk_icon_view_get_model(GTK_ICON_VIEW(servicelist.icon_view));
	gtk_tree_model_foreach(GTK_TREE_MODEL(model), (GtkTreeModelForeachFunc)servicelist_remove_net_services_foreach, &rr_list);
	
	for (node = rr_list;  node != NULL;  node = node->next) {
		if ((path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data)) != NULL) {
			if (gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path)) {
				gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
			}
		}
	}
	
	g_list_foreach(rr_list, (GFunc)gtk_tree_row_reference_free, NULL);
	g_list_free(rr_list);
	
	return TRUE;
}

gboolean servicelist_reload_net_services_cb(gpointer data)
{
	if (servicelist.visible) {
		servicelist_remove_net_services();		
		service_net_read_foreach(servicelist_add_netservice_cb);
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_netreload), TRUE);
		if (service_net_get_number() > 0)
			gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_netclear), TRUE);
	}
	
	return FALSE;
}

void servicelist_udpclient_status_cb(gint event, gpointer data)
{
	if (event == UDPCLIENT_STATUS_LIST_REFRESH) {
		g_idle_add(servicelist_reload_net_services_cb, data);
	}
}

static gboolean servicelist_get_selected_service(gboolean netservices, guint *id, guint *type)
{
	GList *selitems;
	GtkTreePath *path;
	GtkTreeModel *model;
	GtkTreeIter iter;
	guint svcid;
	guint svctype;
	gboolean result;
	
	result = FALSE;
	
	selitems = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(servicelist.icon_view));
	
	if (selitems != NULL) {
		if (!((netservices) && (id == NULL) && (type == NULL))) {
			model = gtk_icon_view_get_model(GTK_ICON_VIEW(servicelist.icon_view));
			if (model != NULL) {
				path = (GtkTreePath *)g_list_nth_data(selitems, 0);
				gtk_tree_model_get_iter(model, &iter, path);
				gtk_tree_model_get(model, &iter, SERVICELIST_COL_ID, &svcid, SERVICELIST_COL_TYPE, &svctype, -1);
				
				if (id != NULL) *id = svcid;
				if (type != NULL) *type = svctype;
				
				if ((!netservices) && (svctype == SERVICE_NETSHARE)) {
					result = FALSE;
				} else {
					result = TRUE;
				}
			}
		} else {
			result = TRUE;
		}
		g_list_foreach(selitems, (GFunc)gtk_tree_path_free, NULL);
		g_list_free(selitems);
	}
	
	return result;
}

static void servicelist_popup_menu_show(GtkWidget *widget, GdkEventButton *event)
{
	gint button, event_time;
	
	if (!servicelist_get_selected_service(FALSE, NULL, NULL)) {
		gtk_widget_set_sensitive(servicelist.pm_edit, FALSE);
		gtk_widget_set_sensitive(servicelist.pm_delete, FALSE);
	} else {
		gtk_widget_set_sensitive(servicelist.pm_edit, TRUE);
		gtk_widget_set_sensitive(servicelist.pm_delete, TRUE);
	}
	
	if (!servicelist_get_selected_service(TRUE, NULL, NULL)) {
		gtk_widget_set_sensitive(servicelist.pm_tune, FALSE);
		gtk_widget_set_sensitive(servicelist.pm_properties, FALSE);
	} else {
		gtk_widget_set_sensitive(servicelist.pm_tune, TRUE);
		gtk_widget_set_sensitive(servicelist.pm_properties, TRUE);
	}
	
	if (event) {
		button = event->button;
		event_time = event->time;
	} else {
		button = 0;
		event_time = gtk_get_current_event_time();
	}
	
	
	gtk_menu_popup(GTK_MENU(servicelist.pm_menu), NULL, NULL, NULL, NULL, button, event_time);
}

static gboolean servicelist_button_press(GtkWidget *widget, GdkEventButton *event)
{
	if ((event->button == 3) && (event->type == GDK_BUTTON_PRESS)) {
		servicelist_popup_menu_show(widget, event);
		return TRUE;
    }
	
	return FALSE;
}

static gboolean servicelist_popup_menu(GtkWidget *widget)
{
	servicelist_popup_menu_show(widget, NULL);
	return TRUE;
}

static void servicelist_selection_changed(GtkIconView *iconview, gpointer data)
{
	if (!servicelist_get_selected_service(FALSE, NULL, NULL)) {
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_edit), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_remove), FALSE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_edit), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_remove), TRUE);
	}
	
	if (!servicelist_get_selected_service(TRUE, NULL, NULL)) {
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_tune), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_properties), FALSE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_tune), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_properties), TRUE);
	}
}

gboolean servicelist_new(GtkWindow *rootwindow)
{
	GtkWidget *listscroll;
    GtkCellRenderer *renderer;
	GtkWidget *vbox;
	GtkToolItem *labelitem;
	GtkToolItem *searchitem;
	struct _netgate *netgate;
	#if GTK_CHECK_VERSION(3, 0, 0)
		GtkStyleContext *context;
	#endif
		
	netgate = gsfcore_get_netgate();
	
	if (!servicelist.visible) {
		servicelist.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_transient_for(GTK_WINDOW(servicelist.win), GTK_WINDOW(rootwindow));
		gtk_window_set_modal(GTK_WINDOW(servicelist.win), TRUE);
		gtk_window_set_position(GTK_WINDOW(servicelist.win), GTK_WIN_POS_CENTER_ON_PARENT);
		gtk_window_set_title(GTK_WINDOW(servicelist.win), _("Service browser"));
		gtk_widget_set_size_request(servicelist.win, 650, 400);
		gtk_window_set_default_size(GTK_WINDOW(servicelist.win), 650, 400);
		
		g_signal_connect(servicelist.win, "destroy", G_CALLBACK(servicelist_close_window), NULL);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
		#else
			vbox = gtk_vbox_new(FALSE, 0);
		#endif
		
		servicelist.toolbar = gtk_toolbar_new();
		
		servicelist.tb_tune = gtk_tool_button_new_from_stock(GTK_STOCK_APPLY);
		gtk_tool_button_set_label(GTK_TOOL_BUTTON(servicelist.tb_tune), _("Tune"));
		gtk_widget_set_tooltip_text(GTK_WIDGET(servicelist.tb_tune), _("Tune service"));
		g_object_set(servicelist.tb_tune, "is-important", TRUE, NULL);
		servicelist.tb_properties = gtk_tool_button_new_from_stock(GTK_STOCK_PROPERTIES);
		gtk_widget_set_tooltip_text(GTK_WIDGET(servicelist.tb_properties), _("View service properties"));
		//g_object_set(servicelist.tb_properties, "is-important", TRUE, NULL);
		servicelist.tb_separator1 = gtk_separator_tool_item_new();
		servicelist.tb_new = gtk_tool_button_new_from_stock(GTK_STOCK_NEW);
		gtk_widget_set_tooltip_text(GTK_WIDGET(servicelist.tb_new), _("Add new service"));
		g_object_set(servicelist.tb_new, "is-important", TRUE, NULL);
		servicelist.tb_edit = gtk_tool_button_new_from_stock(GTK_STOCK_EDIT);
		gtk_widget_set_tooltip_text(GTK_WIDGET(servicelist.tb_edit), _("Edit service parameters"));
		g_object_set(servicelist.tb_edit, "is-important", TRUE, NULL);
		servicelist.tb_remove = gtk_tool_button_new_from_stock(GTK_STOCK_REMOVE);
		gtk_widget_set_tooltip_text(GTK_WIDGET(servicelist.tb_remove), _("Remove service"));
		g_object_set(servicelist.tb_remove, "is-important", TRUE, NULL);
		servicelist.tb_separator2 = gtk_separator_tool_item_new();
		servicelist.tb_netreload = gtk_tool_button_new_from_stock(GTK_STOCK_REFRESH);
		gtk_widget_set_tooltip_text(GTK_WIDGET(servicelist.tb_netreload), _("Reload shared services list"));
		//g_object_set(servicelist.tb_netreload, "is-important", TRUE, NULL);
		servicelist.tb_netclear = gtk_tool_button_new_from_stock(GTK_STOCK_CLEAR);
		gtk_widget_set_tooltip_text(GTK_WIDGET(servicelist.tb_netclear), _("Clear shared services"));
		//g_object_set(servicelist.tb_netclear, "is-important", TRUE, NULL);
		
		//servicelist.tb_connect = gtk_toggle_tool_button_new_from_stock(GTK_STOCK_CONNECT);
						
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_edit), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_remove), FALSE);
		
		if (!netgate->clientconnected)
			gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_netreload), FALSE);
		
		if ((!netgate->clientconnected) || (service_net_get_number() == 0))
			gtk_widget_set_sensitive(GTK_WIDGET(servicelist.tb_netclear), FALSE);
		
		gtk_toolbar_set_show_arrow(GTK_TOOLBAR(servicelist.toolbar), TRUE);
		gtk_toolbar_set_style(GTK_TOOLBAR(servicelist.toolbar), GTK_TOOLBAR_BOTH_HORIZ);
		
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), servicelist.tb_tune, 0);
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), servicelist.tb_properties, 1);
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), servicelist.tb_separator1, 2);
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), servicelist.tb_new, 3);
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), servicelist.tb_edit, 4);
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), servicelist.tb_remove, 5);
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), servicelist.tb_separator2, 6);
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), servicelist.tb_netreload, 7);
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), servicelist.tb_netclear, 8);
		
		
		labelitem = gtk_tool_item_new();
		
		gtk_tool_item_set_expand(GTK_TOOL_ITEM(labelitem), TRUE);
		
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), GTK_TOOL_ITEM(labelitem), 9);
		
		servicelist.serch_tb = gtk_entry_new();
		
		#if GTK_CHECK_VERSION(2, 16, 0)
			gtk_entry_set_icon_from_stock(GTK_ENTRY(servicelist.serch_tb), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_FIND);
		#endif
		
		searchitem = gtk_tool_item_new();
		
		gtk_container_add(GTK_CONTAINER(searchitem), GTK_WIDGET(servicelist.serch_tb));
		
		gtk_toolbar_insert(GTK_TOOLBAR(servicelist.toolbar), GTK_TOOL_ITEM(searchitem), 10);
		
		g_signal_connect(G_OBJECT(servicelist.serch_tb), "activate", G_CALLBACK(servicelist_search), NULL);
		
		#if GTK_CHECK_VERSION(2, 16, 0)
			g_signal_connect(G_OBJECT(servicelist.serch_tb), "icon-press", G_CALLBACK(servicelist_search_icon), NULL);
		#endif
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			context = gtk_widget_get_style_context(GTK_WIDGET(servicelist.toolbar));
			gtk_style_context_add_class(context, GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
		#endif
		
		gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(servicelist.toolbar), FALSE, TRUE, 0);
		
		rebuild_index = FALSE;
		
		servicelist.store = gtk_list_store_new(SERVICELIST_NUM_COLS, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_UINT, G_TYPE_UCHAR, G_TYPE_STRING);
		gtk_list_store_clear(servicelist.store);
		
		service_read_foreach(servicelist_add_service_cb, servicelist_add_netservice_cb, NULL);
		
		g_signal_connect(servicelist.store, "row-deleted", G_CALLBACK(servicelist_row_deleted), NULL);
		g_signal_connect(servicelist.store, "row-inserted", G_CALLBACK(servicelist_row_inserted), NULL);
		
		servicelist.icon_view = gtk_icon_view_new_with_model(GTK_TREE_MODEL(servicelist.store));
		g_signal_connect(servicelist.icon_view, "item-activated", G_CALLBACK(servicelist_item_activated), NULL);
		g_signal_connect(servicelist.icon_view, "button-press-event", G_CALLBACK(servicelist_button_press), NULL);
		g_signal_connect(servicelist.icon_view, "popup-menu", G_CALLBACK(servicelist_popup_menu), NULL);
		g_signal_connect(servicelist.icon_view, "selection-changed", G_CALLBACK(servicelist_selection_changed), NULL);
				
		g_object_unref(servicelist.store);
		
		//if (current_service != 0) gtk_tree_model_foreach(GTK_TREE_MODEL(servicelist.store), servicelist_preselect_service_foreach, NULL);
		
		g_signal_connect_swapped(G_OBJECT(servicelist.tb_tune), "clicked", G_CALLBACK(servicelist_tune_service), NULL);
		g_signal_connect_swapped(G_OBJECT(servicelist.tb_properties), "clicked", G_CALLBACK(servicelist_properties_service), NULL);
		g_signal_connect_swapped(G_OBJECT(servicelist.tb_new), "clicked", G_CALLBACK(servicelist_new_service), NULL);
		g_signal_connect_swapped(G_OBJECT(servicelist.tb_edit), "clicked", G_CALLBACK(servicelist_edit_service), NULL);
		g_signal_connect_swapped(G_OBJECT(servicelist.tb_remove), "clicked", G_CALLBACK(servicelist_remove_service), NULL);
		g_signal_connect_swapped(G_OBJECT(servicelist.tb_netreload), "clicked", G_CALLBACK(servicelist_reload_net_services), NULL);
		g_signal_connect_swapped(G_OBJECT(servicelist.tb_netclear), "clicked", G_CALLBACK(servicelist_clear_net_services), NULL);
				
		//g_signal_connect_swapped(G_OBJECT(selectall), "clicked", G_CALLBACK(select_all), entry);
		
		servicelist.pm_menu = gtk_menu_new();
		servicelist.pm_tune = gtk_image_menu_item_new_from_stock(GTK_STOCK_APPLY, NULL);
		gtk_menu_item_set_label(GTK_MENU_ITEM(servicelist.pm_tune), _("_Tune"));
		servicelist.pm_sep1 = gtk_separator_menu_item_new();
		servicelist.pm_new = gtk_image_menu_item_new_from_stock(GTK_STOCK_NEW, NULL);
		servicelist.pm_edit = gtk_image_menu_item_new_from_stock(GTK_STOCK_EDIT, NULL);
		servicelist.pm_delete = gtk_image_menu_item_new_from_stock(GTK_STOCK_DELETE, NULL);
		servicelist.pm_sep2 = gtk_separator_menu_item_new();
		servicelist.pm_properties = gtk_image_menu_item_new_from_stock(GTK_STOCK_PROPERTIES, NULL);
		
		servicelist.pm_group = gtk_accel_group_new();
		gtk_window_add_accel_group(GTK_WINDOW(servicelist.win), servicelist.pm_group);
		gtk_menu_set_accel_group(GTK_MENU(servicelist.pm_menu), servicelist.pm_group);
		
		#if GTK_CHECK_VERSION(2, 22, 0)
			gtk_widget_add_accelerator(servicelist.pm_tune, "activate", servicelist.pm_group, GDK_KEY_T, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
			gtk_widget_add_accelerator(servicelist.pm_new, "activate", servicelist.pm_group, GDK_KEY_N, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
			gtk_widget_add_accelerator(servicelist.pm_edit, "activate", servicelist.pm_group, GDK_KEY_E, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
			gtk_widget_add_accelerator(servicelist.pm_delete, "activate", servicelist.pm_group, GDK_KEY_D, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
			gtk_widget_add_accelerator(servicelist.pm_properties, "activate", servicelist.pm_group, GDK_KEY_P, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		#else
			gtk_widget_add_accelerator(servicelist.pm_tune, "activate", servicelist.pm_group, GDK_T, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
			gtk_widget_add_accelerator(servicelist.pm_new, "activate", servicelist.pm_group, GDK_N, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
			gtk_widget_add_accelerator(servicelist.pm_edit, "activate", servicelist.pm_group, GDK_E, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
			gtk_widget_add_accelerator(servicelist.pm_delete, "activate", servicelist.pm_group, GDK_D, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
			gtk_widget_add_accelerator(servicelist.pm_properties, "activate", servicelist.pm_group, GDK_P, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		#endif
		
		g_signal_connect(G_OBJECT(servicelist.pm_tune), "activate", G_CALLBACK(servicelist_tune_service), NULL);
		g_signal_connect(G_OBJECT(servicelist.pm_properties), "activate", G_CALLBACK(servicelist_properties_service), NULL);
		g_signal_connect(G_OBJECT(servicelist.pm_new), "activate", G_CALLBACK(servicelist_new_service), NULL);
		g_signal_connect(G_OBJECT(servicelist.pm_edit), "activate", G_CALLBACK(servicelist_edit_service), NULL);
		g_signal_connect(G_OBJECT(servicelist.pm_delete), "activate", G_CALLBACK(servicelist_remove_service), NULL);
		
		gtk_menu_shell_append(GTK_MENU_SHELL(servicelist.pm_menu), servicelist.pm_tune);
		gtk_menu_shell_append(GTK_MENU_SHELL(servicelist.pm_menu), servicelist.pm_sep1);
		gtk_menu_shell_append(GTK_MENU_SHELL(servicelist.pm_menu), servicelist.pm_new);
		gtk_menu_shell_append(GTK_MENU_SHELL(servicelist.pm_menu), servicelist.pm_edit);
		gtk_menu_shell_append(GTK_MENU_SHELL(servicelist.pm_menu), servicelist.pm_delete);
		gtk_menu_shell_append(GTK_MENU_SHELL(servicelist.pm_menu), servicelist.pm_sep2);
		gtk_menu_shell_append(GTK_MENU_SHELL(servicelist.pm_menu), servicelist.pm_properties);
		
		gtk_widget_show_all(servicelist.pm_menu);
		gtk_menu_attach_to_widget(GTK_MENU(servicelist.pm_menu), servicelist.icon_view, NULL);
		
		gtk_icon_view_set_selection_mode(GTK_ICON_VIEW(servicelist.icon_view), GTK_SELECTION_SINGLE);
		
		#if GTK_CHECK_VERSION(2, 22, 0)
			gtk_icon_view_set_item_orientation(GTK_ICON_VIEW(servicelist.icon_view), GTK_ORIENTATION_HORIZONTAL);
		#else
			gtk_icon_view_set_orientation(GTK_ICON_VIEW(servicelist.icon_view), GTK_ORIENTATION_HORIZONTAL);
		#endif
		//gtk_icon_view_set_columns(GTK_ICON_VIEW(servicelist.icon_view), 2);
		gtk_icon_view_set_reorderable(GTK_ICON_VIEW(servicelist.icon_view), TRUE);
		
		renderer = gtk_cell_renderer_pixbuf_new();
		gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(servicelist.icon_view), renderer, TRUE);
		gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(servicelist.icon_view), renderer, "pixbuf", SERVICELIST_COL_PIXBUF, NULL);
		
		renderer = gtk_cell_renderer_text_new();
		gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(servicelist.icon_view), renderer, TRUE);
		gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(servicelist.icon_view), renderer, "markup", SERVICELIST_COL_TEXT, NULL);
		
		listscroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(listscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(listscroll), GTK_SHADOW_IN);
		gtk_container_set_border_width(GTK_CONTAINER(listscroll), 5);
		gtk_container_add(GTK_CONTAINER(listscroll), servicelist.icon_view);
		
		gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(listscroll), TRUE, TRUE, 0);
		
		gtk_container_add(GTK_CONTAINER(servicelist.win), GTK_WIDGET(vbox));
		
		gtk_widget_show_all(servicelist.win);
		
		servicelist.visible = TRUE;
		
		if (current_service == 0) {
			current_service = (guint)settings_read_int("servicelist_service_id", 0);
			current_service_type = (guchar)settings_read_int("servicelist_service_type", SERVICE_TRANSPONDER);
		}
		
		if (current_service != 0) {
			gtk_tree_model_foreach(GTK_TREE_MODEL(servicelist.store), servicelist_preselect_service_foreach, NULL);
		}
		
		udpclient_set_status_callback(servicelist_udpclient_status_cb);
	} else {
		gtk_window_present(GTK_WINDOW(servicelist.win));
	}
	
	return TRUE;
}
