/*
 *      statuswindow.c
 *      
 *      Copyright 2009 - 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gi18n.h>
#include <signal.h>
#include <stdint.h>
#include <stdlib.h>
#include <math.h>

#include "statuswindow.h"
#include "settings.h"
#include "gsfcore.h"
#include "dvbname.h"
#include "dvbstatus.h"
#include "communication.h"
#include "udpclient.h"
#include "dbstorage.h"
#include "netwindow.h"
#include "servicelist.h"
#include "streamconfig.h"
#include "sound.h"
#include "dialogs.h"
#include "grstatus.h"
#include "prefwindow.h"
#include "devwindow.h"
#include "calcwindow.h"
#include "scanwindow.h"
#include "satscanwindow.h"
#include "streamwindow.h"
#include "strcliwindow.h"
#include "tswindow.h"
#include "fswindow.h"
#include "ttwindow.h"
#include "resources.h"


#define THREAD_TIMEOUT  1000000

#define GSF_GETTEXT_PACKAGE "gsf"

struct sstatuswindow {
	//main window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *quit, *pref, *tp, *sound;
	GtkWidget *snr, *agc, *ber;
	GtkStatusbar *statusbar;
	GtkGrstatus *grstatus;
	GtkWidget *statussep;
	
	GtkWidget *menubar, *actionsmenu, *actions_menu, *search_menu, *network_menu, *scan_menu, *satscan_menu, *epg_menu, *teletext_menu, *sep_menu, *quit_menu, *editmenu,
	          *edit_menu, *preferences_menu, *device_menu, *sound_menu, *viewmenu, *view_menu, *fullscreen_menu, *streamingmenu, *streaming_menu,
	          *serverconfig_menu, *serverclients_menu, *tsstruct_menu, *record_menu, *udpbc_menu, *udpmc_menu, *tcpuc_menu, *helpmenu, *help_menu,
	          *calc_menu, *about_menu;
	GtkAccelGroup *accel_group;
	GtkToolbar *toolbar;
	GtkToolItem *search_tb, *device_tb, *sound_tb, *fullscreen_tb, *preferences_tb, *exit_tb, *separator1_tb, *separator2_tb;
	GtkWidget *historymenu_tb;
	gboolean sound_lock;
	gboolean streaming_lock;
	GtkStatusIcon *trayicon;
	GtkWidget *traymenu, *servicesmenu;
	GtkWidget *search_tm, *services_tm, *sep_tm, *quit_tm, *entry_sm;
};
struct sstatuswindow statuswindow;

struct _historyentry {
	guint32 id;
	guchar type;
	gchar name[256*4];
};

#if GTK_CHECK_VERSION(3, 0, 0)
	static GtkApplication *app;
#endif

static GArray *history = NULL;
static gchar historylength = 0;

struct _globalsettings globalsettings;

static void statuswindow_tune_problem_message_response(GtkDialog *dialog, gint response_id, gpointer data);
static gboolean statuswindow_tune_problem_message(gpointer data);
static void statuswindow_streamserver_restart_response(GtkDialog *dialog, gint response_id, gpointer data);
static gboolean statuswindow_streamserver_restart(gpointer data);
static void statuswindow_stop_signal_handler(int sig, siginfo_t *si, void *unused);
static void statuswindow_service_signal_handler(int sig, siginfo_t *si, void *unused);
static gboolean statuswindow_stop_program(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_close_main_window(GtkWidget *widget, GdkEvent  *event, gpointer data);
static gboolean statuswindow_search_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_devconfig_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_preferences_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_network_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_scan_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_epg_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_teletext_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_sconf_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_tsstruct_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_calculations_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_about_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_fullscreen_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean statuswindow_toggle_sound_toolbar(GtkToggleToolButton *toggle_tool_button, gpointer data);
static gboolean statuswindow_toggle_sound_menu(GtkCheckMenuItem *checkmenuitem, gpointer data);
static gboolean statuswindow_toggle_streaming_menu(GtkCheckMenuItem *checkmenuitem, gpointer data);
static gboolean statuswindow_status_message(gpointer data);
static void statuswindow_trayaction(GtkStatusIcon *status_icon, gpointer data);
static void statuswindow_traypopup(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer data);
static void statuswindow_traytune(GtkMenuItem *menuitem, gpointer data);
static gboolean statuswindow_traytooltip(GtkStatusIcon *status_icon, gint x, gint y, gboolean keyboard_mode, GtkTooltip *tooltip, gpointer data);

static GtkWidget *statuswindow_build_window(void);
static void statuswindow_event_cb(gint event, gpointer data);
static void statuswindow_init_settings(void);
static void statuswindow_init_threads(void);

int main(int argc, char *argv[]);


static void statuswindow_tune_problem_message_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

static gboolean statuswindow_tune_problem_message(gpointer data)
{
	GtkWidget *dialog;
	gint res;
	gchar *message;
	
	switch (GPOINTER_TO_INT(data)) {
		case DVBSTATUS_VALUES_IF_WRONG:
			message = _("Service frequency not supported by selected device tuner or driver");
			break;
		case DVBSTATUS_VALUES_SR_WRONG:
			message = _("Service symbol rate not supported by selected device demodulator or driver");
			break;
		default:
			message = _("Service parameters not supported by selected hardware or driver");
			break;
	}
	
	dialog = gtk_message_dialog_new(GTK_WINDOW(statuswindow.win),
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
									message,
									NULL);
	
	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(statuswindow_tune_problem_message_response), NULL);
	gtk_widget_show(dialog);
	
	return FALSE;
}

void statuswindow_graph_set_colors(GdkColor *mincolor, GdkColor *maxcolor)
{
	gchar *colorstr;
	
	if ((mincolor == NULL) || (maxcolor == NULL)) return;
	
	colorstr = gdk_color_to_string(mincolor);
	settings_append_string("graph_low_color", colorstr);
	g_free(colorstr);
	
	colorstr = gdk_color_to_string(maxcolor);
	settings_append_string("graph_high_color", colorstr);
	g_free(colorstr);
	
	gtk_grstatus_set_colors(statuswindow.grstatus, mincolor, maxcolor);
}

static void statuswindow_streamserver_restart_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	if (response_id == GTK_RESPONSE_YES) {
		gsfcore_restart_streamserver();
	}
	
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

static gboolean statuswindow_streamserver_restart(gpointer data)
{
	GtkWidget *dialog;
	gint res;
	
	dialog = gtk_message_dialog_new(GTK_WINDOW(statuswindow.win),
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_QUESTION,
									GTK_BUTTONS_YES_NO,
									_("Streaming server service stopped. Do you want to restart it?"),
									NULL);
	
	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(statuswindow_streamserver_restart_response), NULL);
	gtk_widget_show(dialog);
	
	return FALSE;
}

static void statuswindow_stop_signal_handler(int sig, siginfo_t *si, void *unused)
{
	gsfcore_stop();
	
	settings_close();

	printf("[-] System signal recieved: %s from %ld\n", sys_siglist[sig], si->si_pid);
	
	exit(EXIT_FAILURE);
}

static void statuswindow_service_signal_handler(int sig, siginfo_t *si, void *unused)
{
	if (gsfcore_handle_streamserver_stop_signal(si->si_pid)) {
		printf("[-] Stream server service stopped (PID: %ld)\n", si->si_pid);
		g_idle_add(statuswindow_streamserver_restart, NULL);
	}
}

static gboolean statuswindow_stop_program(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	GtkWidget *win;
	GList *wlist, *wnext;
	
	gsfcore_stop();
	
	settings_close();
	
	gtk_widget_destroy(statuswindow.win);
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		#if GLIB_CHECK_VERSION(2, 32, 0)
			g_application_quit(G_APPLICATION(app));
		#else
			wlist = gtk_application_get_windows(GTK_APPLICATION(app));
			while (wlist) {
				win = wlist->data;
				wnext = wlist->next;
				
				gtk_widget_destroy(GTK_WIDGET(win));
				
				wlist = wnext;
			}
		#endif
	#else
		gtk_main_quit();
	#endif
	
	return FALSE;
}

static gboolean statuswindow_close_main_window(GtkWidget *widget, GdkEvent  *event, gpointer data)
{
	if (settings_read_boolean("program_exit_on_close", FALSE)) {
		g_signal_emit_by_name(statuswindow.win, "destroy", NULL);
		return FALSE;
	} else {
		return gtk_widget_hide_on_delete(widget);
	}
}

static gboolean statuswindow_satscan_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	satscan_window_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_search_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	servicelist_new(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_devconfig_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	devwindow_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_preferences_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	prefwindow_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_network_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	netwindow_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_scan_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	scanwindow_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_epg_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	epgwindow_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_teletext_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	if (gsfcore_service_has_pid_type(PID_TYPE_TELETEXT, NULL)) {
		ttwindow_create(GTK_WINDOW(statuswindow.win));
	} else {
		printf("[-] Failed to find teletext PID\n");
	}
	
	return FALSE;
}

static gboolean statuswindow_sconf_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	streamwindow_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_sclients_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	strcliwindow_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_tsstruct_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	tswindow_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_calculations_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	calcwindow_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_about_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	dialog_about(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean statuswindow_fullscreen_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	fswindow_create();
	return FALSE;
}

static gboolean statuswindow_toggle_sound_toolbar(GtkToggleToolButton *toggle_tool_button, gpointer data)
{
	if (!statuswindow.sound_lock) {
		statuswindow.sound_lock = TRUE;
		if (gtk_toggle_tool_button_get_active(toggle_tool_button)) {
			gsfcore_sound_set_enabled(TRUE);
			g_print("Sounds: on\n");
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.sound_menu),TRUE);
		} else {
			gsfcore_sound_set_enabled(FALSE);
			g_print("Sounds: off\n");
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.sound_menu),FALSE);
		}
		statuswindow.sound_lock = FALSE;
	}
	
	return FALSE;
}

static gboolean statuswindow_toggle_sound_menu(GtkCheckMenuItem *checkmenuitem, gpointer data)
{
	if (!statuswindow.sound_lock) {
		statuswindow.sound_lock = TRUE;
		if (gtk_check_menu_item_get_active(checkmenuitem)) {
			gsfcore_sound_set_enabled(TRUE);
			g_print("Sounds: on\n");
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(statuswindow.sound_tb),TRUE);
		} else {
			gsfcore_sound_set_enabled(FALSE);
			g_print("Sounds: off\n");
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(statuswindow.sound_tb),FALSE);
		}
		statuswindow.sound_lock = FALSE;
	}
	
	return FALSE;
}

static gboolean statuswindow_toggle_streaming_menu(GtkCheckMenuItem *checkmenuitem, gpointer data)
{
	gint option = GPOINTER_TO_INT(data);
	gboolean value = FALSE;
	
	if (!statuswindow.streaming_lock) {
		statuswindow.streaming_lock = TRUE;
		
		switch (option) {
			case 0: {
				value = gtk_check_menu_item_get_active(checkmenuitem);
				if (gsfcore_set_streamserver_record_status(value)) {
					gtk_check_menu_item_set_active(checkmenuitem, value);
				}
				break;
			}
			
			case 1: {
				value = gtk_check_menu_item_get_active(checkmenuitem);
				if (gsfcore_set_streamserver_udpbc_status(value)) {
					gtk_check_menu_item_set_active(checkmenuitem, value);
				}
				break;
			}
			
			case 2: {
				value = gtk_check_menu_item_get_active(checkmenuitem);
				if (gsfcore_set_streamserver_udpmc_status(value)) {
					gtk_check_menu_item_set_active(checkmenuitem, value);
				}
				break;
			}
			
			case 3: {
				value = gtk_check_menu_item_get_active(checkmenuitem);
				if (gsfcore_set_streamserver_tcpuc_status(value)) {
					gtk_check_menu_item_set_active(checkmenuitem, value);
				}
				break;
			}
		
			default: break;
		}
		statuswindow.streaming_lock = FALSE;
	}
	
	return FALSE;
}

static gboolean statuswindow_streaming_menu_init(GtkMenuItem *menuitem, gpointer data)
{
	gboolean sensitive = FALSE;
	gboolean record = FALSE, udpbc = FALSE, udpmc = FALSE, tcpuc = FALSE;
	
	if (!gsfcore_get_streamserver_available()) {
		sensitive = FALSE;
	} else {
		gsfcore_get_streamserver_services(&record, &udpbc, &udpmc, &tcpuc);
		sensitive = TRUE;
	}
	
	gtk_widget_set_sensitive(statuswindow.serverclients_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.tsstruct_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.record_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.udpbc_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.udpmc_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.tcpuc_menu, sensitive);
	
	if (sensitive) {
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.record_menu), record);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.udpbc_menu), udpbc);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.udpmc_menu), udpmc);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.tcpuc_menu), tcpuc);
	}
	
	return FALSE;
}

static void statuswindow_history_tune_start(GtkMenuItem *menuitem, gpointer data)
{
	gint historyid;
	struct _historyentry *currenthe;
	
	historyid = GPOINTER_TO_INT(data);
	currenthe = &g_array_index(history, struct _historyentry, historyid);
	
	if (currenthe != NULL) gsfcore_tune_start(currenthe->id, currenthe->type);
}


static gboolean statuswindow_update_signal(gpointer data)
{
	frontend_t frontend;
	gchar str[30];
	gdouble level;
	
	frontend = (frontend_t)data;
	
	if (frontend == NULL) return FALSE;
	
	if ((level = (gdouble)(frontend->reset ? 0 : frontend->snr)/settings_read_float("scale_snr", 100.0)) > 1.0) level = 1.0;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(statuswindow.snr), level);
	memset(str, 0, sizeof(str));
	if (frontend->lock) g_sprintf(&str, "%u [LOCK]", frontend->reset ? 0 : frontend->snr); else g_sprintf(&str, "%u", frontend->reset ? 0 : frontend->snr);
	gtk_grstatus_set_value(statuswindow.grstatus, (gchar)(level > 1.0 ? 100 : level*100));
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.snr), str);
	
	if ((level = (gdouble)((frontend->signal)/settings_read_float("scale_agc", 1000.0))) > 1.0) level = 1.0;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(statuswindow.agc), level);
	memset(str, 0, sizeof(str));
	if (frontend->carrier) g_sprintf(&str,"%u [CARRIER]", frontend->signal); else g_sprintf(&str, "%u", frontend->signal);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.agc), str);
	
	if ((level = (gdouble)((frontend->ber)/settings_read_float("scale_ber", 65535.0))) > 1.0) level = 1.0;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(statuswindow.ber), level);
	memset(str, 0, sizeof(str));
	g_sprintf(&str, "%u", frontend->ber);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.ber), str);
	
	/*if (svc != NULL) {
		switch (svc->type) {
			case SERVICE_TV_RADIO:
			case SERVICE_RADIO:
			case SERVICE_DATA:
			case SERVICE_TRANSPONDER:
				
				break;
			
			case SERVICE_NETSHARE:
				g_printf("[+] Server signal status: %s@%s SNR: %u, AGC: %u, BER: %u, LOCK: %u, CARRIER: %u\n", svc->name, svc->node, frontend.snr, frontend.signal, frontend.ber, frontend.lock, frontend.carrier);
				break;
			
			default: break;
		}
	}*/
	
	fswindow_display_signal_status(frontend);

	return FALSE;
}

static gboolean statuswindow_transponder_selected(gpointer data)
{
	service_t svc;
	GtkWidget *history_submenu;
	struct _historyentry historyentry;
	gint i;
	struct _historyentry *currenthe;
	
	svc = (service_t)data;
	
	if (svc == NULL) return FALSE;
	
	memset(historyentry.name, 0, sizeof(historyentry.name));
	sprintf(historyentry.name, "%s", svc->name);
	
	historyentry.id = svc->id;
	historyentry.type = svc->type;
	g_array_prepend_val(history, historyentry);
	if (historylength < 10) historylength++;
	
	if (statuswindow.historymenu_tb != NULL) gtk_widget_destroy(statuswindow.historymenu_tb);
	statuswindow.historymenu_tb = gtk_menu_new();
	
	for (i=0; i<historylength; i++) {
		if ((currenthe = &g_array_index(history, struct _historyentry, i)) != NULL) {
			history_submenu = gtk_menu_item_new_with_label(currenthe->name);
			gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.historymenu_tb), history_submenu);
			g_signal_connect(G_OBJECT(history_submenu), "activate", G_CALLBACK(statuswindow_history_tune_start), GINT_TO_POINTER(i));
			gtk_widget_show_all(history_submenu);
		}
	}
	gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(statuswindow.search_tb), GTK_WIDGET(statuswindow.historymenu_tb));
	
	return FALSE;
}

static gboolean statuswindow_update_tune_params(gpointer data)
{
	devwindow_update_page(DEV_PREFERENCE_ID_FRONTEND);
	devwindow_update_page(DEV_PREFERENCE_ID_FILTERS);
	devwindow_update_page(DEV_PREFERENCE_ID_MENU);
	
	return FALSE;
}

static gboolean statuswindow_update_device_params(gpointer data)
{
	devwindow_update_page(DEV_PREFERENCE_ID_DRIVER);
	devwindow_update_page(DEV_PREFERENCE_ID_MENU);
	
	return FALSE;
}

static gboolean statuswindow_status_message(gpointer data)
{
	guint context_id;
	gint message_id;
	gchar *message;
	service_t svc;
	
	message_id = GPOINTER_TO_INT(data);
	
	message = NULL;
	
	switch (message_id) {
		case STATUS_MESSAGE_SVC_NETSARE: {
			svc = gsfcore_get_service();
			if (svc != NULL) {
				message = g_strdup_printf(_("Service %s at host %s"), svc->name, svc->node);
				fswindow_display_infobar(svc);
			}
			break;
		}
		
		case STATUS_MESSAGE_SVC: {
			svc = gsfcore_get_service();
			if (svc != NULL) {
				switch (svc->type) {
					case SERVICE_TV_RADIO: {
						message = g_strdup_printf(_("Service %s"), svc->name);
						break;
					}
					case SERVICE_RADIO: {
						message = g_strdup_printf(_("Radio service %s"), svc->name);
						break;
					}
					case SERVICE_DATA: {
						message = g_strdup_printf(_("Data service %s"), svc->name);
						break;
					}
					case SERVICE_TRANSPONDER: {
						message = g_strdup_printf(_("Transponder %s"), svc->name);
						break;
					}
					case SERVICE_NETSHARE: {
						message = g_strdup_printf(_("Service %s at host %s"), svc->name, svc->node);
						break;
					}
					default: {
						message = g_strdup_printf(_("Service %s"), svc->name);
						break;
					}
				}
				fswindow_display_infobar(svc);
			}
			break;
		}
		
		case STATUS_MESSAGE_FR_OPENING: {
			message = g_strdup(_("Opening frontend..."));
			break;
		}
		
		case STATUS_MESSAGE_FR_OPEN_FAIL: {
			message = g_strdup(_("Failed to open frontend"));
			break;
		}
		
		case STATUS_MESSAGE_TP_SELECTION: {
			message = g_strdup(_("Select transponder to tune..."));
			break;
		}
		
		case STATUS_MESSAGE_FR_READONLY_MODE: {
			message = g_strdup(_("Device opened in readonly mode..."));
			svc = gsfcore_get_service();
			if (svc != NULL) {
				fswindow_display_infobar(svc);
			}
			break;
		}
		
		case STATUS_MESSAGE_FR_NOT_FOUND: {
			message = g_strdup(_("Unable to find any frontend to open"));
			break;
		}
		
		case STATUS_MESSAGE_TP_TUNING: {
			message = g_strdup(_("Tuning to transponder..."));
			break;
		}
		
		case STATUS_MESSAGE_TP_TUNING_FAIL: {
			message = g_strdup(_("Failed to tune transponder"));
			break;
		}
		
		case STATUS_MESSAGE_DEV_NOT_FOUND: {
			message = g_strdup(_("No devices found in system..."));
			break;
		}
		
		default: break;
	}
	
	if (message != NULL) {
		context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statuswindow.statusbar), message);
		gtk_statusbar_push(GTK_STATUSBAR(statuswindow.statusbar), context_id, message);	
		g_free(message);
	}
	
	return FALSE;
}

static gboolean statuswindow_lock_sound_buttons(gpointer data)
{
	gboolean lock;
	
	lock = GPOINTER_TO_INT(data);
	
	if (lock) {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.sound_tb),FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.sound_menu),FALSE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.sound_tb),TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.sound_menu),TRUE);
	}
   	
	return FALSE;
}

static gboolean statuswindow_lock_search_buttons(gpointer data)
{
	gboolean lock;
	frontend_t frontend;
	
	lock = GPOINTER_TO_INT(data);
	
	if (lock) {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_tb), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_menu), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_tm), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.services_tm), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.satscan_menu), FALSE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_tb), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_menu), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_tm), TRUE);
		frontend = gsfcore_get_frontend();
		if ((frontend != NULL) && (dvb_frontend_opened(frontend->frontend_fd))) {
			gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.services_tm), TRUE);
			gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.satscan_menu), TRUE);
		}
	}
   	
	return FALSE;
}

static gboolean statuswindow_lock_scan_buttons(gpointer data)
{
	gboolean lock;
	service_t svc; 
	
	lock = GPOINTER_TO_INT(data);
	
	if (lock) {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.scan_menu), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.epg_menu), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.teletext_menu), FALSE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.scan_menu), TRUE);
		svc = gsfcore_get_service();
		if (svc != NULL) {
			if (((svc->type == SERVICE_TV_RADIO) || (svc->type == SERVICE_RADIO)) && (svc->sid > 0)) {
				gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.epg_menu), TRUE);
			}
			if (((svc->type == SERVICE_TV_RADIO) || (svc->type == SERVICE_RADIO)) && (gsfcore_service_has_pid_type(PID_TYPE_TELETEXT, NULL))) {
				gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.teletext_menu), TRUE);
			}
		}
	}
   	
	return FALSE;
}

static void statuswindow_trayaction(GtkStatusIcon *status_icon, gpointer data)
{
	#if GTK_CHECK_VERSION(2, 20, 0)
		if (gtk_widget_get_visible(statuswindow.win)) {
	#else
		if (GTK_WIDGET_VISIBLE(statuswindow.win)) {
	#endif
		#if GTK_CHECK_VERSION(2, 24, 0)
			gtk_widget_hide(statuswindow.win);
		#else
			gtk_widget_hide_all(statuswindow.win);
		#endif
	} else {
		gtk_widget_show_all(statuswindow.win);
	}
}

static gboolean statuswindow_servicemenu_add_service_cb(service *currentsvc)
{
	service_t svc;
	
	statuswindow.entry_sm = gtk_menu_item_new_with_label(currentsvc->name);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.servicesmenu), statuswindow.entry_sm);
	svc = gsfcore_get_service();
	if (svc != NULL) {
		if (currentsvc->id == svc->id) {
			gtk_widget_set_sensitive(statuswindow.entry_sm, FALSE);
		}
	}
	g_signal_connect(G_OBJECT(statuswindow.entry_sm), "activate", G_CALLBACK(statuswindow_traytune), GUINT_TO_POINTER(currentsvc->id));
	gtk_widget_show_all(statuswindow.entry_sm);
}

static void statuswindow_traypopup(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer data)
{
	if (statuswindow.servicesmenu != NULL) gtk_widget_destroy(statuswindow.servicesmenu);
	
	statuswindow.servicesmenu = gtk_menu_new();
		
	service_read_foreach(statuswindow_servicemenu_add_service_cb, NULL, NULL);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.services_tm), statuswindow.servicesmenu);
	gtk_menu_popup(GTK_MENU(data), NULL, NULL, gtk_status_icon_position_menu, status_icon, button, activate_time);
}

static void statuswindow_traytune(GtkMenuItem *menuitem, gpointer data)
{
	guint32 currensvcid;
	service *currentsvc;
	
	currensvcid = GPOINTER_TO_UINT(data);
	currentsvc = service_read(currensvcid);
	
	if (currentsvc != NULL) {
		gsfcore_tune_start(currentsvc->id, currentsvc->type);
	}
}

static gboolean statuswindow_traytooltip(GtkStatusIcon *status_icon, gint x, gint y, gboolean keyboard_mode, GtkTooltip *tooltip, gpointer data)
{
	frontend_t frontend;
	service_t svc;
	gchar *status;
	gint level;
		
	frontend = gsfcore_get_frontend();
	svc = gsfcore_get_service();
	
	level = round((frontend->reset ? 0 : frontend->snr)*100/(settings_read_int("snr_scale",100)));
	
	if (level > 100) level = 100;
	
	if ((frontend != NULL) && ((svc != NULL) || (!frontend->tuned))) {
		status = g_strdup_printf(_("<b>Signal level: %i%%</b>\n<small>Transponder: %s\nSNR: %i %s\nAGC: %i %s\nBER: %i</small>"), level, (frontend->tuned) ? svc->name : _("Not tuned"), frontend->snr, (frontend->lock) ? "[LOCK]" : "", frontend->signal, (frontend->carrier) ? "[CARRIER]" : "", frontend->ber);
		gtk_tooltip_set_markup(tooltip, status);
		g_free(status);
	}
		
	return TRUE;
} 

static GtkWidget *statuswindow_build_window(void)
{
	GdkDisplay *display;
	GdkScreen *screen;
	#if GTK_CHECK_VERSION(3, 0, 0)
		GtkStyleContext *context;
		GtkCssProvider *provider;
	#endif
	GdkPixbuf *icon;
	GtkWidget *vbox;
	GtkWidget *align;
	GtkWidget *snr_l, *agc_l, *ber_l;
	GtkWidget *statustable;
	GtkWidget *image_menu, *image_tb;
	GdkColor lowcolor, hicolor;
	gchar *lowcolorstr, *hicolorstr;
		
	
	display = gdk_display_get_default();
	screen = gdk_display_get_default_screen(display);
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		provider = gtk_css_provider_new();
		gtk_css_provider_load_from_data(GTK_CSS_PROVIDER(provider),
										"GtkProgressBar {\n"
										"	-GtkProgressBar-xspacing: 7px ;\n"
										"	-GtkProgressBar-yspacing: 7px ;\n"
										"}\n", -1, NULL);
		gtk_style_context_add_provider_for_screen(screen, GTK_STYLE_PROVIDER(provider), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
		g_object_unref(provider);
	#else
		gtk_rc_parse_string("style \"gsf-progressbar\" {\n"
							"	GtkWidget::xspacing = 7\n"
							"	GtkWidget::yspacing = 7\n"
							"} widget \"GtkProgressBar\" style \"gsf-progressbar\"");
		
	#endif
	
	statuswindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	statuswindow.title = _("Signal status - GSF");
	gtk_window_set_position(GTK_WINDOW(statuswindow.win), GTK_WIN_POS_CENTER);
	gtk_window_set_title(GTK_WINDOW(statuswindow.win), statuswindow.title);
	//gtk_widget_set_size_request(statuswindow.win, 490, 240);
	gtk_window_set_default_size(GTK_WINDOW(statuswindow.win), 500, 250);
	
	icon = gdk_pixbuf_new_from_file(RESOURCE_MAINWINDOW_MAINICON, NULL);
	if (icon != NULL) {
		gtk_window_set_icon(GTK_WINDOW(statuswindow.win), icon);
		g_object_unref(G_OBJECT(icon));
	}
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
	#else
		vbox = gtk_vbox_new(FALSE, 0);
	#endif

	//Menu bar
	statuswindow.menubar = gtk_menu_bar_new();
	statuswindow.actionsmenu = gtk_menu_new();
	
	statuswindow.accel_group = gtk_accel_group_new();
	gtk_window_add_accel_group(GTK_WINDOW(statuswindow.win), statuswindow.accel_group);
	
	statuswindow.actions_menu = gtk_menu_item_new_with_mnemonic(_("_Actions"));
	statuswindow.search_menu = gtk_image_menu_item_new_with_label(_("Tune"));
	statuswindow.network_menu = gtk_image_menu_item_new_with_label(_("Network status"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.network_menu), image_menu);
	image_menu = gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.search_menu), image_menu);
	statuswindow.scan_menu = gtk_image_menu_item_new_with_label(_("Transponder scan"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.scan_menu), image_menu);
	statuswindow.epg_menu = gtk_image_menu_item_new_with_label(_("View EPG"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_SPELL_CHECK, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.epg_menu), image_menu);
	statuswindow.teletext_menu = gtk_image_menu_item_new_with_label(_("Read teletext"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_INDEX, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.teletext_menu), image_menu);
	statuswindow.satscan_menu = gtk_image_menu_item_new_with_label(_("Satellite scan"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.satscan_menu), image_menu);
	statuswindow.sep_menu = gtk_separator_menu_item_new();
	statuswindow.quit_menu = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
	
	statuswindow.editmenu = gtk_menu_new();
	statuswindow.edit_menu = gtk_menu_item_new_with_mnemonic(_("_Edit"));
	statuswindow.preferences_menu = gtk_image_menu_item_new_with_label(_("Preferences"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.preferences_menu), image_menu);
	statuswindow.device_menu = gtk_image_menu_item_new_with_label(_("Device"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.device_menu), image_menu);
	statuswindow.sound_menu = gtk_check_menu_item_new_with_label(_("Enable sound"));
	
	statuswindow.viewmenu = gtk_menu_new();
	statuswindow.view_menu = gtk_menu_item_new_with_mnemonic(_("_View"));
	statuswindow.fullscreen_menu = gtk_image_menu_item_new_with_label(_("Fullscreen"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_FULLSCREEN, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.fullscreen_menu), image_menu);
	
	statuswindow.streamingmenu = gtk_menu_new();
	statuswindow.streaming_menu = gtk_menu_item_new_with_mnemonic(_("_Streaming"));
	statuswindow.serverconfig_menu = gtk_image_menu_item_new_with_label(_("Configure server"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.serverconfig_menu), image_menu);
	statuswindow.serverclients_menu = gtk_image_menu_item_new_with_label(_("Streaming clients"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_CONNECT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.serverclients_menu), image_menu);
	statuswindow.tsstruct_menu = gtk_image_menu_item_new_with_label(_("TS structure"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.tsstruct_menu), image_menu);
	statuswindow.record_menu = gtk_check_menu_item_new_with_label(_("Enable record"));
	statuswindow.udpbc_menu = gtk_check_menu_item_new_with_label(_("Enable UDP Broadcast"));
	statuswindow.udpmc_menu = gtk_check_menu_item_new_with_label(_("Enable UDP Multicast"));
	statuswindow.tcpuc_menu = gtk_check_menu_item_new_with_label(_("Enable TCP Unicast"));
	
	statuswindow.helpmenu = gtk_menu_new();
	statuswindow.help_menu = gtk_menu_item_new_with_mnemonic(_("_Help"));
	statuswindow.calc_menu = gtk_image_menu_item_new_with_label(_("Calculations"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_CONVERT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.calc_menu), image_menu);
	statuswindow.about_menu = gtk_image_menu_item_new_with_label(_("About"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.about_menu), image_menu);
	
	#if GTK_CHECK_VERSION(2, 22, 0)
		gtk_widget_add_accelerator(statuswindow.quit_menu, "activate", statuswindow.accel_group,
									GDK_KEY_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.search_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.network_menu, "activate", statuswindow.accel_group,
									GDK_KEY_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.scan_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_f, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.satscan_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_g, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.epg_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_w, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.teletext_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_j, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.preferences_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.device_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_d, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.fullscreen_menu, "activate", statuswindow.accel_group,
									GDK_KEY_F11, 0, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.sound_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.serverconfig_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_e, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.serverclients_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_l, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.tsstruct_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_t, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);						 
		gtk_widget_add_accelerator(statuswindow.record_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.udpbc_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_b, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.udpmc_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_m, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.tcpuc_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_u, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.about_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_h, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	#else
		gtk_widget_add_accelerator(statuswindow.quit_menu, "activate", statuswindow.accel_group,
									GDK_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.search_menu, "activate", statuswindow.accel_group, 
									GDK_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.network_menu, "activate", statuswindow.accel_group,
									GDK_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.scan_menu, "activate", statuswindow.accel_group, 
									GDK_f, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.satscan_menu, "activate", statuswindow.accel_group, 
									GDK_g, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.epg_menu, "activate", statuswindow.accel_group, 
									GDK_w, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.teletext_menu, "activate", statuswindow.accel_group, 
									GDK_j, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.preferences_menu, "activate", statuswindow.accel_group, 
									GDK_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.device_menu, "activate", statuswindow.accel_group, 
									GDK_d, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.fullscreen_menu, "activate", statuswindow.accel_group,
									GDK_F11, 0, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.sound_menu, "activate", statuswindow.accel_group, 
									GDK_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.serverconfig_menu, "activate", statuswindow.accel_group, 
									GDK_e, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.serverclients_menu, "activate", statuswindow.accel_group, 
									GDK_l, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.tsstruct_menu, "activate", statuswindow.accel_group, 
									GDK_t, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);						 
		gtk_widget_add_accelerator(statuswindow.record_menu, "activate", statuswindow.accel_group, 
									GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.udpbc_menu, "activate", statuswindow.accel_group, 
									GDK_b, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.udpmc_menu, "activate", statuswindow.accel_group, 
									GDK_m, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.tcpuc_menu, "activate", statuswindow.accel_group, 
									GDK_u, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.about_menu, "activate", statuswindow.accel_group, 
									GDK_h, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	#endif
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.actions_menu), statuswindow.actionsmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.search_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.network_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.scan_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.epg_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.teletext_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.satscan_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.sep_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.quit_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.actions_menu);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.edit_menu), statuswindow.editmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.editmenu), statuswindow.preferences_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.editmenu), statuswindow.device_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.editmenu), statuswindow.sound_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.edit_menu);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.view_menu), statuswindow.viewmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.viewmenu), statuswindow.fullscreen_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.view_menu);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.streaming_menu), statuswindow.streamingmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.serverconfig_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.serverclients_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.tsstruct_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.record_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.udpbc_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.udpmc_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.tcpuc_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.streaming_menu);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.help_menu), statuswindow.helpmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.helpmenu), statuswindow.calc_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.helpmenu), statuswindow.about_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.help_menu);
	
	g_signal_connect(G_OBJECT(statuswindow.quit_menu), "activate", G_CALLBACK(statuswindow_stop_program), NULL);
	g_signal_connect(G_OBJECT(statuswindow.search_menu), "activate", G_CALLBACK(statuswindow_search_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.network_menu), "activate", G_CALLBACK(statuswindow_network_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.scan_menu), "activate", G_CALLBACK(statuswindow_scan_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.epg_menu), "activate", G_CALLBACK(statuswindow_epg_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.teletext_menu), "activate", G_CALLBACK(statuswindow_teletext_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.satscan_menu), "activate", G_CALLBACK(statuswindow_satscan_open), NULL);
	
	g_signal_connect(G_OBJECT(statuswindow.device_menu), "activate", G_CALLBACK(statuswindow_devconfig_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.preferences_menu), "activate", G_CALLBACK(statuswindow_preferences_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.fullscreen_menu), "activate", G_CALLBACK(statuswindow_fullscreen_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.sound_menu), "toggled", G_CALLBACK(statuswindow_toggle_sound_menu), NULL);
	
	g_signal_connect(G_OBJECT(statuswindow.streaming_menu), "activate", G_CALLBACK(statuswindow_streaming_menu_init), NULL);
	g_signal_connect(G_OBJECT(statuswindow.serverconfig_menu), "activate", G_CALLBACK(statuswindow_sconf_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.serverclients_menu), "activate", G_CALLBACK(statuswindow_sclients_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.tsstruct_menu), "activate", G_CALLBACK(statuswindow_tsstruct_open), NULL);
		
	g_signal_connect(G_OBJECT(statuswindow.record_menu), "toggled", G_CALLBACK(statuswindow_toggle_streaming_menu), GINT_TO_POINTER(0));
	g_signal_connect(G_OBJECT(statuswindow.udpbc_menu), "toggled", G_CALLBACK(statuswindow_toggle_streaming_menu), GINT_TO_POINTER(1));
	g_signal_connect(G_OBJECT(statuswindow.udpmc_menu), "toggled", G_CALLBACK(statuswindow_toggle_streaming_menu), GINT_TO_POINTER(2));
	g_signal_connect(G_OBJECT(statuswindow.tcpuc_menu), "toggled", G_CALLBACK(statuswindow_toggle_streaming_menu), GINT_TO_POINTER(3));
	
	g_signal_connect(G_OBJECT(statuswindow.calc_menu), "activate", G_CALLBACK(statuswindow_calculations_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.about_menu), "activate", G_CALLBACK(statuswindow_about_open), NULL);
	gtk_box_pack_start(GTK_BOX(vbox), statuswindow.menubar, FALSE, FALSE, 0);

	//Toolbar
	statuswindow.toolbar = GTK_TOOLBAR(gtk_toolbar_new());
	image_tb = gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.historymenu_tb = gtk_menu_new();
	statuswindow.search_tb = gtk_menu_tool_button_new(image_tb,_("Tune"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.search_tb), _("Tune transponder"));
    g_object_set(statuswindow.search_tb, "is-important", TRUE, NULL);
  	//gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(statuswindow.search_tb), GTK_WIDGET(statuswindow.historymenu_tb));
  	image_tb = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.device_tb = gtk_tool_button_new(image_tb,_("Device"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.device_tb), _("DVB device parameters"));
	image_tb = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.preferences_tb = gtk_tool_button_new(image_tb, _("Configure"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.preferences_tb), _("Configure program"));
	statuswindow.separator1_tb = gtk_separator_tool_item_new();
	image_tb = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.sound_tb = gtk_toggle_tool_button_new();
	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(statuswindow.sound_tb),image_tb);
	gtk_tool_button_set_label(GTK_TOOL_BUTTON(statuswindow.sound_tb),_("Sound"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.sound_tb), _("Enable/disable sound"));
	g_object_set(statuswindow.sound_tb, "is-important", TRUE, NULL);
	image_tb = gtk_image_new_from_stock(GTK_STOCK_FULLSCREEN, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.fullscreen_tb = gtk_tool_button_new(image_tb, _("Fullscreen"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.fullscreen_tb), _("Fullscreen mode"));
	image_tb = gtk_image_new_from_stock(GTK_STOCK_QUIT, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.exit_tb = gtk_tool_button_new(image_tb, _("Exit"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.exit_tb), _("Exit program"));
	statuswindow.separator2_tb = gtk_separator_tool_item_new();
	
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.search_tb, 0);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.device_tb, 1);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.preferences_tb, 2);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.fullscreen_tb, 3);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.separator1_tb, 4);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.sound_tb, 5);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.separator2_tb, 6);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.exit_tb, 7);
	
	g_signal_connect(G_OBJECT(statuswindow.search_tb), "clicked", G_CALLBACK(statuswindow_search_open), NULL);
	//g_signal_connect(G_OBJECT(statuswindow.search_tb), "show-menu", G_CALLBACK(statuswindow_show_history_menu), NULL);
	g_signal_connect(G_OBJECT(statuswindow.device_tb), "clicked", G_CALLBACK(statuswindow_devconfig_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.preferences_tb), "clicked", G_CALLBACK(statuswindow_preferences_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.sound_tb), "toggled", G_CALLBACK(statuswindow_toggle_sound_toolbar), NULL);
	g_signal_connect(G_OBJECT(statuswindow.fullscreen_tb), "clicked", G_CALLBACK(statuswindow_fullscreen_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.exit_tb), "clicked", G_CALLBACK(statuswindow_stop_program), NULL);
	
	gtk_toolbar_set_show_arrow(GTK_TOOLBAR(statuswindow.toolbar),TRUE);
	gtk_toolbar_set_style(GTK_TOOLBAR(statuswindow.toolbar), GTK_TOOLBAR_BOTH_HORIZ);
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		context = gtk_widget_get_style_context(GTK_WIDGET(statuswindow.toolbar));
		gtk_style_context_add_class(context, GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
	#endif
	
	gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(statuswindow.toolbar),FALSE,FALSE,0);
	
	//Progressbars
	statustable = gtk_table_new(3, 2, FALSE);
		
	snr_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(snr_l), "<b>SNR:</b>");
	gtk_table_attach(GTK_TABLE(statustable), snr_l, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 5, 5);
		
	statuswindow.snr = gtk_progress_bar_new();
	gtk_table_attach(GTK_TABLE(statustable), statuswindow.snr, 1, 2, 0, 1, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 5, 5);
	
	agc_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(agc_l), "<b>AGC:</b>");
	gtk_table_attach(GTK_TABLE(statustable), agc_l, 0, 1, 1, 2, GTK_SHRINK, GTK_SHRINK, 5, 5);
	
	statuswindow.agc = gtk_progress_bar_new();
	gtk_table_attach(GTK_TABLE(statustable), statuswindow.agc, 1, 2, 1, 2, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 5, 5);
	
	ber_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(ber_l), "<b>BER:</b>");
	gtk_table_attach(GTK_TABLE(statustable), ber_l, 0, 1, 2, 3, GTK_SHRINK, GTK_SHRINK, 5, 5);
		
	statuswindow.ber = gtk_progress_bar_new();
	gtk_table_attach(GTK_TABLE(statustable), statuswindow.ber, 1, 2, 2, 3, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 5, 5);
	
	gtk_box_pack_start(GTK_BOX(vbox), statustable, TRUE, TRUE, 10);
	
	statuswindow.statusbar = GTK_STATUSBAR(gtk_statusbar_new());
	#if GTK_CHECK_VERSION(3, 0, 0)
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.snr), " ");
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.agc), " ");
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.ber), " ");
		gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(statuswindow.snr), TRUE);
		gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(statuswindow.agc), TRUE);
		gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(statuswindow.ber), TRUE);
	#else
		gtk_statusbar_set_has_resize_grip(statuswindow.statusbar, FALSE);
	#endif
	
	statuswindow.grstatus = GTK_GRSTATUS(gtk_grstatus_new(100));
	lowcolorstr = settings_read_string("graph_low_color", "white");
	gdk_color_parse(lowcolorstr, &lowcolor);
	g_free(lowcolorstr);
	hicolorstr = settings_read_string("graph_high_color","green3");
	gdk_color_parse(hicolorstr, &hicolor);
	g_free(hicolorstr);
	gtk_grstatus_set_colors(statuswindow.grstatus, &lowcolor, &hicolor);
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		statuswindow.statussep = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
	#else
		statuswindow.statussep = gtk_vseparator_new();
	#endif
	gtk_box_pack_start(GTK_BOX(statuswindow.statusbar), GTK_WIDGET(statuswindow.statussep), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(statuswindow.statusbar), GTK_WIDGET(statuswindow.grstatus), FALSE, FALSE, 10);
	
	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(statuswindow.statusbar), FALSE, FALSE, 5);
	g_signal_connect(G_OBJECT(statuswindow.win), "delete-event", G_CALLBACK(statuswindow_close_main_window), NULL);
	
	gtk_container_add(GTK_CONTAINER(statuswindow.win), vbox);
	
	statuswindow.trayicon = gtk_status_icon_new_from_file(RESOURCE_MAINWINDOW_MAINICON);
	g_signal_connect(G_OBJECT(statuswindow.trayicon), "activate",
						G_CALLBACK(statuswindow_trayaction), G_OBJECT(statuswindow.trayicon));
	
	statuswindow.traymenu = gtk_menu_new();
	
	statuswindow.search_tm = gtk_image_menu_item_new_with_label(_("Tune"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.search_tm), image_menu);
	
	statuswindow.services_tm = gtk_image_menu_item_new_with_label(_("Services"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.services_tm), image_menu);
	
	statuswindow.sep_tm = gtk_separator_menu_item_new();
	
	statuswindow.quit_tm = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
	
	g_signal_connect(G_OBJECT(statuswindow.search_tm), "activate", G_CALLBACK(statuswindow_search_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.quit_tm), "activate", G_CALLBACK(statuswindow_stop_program), NULL);
	
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.traymenu), statuswindow.search_tm);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.traymenu), statuswindow.services_tm);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.traymenu), statuswindow.sep_tm);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.traymenu), statuswindow.quit_tm);
	
	gtk_widget_show_all(statuswindow.traymenu);
	g_signal_connect(G_OBJECT(statuswindow.trayicon), "popup-menu", G_CALLBACK(statuswindow_traypopup), G_OBJECT(statuswindow.traymenu));
	
	#if GTK_CHECK_VERSION(2, 16, 0)
		gtk_status_icon_set_has_tooltip(statuswindow.trayicon, TRUE);
		g_signal_connect(G_OBJECT(statuswindow.trayicon), "query-tooltip", G_CALLBACK(statuswindow_traytooltip), G_OBJECT(statuswindow.trayicon));
	#endif
	
	statuswindow_lock_search_buttons(GINT_TO_POINTER(TRUE));
	statuswindow_lock_scan_buttons(GINT_TO_POINTER(TRUE));
	
	history = g_array_sized_new(FALSE, TRUE, sizeof(struct _historyentry), 10);
	
	return statuswindow.win;
}

static void statuswindow_event_cb(gint event, gpointer data)
{
	switch (event) {
		case GSF_EVENT_UPDATE_SIGNAL:
			g_idle_add(statuswindow_update_signal, data);
			break;
		case GSF_EVENT_TRANSPONDER_SELECTED:
			g_idle_add(statuswindow_transponder_selected, data);
			break;
		case GSF_EVENT_TRANSPONDER_TUNED:
			//TODO: add function
			break;
		case GSF_EVENT_UPDATE_TUNE_PARAMS:
			g_idle_add(statuswindow_update_tune_params, NULL);
			break;
		case GSF_EVENT_UPDATE_DEVICE_PARAMS:
			g_idle_add(statuswindow_update_device_params, NULL);
			break;
		case GSF_EVENT_STATUS_MESSAGE:
			g_idle_add(statuswindow_status_message, data);
			break;
		case GSF_EVENT_ERROR_MESSAGE:
			g_idle_add(message_error_nonmodal, data);
			break;
		case GSF_EVENT_TUNE_ERROR_MESSAGE:
			g_idle_add(statuswindow_tune_problem_message, data);
			break;
		case GSF_EVENT_LOCK_TUNE_BUTTONS:
			g_idle_add(statuswindow_lock_search_buttons, data);
			break;
		case GSF_EVENT_LOCK_SCAN_BUTTONS:
			g_idle_add(statuswindow_lock_scan_buttons, data);
			break;
		case GSF_EVENT_LOCK_SOUND_BUTTONS:
			g_idle_add(statuswindow_lock_sound_buttons, data);
			break;
		case GSF_EVENT_SATSCAN_STATUS:
			g_idle_add(satscan_window_scan_status_fill, NULL);
			break;
		default:
			
			break;
	}
}

static void statuswindow_init_settings(void)
{
	settings_open(FALSE);
	
	service_storage_init();
	
	fswindow_init();
}

static void statuswindow_init_threads(void)
{
	gsfcore_init(globalsettings.nonetwork, globalsettings.nostreaming, statuswindow_event_cb);
}

#if GTK_CHECK_VERSION(3, 0, 0)
static void statuswindow_activate(GtkApplication *app)
{
	GList *list;
	GtkWidget *window;
	
	list = gtk_application_get_windows(app);
	
	if (list) {
		gtk_window_present(GTK_WINDOW(g_list_nth(list, 0)));
	} else {
		window = statuswindow_build_window();
		gtk_window_set_application(GTK_WINDOW(window), app);
		if (!settings_read_boolean("program_start_in_tray", FALSE)) {
			gtk_widget_show_all(window);
			gtk_window_set_focus(GTK_WINDOW(window), NULL);
		}
		statuswindow_init_threads();
	}
}
#endif

int main(int argc, char *argv[])
{
	struct sigaction sa;
	sigset_t sigmask;
	gint status;
	GOptionContext *context;
	GError *error = NULL;
	#if !GTK_CHECK_VERSION(3, 0, 0)
		/*GtkApplication *app;
	#else*/
		GtkWidget *window;
	#endif
	
	static GOptionEntry entries[] = {
		{ "nonetwork", 'n', 0, G_OPTION_ARG_NONE, &globalsettings.nonetwork, "Disable network and web interfaces", NULL },
		{ "nostreaming", 's', 0, G_OPTION_ARG_NONE, &globalsettings.nostreaming, "Disable streaming support", NULL },
		{ NULL }
	};
	
	#ifndef LC_ALL
		#define LC_ALL 0
	#endif
	
	setlocale(LC_ALL, "");
	bindtextdomain(GSF_GETTEXT_PACKAGE, RESOURCE_DIR_LOCALE);
	bind_textdomain_codeset(GSF_GETTEXT_PACKAGE, "UTF-8");
	textdomain(GSF_GETTEXT_PACKAGE);
	
	g_set_application_name("GSF software satfinder");
	
	context = g_option_context_new("- tune satellite antenna");
	g_option_context_add_main_entries(context, entries, GSF_GETTEXT_PACKAGE);
	g_option_context_add_group(context, gtk_get_option_group(TRUE));
	
	if (!g_option_context_parse(context, &argc, &argv, &error)) {
		g_print("[-] Command line option parsing failed: %s\n", error->message);
		g_error_free(error);
		exit(EXIT_FAILURE);
	}
	
	sa.sa_flags = SA_SIGINFO;
	sigemptyset(&sa.sa_mask);
	sa.sa_sigaction = statuswindow_stop_signal_handler;
	
	sigaction(SIGTERM, &sa, NULL);
	sigaction(SIGINT, &sa, NULL);
	sigaction(SIGSEGV, &sa, NULL);
	
	sa.sa_flags = SA_SIGINFO;
	sigemptyset(&sa.sa_mask);
	sa.sa_sigaction = statuswindow_service_signal_handler;
	
	sigaction(SIGCHLD, &sa, NULL);
	
	g_thread_init(NULL);
		
	statuswindow_init_settings();
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		app = gtk_application_new("org.gtk.GSF", 0);
		g_signal_connect(app, "activate", G_CALLBACK(statuswindow_activate), NULL);
	#else
		gdk_threads_init();
		gtk_init(&argc, &argv);
		window = statuswindow_build_window();
		if (!settings_read_boolean("program_start_in_tray", FALSE)) {
			gtk_widget_show_all(window);
			gtk_window_set_focus(GTK_WINDOW(window), NULL);
		}
		statuswindow_init_threads();
	#endif
	
	status = 0;
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		status = g_application_run(G_APPLICATION(app), argc, argv);
	#else
		gdk_threads_enter();
		gtk_main();
		gdk_threads_leave();
	#endif
		
	#if GTK_CHECK_VERSION(3, 0, 0)
		g_object_unref(app);
	#endif
	
	return status;
}
