/*
 *      streamconfig.h
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __STREAMCONFIG_H__
#define __STREAMCONFIG_H__

#include <glib.h>
#include <sys/un.h>
#include <net/if.h>
#include <net/route.h>
#include <stdio.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdint.h>
#include "gsfstrsvr/streamserver.h"
#include "gsfstrsvr/transportstream.h"

struct _streamconfig {
	gint clientsock;
	struct sockaddr_un clientaddr;
	gchar *clientsockpath;
	struct sockaddr_un serveraddr;
	gchar *serversockpath;
	gchar mcintname[IFNAMSIZ];
	gboolean fileopened;
	GKeyFile *configfile;
	gchar *filename;
	gsize filebytes;
	pid_t processpid;
	gboolean serverctl;
	gboolean netintlist;
	gboolean netintctl;
};

typedef struct _streamconfig *streamconfig_t;

struct _netinterface {
	gchar ifname[IFNAMSIZ];
	guint ifindex;
	gchar ifipaddr[INET6_ADDRSTRLEN];
	gchar ifnetmask[INET6_ADDRSTRLEN];
	gchar ifmacaddr[IFHWADDRLEN];
	gchar ifbcipaddr[INET_ADDRSTRLEN];
	gboolean ifup;
	gboolean ifbc;
	gboolean ifmc;
	gboolean ifloop;
	gboolean ifptp;
};

typedef struct _netinterface *netinterface_t;

struct _streamconnparams {
	/*UDP BROADCAST*/
	gchar udpbcint[IFNAMSIZ];
	gushort udpbcport;
	/*UDP MULTICAST*/
	gchar udpmcint[IFNAMSIZ];
	gchar udpmcgroup[INET_ADDRSTRLEN];
	gushort udpmcport;
	gchar udpmcloop;
	gchar udpmcttl;
	/*TCP UNICAST*/
	gushort tcpucport;
	gchar tcpucclients;
	gchar tcpucqueue;
};

typedef struct _streamconnparams *streamconnparams_t;

struct _strclient {
	gboolean sticky;
	gchar family;
	gchar type;
	guint pid;
	guint seq;
	gushort pidfilter;
	gchar address[55];
	gchar useragent[64];
	unsigned long long transmittedbytes;
	unsigned long long starttime;
};

typedef struct _strclient *strclient_t;

struct _strtspid {
	guchar type;
	guchar format;
	guchar value;
	guchar percentage;
	gushort pid;
	gboolean sticky;
	guint bandwidth;
	guint revision;
	gchar description[64];
	union _streamparams params;
};

typedef struct _strtspid *strtspid_t;

GSList *streamconfig_interfaces_get_with_multicast(void);
GSList *streamconfig_interfaces_get_with_broadcast(void);
gint streamconfig_interfaces_list_build(void);

gint streamconfig_service_state(struct _streamconfig *sc);

gint streamconfig_service_state(struct _streamconfig *sc);
gboolean streamconfig_service_restart(struct _streamconfig *sc);
gboolean streamconfig_init(struct _streamconfig *sc);
gboolean streamconfig_close(struct _streamconfig *sc);
gboolean streamconfig_set_record_status(struct _streamconfig *sc, gboolean status);
gboolean streamconfig_set_udp_broadcast_status(struct _streamconfig *sc, gboolean status);
gboolean streamconfig_set_udp_multicast_status(struct _streamconfig *sc, gboolean status);
gboolean streamconfig_set_tcp_unicast_status(struct _streamconfig *sc, gboolean status);
gboolean streamconfig_set_stream_data(struct _streamconfig *sc, gchar *svcname, GSList *mediastreams, guint16 caid, gchar *cakey);
gboolean streamconfig_set_connection_details(struct _streamconfig *sc, struct _streamconnparams *sendparams);
gboolean streamconfig_set_device(struct _streamconfig *sc, gchar adapter, gchar dvr);
gboolean streamconfig_stop_client(struct _streamconfig *sc, guint clientid);
GSList *streamconfig_get_clients(struct _streamconfig *sc);
void streamconfig_free_clients(GSList *clients);
GSList *streamconfig_get_tsstruct(struct _streamconfig *sc);
void streamconfig_free_tsstruct(GSList *tsstruct);
gboolean streamconfig_get_statistics(struct _streamconfig *sc, guint *udpmccli, guint *udpbccli, guint *tcpuccli, guint *tcpucclimax, glong *sessiontime);
gboolean streamconfig_get_service_flags(struct _streamconfig *sc, gboolean *record, gboolean *udpbc, gboolean *udpmc, gboolean *tcpuc);
gboolean streamconfig_get_connection_details(struct _streamconfig *sc, struct _streamconnparams *recvparams);

#endif /* __STREAMCONFIG_H__ */
