/*
 *      streamwindow.c
 *      
 *      Copyright 2011-2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>

#include "gsfcore.h"
//remove
//#include "mainwindow.h"
#include "streamconfig.h"
#include "streamwindow.h"

#define STREAMWINDOW_UDP_GROUP_REGEX "^(22[4-9]|23[0-9])(\\.(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])){3}$"

enum {
	STREAM_NETINTERFACE_NAME = 0,
	STREAM_NETINTERFACE_NUMBER,
	STREAM_NETINTERFACE_COLUMNS
};

enum {
	STREAM_ENTRIES_COL_ICON = 0,
	STREAM_ENTRIES_COL_ICON_SIZE,
	STREAM_ENTRIES_COL_NAME,
	STREAM_ENTRIES_COL_ID,
	STREAM_ENTRIES_COLUMNS
};

struct sstreamwindow {
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	GtkWidget *apply;
	GtkWidget *list;
	GtkTreeSelection *selection;
	GtkWidget *panelcaption;
	GtkWidget *panel;
	gint pageid;
	GtkWidget *page;
	gulong changepagesignal;
};
struct sstreamwindow streamwindow;

gint streamwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
void streamwindow_init_list(GtkWidget *list);
static gchar *streamwindow_format_elapsed_time(time_t start, time_t end, gchar *buffer, gsize size);
static void streamwindow_start_server(GtkButton *button, gpointer data);

GtkWidget **streamwindow_stats_settings_create(void);
GtkWidget **streamwindow_udpbc_settings_create(void);
GtkWidget **streamwindow_udpmc_settings_create(void);
GtkWidget **streamwindow_tcpuc_settings_create(void);

void streamwindow_stats_settings_configure(gpointer page);
void streamwindow_udpbc_settings_configure(gpointer page);
void streamwindow_udpmc_settings_configure(gpointer page);
void streamwindow_tcpuc_settings_configure(gpointer page);

void streamwindow_udpbc_settings_apply(gpointer page);
void streamwindow_udpmc_settings_apply(gpointer page);
void streamwindow_tcpuc_settings_apply(gpointer page);

typedef GtkWidget **(*stream_page_init_func)(void);
typedef void (*stream_page_configure_func)(gpointer page);
typedef void (*stream_page_apply_func)(gpointer page);

typedef struct {
	GtkWidget **widgets;
	gboolean expired;
	const guint preferenceid;
	const gchar *title;
	const gchar *description;
	const gchar *stock;
	stream_page_init_func init_func;
	stream_page_configure_func configure_func;
	stream_page_apply_func apply_func;
} streampage;

static streampage stream_pages[4] = {
	{ NULL, FALSE, STREAM_PREFERENCE_ID_STATS, N_("Statistics"),    N_("Server state"),   GTK_STOCK_EXECUTE, streamwindow_stats_settings_create, streamwindow_stats_settings_configure, NULL},
	{ NULL, FALSE, STREAM_PREFERENCE_ID_UDPBC, N_("UDP broadcast"), N_("IPv4 broadcast"), GTK_STOCK_NETWORK, streamwindow_udpbc_settings_create, streamwindow_udpbc_settings_configure, streamwindow_udpbc_settings_apply},
	{ NULL, FALSE, STREAM_PREFERENCE_ID_UDPMC, N_("UDP multicast"), N_("IPv4 multicast"), GTK_STOCK_NETWORK, streamwindow_udpmc_settings_create, streamwindow_udpmc_settings_configure, streamwindow_udpmc_settings_apply},
	{ NULL, FALSE, STREAM_PREFERENCE_ID_TCPUC, N_("TCP unicast"),   N_("IPv6 unicast"),   GTK_STOCK_NETWORK, streamwindow_tcpuc_settings_create, streamwindow_tcpuc_settings_configure, streamwindow_tcpuc_settings_apply},
};

struct _ifwidget {
	GtkWidget *widget;
	gchar *ifname;
};

static struct _streamconnparams *connparams;
static gint timer;
static GRegex *groupregex;
static time_t svrstart;

gint streamwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gint i;
	
	streamwindow.visible = FALSE;
	
	if (g_signal_handler_is_connected(G_OBJECT(streamwindow.list), streamwindow.changepagesignal)) {
		g_signal_handler_disconnect(G_OBJECT(streamwindow.list), streamwindow.changepagesignal);
	}
	
	for (i=0; i<(sizeof(stream_pages)/sizeof(streampage)); i++) {
		if (stream_pages[i].widgets != NULL) {
			gtk_widget_destroy(stream_pages[i].widgets[0]);
			g_free(stream_pages[i].widgets);
			stream_pages[i].widgets = NULL;
		}
	}
	
	if (timer > 0) {
		//Stop timer
		g_source_remove(timer);
		timer = 0;
		//Destroy regex
		g_regex_unref(groupregex);
		groupregex = NULL;
		//Destroy params
		g_free(connparams);
		connparams = NULL;
	}
	
	gtk_widget_destroy(streamwindow.win);
	
	return FALSE;
}

/*gint streamwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gint i;
	streamconfig_t streamconfig;
		
	streamconfig = gsfcore_get_streamconfig();
	if (connparams != NULL) {
		for (i=0; i<(sizeof(stream_pages)/sizeof(streampage)); i++) {
			if ((stream_pages[i].widgets != NULL) && (stream_pages[i].apply_func != NULL)) {
				stream_pages[i].apply_func(&stream_pages[i]);
			}
		}
		
		streamconfig_set_connection_details(streamconfig, connparams);
	}
	
	g_signal_emit_by_name(G_OBJECT(streamwindow.win), "destroy", NULL);
	
	return 0;
}*/

static void streamwindow_action_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gint i;
	streamconfig_t streamconfig;
		
	if (response_id == GTK_RESPONSE_APPLY) {
		streamconfig = gsfcore_get_streamconfig();
		if (connparams != NULL) {
			for (i=0; i<(sizeof(stream_pages)/sizeof(streampage)); i++) {
				if ((stream_pages[i].widgets != NULL) && (stream_pages[i].apply_func != NULL)) {
					stream_pages[i].apply_func(&stream_pages[i]);
				}
			}
			streamconfig_set_connection_details(streamconfig, connparams);
		}
		g_signal_emit_by_name(G_OBJECT(streamwindow.win), "destroy", NULL);
	} else if (response_id == GTK_RESPONSE_CLOSE) {
		g_signal_emit_by_name(G_OBJECT(streamwindow.win), "destroy", NULL);
	} else if (response_id == GTK_RESPONSE_HELP) {
		
	}
}

void streamwindow_init_list(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2;
	GtkListStore       *store;
		
	
	renderer = gtk_cell_renderer_pixbuf_new();
	column1 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column1, NULL);
	gtk_tree_view_column_pack_start(column1, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column1, renderer, "stock-id", STREAM_ENTRIES_COL_ICON, "stock-size", STREAM_ENTRIES_COL_ICON_SIZE, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, "ellipsize-set", TRUE, NULL);
	column2 = gtk_tree_view_column_new_with_attributes(_("Device"), renderer, "markup", STREAM_ENTRIES_COL_NAME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	
	store = gtk_list_store_new(STREAM_ENTRIES_COLUMNS, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

gboolean streamwindow_fill_entries(void)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar caption[256];
	gint len;
	gint i;
	
	
	if (streamwindow.visible) {
		if ((model = gtk_tree_view_get_model(GTK_TREE_VIEW(streamwindow.list))) != NULL) {
			g_object_ref(model);
			gtk_tree_view_set_model(GTK_TREE_VIEW(streamwindow.list), NULL);
			gtk_list_store_clear(GTK_LIST_STORE(model));
			for (i=0; i<(sizeof(stream_pages)/sizeof(streampage)); i++) {
				if (stream_pages[i].widgets != NULL) continue;
				len = snprintf((gchar *)&caption, sizeof(caption), "<small><b>%s</b>\n%s</small>", gettext(stream_pages[i].title), gettext(stream_pages[i].description));
				caption[len] = '\0';
				gtk_list_store_append(GTK_LIST_STORE(model), &iter);
				gtk_list_store_set(GTK_LIST_STORE(model), &iter, STREAM_ENTRIES_COL_ICON, stream_pages[i].stock,
																STREAM_ENTRIES_COL_ICON_SIZE, GTK_ICON_SIZE_BUTTON,
																STREAM_ENTRIES_COL_NAME, caption,
																STREAM_ENTRIES_COL_ID, i,
																-1);
			}
			gtk_tree_view_set_model(GTK_TREE_VIEW(streamwindow.list), model);
			g_object_unref(model);
		}
	}
	
	return FALSE;
}


void streamwindow_entry_selected(GtkTreeView *tree_view, gpointer data)
{
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint id;
	gchar caption[256];
	gint len;
		
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, STREAM_ENTRIES_COL_ID, &id, -1);
		
		if (((streamwindow.pageid == -1) || (streamwindow.pageid != id)) && (id < (sizeof(stream_pages)/sizeof(streampage)))) {
			if ((stream_pages[id].widgets == NULL) && (stream_pages[id].init_func != NULL)) {
				//Create new page and configure it
				stream_pages[id].widgets = stream_pages[id].init_func();
				if ((stream_pages[id].widgets != NULL) && (stream_pages[id].configure_func != NULL)) {
					stream_pages[id].configure_func(&stream_pages[id]);
				}
			} else if (stream_pages[id].init_func == NULL) {
				g_warning("No constructor for page %i (%s)", id, stream_pages[id].title);
			}
				
			if (stream_pages[id].widgets != NULL) {
				if (streamwindow.page != NULL) {
					if (streamwindow.pageid == -1) {
						//After start page selected
						gtk_widget_set_sensitive(streamwindow.panel, TRUE);
						gtk_dialog_set_response_sensitive(GTK_DIALOG(streamwindow.win), GTK_RESPONSE_APPLY, TRUE);
						//gtk_widget_set_sensitive(streamwindow.apply, TRUE);
					} else {
						//Increase refcount to avoid page re-creation
						g_object_ref(streamwindow.page);
					}
					//Remove old page
					gtk_container_remove(GTK_CONTAINER(streamwindow.panel), streamwindow.page);
				}
				//Add new page
				streamwindow.page = stream_pages[id].widgets[0];
				gtk_box_pack_end(GTK_BOX(streamwindow.panel), streamwindow.page, TRUE, TRUE, 10);
				streamwindow.pageid = id;
				len = snprintf((gchar *)&caption, sizeof(caption), "<b>%s</b>", gettext(stream_pages[id].title));
				caption[len] = '\0';
				gtk_label_set_markup(GTK_LABEL(streamwindow.panelcaption), caption);
				gtk_widget_show_all(streamwindow.panel);
			}
			
		}
		
	}
}	

gboolean streamwindow_update_stats(gpointer data)
{
	streamwindow_update_page(STREAM_PREFERENCE_ID_STATS);
	
	return TRUE;
}

static void streamwindow_validate_regex(GtkEditable *editable, gpointer data)
{
	gchar *txt;
	GRegex *regex;
	
	txt = g_strdup(gtk_entry_get_text(GTK_ENTRY(editable)));
	regex = g_regex_ref(groupregex);
	
	if (!g_regex_match_all(regex, txt, 0, NULL)) {
		gtk_dialog_set_response_sensitive(GTK_DIALOG(streamwindow.win), GTK_RESPONSE_APPLY, FALSE);
		//gtk_widget_set_sensitive(streamwindow.apply, FALSE);
	} else {
		gtk_dialog_set_response_sensitive(GTK_DIALOG(streamwindow.win), GTK_RESPONSE_APPLY, TRUE);
		//gtk_widget_set_sensitive(streamwindow.apply, TRUE);
	}
	
	g_regex_unref(regex);
	g_free(txt);
}

void streamwindow_interfaces_foreach(gpointer data, GtkListStore **store)
{
	GtkTreeIter iter;
	struct _netinterface *interface;
	
	interface = (struct _netinterface *)data;
	
	gtk_list_store_append(GTK_LIST_STORE(*store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(*store), &iter, STREAM_NETINTERFACE_NAME, interface->ifname, STREAM_NETINTERFACE_NUMBER, interface->ifindex, -1);
}

gboolean streamwindow_interfaces_select_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	gchar *ifname;
	struct _ifwidget *ifwidget;
	
	ifwidget = (struct _ifwidget *)data;
	
	if ((ifwidget == NULL) || (ifwidget->widget == NULL) || (ifwidget->ifname == NULL)) return TRUE;
	
	gtk_tree_model_get(model, iter, STREAM_NETINTERFACE_NAME, &ifname, -1);
	
	if (g_str_equal(ifwidget->ifname, ifname)) {
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(ifwidget->widget), iter);
		g_free(ifname);
		return TRUE;
	}
	g_free(ifname);
		
	return FALSE;
}

GtkWidget **streamwindow_stats_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *state, *state_l;
	GtkWidget *udpbc, *udpbc_l;
	GtkWidget *udpmc, *udpmc_l;
	GtkWidget *tcpuc, *tcpuc_l;
	GtkWidget *session, *session_l;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 5, FALSE);
	
	state_l = gtk_label_new(_("Server state"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), state_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	state = gtk_button_new_with_label(_("Starting..."));
	g_signal_connect(G_OBJECT(state), "clicked", G_CALLBACK(streamwindow_start_server), NULL);
	gtk_widget_set_sensitive(state, FALSE);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), state);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
		
	udpbc_l = gtk_label_new(_("UDP Broadcast clients"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), udpbc_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	udpbc = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(udpbc), TRUE);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), udpbc);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	udpmc_l = gtk_label_new(_("UDP Multicast clients"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), udpmc_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	udpmc = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(udpmc), TRUE);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), udpmc);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	tcpuc_l = gtk_label_new(_("TCP Unicast clients"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), tcpuc_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	tcpuc = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(tcpuc), TRUE);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), tcpuc);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	session_l = gtk_label_new(_("Last session time"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), session_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	session = gtk_label_new(NULL);
	gtk_label_set_selectable(GTK_LABEL(session), TRUE);
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), session);
	gtk_table_attach(GTK_TABLE(table), align, 1, 2, 4, 5, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 3);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(6*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = state;
	widgets[2] = udpbc;
	widgets[3] = udpmc;
	widgets[4] = tcpuc;
	widgets[5] = session;
	
	return widgets;
}

static gchar *streamwindow_format_elapsed_time(time_t start, time_t end, gchar *buffer, gsize size)
{
	gdouble sessionperiod;
	guint i;
	guint timeparts[3];
	gchar timespecs[3][3];
	
	if ((buffer == NULL) || (size < 8)) return NULL;
	
	sessionperiod = difftime(end, start);
	timeparts[0] = (guint)(((gint)sessionperiod) / 3600);
	timeparts[1] = (guint)(((gint)sessionperiod) / 60);
	timeparts[2] = (guint)(((gint)sessionperiod) % 60);
	
	for (i=0; i<3; i++) {
		if (timeparts[i] < 10) {
			g_snprintf((gchar *)&timespecs[i][0], sizeof(timespecs[i]), "0%u", timeparts[i]);
		} else {
			g_snprintf((gchar *)&timespecs[i][0], sizeof(timespecs[i]), "%u", timeparts[i]);
		}
	}
	
	g_snprintf(buffer, size, "%s:%s:%s", timespecs[0], timespecs[1], timespecs[2]);
	
	return buffer;
}

static void streamwindow_start_server(GtkButton *button, gpointer data)
{
	//streamconfig_t *streamconfig;
	//globalsettings_t globalsettings;
	
	svrstart = time(NULL);
	
	//streamconfig = gsfcore_get_streamconfig();
	//globalsettings = mainwindow_get_globalsettings();
	
	if (!(gsfcore_get_streaming_disabled())/*globalsettings->nostreaming*/) {
		//streamconfig->serverctl = streamconfig_service_restart(streamconfig);
		gsfcore_restart_streamserver();
		gtk_button_set_label(GTK_BUTTON(button), _("Starting..."));
		gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
	} else {
		gtk_button_set_label(GTK_BUTTON(button), _("Disabled"));
		gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
	}
}

void streamwindow_stats_settings_configure(gpointer page)
{
	streampage *pageptr;
	streamconfig_t streamconfig;
	//globalsettings_t globalsettings;
	guint udpmcclinum, udpbcclinum, tcpucclinum, tcpucclimax;
	glong sessiontime;
	gchar strbuf[256];
	time_t presenttime;
	
	pageptr = (streampage *)page;
	streamconfig = gsfcore_get_streamconfig();
	//globalsettings = mainwindow_get_globalsettings();
	presenttime = time(NULL);
	if (gsfcore_get_streamserver_control()) {
		g_sprintf(strbuf, _("Running (%u)"), streamconfig->processpid);
		gtk_button_set_label(GTK_BUTTON(pageptr->widgets[1]), strbuf);
		
		if (gtk_widget_get_sensitive(pageptr->widgets[1])) {
			gtk_widget_set_sensitive(pageptr->widgets[1], FALSE);
		}
		
		if (streamconfig_get_statistics(streamconfig, &udpmcclinum, &udpbcclinum, &tcpucclinum, &tcpucclimax, &sessiontime)) {
			g_snprintf((gchar *)&strbuf, sizeof(strbuf), "%u", udpbcclinum);
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), strbuf);
			g_snprintf((gchar *)&strbuf, sizeof(strbuf), "%u", udpmcclinum);
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), strbuf);
			g_snprintf((gchar *)&strbuf, sizeof(strbuf), "%u (%u)", tcpucclinum, tcpucclimax);
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), strbuf);
			streamwindow_format_elapsed_time(sessiontime, time(NULL), (gchar *)&strbuf, sizeof(strbuf));
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), strbuf);
		} else {
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), "--");
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), "--");
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), "--");
			gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), "--");
		}
	} else {
		if (!(gsfcore_get_streaming_disabled())/*globalsettings->nostreaming*/) {
			if ((svrstart != 0) && (difftime(presenttime, svrstart) < 5.0)) {
				gtk_button_set_label(GTK_BUTTON(pageptr->widgets[1]), _("Starting..."));
			} else if ((svrstart == 0) || ((svrstart != 0) && (difftime(presenttime, svrstart) > 5.0))) {
				gtk_button_set_label(GTK_BUTTON(pageptr->widgets[1]), _("Start server"));
				if (svrstart != 0) svrstart = 0;
			}
		} else {
			gtk_button_set_label(GTK_BUTTON(pageptr->widgets[1]), _("Disabled"));
		}
		
		if (!gtk_widget_get_sensitive(pageptr->widgets[1])) {
			if (!(gsfcore_get_streaming_disabled())/*globalsettings->nostreaming*/) {
				if ((svrstart == 0) || ((svrstart != 0) && (difftime(presenttime, svrstart) > 5.0))) {
					gtk_widget_set_sensitive(pageptr->widgets[1], TRUE);
				}
			}
		}
			
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[2]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[3]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[4]), "--");
		gtk_label_set_text(GTK_LABEL(pageptr->widgets[5]), "--");
	}
}

GtkWidget **streamwindow_udpbc_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *interface, *interface_l;
	GtkWidget *port, *port_l;
	GtkCellRenderer *renderer;
	GtkListStore *intstore;
	GtkTreeIter iter;
	GSList *interfaces;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 2, FALSE);
	
	interface_l = gtk_label_new(_("Network interface"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), interface_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 5, 5);
	
	interface = gtk_combo_box_new();
	
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(interface), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(interface), renderer, "markup", STREAM_NETINTERFACE_NAME, NULL);
	
	intstore = gtk_list_store_new(STREAM_NETINTERFACE_COLUMNS, G_TYPE_STRING, G_TYPE_ULONG);
	
	if ((interfaces = streamconfig_interfaces_get_with_broadcast()) != NULL) {
		g_slist_foreach(interfaces, (GFunc)streamwindow_interfaces_foreach, &intstore);
		g_slist_free(interfaces);
	} else {
		gtk_list_store_append(GTK_LIST_STORE(intstore), &iter);
		gtk_list_store_set(GTK_LIST_STORE(intstore), &iter, STREAM_NETINTERFACE_NAME, "lo", STREAM_NETINTERFACE_NUMBER, 1, -1);
	}
	
	gtk_combo_box_set_model(GTK_COMBO_BOX(interface), GTK_TREE_MODEL(intstore));
	g_object_unref(intstore);
	
	gtk_table_attach(GTK_TABLE(table), interface, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
		
	port_l = gtk_label_new(_("Port number"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), port_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 5, 5);
	
	port = gtk_spin_button_new_with_range(1024.0, 65535.0, 1.0);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(port), 0);
	gtk_table_attach(GTK_TABLE(table), port, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(3*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = interface;
	widgets[2] = port;
	
	return widgets;
}

void streamwindow_udpbc_settings_configure(gpointer page)
{
	streampage *pageptr;
	struct _ifwidget ifwidget;
	GtkTreeModel *intmodel;
	
	pageptr = (streampage *)page;
	if (connparams != NULL) {
		if (connparams->udpbcint[0] != '\0') {
			intmodel = gtk_combo_box_get_model(GTK_COMBO_BOX(pageptr->widgets[1]));
			ifwidget.widget = pageptr->widgets[1];
			ifwidget.ifname = connparams->udpbcint;
			gtk_tree_model_foreach(intmodel, streamwindow_interfaces_select_foreach, &ifwidget);
		} else {
			gtk_combo_box_set_active(GTK_COMBO_BOX(pageptr->widgets[1]), 0);
		}
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[2]), (gdouble)connparams->udpbcport);
	} else {
		gtk_combo_box_set_active(GTK_COMBO_BOX(pageptr->widgets[1]), 0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[2]), 4000);
	}
}

void streamwindow_udpbc_settings_apply(gpointer page)
{
	streampage *pageptr;
	gchar *ifname;
	GtkTreeModel *model;
	GtkTreeIter iter;
	
	pageptr = (streampage *)page;
	
	model = gtk_combo_box_get_model(GTK_COMBO_BOX(pageptr->widgets[1]));
	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(pageptr->widgets[1]), &iter)) {
		gtk_tree_model_get(model, &iter, STREAM_NETINTERFACE_NAME, &ifname, -1);
		g_snprintf(connparams->udpbcint, sizeof(connparams->udpbcint), ifname);
		g_free(ifname);
	}
	connparams->udpbcport = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pageptr->widgets[2]));
}

GtkWidget **streamwindow_udpmc_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *interface, *interface_l;
	GtkWidget *group, *group_l;
	GtkWidget *port, *port_l;
	GtkWidget *loopback, *ttl, *loopback_l;
	GtkCellRenderer *renderer;
	GtkListStore *intstore;
	GtkTreeIter iter;
	GSList *interfaces;
	GtkWidget **widgets;
	
	table = gtk_table_new(3, 4, FALSE);
	
	interface_l = gtk_label_new(_("Network interface"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), interface_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 5, 5);
	
	interface = gtk_combo_box_new();
	
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(interface), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(interface), renderer, "markup", STREAM_NETINTERFACE_NAME, NULL);
	
	intstore = gtk_list_store_new(STREAM_NETINTERFACE_COLUMNS, G_TYPE_STRING, G_TYPE_ULONG);
	
	if ((interfaces = streamconfig_interfaces_get_with_multicast()) != NULL) {
		g_slist_foreach(interfaces, (GFunc)streamwindow_interfaces_foreach, &intstore);
		g_slist_free(interfaces);
	} else {
		gtk_list_store_append(GTK_LIST_STORE(intstore), &iter);
		gtk_list_store_set(GTK_LIST_STORE(intstore), &iter, STREAM_NETINTERFACE_NAME, "lo", STREAM_NETINTERFACE_NUMBER, 1, -1);
	}
	
	gtk_combo_box_set_model(GTK_COMBO_BOX(interface), GTK_TREE_MODEL(intstore));
	g_object_unref(intstore);
	gtk_table_attach(GTK_TABLE(table), interface, 1, 3, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	group_l = gtk_label_new(_("Multicast group"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), group_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 5, 5);
	
	group = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(group), INET6_ADDRSTRLEN);
	gtk_table_attach(GTK_TABLE(table), group, 1, 3, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	g_signal_connect(G_OBJECT(group), "changed", G_CALLBACK(streamwindow_validate_regex), NULL);
	
	port_l = gtk_label_new(_("Port number"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), port_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 2, 3, GTK_FILL, GTK_SHRINK, 5, 5);
	
	port = gtk_spin_button_new_with_range(1024.0, 65535.0, 1.0);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(port), 0);
	gtk_table_attach(GTK_TABLE(table), port, 1, 3, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	loopback_l = gtk_label_new(_("Loopback/TTL"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), loopback_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 3, 4, GTK_FILL, GTK_SHRINK, 5, 5);
	
	loopback = gtk_spin_button_new_with_range(0.0, 1.0, 1.0);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(loopback), 0);
	gtk_table_attach(GTK_TABLE(table), loopback, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	ttl = gtk_spin_button_new_with_range(1.0, 255.0, 1.0);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(ttl), 0);
	gtk_table_attach(GTK_TABLE(table), ttl, 2, 3, 3, 4, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(6*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = interface;
	widgets[2] = group;
	widgets[3] = port;
	widgets[4] = loopback;
	widgets[5] = ttl;
	
	return widgets;
}

void streamwindow_udpmc_settings_configure(gpointer page)
{
	streampage *pageptr;
	struct _ifwidget ifwidget;
	GtkTreeModel *intmodel;
	
	pageptr = (streampage *)page;
	if (connparams != NULL) {
		if (connparams->udpmcint[0] != '\0') {
			intmodel = gtk_combo_box_get_model(GTK_COMBO_BOX(pageptr->widgets[1]));
			ifwidget.widget = pageptr->widgets[1];
			ifwidget.ifname = connparams->udpmcint;
			gtk_tree_model_foreach(intmodel, streamwindow_interfaces_select_foreach, &ifwidget);
		} else {
			gtk_combo_box_set_active(GTK_COMBO_BOX(pageptr->widgets[1]), 0);
		}
		if (connparams->udpmcgroup[0] != '\0') {
			gtk_entry_set_text(GTK_ENTRY(pageptr->widgets[2]), connparams->udpmcgroup);
		}
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[3]), (gdouble)connparams->udpmcport);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[4]), (gdouble)connparams->udpmcloop);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[5]), (gdouble)connparams->udpmcttl);
	} else {
		gtk_combo_box_set_active(GTK_COMBO_BOX(pageptr->widgets[1]), 0);
		gtk_entry_set_text(GTK_ENTRY(pageptr->widgets[2]), "225.0.0.0");
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[3]), 4000);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[4]), 0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[5]), 1);
	}
}

void streamwindow_udpmc_settings_apply(gpointer page)
{
	streampage *pageptr;
	gchar *ifname;
	const gchar *group;
	GtkTreeModel *model;
	GtkTreeIter iter;
	
	pageptr = (streampage *)page;
	model = gtk_combo_box_get_model(GTK_COMBO_BOX(pageptr->widgets[1]));
	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(pageptr->widgets[1]), &iter)) {
		gtk_tree_model_get(model, &iter, STREAM_NETINTERFACE_NAME, &ifname, -1);
		g_snprintf(connparams->udpmcint, sizeof(connparams->udpmcint), ifname);
		g_free(ifname);
	}
	
	group = gtk_entry_get_text(GTK_ENTRY(pageptr->widgets[2]));
	strncpy(connparams->udpmcgroup, group, strlen(group));
	connparams->udpmcport = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pageptr->widgets[3]));
	connparams->udpmcloop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pageptr->widgets[4]));
	connparams->udpmcttl = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pageptr->widgets[5]));
}

GtkWidget **streamwindow_tcpuc_settings_create(void)
{
	GtkWidget *table;
	GtkWidget *align;
	GtkWidget *clients, *clients_l;
	GtkWidget *port, *port_l;
	GtkWidget *queue, *queue_l;
	GtkWidget **widgets;
	
	table = gtk_table_new(2, 3, FALSE);
	
	clients_l = gtk_label_new(_("Clients number"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), clients_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 5, 5);
	
	clients = gtk_spin_button_new_with_range(1.0, 90.0, 1.0);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(clients), 0);
	gtk_table_attach(GTK_TABLE(table), clients, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
		
	port_l = gtk_label_new(_("Port number"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), port_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 5, 5);
	
	port = gtk_spin_button_new_with_range(1024.0, 65535.0, 1.0);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(port), 0);
	gtk_table_attach(GTK_TABLE(table), port, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	queue_l = gtk_label_new(_("Queue length"));
	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), queue_l);
	gtk_table_attach(GTK_TABLE(table), align, 0, 1, 2, 3, GTK_FILL, GTK_SHRINK, 5, 5);
	
	queue = gtk_spin_button_new_with_range(1.0, 100.0, 1.0);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(queue), 0);
	gtk_table_attach(GTK_TABLE(table), queue, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
	
	align = gtk_alignment_new(0.5, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), table);
	
	widgets = (GtkWidget **)g_malloc0(4*sizeof(GtkWidget *));
	widgets[0] = align;
	widgets[1] = clients;
	widgets[2] = port;
	widgets[3] = queue;
	
	return widgets;
}

void streamwindow_tcpuc_settings_configure(gpointer page)
{
	streampage *pageptr;
	
	pageptr = (streampage *)page;
	if (connparams != NULL) {
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[1]), (gdouble)connparams->tcpucclients);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[2]), (gdouble)connparams->tcpucport);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[3]), (gdouble)connparams->tcpucqueue);
	} else {
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[1]), 20);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[2]), 4000);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pageptr->widgets[3]), 10);
	}
}

void streamwindow_tcpuc_settings_apply(gpointer page)
{
	streampage *pageptr;
	
	pageptr = (streampage *)page;
	connparams->tcpucclients = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pageptr->widgets[1]));
	connparams->tcpucport = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pageptr->widgets[2]));
	connparams->tcpucqueue = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pageptr->widgets[3]));
}

void streamwindow_set_expired_page(gint preference)
{
	gint i;
	
	if (streamwindow.visible) {
		for (i=0; i<(sizeof(stream_pages)/sizeof(streampage)); i++) {
			if (stream_pages[i].preferenceid == preference) {
				if ((stream_pages[i].configure_func != NULL) && (stream_pages[i].widgets != NULL)) {
					stream_pages[i].expired = TRUE;
				}
			}
		}
	}
}

void streamwindow_update_page(gint preference)
{
	gint i;
	
	if (streamwindow.visible) {
		if (preference == STREAM_PREFERENCE_ID_MENU) {
			streamwindow_fill_entries();
		} else {
			for (i=0; i<(sizeof(stream_pages)/sizeof(streampage)); i++) {
				if (stream_pages[i].preferenceid == preference) {
					if ((stream_pages[i].configure_func != NULL) && (stream_pages[i].widgets != NULL)) {
						stream_pages[i].configure_func(&stream_pages[i]);
					} 
				}
			}
		}
	}
}

gboolean streamwindow_create(GtkWindow *parentwin)
{
	GtkWidget *vbox, *hbox, *hpaned;
	GtkWidget *align;
	GtkWidget *listscroll, *panelscroll;
	streamconfig_t streamconfig;
	
	if (!streamwindow.visible) {
		streamwindow.visible = TRUE;
		
		/*streamwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		streamwindow.title = _("Streaming server settings");
		gtk_window_set_position(GTK_WINDOW(streamwindow.win), GTK_WIN_POS_CENTER);
		gtk_window_set_title(GTK_WINDOW(streamwindow.win), streamwindow.title);
		gtk_widget_set_size_request(streamwindow.win, 500, 300);
		gtk_window_set_default_size(GTK_WINDOW(streamwindow.win), 500, 300);
		gtk_window_set_resizable(GTK_WINDOW(streamwindow.win), FALSE);
		gtk_window_set_transient_for(GTK_WINDOW(streamwindow.win), parentwin);*/
		
		streamwindow.win = gtk_dialog_new_with_buttons(_("Streaming server settings"), 
														GTK_WINDOW(parentwin),
														GTK_DIALOG_DESTROY_WITH_PARENT,
														/*GTK_STOCK_HELP, GTK_RESPONSE_HELP,*/
														GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
														GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
														NULL);
		
		gtk_window_set_modal(GTK_WINDOW(streamwindow.win), TRUE);
		gtk_widget_set_size_request(GTK_WIDGET(streamwindow.win), 500, 300);
		gtk_window_set_default_size(GTK_WINDOW(streamwindow.win), 500, 300);
		gtk_window_set_resizable(GTK_WINDOW(streamwindow.win), FALSE);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
			hpaned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
		#else
			vbox = gtk_vbox_new(FALSE, 0);
			hpaned = gtk_hpaned_new();
		#endif
		gtk_container_set_border_width(GTK_CONTAINER(hpaned), 5);
		
		listscroll = gtk_scrolled_window_new(NULL, NULL);
		
		streamwindow.list = gtk_tree_view_new();
		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(streamwindow.list), FALSE);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(listscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(listscroll), GTK_SHADOW_ETCHED_IN);
		
		gtk_container_add(GTK_CONTAINER (listscroll), streamwindow.list);
		streamwindow.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(streamwindow.list));
		gtk_tree_selection_set_mode(streamwindow.selection, GTK_SELECTION_BROWSE);
		
		streamwindow_init_list(streamwindow.list);
		
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(streamwindow.list), TRUE);
		
		gtk_paned_pack1(GTK_PANED(hpaned), listscroll, TRUE, FALSE);
		
		panelscroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(panelscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(panelscroll), GTK_SHADOW_NONE);
		
		gtk_widget_set_size_request(listscroll, 150, -1);
		gtk_widget_set_size_request(panelscroll, 300, -1);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			streamwindow.panel = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
		#else
			streamwindow.panel = gtk_vbox_new(FALSE, 0);
		#endif
		
		streamwindow.panelcaption = gtk_label_new(NULL);
		gtk_label_set_markup(GTK_LABEL(streamwindow.panelcaption), _("<b>Streaming server settings</b>"));
		gtk_label_set_max_width_chars(GTK_LABEL(streamwindow.panelcaption), 30);
		gtk_label_set_ellipsize(GTK_LABEL(streamwindow.panelcaption), PANGO_ELLIPSIZE_END);
		gtk_box_pack_start(GTK_BOX(streamwindow.panel), streamwindow.panelcaption, FALSE, FALSE, 10);
		
		streamwindow.page = gtk_image_new_from_stock(GTK_STOCK_NETWORK, GTK_ICON_SIZE_DIALOG);
		gtk_box_pack_end(GTK_BOX(streamwindow.panel), streamwindow.page, TRUE, FALSE, 10);
		
		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(panelscroll), streamwindow.panel);
		
		align = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
		gtk_container_add(GTK_CONTAINER(align), panelscroll);
		
		gtk_paned_pack2(GTK_PANED(hpaned), align, TRUE, FALSE);
		
		gtk_box_pack_start(GTK_BOX(vbox), hpaned, TRUE, TRUE, 0);
		
		/*streamwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
		streamwindow.apply = gtk_button_new_from_stock(GTK_STOCK_APPLY);
		
		gtk_widget_set_sensitive(streamwindow.panel, FALSE);
		gtk_widget_set_sensitive(streamwindow.apply, FALSE);
		
		hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), streamwindow.apply, FALSE, FALSE, 5);
		gtk_box_pack_end(GTK_BOX(hbox), streamwindow.close, FALSE, FALSE, 5);
			
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
		
		g_signal_connect(G_OBJECT(streamwindow.win), "destroy", G_CALLBACK(streamwindow_close), NULL);
		g_signal_connect(G_OBJECT(streamwindow.close), "clicked", G_CALLBACK(streamwindow_close), NULL);
		g_signal_connect(G_OBJECT(streamwindow.apply), "clicked", G_CALLBACK(streamwindow_apply), NULL);
		g_signal_connect(G_OBJECT(streamwindow.list), "cursor-changed", G_CALLBACK(streamwindow_entry_selected), NULL);*/
		
		
		streamconfig = gsfcore_get_streamconfig();
		
		connparams = (struct _streamconnparams *)g_malloc0(sizeof(struct _streamconnparams));
		
		streamconfig_get_connection_details(streamconfig, connparams);
		
		groupregex = g_regex_new(STREAMWINDOW_UDP_GROUP_REGEX, G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
		
		svrstart = 0;
		
		timer = g_timeout_add(1000, streamwindow_update_stats, NULL);
		
		streamwindow.pageid = -1;
		streamwindow_fill_entries();
		
		/*gtk_container_add(GTK_CONTAINER(streamwindow.win),vbox);
		gtk_widget_show_all(streamwindow.win);
		
		gtk_window_set_focus(GTK_WINDOW(streamwindow.win), streamwindow.list);*/
		
		GtkWidget *dialogarea = gtk_dialog_get_content_area(GTK_DIALOG(streamwindow.win));
		
		gtk_box_pack_start(GTK_BOX(dialogarea), vbox, TRUE, TRUE, 0);
		gtk_widget_show_all(streamwindow.win);
		
		gtk_dialog_set_response_sensitive(GTK_DIALOG(streamwindow.win), GTK_RESPONSE_APPLY, FALSE);
		
		//gtk_window_set_focus(GTK_WINDOW(prefwindow.win), prefwindow.list);
		
		g_signal_connect(G_OBJECT(streamwindow.win), "destroy", G_CALLBACK(streamwindow_close), NULL);
		g_signal_connect(G_OBJECT(streamwindow.win), "response", G_CALLBACK(streamwindow_action_response), NULL);
		streamwindow.changepagesignal = g_signal_connect(G_OBJECT(streamwindow.list), "cursor-changed", G_CALLBACK(streamwindow_entry_selected), NULL);
	} else {
		gtk_window_present(GTK_WINDOW(streamwindow.win));
	}
	
	return TRUE;
}
