/*
 *      svcparams.c
 *      
 *      Copyright 2009 - 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <glib/gstdio.h>
#include <string.h>
#include <errno.h>
#include "dbstorage.h"
#include "servicelist.h"
#include "gsfcore.h"
#include "calculations.h"
#include "resources.h"

static gboolean switch_params_page_select_lnb_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
static gboolean switch_params_page_custom_lnb_separator(GtkTreeModel *model, GtkTreeIter *iter, gpointer data);
static void switch_params_page_custom_lnb(GtkComboBox *widget, service *svc);
//static void service_assistant_prepare(GtkAssistant *assistant, GtkWidget *page, service *svc);
gint service_assistant_page_func(gint current_page, gpointer data);

static void service_assistant_apply(GtkAssistant *assistant, gpointer data);

static void service_assistant_cancel(GtkAssistant *assistant, service *svc);
static void service_assistant_close(GtkAssistant *assistant, service *svc);

GRegex *ipregex, *macregex;

typedef struct {
     GtkWidget *widget;
     gint index;
     const gchar *title;
     GtkAssistantPageType type;
     gboolean complete;
} pagedescr;

static pagedescr page[7] = {
	{ NULL, -1, N_("Introduction"),           GTK_ASSISTANT_PAGE_INTRO,   TRUE},
	{ NULL, -1, N_("Tuner parameters"),       GTK_ASSISTANT_PAGE_CONTENT, TRUE},
	{ NULL, -1, N_("Switch/LNB parameters"),  GTK_ASSISTANT_PAGE_CONTENT, TRUE},
	{ NULL, -1, N_("Positioner parameters"),  GTK_ASSISTANT_PAGE_CONTENT, TRUE},	
	{ NULL, -1, N_("Satellite internet"),     GTK_ASSISTANT_PAGE_CONTENT, TRUE},
	{ NULL, -1, N_("TV/Radio broadcast"),     GTK_ASSISTANT_PAGE_CONTENT, FALSE},
	{ NULL, -1, N_("Confirmation"),           GTK_ASSISTANT_PAGE_CONFIRM, TRUE},
};

struct swidgets {
	GtkWidget *svcname, *svctype, *svcposition, *svcimage;
	GtkWidget *frequency, *sr, *polarisation, *fec, *system, *modulation, *rolloff;
	GtkWidget *port11, *repeats11, *port10, *lnbtype, *lnb22khz;
	GtkWidget *pos_disabled, *pos_position, *pos_satposition, *pos_number, *pos_satnumber;
	GtkWidget *npid, *nip, *nm, *nmac, *nprocess;
	GtkWidget *pidslist; GtkTreeSelection  *pidsselection;
	
} widgets;

enum {
	PID_STREAM_ENABLED = 0,
	PID_STREAM_TYPE,
	PID_STREAM_VALUE,
	PID_STREAM_LENGTH
};

struct _pval {
	GHashTable *hash;
	guint vcount;
	guint acount;
	guint dcount;
	guint tcount;
	guint scount;
	gboolean valid;
};

enum {
	CAID_VALUE_STRING = 0,
	CAID_VALUE_ID,
	CAID_NUM_COLUMNS
};

enum {
	LNBTYPE_STRING = 0,
	LNBTYPE_ID,
	LNBTYPE_COLUMNS
};

static void tuner_params_page_validate(GtkComboBox *widget, GtkAssistant *assistant)
{
	gint fec, sys, mod, rof;
	gint num;
	GtkWidget *page;
	
	fec = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.fec));
	sys = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.system));
	mod = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.modulation));
	rof = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.rolloff));
	
	num = gtk_assistant_get_current_page(assistant);
	page = gtk_assistant_get_nth_page(assistant, num);
		
	if ((sys == 0) && ((fec == 4) || (fec == 5) || (fec == 7) || (fec == 9) || (fec == 10) || (mod == 1) || (rof == 2) || (rof == 3))) {
		gtk_assistant_set_page_complete(assistant, page, FALSE);
	} else {
		gtk_assistant_set_page_complete(assistant, page, TRUE);
	}
	
}

static void tuner_params_page_finalize(service *svc)
{
	svc->freq = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgets.frequency));
	svc->sr = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgets.sr));
	svc->pol = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.polarisation));
	svc->fec = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.fec));
	svc->sys = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.system));
	svc->mod = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.modulation));
	svc->rof = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.rolloff));
}

GtkWidget *tuner_params_page_new(GtkAssistant *assistant, service *svc)
{
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *frequency_l, *sr_l, *polarisation_l, *fec_l, *system_l, *modulation_l, *rolloff_l, *port_l;
	
	gentable = gtk_table_new(7, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	frequency_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(frequency_l),_("<b>Frequency:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), frequency_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,0,1);
	
	widgets.frequency = gtk_spin_button_new_with_range(3400,12750,1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widgets.frequency), svc->freq);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.frequency);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,0,1);
	
	sr_l = gtk_label_new(NULL);
  	gtk_label_set_markup(GTK_LABEL(sr_l),_("<b>Symbol rate:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), sr_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,1,2);
	
	widgets.sr = gtk_spin_button_new_with_range(1000,45000,1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widgets.sr), svc->sr);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.sr);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,1,2);
	
	polarisation_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(polarisation_l),_("<b>Polarisation:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), polarisation_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,2,3);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.polarisation = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.polarisation),"V");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.polarisation),"H");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.polarisation),"L");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.polarisation),"R");
	#else
		widgets.polarisation = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.polarisation),"V");
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.polarisation),"H");
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.polarisation),"L");
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.polarisation),"R");
	#endif
		
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.polarisation), svc->pol);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.polarisation);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,2,3);
	
	fec_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(fec_l),_("<b>FEC:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), fec_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,3,4);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.fec = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),_("AUTO"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"1/2");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"2/3");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"3/4");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"3/5");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"4/5");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"5/6");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"6/7");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"7/8");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"8/9");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.fec),"9/10");
	#else
		widgets.fec = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec), _("AUTO"));//999=AUTO
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"1/2"); //12=1/2
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"2/3"); //23=2/3
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"3/4"); //34=3/4
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"3/5"); //35=3/5
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"4/5"); //45=4/5
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"5/6"); //56=5/6
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"6/7"); //67=6/7
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"7/8"); //
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"8/9"); //89=8/9
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.fec),"9/10");//910=9/10
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.fec), svc->fec);
	g_signal_connect(G_OBJECT(widgets.fec), "changed", G_CALLBACK(tuner_params_page_validate), (gpointer)assistant);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.fec);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,3,4);
	
	system_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(system_l),_("<b>System:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), system_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,4,5);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.system = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.system),"DVB-S");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.system),"DVB-S2");
	#else
		widgets.system = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.system),"DVB-S");
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.system),"DVB-S2");
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.system), svc->sys);
	g_signal_connect(G_OBJECT(widgets.system), "changed", G_CALLBACK(tuner_params_page_validate), (gpointer)assistant);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.system);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,4,5);
	
	modulation_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(modulation_l),_("<b>Modulation:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), modulation_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,5,6);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.modulation = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.modulation),"QPSK");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.modulation),"8PSK");
	#else
		widgets.modulation = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.modulation),"QPSK");
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.modulation),"8PSK");
	#endif
		
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.modulation), svc->mod);
	g_signal_connect(G_OBJECT(widgets.modulation), "changed", G_CALLBACK(tuner_params_page_validate), (gpointer)assistant);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.modulation);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,5,6);
	
	rolloff_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(rolloff_l),_("<b>Rolloff:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), rolloff_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,6,7);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.rolloff = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.rolloff),_("Unknown"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.rolloff),"0.35");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.rolloff),"0.25");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.rolloff),"0.20");
	#else
		widgets.rolloff = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.rolloff), _("Unknown"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.rolloff),"0.35");
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.rolloff),"0.25");
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.rolloff),"0.20");
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.rolloff), svc->rof);
	g_signal_connect(G_OBJECT(widgets.rolloff), "changed", G_CALLBACK(tuner_params_page_validate), (gpointer)assistant);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.rolloff);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,6,7);
	
	return gentable;
}

static void switch_params_page_finalize(service *svc)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	
	svc->lnbtype = LNB_TYPE_AUTO;
	
	if ((model = gtk_combo_box_get_model(GTK_COMBO_BOX(widgets.lnbtype))) != NULL) {
		if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(widgets.lnbtype), &iter)) {
			gtk_tree_model_get(model, &iter, LNBTYPE_ID, &svc->lnbtype, -1);
		}
	}
		
	svc->port11 = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.port11));
	svc->rep11 = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.repeats11));
	svc->port10 = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.port10));
	svc->lnb22khz = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.lnb22khz));
}

static gboolean switch_params_page_select_lnb_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	service *svc;
	gint id;
	
	svc = (service *)data;
	gtk_tree_model_get(model, iter, LNBTYPE_ID, &id, -1);
	
	if (id == svc->lnbtype) {
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(widgets.lnbtype), iter);
		return TRUE;
	}
	
	return FALSE;
}

static gboolean switch_params_page_custom_lnb_separator(GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	gint id;
		
	gtk_tree_model_get(model, iter, LNBTYPE_ID, &id, -1);
	if (id == LNB_TYPE_NULL) return TRUE; else return FALSE;
}

static void switch_params_page_custom_lnb(GtkComboBox *widget, service *svc)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint id;
	GtkWidget *dialog, *table, *hifreq, *lofreq, *swfreq;
	GtkWidget *dialogarea;
	GtkWidget *hifreq_l, *lofreq_l, *swfreq_l, *align;
	gint res, hifv, lofv, swfv;
	gchar customstr[256];
			
	if ((model = gtk_combo_box_get_model(widget)) != NULL) {
		if (gtk_combo_box_get_active_iter(widget, &iter)) {
			gtk_tree_model_get(model, &iter, LNBTYPE_ID, &id, -1);
			if (id == LNB_TYPE_CUSTOM) {
				dialog = gtk_dialog_new_with_buttons(_("LNB parameters"), 
													GTK_WINDOW(gtk_widget_get_parent(page[2].widget)),
													GTK_DIALOG_DESTROY_WITH_PARENT,
													GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
													GTK_STOCK_APPLY, GTK_RESPONSE_OK,
													NULL);
				
				table = gtk_table_new(2, 3, FALSE);
				gtk_table_set_row_spacings(GTK_TABLE(table), 5);
				gtk_table_set_col_spacings(GTK_TABLE(table), 5);
				gtk_container_set_border_width(GTK_CONTAINER(table), 5);
				
				lofreq_l = gtk_label_new(NULL);
				gtk_label_set_markup(GTK_LABEL(lofreq_l),_("<b>LOF 1:</b>"));
				align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
				gtk_container_add(GTK_CONTAINER(align), lofreq_l);
				gtk_table_attach_defaults(GTK_TABLE(table), align, 0, 1, 0, 1);
				lofreq = gtk_spin_button_new_with_range(3400.0, 12750.0, 1.0);
				gtk_spin_button_set_value(GTK_SPIN_BUTTON(lofreq), (gdouble)svc->lnblofreq);
				gtk_table_attach(GTK_TABLE(table), lofreq, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
	
				hifreq_l = gtk_label_new(NULL);
				gtk_label_set_markup(GTK_LABEL(hifreq_l),_("<b>LOF 2:</b>"));
				align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
				gtk_container_add(GTK_CONTAINER(align), hifreq_l);
				gtk_table_attach_defaults(GTK_TABLE(table), align, 0, 1, 1, 2);
				hifreq = gtk_spin_button_new_with_range(3400.0, 12750.0, 1.0);
				gtk_spin_button_set_value(GTK_SPIN_BUTTON(hifreq), (gdouble)svc->lnbhifreq);
				gtk_table_attach(GTK_TABLE(table), hifreq, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
	
				swfreq_l = gtk_label_new(NULL);
				gtk_label_set_markup(GTK_LABEL(swfreq_l),_("<b>Switch:</b>"));
				align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
				gtk_container_add(GTK_CONTAINER(align), swfreq_l);
				gtk_table_attach_defaults(GTK_TABLE(table), align, 0, 1, 2, 3);
				swfreq = gtk_spin_button_new_with_range(3400.0, 12750.0, 1.0);
				gtk_spin_button_set_value(GTK_SPIN_BUTTON(swfreq), (gdouble)svc->lnbswfreq);
				gtk_table_attach(GTK_TABLE(table), swfreq, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
				
				dialogarea = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
				
				//gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);
				gtk_container_add(GTK_CONTAINER(dialogarea), table);
				gtk_widget_show_all(dialog);
				res = gtk_dialog_run(GTK_DIALOG(dialog));
				
				if (res == GTK_RESPONSE_OK) {
					hifv = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(hifreq));
					lofv = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lofreq));
					swfv = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(swfreq));
					
					if ((swfv >= hifv) && (hifv >= lofv)) {
						g_snprintf((gchar *)&customstr, sizeof(customstr), _("Custom LNB\n<small>%u/%u/%u MHz</small>"), hifv, lofv, swfv);
						gtk_list_store_set(GTK_LIST_STORE(model), &iter, LNBTYPE_STRING, customstr, -1);
						
						svc->lnbhifreq = hifv;
						svc->lnblofreq = lofv;
						svc->lnbswfreq = swfv;
					}
				} else if (res == GTK_RESPONSE_CANCEL) {
					if (svc->lnbtype != LNB_TYPE_CUSTOM) {
						gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)switch_params_page_select_lnb_foreach, (gpointer)svc);
					}
				}
			
				gtk_widget_destroy(dialog);
			}
		}
	}
}

GtkWidget *switch_params_page_new(GtkAssistant *assistant, service *svc)
{
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *port11_l, *repeats11_l, *port10_l, *lnbtype_l, *lnb22khz_l;
	GtkCellRenderer *renderer;
	GtkListStore *lnbtypestore;
	GtkTreeIter iter;
	gchar customstr[256];
	
		
	gentable = gtk_table_new(6, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	port10_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(port10_l),_("<b>Diseqc 1.0 port:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), port10_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,0,1);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.port10 = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port10),_("Unused"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port10),_("Port A"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port10),_("Port B"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port10),_("Port C"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port10),_("Port D"));
	#else
		widgets.port10 = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port10), _("Unused"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port10), _("Port A"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port10), _("Port B"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port10), _("Port C"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port10), _("Port D"));
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.port10), svc->port10);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.port10);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,0,1);
		
	port11_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(port11_l),_("<b>Diseqc 1.1 port:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), port11_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,1,2);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.port11 = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port11),_("Unused"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port11),_("Port 1"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port11),_("Port 2"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port11),_("Port 3"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port11),_("Port 4"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port11),_("Port 5"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port11),_("Port 6"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port11),_("Port 7"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.port11),_("Port 8"));
	#else
		widgets.port11 = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port11), _("Unused"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port11), _("Port 1"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port11), _("Port 2"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port11), _("Port 3"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port11), _("Port 4"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port11), _("Port 5"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port11), _("Port 6"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port11), _("Port 7"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.port11), _("Port 8"));
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.port11), svc->port11);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.port11);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,1,2);
	
	repeats11_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(repeats11_l),_("<b>Diseqc 1.1 repeats:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), repeats11_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,2,3);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.repeats11 = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.repeats11),"0");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.repeats11),"1");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.repeats11),"2");
	#else
		widgets.repeats11 = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.repeats11),"0");
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.repeats11),"1");
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.repeats11),"2");
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.repeats11), svc->rep11);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.repeats11);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,2,3);
	
		
	lnbtype_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(lnbtype_l),_("<b>LNB type:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), lnbtype_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,3,4);
	
	if ((svc->lnbhifreq == 0) || (svc->lnblofreq == 0) || (svc->lnbswfreq == 0)) {
		g_snprintf((gchar *)&customstr, sizeof(customstr), _("Custom LNB\n<small>10600/9750/11700 MHz</small>"));
	} else {
		g_snprintf((gchar *)&customstr, sizeof(customstr), _("Custom LNB\n<small>%u/%u/%u MHz</small>"), svc->lnbhifreq, svc->lnblofreq, svc->lnbswfreq);
	}
	
	widgets.lnbtype = gtk_combo_box_new();
	
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(widgets.lnbtype), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(widgets.lnbtype), renderer, "markup", LNBTYPE_STRING, NULL);
	
	lnbtypestore = gtk_list_store_new(LNBTYPE_COLUMNS, G_TYPE_STRING, G_TYPE_INT);
	
	gtk_list_store_append(GTK_LIST_STORE(lnbtypestore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lnbtypestore), &iter, LNBTYPE_STRING, _("Auto"), LNBTYPE_ID, LNB_TYPE_AUTO, -1);	
	gtk_list_store_append(GTK_LIST_STORE(lnbtypestore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lnbtypestore), &iter, LNBTYPE_STRING, _("Universal Ku Band\n<small>10600/9750/11700 MHz</small>"), LNBTYPE_ID, LNB_TYPE_KU_UNIVERSAL, -1);
	gtk_list_store_append(GTK_LIST_STORE(lnbtypestore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lnbtypestore), &iter, LNBTYPE_STRING, _("WideBand Ku Band\n<small>10750/9750/11700 MHz</small>"), LNBTYPE_ID, LNB_TYPE_KU_WIDEBAND, -1);
	gtk_list_store_append(GTK_LIST_STORE(lnbtypestore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lnbtypestore), &iter, LNBTYPE_STRING, _("Circular Ku Band\n<small>10750 MHz</small>"), LNBTYPE_ID, LNB_TYPE_KU_CIRCULAR, -1);
	gtk_list_store_append(GTK_LIST_STORE(lnbtypestore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lnbtypestore), &iter, LNBTYPE_STRING, _("Standard C Band\n<small>5150 MHz</small>"), LNBTYPE_ID, LNB_TYPE_C_STANDARD, -1);
	gtk_list_store_append(GTK_LIST_STORE(lnbtypestore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lnbtypestore), &iter, LNBTYPE_STRING, _("SingleSolution C Band\n<small>5750 MHz</small>"), LNBTYPE_ID, LNB_TYPE_C_SINGLE_SOLUTION, -1);
	gtk_list_store_append(GTK_LIST_STORE(lnbtypestore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lnbtypestore), &iter, LNBTYPE_STRING, NULL, LNBTYPE_ID, LNB_TYPE_NULL, -1);
	gtk_list_store_append(GTK_LIST_STORE(lnbtypestore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lnbtypestore), &iter, LNBTYPE_STRING, customstr, LNBTYPE_ID, LNB_TYPE_CUSTOM, -1);
	
	gtk_combo_box_set_row_separator_func(GTK_COMBO_BOX(widgets.lnbtype), (GtkTreeViewRowSeparatorFunc)switch_params_page_custom_lnb_separator, NULL, NULL);
	
	gtk_combo_box_set_model(GTK_COMBO_BOX(widgets.lnbtype), GTK_TREE_MODEL(lnbtypestore));
	g_object_unref(lnbtypestore);
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.lnbtype), 0);
	
	gtk_tree_model_foreach(GTK_TREE_MODEL(lnbtypestore), (GtkTreeModelForeachFunc)switch_params_page_select_lnb_foreach, (gpointer)svc);
	
	g_signal_connect(G_OBJECT(widgets.lnbtype), "changed", G_CALLBACK(switch_params_page_custom_lnb), (gpointer)svc);
	
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.lnbtype);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,3,4);
	
		
	lnb22khz_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(lnb22khz_l),_("<b>22kHz tone:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), lnb22khz_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,4,5);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.lnb22khz = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.lnb22khz), _("Auto"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.lnb22khz), _("On"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.lnb22khz), _("Off"));
	#else
		widgets.lnb22khz = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.lnb22khz), _("Auto"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.lnb22khz), _("On"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.lnb22khz), _("Off"));
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.lnb22khz), svc->lnb22khz);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.lnb22khz);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,4,5);
	
	return gentable;
}

////////////////////////////////////////////////////////////////////////////////////
static void svcparams_positioner_params_page_finalize(service *svc)
{
	svc->possatnum = (guchar)gtk_spin_button_get_value(GTK_SPIN_BUTTON(widgets.pos_satnumber));
	
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets.pos_position))) {
		svc->positioner = POSITIONER_POSITION;
	} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets.pos_number))) {
		svc->positioner = POSITIONER_NUMBER;
	} else {
		svc->positioner = POSITIONER_DIASABLED;
	}
}

GtkWidget *svcparams_positioner_params_page_new(GtkAssistant *assistant, service *svc)
{
	GtkWidget *align;
	GtkWidget *gentable;
	
	gentable = gtk_table_new(4, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	widgets.pos_disabled = gtk_radio_button_new(NULL);
	gtk_button_set_label(GTK_BUTTON(widgets.pos_disabled), _("Positioner not used")); 
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.pos_disabled);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 2, 0, 1);
	
	widgets.pos_position = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(widgets.pos_disabled));
	gtk_button_set_label(GTK_BUTTON(widgets.pos_position), _("Use satellite position")); 
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.pos_position);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 1, 2);
	
	widgets.pos_satposition = gtk_label_new(NULL);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.pos_satposition);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 2, 1, 2);
	
	widgets.pos_number = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(widgets.pos_disabled));
	gtk_button_set_label(GTK_BUTTON(widgets.pos_number), _("Use satellite number")); 
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.pos_number);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 2, 3);
	
	widgets.pos_satnumber = gtk_spin_button_new_with_range(0, 128, 1);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.pos_satnumber);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 2, 2, 3);
	
	switch (svc->positioner) {
		case POSITIONER_DIASABLED:
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.pos_disabled), TRUE);
			break;
		case POSITIONER_POSITION:
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.pos_position), TRUE);
			break;
		case POSITIONER_NUMBER:
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.pos_number), TRUE);
			break;
		default:
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.pos_disabled), TRUE);
			break;
	}
	
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widgets.pos_satnumber), (gdouble)svc->possatnum);
	
	return gentable;
}

///////////////////////////////////////////////////////////////////////////////////
static void satinet_params_page_finalize(service *svc)
{
	svc->npid = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgets.npid));
	if (svc->nip != NULL) g_free(svc->nip);
	svc->nip = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.nip)));
	if (svc->nmask != NULL) g_free(svc->nmask);
	svc->nmask = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.nm)));
	if (svc->nmac != NULL) g_free(svc->nmac);
	svc->nmac = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.nmac)));
	if (svc->nprog != NULL) g_free(svc->nprog);
	svc->nprog = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widgets.nprocess)));
}

static void satinet_params_page_validate(GtkEditable *editable, GtkAssistant *assistant)
{
	gchar *ip, *mask, *mac;
	gint num;
	GtkWidget *page;
	
	ip = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.nip)));
	mask = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.nm)));
	mac = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.nmac)));
	
	num = gtk_assistant_get_current_page(assistant);
	page = gtk_assistant_get_nth_page(assistant, num);
	
	if (g_regex_match_all(ipregex, ip, 0, NULL) && g_regex_match_all(ipregex, mask, 0, NULL) && g_regex_match_all(macregex, mac, 0, NULL)) {
		gtk_assistant_set_page_complete(assistant, page, TRUE);
	} else {
		gtk_assistant_set_page_complete(assistant, page, FALSE);
	}
	
	g_free(ip);
	g_free(mask);
	g_free(mac);
}

GtkWidget *satinet_params_page_new(GtkAssistant *assistant, service *svc)
{
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *npid_l, *nip_l, *nm_l, *nmac_l, *nprocess_l;
	GtkFileFilter *filter;
	
	gentable = gtk_table_new(5, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	npid_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(npid_l), _("<b>Network provider PID:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), npid_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,0,1);
	
	widgets.npid = gtk_spin_button_new_with_range(0x001f, 0x1FFF, 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widgets.npid), svc->npid);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.npid);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,0,1);
	
	nip_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(nip_l),_("<b>IP address:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), nip_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,1,2);
	
	widgets.nip = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(widgets.nip), 15);
	gtk_entry_set_text(GTK_ENTRY(widgets.nip), svc->nip);
	g_signal_connect(G_OBJECT(widgets.nip), "changed", G_CALLBACK(satinet_params_page_validate), (gpointer)assistant);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.nip);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,1,2);
	
	nm_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(nm_l),_("<b>Netmask:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), nm_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,2,3);
	
	widgets.nm = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(widgets.nm), 15);
	gtk_entry_set_text(GTK_ENTRY(widgets.nm), svc->nmask);
	g_signal_connect(G_OBJECT(widgets.nm), "changed", G_CALLBACK(satinet_params_page_validate), (gpointer)assistant);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.nm);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,2,3);
	
	nmac_l = gtk_label_new(NULL);
  	gtk_label_set_markup(GTK_LABEL(nmac_l),_("<b>MAC address:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), nmac_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,3,4);
	
	widgets.nmac = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(widgets.nmac), 17);
	gtk_entry_set_text(GTK_ENTRY(widgets.nmac), svc->nmac);
	g_signal_connect(G_OBJECT(widgets.nmac), "changed", G_CALLBACK(satinet_params_page_validate), (gpointer)assistant);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.nmac);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,3,4);
	
	nprocess_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(nprocess_l),_("<b>Start process:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), nprocess_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,4,5);
	
	widgets.nprocess = gtk_file_chooser_button_new(_("Choose executable file"), GTK_FILE_CHOOSER_ACTION_OPEN);
	//gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(ntpwindow.nprocess), g_get_home_dir());
	filter = gtk_file_filter_new();
	gtk_file_filter_add_mime_type(filter, "application/x-executable");
	gtk_file_filter_set_name(filter, _("Executable file"));
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widgets.nprocess), filter);
	if (svc->nprog != NULL) gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(widgets.nprocess), svc->nprog);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.nprocess);
	gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,4,5);
	
	return gentable;
}

static void service_params_page_finalize(service *svc)
{
	gdouble longitude, latitude, azimuth, elevation, hourangle;
	gchar position[128];
	
	if (svc->name != NULL) g_free(svc->name);
	svc->name = g_strdup(gtk_entry_get_text(GTK_ENTRY(widgets.svcname)));
	svc->type = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets.svctype));
	svc->pos = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(widgets.svcposition))*10);
	if (svc->picture != NULL) g_free(svc->picture);
	svc->picture = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widgets.svcimage)));
		
	gsfcore_get_location(&longitude, &latitude);
	
	azimuth = calculations_azimuth(longitude, latitude, svc->pos/10.0, 200.0);
	elevation = calculations_elevation(longitude, latitude, svc->pos/10.0, 200.0);
	hourangle = calculations_hourangle(latitude, azimuth, elevation);
	
	memset(position, 0, sizeof(position));
	g_snprintf(position, sizeof(position), _("<small><b>Azimuth: %.2f\nElevation: %.2f\nHourangle: %.2f</b></small>"), azimuth, elevation, hourangle);
	
	gtk_label_set_markup(GTK_LABEL(widgets.pos_satposition), (const gchar*)position);
}

static void service_params_page_validate(GtkEditable *entry, GtkAssistant *assistant)
{
	const gchar *text;
	gint num;
	GtkWidget *page;
	
	text = gtk_entry_get_text(GTK_ENTRY(entry));
	num = gtk_assistant_get_current_page(assistant);
	page = gtk_assistant_get_nth_page(assistant, num);
	gtk_assistant_set_page_complete(assistant, page, (strlen(text) > 0));
}

void service_params_page_set_image(GtkFileChooserButton *widget, service *svc)
{
	guint height, width, hscale, wscale;
	GdkPixbuf *normal, *scaled;
	gchar *fname, *tbfname;
	GError *error;
		
	if ((fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget))) != NULL) {
		if (g_file_test(fname, G_FILE_TEST_EXISTS)) {
			error = NULL;
			normal = gdk_pixbuf_new_from_file(fname, &error);
			if (error != NULL) {
				g_warning(error->message);
				g_error_free(error);
				return;
			}
			height = gdk_pixbuf_get_height(GDK_PIXBUF(normal));
			width = gdk_pixbuf_get_width(GDK_PIXBUF(normal));
			if (height > width) {
				hscale = 48;
				wscale = (guint)(((gdouble)hscale/(gdouble)height)*(gdouble)width);
			} else if (width > height) {
				wscale = 48;
				hscale = (guint)(((gdouble)wscale/(gdouble)width)*(gdouble)height);
			} else {
				hscale = 48;
				wscale = 48;
			}
			scaled = gdk_pixbuf_scale_simple(normal, wscale, hscale, GDK_INTERP_BILINEAR);
			g_object_unref(normal);
			
			tbfname = g_strdup_printf("%s/%u.png", g_get_tmp_dir(), svc->id);
			error = NULL;
			gdk_pixbuf_save(scaled, tbfname, "png", &error, "compression", "9", NULL);
			if (error != NULL) {
				g_warning(error->message);
				g_error_free(error);
			}
			
			g_object_unref(scaled);
			g_free(tbfname);
		}
	}
	
}

GtkWidget *service_params_page_new(GtkAssistant *assistant, service *svc)
{
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *svcname_l, *svctype_l, *svcposition_l, *svcimage_l, *intro_l;
	GtkFileFilter *filter;
	
	gentable = gtk_table_new(4, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	intro_l = gtk_label_new(_("This wizard will guide you through service creation\nand edition process. By clicking the forward button,\nyou can continue to the next step"));
	gtk_table_attach_defaults(GTK_TABLE(gentable), intro_l, 0, 2, 0, 1);
	
	svcname_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(svcname_l),_("<b>Service name:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), svcname_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 1, 2);
	
	widgets.svcname = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(widgets.svcname), 256);
	gtk_entry_set_text(GTK_ENTRY(widgets.svcname), svc->name);
	g_signal_connect(G_OBJECT(widgets.svcname), "changed", G_CALLBACK(service_params_page_validate), (gpointer)assistant);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.svcname);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 2, 1, 2);
		
	svctype_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(svctype_l),_("<b>Service type:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), svctype_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 2, 3);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		widgets.svctype = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.svctype), _("TV/Radio broadcast"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.svctype), _("Satellite internet"));
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.svctype), _("Empty transponder"));
	#else
		widgets.svctype = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.svctype), _("TV/Radio broadcast"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.svctype), _("Satellite internet"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(widgets.svctype), _("Empty transponder"));
	#endif
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.svctype), svc->type);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.svctype);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 2, 2, 3);
	
	
	svcposition_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(svcposition_l),_("<b>Orbital position:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), svcposition_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 3, 4);
	
	widgets.svcposition = gtk_spin_button_new_with_range(-180.0, 180.0, 0.1);
	//g_signal_connect(G_OBJECT(widgets.svcimage), "file-set", G_CALLBACK(service_params_page_set_image), (gpointer)svc);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widgets.svcposition), svc->pos/10.0);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.svcposition);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 2, 3, 4);
	
	
	svcimage_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(svcimage_l),_("<b>Service image:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), svcimage_l);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 4, 5);
	
	widgets.svcimage = gtk_file_chooser_button_new(_("Choose image"), GTK_FILE_CHOOSER_ACTION_OPEN);
	g_signal_connect(G_OBJECT(widgets.svcimage), "file-set", G_CALLBACK(service_params_page_set_image), (gpointer)svc);
	filter = gtk_file_filter_new();
	gtk_file_filter_add_mime_type(filter, "image/jpeg");
	gtk_file_filter_add_mime_type(filter, "image/gif");
	gtk_file_filter_add_mime_type(filter, "image/png");
	gtk_file_filter_set_name(filter, _("Image"));
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widgets.svcimage), filter);
	if ((svc->picture != NULL) && (g_file_test(svc->picture, G_FILE_TEST_EXISTS)))
		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(widgets.svcimage), svc->picture);
	align = gtk_alignment_new(0.0, 0.5, 1.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), widgets.svcimage);
	gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 2, 4, 5);
	
	return gentable;
}

gboolean service_pids_page_finalize_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, service *svc)
{
	gchar *stype;
	gboolean enabled;
	gchar type;
	gint  value;
	pid   *curpid; 
	
	gtk_tree_model_get(model, iter, PID_STREAM_ENABLED, &enabled, PID_STREAM_TYPE, &stype, PID_STREAM_VALUE, &value, -1);
	
	switch (stype[0]) {
		case 'V': type = PID_TYPE_VIDEO; break;
		case 'A': type = PID_TYPE_AUDIO; break;
		case 'D': type = PID_TYPE_DATA; break;
		case 'S': type = PID_TYPE_SUBTITLES; break;
		case 'T': type = PID_TYPE_TELETEXT; break;
		default:  type = PID_TYPE_DATA; break;
	}
	
	g_free(stype);
	
	if ((svc->type == SERVICE_RADIO) && (type == PID_TYPE_VIDEO)) svc->type = SERVICE_TV_RADIO;
	
	curpid = (pid *)g_malloc0(sizeof(pid));
	curpid->enabled = enabled;
	curpid->type = type;
	curpid->value = value;
	
	svc->pids = g_slist_prepend(svc->pids, curpid);
		
	return FALSE;
}

static void service_pids_page_finalize(service *svc)
{
	GtkTreeModel *model;
	
	if (svc->pids != NULL) service_pids_free(svc);
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widgets.pidslist));
	if (svc->type == SERVICE_TV_RADIO) svc->type = SERVICE_RADIO;
	gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)service_pids_page_finalize_foreach, (gpointer)svc);
	svc->pids = g_slist_reverse(svc->pids);
}

gboolean service_pids_page_validate_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	gchar *stype;
	guint  value;
	guint *hashval;
	struct _pval *pval = (struct _pval *)data;
		
	gtk_tree_model_get(model, iter, PID_STREAM_TYPE, &stype, PID_STREAM_VALUE, &value, -1);
	
	if (value <= 0x001f) {
		g_free(stype);
		return FALSE;
	}
	
	switch (stype[0]) {
		case 'V': pval->vcount += 1; break;
		case 'A': pval->acount += 1; break;
		case 'D': pval->dcount += 1; break;
		case 'S': pval->scount += 1; break;
		case 'T': pval->tcount += 1; break;
		default: break;
	}
	
	g_free(stype);
	
	if ((pval->vcount > 1) || (pval->scount > 1) || (pval->tcount > 1)) {
		pval->valid = FALSE;
		return TRUE;
	}
	
	if (g_hash_table_lookup(pval->hash, &value) == NULL) {
		hashval = (guint *)g_malloc0(sizeof(guint));
		*hashval = value;
		g_hash_table_insert(pval->hash, hashval, hashval);
	} else {
		pval->valid = FALSE;
		return TRUE;
	}
	
	return FALSE;
}

//Clean up hash table contents
void service_pids_page_free_pval(gpointer data)
{
	struct _pval *pval = (struct _pval *)data;
	
	g_free(pval);
}


static void service_pids_page_validate(GtkTreeModel *model, GtkAssistant *assistant)
{
	gint num;
	GtkWidget *page;
	struct _pval pval;
	guint svccount;
	
	svccount = 0;
	pval.hash = g_hash_table_new_full(g_int_hash, g_int_equal, NULL, (GDestroyNotify)service_pids_page_free_pval);
	pval.valid = TRUE; 
	pval.vcount = 0; 
	pval.acount = 0; 
	pval.dcount = 0; 
	pval.tcount = 0; 
	pval.scount = 0;
	gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)service_pids_page_validate_foreach, (gpointer)&pval);
	g_hash_table_destroy(pval.hash);
	svccount = pval.vcount + pval.acount + pval.dcount + pval.tcount + pval.scount;
	
	if ((svccount == 0) || ((pval.vcount == 0) && (pval.acount == 0) && (pval.dcount == 0))) pval.valid = FALSE;
		
	num = gtk_assistant_get_current_page(assistant);
	page = gtk_assistant_get_nth_page(assistant, num);
	gtk_assistant_set_page_complete(assistant, page, pval.valid);
}

void service_pids_page_add_row(GtkButton *button, GtkAssistant *assistant)
{
	GtkTreeModel *model;
	GtkTreeIter  iter;
		
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widgets.pidslist));
	gtk_list_store_append(GTK_LIST_STORE(model), &iter);
	gtk_list_store_set(GTK_LIST_STORE(model), &iter, PID_STREAM_ENABLED, TRUE, PID_STREAM_TYPE, "Audio", PID_STREAM_VALUE, 32, -1);
	service_pids_page_validate(model, GTK_ASSISTANT(assistant));
}

void service_pids_page_rm_row(GtkButton *button, GtkAssistant *assistant)
{
	GtkTreeModel *model;
	GtkTreeIter  iter;
	
	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(widgets.pidsselection), &model, &iter)) {
		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
		service_pids_page_validate(model, GTK_ASSISTANT(assistant));
	}
}

void service_pids_page_set_caid(GtkComboBox *widget, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	guint curid;
	GtkWidget *keyentry;
	
	if ((model = gtk_combo_box_get_model(widget)) != NULL) {
		keyentry = GTK_WIDGET(data);
		gtk_combo_box_get_active_iter(widget, &iter);
		gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, CAID_VALUE_ID, &curid, -1);
		if ((curid == 0x0000) || (curid == 0xffff)) {
			gtk_widget_set_sensitive(keyentry, FALSE);
		} else {
			gtk_widget_set_sensitive(keyentry, TRUE);
		}
	}
	
}

void service_pids_page_properties(GtkButton *button, service *svc)
{
	GtkWidget *dialog, *table, *encsys, *enckey, *svcsid;
	GtkWidget *dialogarea;
	GtkWidget *encsys_l, *enckey_l, *svcsid_l, *align;
	gint res, sid;
	gchar *key;
	GRegex *keyregex;
	GtkCellRenderer *renderer;
	GtkListStore *caidstore;
	GtkTreeIter iter;
	gboolean valid;
	guint curid;
	
	
	dialog = gtk_dialog_new_with_buttons(_("Extended settings"), 
										GTK_WINDOW(gtk_widget_get_parent(page[5].widget)),
										GTK_DIALOG_DESTROY_WITH_PARENT,
										GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
										GTK_STOCK_APPLY, GTK_RESPONSE_OK,
										NULL);
	
	enckey = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(enckey), 256);
	if (svc->key != NULL) gtk_entry_set_text(GTK_ENTRY(enckey), svc->key);
	
	encsys = gtk_combo_box_new();
	
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(encsys), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(encsys), renderer, "markup", CAID_VALUE_STRING, NULL);
	
	caidstore = gtk_list_store_new(CAID_NUM_COLUMNS, G_TYPE_STRING, G_TYPE_UINT);
	
	gtk_list_store_append(GTK_LIST_STORE(caidstore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(caidstore), &iter, CAID_VALUE_STRING, "FTA", CAID_VALUE_ID, 0x0000, -1);	
	gtk_list_store_append(GTK_LIST_STORE(caidstore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(caidstore), &iter, CAID_VALUE_STRING, "BISS 1", CAID_VALUE_ID, 0x2600, -1);
	gtk_list_store_append(GTK_LIST_STORE(caidstore), &iter);
	gtk_list_store_set(GTK_LIST_STORE(caidstore), &iter, CAID_VALUE_STRING, "Other", CAID_VALUE_ID, 0xffff, -1);
	
	gtk_combo_box_set_model(GTK_COMBO_BOX(encsys), GTK_TREE_MODEL(caidstore));
	g_object_unref(caidstore);
	
	g_signal_connect(G_OBJECT(encsys), "changed", G_CALLBACK(service_pids_page_set_caid), enckey);
	
	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(caidstore), &iter);
	while (valid) {
		gtk_tree_model_get(GTK_TREE_MODEL(caidstore), &iter, CAID_VALUE_ID, &curid, -1);
		if ((curid == svc->caid) || (curid == 0xffff)) {
			gtk_combo_box_set_active_iter(GTK_COMBO_BOX(encsys), &iter);
			break;
		}
		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(caidstore), &iter);
	}
	
	svcsid = gtk_spin_button_new_with_range(0.0, 65535.0, 1.0);
	if (svc->sid < 65535) gtk_spin_button_set_value(GTK_SPIN_BUTTON(svcsid), (gdouble)svc->sid);
			
	table = gtk_table_new(2, 3, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
	
	encsys_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(encsys_l),_("<b>Encryption system:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), encsys_l);
	gtk_table_attach_defaults(GTK_TABLE(table), align, 0, 1, 0, 1);
	gtk_table_attach(GTK_TABLE(table), encsys, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
	
	enckey_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(enckey_l),_("<b>Encryption key:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), enckey_l);
	gtk_table_attach_defaults(GTK_TABLE(table), align, 0, 1, 1, 2);
	gtk_table_attach(GTK_TABLE(table), enckey, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
	
	svcsid_l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(svcsid_l),_("<b>SID:</b>"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), svcsid_l);
	gtk_table_attach_defaults(GTK_TABLE(table), align, 0, 1, 2, 3);
	gtk_table_attach(GTK_TABLE(table), svcsid, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
	
	dialogarea = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
	
	//gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG (dialog)->vbox), table);
	gtk_container_add(GTK_CONTAINER(dialogarea), table);
	gtk_widget_show_all(dialog);
	res = gtk_dialog_run(GTK_DIALOG(dialog));
	
	if (res == GTK_RESPONSE_OK) {
		gtk_combo_box_get_active_iter(GTK_COMBO_BOX(encsys), &iter);
		gtk_tree_model_get(GTK_TREE_MODEL(caidstore), &iter, CAID_VALUE_ID, &curid, -1);
		key = g_strdup(gtk_entry_get_text(GTK_ENTRY(enckey)));
		sid = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(svcsid));
		
		if (key != NULL) {
			switch (curid) {
				case 0x2600:
					keyregex = g_regex_new("^([0-9a-fA-F]{2}){8}$", G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
					if (g_regex_match_all(keyregex, key, 0, NULL)) {
						svc->caid = 0x2600;
						svc->key = g_strdup(key);
					}
					g_regex_unref(keyregex);
					break;
				
				case 0x0000:
					svc->caid = 0x0000;
					svc->key = NULL;
					break;
					
				default:
					svc->key = NULL;
					break;
			}
		}
		
		g_free(key);
		
		if ((sid != 0) && (sid < 65535)) svc->sid = sid;
	}
	
	gtk_widget_destroy(dialog);
}

void service_pids_page_edit_type(GtkCellRendererText *renderer, gchar *path, gchar *new_text, gpointer assistant)
{
	GtkTreeIter  iter;
	GtkTreeModel *model;
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widgets.pidslist));
	if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, PID_STREAM_TYPE, new_text, -1);
		service_pids_page_validate(model, GTK_ASSISTANT(assistant));
	}
	
}

void service_pids_page_edit_pid(GtkCellRendererText *renderer, gchar *path, gchar *new_text, gpointer assistant)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	guint newvalue;
	gchar *endptr;
				
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widgets.pidslist));
	if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
		errno = 0;
		newvalue = strtoul(new_text, &endptr, 10);
		if ((errno == 0) && (newvalue > 0x001f) && (newvalue < 0x2000)) {
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, PID_STREAM_VALUE, newvalue, -1);
			service_pids_page_validate(model, GTK_ASSISTANT(assistant));
		}
	}
	
}

void service_pids_page_toggle_pid(GtkCellRendererToggle *cell_renderer, gchar *path, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gboolean value;
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widgets.pidslist));
	if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
		gtk_tree_model_get(model, &iter, PID_STREAM_ENABLED, &value, -1);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, PID_STREAM_ENABLED, !value, -1);
	}
}

void service_pids_fill(pid *curpid, GtkListStore *store)
{
	GtkTreeIter iter;
	gchar *types[] = {"Video", "Audio", "Data", "Subtitles", "Teletext"};
	
	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter, PID_STREAM_ENABLED, curpid->enabled, PID_STREAM_TYPE, types[curpid->type], PID_STREAM_VALUE, curpid->value, -1);
}


GtkWidget *service_pids_page_new(GtkAssistant *assistant, service *svc)
{
	GtkWidget         *scrollwin;
	GtkWidget         *addbutton;
	GtkWidget         *rmbutton;
	GtkWidget         *advbutton;
	GtkCellRenderer   *rendererimg, *renderercb, *rendererspin, *renderertoggle;
	GtkTreeViewColumn *column;
	GtkListStore      *store;
	GtkTreeIter       iter;
	GtkAdjustment     *adj;
	GtkListStore      *combomodel;
	GtkWidget         *gentable;
	gint              i;
		
	gentable = gtk_table_new(3, 3, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(gentable), 2);
	gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	
	scrollwin = gtk_scrolled_window_new(NULL, NULL);
	widgets.pidslist = gtk_tree_view_new();
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollwin), GTK_SHADOW_ETCHED_IN);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(widgets.pidslist), TRUE);
	gtk_container_add(GTK_CONTAINER(scrollwin), widgets.pidslist);
	widgets.pidsselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widgets.pidslist));
	gtk_table_attach_defaults(GTK_TABLE(gentable), scrollwin, 0, 3, 0, 1);
	
	renderertoggle = gtk_cell_renderer_toggle_new();
	g_signal_connect(G_OBJECT(renderertoggle), "toggled", G_CALLBACK(service_pids_page_toggle_pid), assistant);
	column = gtk_tree_view_column_new_with_attributes("", renderertoggle, "active", PID_STREAM_ENABLED, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(widgets.pidslist), column);
		
	renderercb = gtk_cell_renderer_combo_new();
	
	combomodel = gtk_list_store_new(1, G_TYPE_STRING);
	gtk_list_store_append(combomodel, &iter);
	gtk_list_store_set(combomodel, &iter, 0, "Video", -1);
	gtk_list_store_append(combomodel, &iter);
	gtk_list_store_set(combomodel, &iter, 0, "Audio", -1);
	gtk_list_store_append(combomodel, &iter);
	gtk_list_store_set(combomodel, &iter, 0, "Data", -1);
	gtk_list_store_append(combomodel, &iter);
	gtk_list_store_set(combomodel, &iter, 0, "Subtitles", -1);
	gtk_list_store_append(combomodel, &iter);
	gtk_list_store_set(combomodel, &iter, 0, "Teletext", -1);
	
	g_object_set(renderercb, "text-column", 0, "editable", TRUE, "has-entry", FALSE, "model", combomodel, NULL);
	g_signal_connect(G_OBJECT(renderercb), "edited", G_CALLBACK(service_pids_page_edit_type), assistant);
	column = gtk_tree_view_column_new_with_attributes(_("Stream type"), renderercb, "text", PID_STREAM_TYPE, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(widgets.pidslist), column);
	
	rendererspin = gtk_cell_renderer_spin_new();
	adj = GTK_ADJUSTMENT(gtk_adjustment_new(100.0, 32.0, 8191.0, 1.0, 2.0, 0.0));
	g_object_set(rendererspin, "editable", TRUE, "adjustment", adj, "digits", 0, NULL);
	g_signal_connect(G_OBJECT(rendererspin), "edited", G_CALLBACK(service_pids_page_edit_pid), assistant);
	column = gtk_tree_view_column_new_with_attributes(_("Stream PID"), rendererspin, "text", PID_STREAM_VALUE, NULL);
	
	gtk_tree_view_append_column(GTK_TREE_VIEW(widgets.pidslist), column);
	
	store = gtk_list_store_new(PID_STREAM_LENGTH, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_INT);
	gtk_tree_view_set_model(GTK_TREE_VIEW(widgets.pidslist), GTK_TREE_MODEL(store));
	g_object_unref(store);
	
	if (svc->pids != NULL) {
		page[5].complete = TRUE; //If there are pids in cloned service, just set page ready
		g_slist_foreach(svc->pids, (GFunc)service_pids_fill, (gpointer)store);
	}
	
	addbutton = gtk_button_new_from_stock(GTK_STOCK_ADD);
	gtk_table_attach(GTK_TABLE(gentable), addbutton, 0, 1, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
	g_signal_connect(G_OBJECT(addbutton), "clicked", G_CALLBACK(service_pids_page_add_row), (gpointer)assistant);
	
	rmbutton = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
	gtk_table_attach(GTK_TABLE(gentable), rmbutton, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
	g_signal_connect(G_OBJECT(rmbutton), "clicked", G_CALLBACK(service_pids_page_rm_row), (gpointer)assistant);
	
	advbutton = gtk_button_new_with_label(_("Extended"));
	gtk_table_attach(GTK_TABLE(gentable), advbutton, 2, 3, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
	g_signal_connect(G_OBJECT(advbutton), "clicked", G_CALLBACK(service_pids_page_properties), (gpointer)svc);
	
	return gentable;
}

gint service_assistant_page_func(gint current_page, gpointer data)
{
	service *svc = (service *)data;
	gint nextpage = current_page + 1;
	
	if ((svc->type == SERVICE_TV_RADIO) || (svc->type == SERVICE_RADIO)) {
		switch (current_page) {
			case 0: 
				service_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 1:
				tuner_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 2:
				switch_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 3:
				svcparams_positioner_params_page_finalize(svc);
				nextpage = current_page + 2;
				break;
			case 5:
				service_pids_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			default:
				//nextpage = 6;
				//nextpage = current_page + 1;
				break;
		}
	} else if (svc->type == SERVICE_DATA) {
		switch (current_page) {
			case 0: 
				service_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 1:
				tuner_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 2:
				switch_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 3:
				svcparams_positioner_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 4:
				satinet_params_page_finalize(svc);
				nextpage = current_page + 2;
				break;
			default:
				//nextpage = 6;
				//nextpage = current_page + 1;
				break;
		}
	} else if (svc->type == SERVICE_TRANSPONDER) {
		switch (current_page) {
			case 0: 
				service_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 1:
				tuner_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 2:
				switch_params_page_finalize(svc);
				nextpage = current_page + 1;
				break;
			case 3:
				svcparams_positioner_params_page_finalize(svc);
				nextpage = current_page + 3;
				break;
			default:
				//nextpage = 6;
				//nextpage = current_page + 1;
				break;
		}
	}
		
	return nextpage;
}

gboolean service_assistant_new(service *svc, GtkWindow *rootwindow)
{
	GtkWidget *assistant;
	guint i;
	#if !GTK_CHECK_VERSION(3, 2, 0)
		GdkPixbuf *pixbuf;
	#endif
	service *dsvc;
		
	ipregex = g_regex_new("^(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])(\\.(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])){3}$", G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
	macregex = g_regex_new("^([0-9a-f]{2}(:|$)){6}$", G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
		
	dsvc = service_clone(svc);
	
	assistant = gtk_assistant_new();
	gtk_widget_set_size_request (assistant, 470, 370);
	gtk_window_set_title (GTK_WINDOW (assistant), _("Service properties editor"));
	gtk_window_set_modal(GTK_WINDOW(assistant), TRUE);
	gtk_window_set_transient_for(GTK_WINDOW(assistant), GTK_WINDOW(rootwindow));
	gtk_window_set_position(GTK_WINDOW(assistant), GTK_WIN_POS_CENTER_ON_PARENT);	
	
	#if !GTK_CHECK_VERSION(3, 2, 0)
		pixbuf = gdk_pixbuf_new_from_file(RESOURCE_SVCPARAMS_WIZARD, NULL);		
	#endif
	
	page[0].widget = service_params_page_new(GTK_ASSISTANT(assistant), dsvc);
	page[1].widget = tuner_params_page_new(GTK_ASSISTANT(assistant), dsvc);
	page[2].widget = switch_params_page_new(GTK_ASSISTANT(assistant), dsvc);
	page[3].widget = svcparams_positioner_params_page_new(GTK_ASSISTANT(assistant), dsvc);
	page[4].widget = satinet_params_page_new(GTK_ASSISTANT(assistant), dsvc);
	page[5].widget = service_pids_page_new(GTK_ASSISTANT(assistant), dsvc);
	page[6].widget = gtk_label_new (_("Service edition process successfully finished.\nClick \"Apply\" if you want to save changes,\notherwise click \"Cancel\"."));
	
	for (i = 0; i <= 6; i++) {
		page[i].index = gtk_assistant_append_page(GTK_ASSISTANT (assistant), page[i].widget);
		gtk_assistant_set_page_title(GTK_ASSISTANT (assistant), page[i].widget, gettext(page[i].title));
		#if !GTK_CHECK_VERSION(3, 2, 0)
			gtk_assistant_set_page_header_image(GTK_ASSISTANT (assistant), page[i].widget, pixbuf);
		#endif
		gtk_assistant_set_page_type(GTK_ASSISTANT (assistant), page[i].widget, page[i].type);
		gtk_assistant_set_page_complete(GTK_ASSISTANT (assistant), page[i].widget, page[i].complete);
	}
	#if !GTK_CHECK_VERSION(3, 2, 0)
		gdk_pixbuf_unref(pixbuf);
	#endif
	
	g_signal_connect (G_OBJECT (assistant), "cancel", G_CALLBACK (service_assistant_cancel), (gpointer)dsvc);
	g_signal_connect (G_OBJECT (assistant), "close", G_CALLBACK (service_assistant_close), (gpointer)dsvc);
	//g_signal_connect (G_OBJECT (assistant), "apply", G_CALLBACK (service_assistant_apply), (gpointer)dsvc);
	//g_signal_connect (G_OBJECT (assistant), "prepare", G_CALLBACK (service_assistant_prepare), (gpointer)dsvc);
	
	gtk_assistant_set_forward_page_func(GTK_ASSISTANT(assistant), service_assistant_page_func, (gpointer)dsvc, NULL);
	
	gtk_widget_show_all(assistant);
	
	return TRUE;
}

/* If the dialog is cancelled, delete it from memory and then clean up after
  * the Assistant structure. */
static void service_assistant_cancel (GtkAssistant *assistant, service *svc)
{
	gchar *tbfname;
	
	gtk_assistant_set_forward_page_func(GTK_ASSISTANT(assistant), NULL, NULL, NULL);
	
	tbfname = g_strdup_printf("%s/%u.png", g_get_tmp_dir(), svc->id);
	if (g_file_test(tbfname, G_FILE_TEST_EXISTS)) {
		g_unlink(tbfname);
	}
	g_free(tbfname);
	
	service_free(svc);
	g_regex_unref(ipregex);
	g_regex_unref(macregex);
	gtk_widget_destroy (GTK_WIDGET (assistant));
}

/* This function is where you would apply the changes and destroy the assistant. */
static void service_assistant_close (GtkAssistant *assistant, service *svc)
{
	gtk_assistant_set_forward_page_func(GTK_ASSISTANT(assistant), NULL, NULL, NULL);
	
	if (svc->id == 0) {
		service_add(svc, TRUE, servicelist_add_service_cb);
	} else {
		service_merge(svc, servicelist_update_service_cb);
	}
	g_regex_unref(ipregex);
	g_regex_unref(macregex);
	gtk_widget_destroy (GTK_WIDGET (assistant));
}

