/*
 *      teletext.h
 *      
 *      Copyright 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TELETEXT_H__
#define __TELETEXT_H__

#include <glib.h>

#define TT_LINE_UTF_MAX_LENGTH  26*3*40

enum {
	TT_LANGUAGE_ENGLISH = 0,
	TT_LANGUAGE_FRENCH,
	TT_LANGUAGE_SWEDISH,
	TT_LANGUAGE_FINNISH,
	TT_LANGUAGE_HUNGARIAN,
	TT_LANGUAGE_CZECH,
	TT_LANGUAGE_SLOVAK,
	TT_LANGUAGE_GERMAN,
	TT_LANGUAGE_PORTUGUESE,
	TT_LANGUAGE_SPANISH,
	TT_LANGUAGE_ITALIAN,
	TT_LANGUAGE_POLISH,
	TT_LANGUAGE_TURKISH,
	TT_LANGUAGE_SERBIAN,
	TT_LANGUAGE_CROATIAN,
	TT_LANGUAGE_SERBIAN_L,
	TT_LANGUAGE_CROATIAN_L,
	TT_LANGUAGE_SLOVENIAN,
	TT_LANGUAGE_RUSSIAN,
	TT_LANGUAGE_BULGARIAN,
	TT_LANGUAGE_ESTONIAN,
	TT_LANGUAGE_UKRAINIAN,
	TT_LANGUAGE_LETTISH,
	TT_LANGUAGE_LITHUANIAN,
	TT_LANGUAGE_RUMANIAN,
	TT_LANGUAGE_GREEK,
	TT_LANGUAGE_ARABIC,
	TT_LANGUAGE_HEBREW
};

enum {
	TT_EVENT_PAGE_NEW = 0,
	TT_EVENT_PAGE_REFRESH,
	TT_EVENT_PAGE_PARSING,
	TT_EVENT_NETWORK,
	TT_EVENT_STOP
};

enum {
	TT_ERROR_NULL = 0,
	TT_ERROR_TIMEOUT,
	TT_ERROR_NO_DATA
};

enum {
	TT_COLOUR_BLACK = 0,
	TT_COLOUR_RED,
	TT_COLOUR_GREEN,
	TT_COLOUR_YELLOW,
	TT_COLOUR_BLUE,
	TT_COLOUR_MAGENTA,
	TT_COLOUR_CYAN,
	TT_COLOUR_WHITE
};

enum {
	TT_SIZE_NORMAL = 0,
	TT_SIZE_DOUBLE_HEIGHT,
	TT_SIZE_DOUBLE_WIDTH,
	TT_SIZE_DOUBLE_SIZE
};

enum {
	TT_TAG_NULL = 0,
	TT_TAG_COLOUR,
	TT_TAG_BG_COLOUR,
	TT_TAG_SIZE,
	TT_TAG_FLASH,
	TT_TAG_BOX
};

typedef void (*tt_parser_event_callback)(gint event, void *data);

struct _ttcolour {
	guchar r;
	guchar g;
	guchar b;
};

typedef struct _ttcolour * ttcolour_t;

struct _tttag {
	guchar tag;
	guchar value;
	gushort start;
	gushort end;
};

typedef struct _tttag * tttag_t;

struct _ttpage {
	guint eraseflag : 1;
	guint newsflashflag : 1;
	guint subtitleflag : 1;
	guint updateflag : 1;
	guint intseqflag : 1;
	guint magserialflag : 1;
	guint language : 3;
	guint primno : 7;
	guint secno : 7;
	guint defscrcolor : 5;
	guint defrowcolor : 5;
	guint bgcolorsub : 1;
	guint visible : 1;
	guint charset[3];
	guchar colormap[2][32][3];
	guint text[2][26][40];
	gushort tags[2][26][40];
	guint32 bitmap;
	guint32 pageid;
	time_t lastupdate;
};

typedef struct _ttpage * ttpage_t;

struct _ttparser {
	gint fd;
	gint ctlfd[2];
	guint adapter;
	guint demux;
	guint pid;
	guint command;
	guint error;
	gboolean running;
	guint language;
	gboolean detectlang;
	guint initialpage;
	guint initialsubpage;
	guint networkident;
	guchar networkname[21];
	time_t svctime;
	GMutex *expmutex;
	GHashTable *pages;
	GThread *thread;
	tt_parser_event_callback callback;
};

typedef struct _ttparser * ttparser_t;


gushort tt_get_pagenum(guint pageid);
gushort tt_get_subpagenum(guint pageid);
gchar *tt_page_get_string(ttparser_t parser, guint32 pageid, guint *length, GSList **tags);
ttcolour_t tt_page_get_colormap(ttparser_t parser, guint32 pageid);
ttparser_t tt_parser_new(guint language, gboolean detectlang, guint datasrc[3], tt_parser_event_callback callback);
void tt_parser_close(ttparser_t parser);
void tt_parser_set_params(ttparser_t parser, guint language, gboolean detectlang);
void tt_parser_get_params(ttparser_t parser, guint *language, gboolean *detectlang);

#endif /* __TELETEXT_H__ */
