/*
 *      tswindow.c
 *      
 *      Copyright 2009-2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>

#include "gsfcore.h"
#include "streamconfig.h"
#include "dbstorage.h"

#define TSWINDOW_STATUSBAR_CONTEXT "tsstatus"
#define TSWINDOW_PLAYLIST "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<playlist version=\"1\" xmlns=\"http://xspf.org/ns/0/\">\n\t<trackList>\n\t\t<track>\n\t\t\t<title>%s</title>\n\t\t\t<location>http://localhost:%u/%u</location>\n\t\t</track>\n\t</trackList>\n</playlist>"

enum {
	TS_COL_ICON = 0,
	TS_COL_PID,
	TS_COL_BANDWIDTH,
	TS_COL_PERCENTAGE,
	TS_COL_STREAMPARAMS,
	TS_COLUMNS
};

struct stswindow {
	//transport stream structure window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *toolbar;
	GtkToolItem *playlist_tb, *sep_tb, *close_tb;
	GtkWidget *statusbar;
	GtkTreeSelection *selection;
	GtkWidget *list;
	GtkTreeModel *model;
	guint timer;
};
struct stswindow tswindow;


gint tswindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
void tswindow_init_list(GtkWidget *list);
void tswindow_edit_stream(struct _strtspid *strtspid, GtkTreeModel *model, GtkTreeIter *iter);
void tswindow_add_stream_foreach(gpointer data, gpointer user_data);
gboolean tswindow_update(gpointer data);


gint tswindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	tswindow.visible = FALSE;
	g_source_remove(tswindow.timer);
	gtk_widget_destroy(tswindow.win);
	return FALSE;
}

static void tswindow_status_refresh(GSList *streams)
{
	guint activestreams;
	gchar *status;
	guint context_id;
	
	if (streams == NULL) return;
	
	activestreams = g_slist_length(streams);
	
	status = g_strdup_printf(_("Active streams: %u"), activestreams);
	
	if (status != NULL) {
		context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(tswindow.statusbar), TSWINDOW_STATUSBAR_CONTEXT);
		gtk_statusbar_pop(GTK_STATUSBAR(tswindow.statusbar), context_id);
		gtk_statusbar_push(GTK_STATUSBAR(tswindow.statusbar), context_id, status);
		g_free(status);
	}
}

static void tswindow_playlist(GtkWidget *widget, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	guint streampid;
	GtkWidget *dialog;
	gint res;
	gchar *filename;
	gchar *message;
	streamconfig_t streamconfig;
	struct _streamconnparams connparams;
	service_t svc;
	
	streamconfig = gsfcore_get_streamconfig();
	svc = gsfcore_get_service();
		
	if ((streamconfig == NULL) || (svc == NULL)) return;
	
	streamconfig_get_connection_details(streamconfig, &connparams);
	
	if (gtk_tree_selection_get_selected(tswindow.selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, TS_COL_PID, &streampid, -1);
		if ((streampid > 32) && (streampid < 8192)) {
			dialog = gtk_file_chooser_dialog_new(_("Save File"),
												GTK_WINDOW(tswindow.win),
												GTK_FILE_CHOOSER_ACTION_SAVE,
												GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
												GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
												NULL);
			
			gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), "playlist.xspf");
			res = gtk_dialog_run(GTK_DIALOG(dialog));
			
			if (res == GTK_RESPONSE_ACCEPT) {
				filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
								
				message = g_strdup_printf(TSWINDOW_PLAYLIST, svc->name, connparams.tcpucport, streampid);
				
				g_file_set_contents(filename, message, -1, NULL);
				
				g_free(filename);
				g_free(message);
			}
			
			gtk_widget_destroy(dialog);
		}
	}
	
}

void tswindow_init_list(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2, *column3, *column4, *column5;
	GtkListStore       *store;
	GtkTreeIter        iter;
	
	
	renderer = gtk_cell_renderer_pixbuf_new();
	column1 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column1, "");
	gtk_tree_view_column_pack_start(column1, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column1, renderer, "stock-id", TS_COL_ICON, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	
	renderer = gtk_cell_renderer_text_new();
	column2 = gtk_tree_view_column_new_with_attributes(_("PID"), renderer, "text", TS_COL_PID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	
	renderer = gtk_cell_renderer_text_new();
	column3 = gtk_tree_view_column_new_with_attributes(_("Bandwidth"), renderer, "text", TS_COL_BANDWIDTH, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column3);
	
	renderer = gtk_cell_renderer_progress_new();
	column4 = gtk_tree_view_column_new_with_attributes(_("Percentage"), renderer, "value", TS_COL_PERCENTAGE, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column4);
	
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, "ellipsize-set", TRUE, NULL);
	column5 = gtk_tree_view_column_new_with_attributes(_("Stream properties"), renderer, "markup", TS_COL_STREAMPARAMS, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column5);
		
	store = gtk_list_store_new(TS_COLUMNS, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

void tswindow_edit_stream(struct _strtspid *strtspid, GtkTreeModel *model, GtkTreeIter *iter)
{
	gchar *iconstr;
	
	if ((strtspid != NULL) && (model != NULL) && (iter != NULL)) {
		if (strtspid->format == TS_STREAM_FORMAT_SPECIAL) {
			iconstr = GTK_STOCK_PROPERTIES;
		} else if (strtspid->format == TS_STREAM_FORMAT_TABLE) {
			iconstr = GTK_STOCK_FILE;
		} else if (strtspid->format == TS_STREAM_FORMAT_UNKNOWN) {
			switch (strtspid->type) {
				case TS_STREAM_PES_PRIVATE_STREAM_1:
					iconstr = GTK_STOCK_PROPERTIES;
					break;
				case TS_STREAM_PES_AUDIO_STREAM:
					iconstr = GTK_STOCK_MEDIA_PLAY;
					break;
				case TS_STREAM_PES_VIDEO_STREAM:
					iconstr = GTK_STOCK_MEDIA_PLAY;
					break;
				default:
					iconstr = GTK_STOCK_PROPERTIES;
					break;
			}
		} else if (strtspid->format == TS_STREAM_FORMAT_ENCRYPTED) {
			iconstr = GTK_STOCK_DIALOG_AUTHENTICATION;
		} else if (strtspid->format == TS_STREAM_FORMAT_VIDEO_H262) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_VIDEO_H264) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_MPEG) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_AC3) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_AAC) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_DTS) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		}
		
		gtk_list_store_set(GTK_LIST_STORE(model), iter, TS_COL_ICON, iconstr, TS_COL_PID, strtspid->pid, TS_COL_BANDWIDTH, strtspid->bandwidth, TS_COL_PERCENTAGE, strtspid->percentage, TS_COL_STREAMPARAMS, strtspid->description, -1);
	}
}

void tswindow_add_stream_foreach(gpointer data, gpointer user_data)
{
	struct _strtspid *strtspid;
	GtkTreeIter iter;
	gchar *iconstr;
	GtkTreeModel *model;
	
	strtspid = (struct _strtspid *)data;
	model = (GtkTreeModel *)user_data;
	
	if ((!strtspid->sticky) && (model != NULL)) {
		if (strtspid->format == TS_STREAM_FORMAT_SPECIAL) {
			iconstr = GTK_STOCK_PROPERTIES;
		} else if (strtspid->format == TS_STREAM_FORMAT_TABLE) {
			iconstr = GTK_STOCK_FILE;
		} else if (strtspid->format == TS_STREAM_FORMAT_UNKNOWN) {
			switch (strtspid->type) {
				case TS_STREAM_PES_PRIVATE_STREAM_1:
					iconstr = GTK_STOCK_PROPERTIES;
					break;
				case TS_STREAM_PES_AUDIO_STREAM:
					iconstr = GTK_STOCK_MEDIA_PLAY;
					break;
				case TS_STREAM_PES_VIDEO_STREAM:
					iconstr = GTK_STOCK_MEDIA_PLAY;
					break;
				default:
					iconstr = GTK_STOCK_PROPERTIES;
					break;
			}
		} else if (strtspid->format == TS_STREAM_FORMAT_ENCRYPTED) {
			iconstr = GTK_STOCK_DIALOG_AUTHENTICATION;
		} else if (strtspid->format == TS_STREAM_FORMAT_VIDEO_H262) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_VIDEO_H264) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_MPEG) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_AC3) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_AAC) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_DTS) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		}
		
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, TS_COL_ICON, iconstr, TS_COL_PID, strtspid->pid, TS_COL_BANDWIDTH, strtspid->bandwidth, TS_COL_PERCENTAGE, strtspid->percentage, TS_COL_STREAMPARAMS, strtspid->description, -1);
	}
	
}

gboolean tswindow_update(gpointer data)
{
	GtkTreeModel *model;
	streamconfig_t streamconfig;
	GSList *tsstruct;
	guint streamnum;
	GtkTreeIter iter;
	gboolean valid;
	gint ident, i;
	struct _strtspid *strtspid;
	gboolean rowfound;
	GList *removelist = NULL;
	GtkTreePath *path;
	GtkTreeRowReference *reference;
	GList *node;
	
	if (!tswindow.visible) return FALSE;
	
	if ((streamconfig = gsfcore_get_streamconfig()) == NULL) return TRUE;
	
	if ((model = gtk_tree_view_get_model(GTK_TREE_VIEW(tswindow.list))) != NULL) {
		if ((tsstruct = streamconfig_get_tsstruct(streamconfig)) != NULL) {
			streamnum = g_slist_length(tsstruct);
			
			valid = gtk_tree_model_get_iter_first(model, &iter);
			while (valid) {
				gtk_tree_model_get(model, &iter, TS_COL_PID, &ident, -1);
				
				rowfound = FALSE;
				for (i=0; i<streamnum; i++) {
					strtspid = (struct _strtspid *)g_slist_nth_data(tsstruct, i);
					if (strtspid->pid == ident) {
						tswindow_edit_stream(strtspid, model, &iter);
						rowfound = TRUE;
						strtspid->sticky = TRUE;
						break;
					}
				}
				
				if (!rowfound) {
					//remove this row from list
					path = gtk_tree_model_get_path(model, &iter);
					reference = gtk_tree_row_reference_new(model, path);
					removelist = g_list_append(removelist, reference);
					gtk_tree_path_free(path);
				}
				
				valid = gtk_tree_model_iter_next(model, &iter);
			}
			
			for (node = removelist; node != NULL; node = node->next) {
				if ((path = gtk_tree_row_reference_get_path((GtkTreeRowReference *)node->data)) != NULL) {
					if (gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path)) {
						gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
					}
				}
			}
			g_list_foreach(removelist, (GFunc)gtk_tree_row_reference_free, NULL);
			g_list_free(removelist);
			
			g_slist_foreach(tsstruct, tswindow_add_stream_foreach, model);
			
			tswindow_status_refresh(tsstruct);
			
			streamconfig_free_tsstruct(tsstruct);
		}
	}
	
	return TRUE;
}

static void tswindow_selection(GtkTreeSelection *selection, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	guint streampid;
	
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, TS_COL_PID, &streampid, -1);
		if ((streampid > 32) && (streampid < 8192)) {
			gtk_widget_set_sensitive(GTK_WIDGET(tswindow.playlist_tb), TRUE);
		} else {
			gtk_widget_set_sensitive(GTK_WIDGET(tswindow.playlist_tb), FALSE);
		}
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(tswindow.playlist_tb), FALSE);
	}
	
}

gboolean tswindow_create(GtkWindow *rootwindow)
{
	GtkWidget *vbox;
	GtkWidget *image;
	GtkWidget *sw;
	
	if (!tswindow.visible) {
		tswindow.visible = TRUE;
		
		tswindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_transient_for(GTK_WINDOW(tswindow.win), GTK_WINDOW(rootwindow));
		gtk_window_set_position(GTK_WINDOW(tswindow.win), GTK_WIN_POS_CENTER_ON_PARENT);
		tswindow.title = _("Transport stream structure");
		gtk_window_set_title(GTK_WINDOW(tswindow.win), tswindow.title);
		gtk_widget_set_size_request(tswindow.win, 570, 300);
		gtk_window_set_default_size(GTK_WINDOW(tswindow.win), 570, 300);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
		#else
			vbox = gtk_vbox_new(TRUE, 0);
		#endif
		
		tswindow.toolbar = gtk_toolbar_new();
		
		gtk_toolbar_set_show_arrow(GTK_TOOLBAR(tswindow.toolbar), TRUE);
		gtk_toolbar_set_style(GTK_TOOLBAR(tswindow.toolbar), GTK_TOOLBAR_BOTH_HORIZ);
		
		image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON);
		tswindow.playlist_tb = gtk_tool_button_new(image, _("Playlist for stream"));
		gtk_tool_item_set_is_important(GTK_TOOL_ITEM(tswindow.playlist_tb), TRUE);
		
		tswindow.sep_tb = gtk_separator_tool_item_new();
		
		image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
		tswindow.close_tb = gtk_tool_button_new(image, _("Close"));
		gtk_tool_item_set_is_important(GTK_TOOL_ITEM(tswindow.close_tb), TRUE);
		
		gtk_toolbar_insert(GTK_TOOLBAR(tswindow.toolbar), tswindow.playlist_tb, 0);
		gtk_toolbar_insert(GTK_TOOLBAR(tswindow.toolbar), tswindow.sep_tb, 1);
		gtk_toolbar_insert(GTK_TOOLBAR(tswindow.toolbar), tswindow.close_tb, 2);
		
		gtk_box_pack_start(GTK_BOX(vbox), tswindow.toolbar, FALSE, FALSE, 0);
		
		sw = gtk_scrolled_window_new(NULL, NULL);
		gtk_container_set_border_width(GTK_CONTAINER(sw), 5);
		tswindow.list = gtk_tree_view_new();
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
		
		gtk_container_add(GTK_CONTAINER(sw), tswindow.list);
		
		tswindow.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tswindow.list));
		gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(tswindow.list)), GTK_SELECTION_BROWSE);
		
		tswindow_init_list(tswindow.list);
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tswindow.list), TRUE);
		
		gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
		
		tswindow.statusbar = gtk_statusbar_new();
		
		gtk_box_pack_start(GTK_BOX(vbox), tswindow.statusbar, FALSE, FALSE, 5);
		
		g_signal_connect(G_OBJECT(tswindow.win), "destroy", G_CALLBACK(tswindow_close), NULL);
		g_signal_connect(G_OBJECT(tswindow.close_tb), "clicked", G_CALLBACK(tswindow_close), NULL);
		g_signal_connect(G_OBJECT(tswindow.playlist_tb), "clicked", G_CALLBACK(tswindow_playlist), NULL);
		g_signal_connect(G_OBJECT(tswindow.selection), "changed", G_CALLBACK(tswindow_selection), NULL);
		
		gtk_container_add(GTK_CONTAINER(tswindow.win), vbox);
		gtk_widget_show_all(tswindow.win);
		
		gtk_widget_set_sensitive(GTK_WIDGET(tswindow.playlist_tb), FALSE);
		
		tswindow_update(NULL);
		tswindow.timer = g_timeout_add(1000, tswindow_update, NULL);
	} else {
		gtk_window_present(GTK_WINDOW(tswindow.win));
	}
	
	return TRUE;
}
