/*
 *      ttwindow.c
 *      
 *      Copyright 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>
#include <time.h>

#include "teletext.h"
#include "gsfcore.h"
#include "dbstorage.h"
#include "settings.h"

enum {
	TT_PAGE_PIXBUF,
	TT_PAGE_NAME,
	TT_PAGE_ID,
	TT_PAGE_COLS
};

struct _ttwindow {
	//teletext window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *toolbar;
	GtkToolItem *settings_tb, *markup_tb, *sep_tb, *close_tb;
	GtkWidget *pagesview;
	GtkListStore *pagesstore;
	GtkTreeSortable *pagessortable;
	GtkWidget *pagetext;
	GtkTextTag *deffonttag, *sizetag, *flashtag;
	GtkTextTag *fgcolormap[16];
	GtkTextTag *bgcolormap[16];
	guint statusbarcontext;
	GtkWidget *statusbar;
	GtkWidget *parsingpage;
	gboolean markupenabled;
	guint32 currentpageid;
	gint deflanguage;
	gboolean detectlang;
	gulong listcursignal;
	ttparser_t parser;
};
struct _ttwindow ttwindow;

static gboolean ttwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
static void ttwindow_settings_dialog(GtkToolButton *toolbutton, gpointer data);
static void ttwindow_markup_toggle(GtkToggleToolButton *toggle_tool_button, gpointer user_data);
static void ttwindow_close_toolbar(GtkToolButton *toolbutton, gpointer data);
static void tt_colour_to_string(ttcolour_t colour, gchar *string, gsize length);
static void tt_system_message(gchar *message);
static void tt_page_place_tags(gpointer data, gpointer user_data);
static void tt_page_open(guint32 pageid);
static void tt_page_selected(GtkTreeView *tree_view, gpointer data);
static gint tt_sort_iter_compare_func(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer userdata);
static GtkWidget *ttwindow_create_pages_list(void);
static GtkWidget *ttwindow_create_text_view(void);
static gboolean tt_parser_page_add(gpointer data);
static gboolean tt_parser_page_refresh(gpointer data);
static gboolean tt_parser_page_parsing(gpointer data);
static gboolean tt_network_event(gpointer data);
static void tt_stop_event_response(GtkDialog *dialog, gint response_id, gpointer data);
static gboolean tt_stop_event(gpointer data);
static void tt_parser_event_cb(gint event, gpointer data);


static gboolean ttwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	if (ttwindow.win) {
		if (g_signal_handler_is_connected(G_OBJECT(ttwindow.pagesview), ttwindow.listcursignal)) {
			g_signal_handler_disconnect(G_OBJECT(ttwindow.pagesview), ttwindow.listcursignal);
		}
		tt_parser_close(ttwindow.parser);
		gtk_widget_destroy(ttwindow.win);
		ttwindow.visible = FALSE;
	}
	
	return FALSE;
}

static void ttwindow_settings_dialog(GtkToolButton *toolbutton, gpointer data)
{
	GtkWidget *dialog, *table, *deflang, *detectlang_cb;
	GtkWidget *dialogarea;
	GtkWidget *deflang_l, *align;
	gint res;
		
	dialog = gtk_dialog_new_with_buttons(_("Teletext settings"), 
										GTK_WINDOW(ttwindow.win),
										GTK_DIALOG_DESTROY_WITH_PARENT,
										GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
										GTK_STOCK_APPLY, GTK_RESPONSE_OK,
										NULL);
	
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
	
	table = gtk_table_new(2, 3, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
	
	deflang_l = gtk_label_new(_("Default language:"));
	align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), deflang_l);
	gtk_table_attach_defaults(GTK_TABLE(table), align, 0, 1, 0, 1);
	
	#if GTK_CHECK_VERSION(2, 24, 0)
		deflang = gtk_combo_box_text_new();
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "English");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "French");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Swedish");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Finnish");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Hungarian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Czech");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Slovak");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "German");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Portuguese");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Spanish");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Italian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Polish");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Turkish");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Serbian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Croatian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Serbian latin");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Croatian latin");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Slovenian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Russian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Bulgarian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Estonian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Ukrainian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Lettish");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Lithuanian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Rumanian");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Greek");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Arabic");
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(deflang), "Hebrew");
	#else
		deflang = gtk_combo_box_new_text();
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "English");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "French");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Swedish");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Finnish");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Hungarian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Czech");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Slovak");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "German");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Portuguese");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Spanish");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Italian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Polish");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Turkish");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Serbian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Croatian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Serbian latin");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Croatian latin");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Slovenian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Russian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Bulgarian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Estonian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Ukrainian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Lettish");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Lithuanian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Rumanian");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Greek");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Arabic");
		gtk_combo_box_append_text(GTK_COMBO_BOX(deflang), "Hebrew");
	#endif
		
	gtk_table_attach(GTK_TABLE(table), deflang, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
	
	detectlang_cb = gtk_check_button_new_with_label(_("Detect page language"));
	gtk_table_attach(GTK_TABLE(table), detectlang_cb, 0, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
	
	dialogarea = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
	
	gtk_container_add(GTK_CONTAINER(dialogarea), table);
	gtk_widget_show_all(dialog);
	
	tt_parser_get_params(ttwindow.parser, &ttwindow.deflanguage, &ttwindow.detectlang);
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(deflang), ttwindow.deflanguage);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(detectlang_cb), ttwindow.detectlang);
	
	res = gtk_dialog_run(GTK_DIALOG(dialog));
	
	if (res == GTK_RESPONSE_OK) {
		ttwindow.deflanguage = gtk_combo_box_get_active(GTK_COMBO_BOX(deflang));
		ttwindow.detectlang = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(detectlang_cb));
		
		settings_append_int("tt_language_id", ttwindow.deflanguage);
		settings_append_boolean("tt_language_detect", ttwindow.detectlang);
		
		tt_parser_set_params(ttwindow.parser, ttwindow.deflanguage, ttwindow.detectlang);
	}
	
	gtk_widget_destroy(dialog);
}

static void ttwindow_markup_toggle(GtkToggleToolButton *toggle_tool_button, gpointer user_data)
{
	ttwindow.markupenabled = gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(ttwindow.markup_tb));
	
	settings_append_boolean("tt_markup_enabled", ttwindow.markupenabled);
	
	if (ttwindow.currentpageid != 0) {
		tt_page_open(ttwindow.currentpageid);
	}
}

static void ttwindow_close_toolbar(GtkToolButton *toolbutton, gpointer data)
{
	g_signal_emit_by_name(ttwindow.win, "destroy", NULL);
}

static void tt_colour_to_string(ttcolour_t colour, gchar *string, gsize length)
{
	gint s, r, g, b;
	
	r = colour->r == 0 ? 0 : colour->r*16-1;
	g = colour->g == 0 ? 0 : colour->g*16-1;
	b = colour->b == 0 ? 0 : colour->b*16-1;
	
	g_sprintf(string, "#%2x%2x%2x", r, g, b);
	
	s = 0;
	
	while ((string[s] != '\0') && (s < length)) {
		if (string[s] == ' ') {
			string[s] = '0';
		}
		s++;
	}
}

static void tt_system_message(gchar *message)
{
	GtkTextBuffer *buffer;
	GdkPixbuf *icon;
	GError *error; 
	GtkTextIter start, end;
	
	error = NULL;
	
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ttwindow.pagetext));
	
	gtk_text_buffer_get_bounds(buffer, &start, &end);
	gtk_text_buffer_delete(buffer, &start, &end);
	
	icon = gdk_pixbuf_new_from_file("teletext.png", &error);
	
	if (icon != NULL) {
		gtk_text_buffer_get_end_iter(buffer, &end);
		gtk_text_buffer_insert_pixbuf(buffer, &end, icon);
		g_object_unref(G_OBJECT(icon));
	}
	
	gtk_text_buffer_get_end_iter(buffer, &end);
	gtk_text_buffer_insert(buffer, &end, "\n", 1);
	
	gtk_text_buffer_get_end_iter(buffer, &end);
	gtk_text_buffer_insert(buffer, &end, message, -1);
}

static void tt_page_place_tags(gpointer data, gpointer user_data)
{
	tttag_t tag;
	guint length;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	guint colourid;
	
	tag = (tttag_t)data;
	length = *(guint *)user_data;
	
	if ((tag->start > length) || (tag->end > length)) return;
	
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ttwindow.pagetext));
	
	gtk_text_buffer_get_iter_at_offset(buffer, &start, tag->start);
	gtk_text_buffer_get_iter_at_offset(buffer, &end, tag->end);
	
	if ((tag->tag == TT_TAG_COLOUR) || (tag->tag == TT_TAG_BG_COLOUR)) {
		switch (tag->value) {
			case TT_COLOUR_WHITE:
				colourid = 0;
				break;
			case TT_COLOUR_RED:
				colourid = 1;
				break;
			case TT_COLOUR_GREEN:
				colourid = 2;
				break;
			case TT_COLOUR_YELLOW:
				colourid = 3;
				break;
			case TT_COLOUR_BLUE:
				colourid = 4;
				break;
			case TT_COLOUR_MAGENTA:
				colourid = 5;
				break;
			case TT_COLOUR_CYAN:
				colourid = 6;
				break;
			case TT_COLOUR_BLACK:
				colourid = 7;
				break;
			default:
				colourid = 0;
				break;
		}
			
		if (tag->tag == TT_TAG_COLOUR) {
			gtk_text_buffer_apply_tag(buffer, ttwindow.fgcolormap[colourid], &start, &end);
		} else {
			gtk_text_buffer_apply_tag(buffer, ttwindow.bgcolormap[colourid], &start, &end);
		}
	} else if (tag->tag == TT_TAG_SIZE) {
		gtk_text_buffer_apply_tag(buffer, ttwindow.sizetag, &start, &end);
	} else if (tag->tag == TT_TAG_FLASH) {
		gtk_text_buffer_apply_tag(buffer, ttwindow.flashtag, &start, &end);
	} else {
		printf("TAG: %u, %u, %u, %u\n", tag->tag, tag->value, tag->start, tag->end);
	}
	
}

static void tt_page_open(guint32 pageid)
{
	gchar *pagetext;
	guint length;
	GSList *pagetags;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	gint i;
	ttcolour_t  colours;
	gchar colourstr[8];
	
	
	if (ttwindow.parser == NULL) {
		tt_system_message(_("Data source not found"));
		return;
	}
	
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ttwindow.pagetext));
		
	pagetext = tt_page_get_string(ttwindow.parser, pageid, &length, &pagetags);
	
	if (pagetext != NULL) {
		gtk_text_buffer_get_bounds(buffer, &start, &end);
		gtk_text_buffer_delete(buffer, &start, &end);
		
		gtk_text_buffer_insert_with_tags(buffer, &start, pagetext, length, ttwindow.deffonttag, NULL);
		
		if (ttwindow.markupenabled) {
			colours = tt_page_get_colormap(ttwindow.parser, pageid);
			
			if (colours != NULL) {
				for (i=0; i<16; i++) {
					tt_colour_to_string(&colours[i], (gchar *)&colourstr, sizeof(colourstr));
					g_object_set_data(G_OBJECT(ttwindow.fgcolormap[i]), "foreground", colourstr);
					g_object_set_data(G_OBJECT(ttwindow.bgcolormap[i]), "background", colourstr);
				}
				g_free(colours);
			}
			
			if (pagetags != NULL) g_slist_foreach(pagetags, tt_page_place_tags, &length);
		}
		
		if (pagetags != NULL) g_slist_free(pagetags);
		
		g_free(pagetext);
	} else {
		tt_system_message(_("Page not found in cache"));
	}
} 

static void tt_page_selected(GtkTreeView *tree_view, gpointer data)
{
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint pageid;
		
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, TT_PAGE_ID, &pageid, -1);
		ttwindow.currentpageid = pageid;
		tt_page_open(pageid);
	}
}

static gint tt_sort_iter_compare_func(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer userdata)
{
	gint ret;
	guint page1, page2;
	
	ret = 0;
	
	gtk_tree_model_get(model, a, TT_PAGE_ID, &page1, -1);
	gtk_tree_model_get(model, b, TT_PAGE_ID, &page2, -1);
	
	if (page1 > page2) {
		ret = 1;
	} else if (page1 < page2) {
		ret = -1;
	} else {
		ret = 0;
	}
	
	return ret;
}

static GtkWidget *ttwindow_create_pages_list(void)
{
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column1, *column2;
	GtkListStore *store;
	GtkTreeIter iter;
	GtkTreeSortable *sortable;
	
	
	ttwindow.pagesview = gtk_tree_view_new();
	
	renderer = gtk_cell_renderer_pixbuf_new();
	column1 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column1, NULL);
	gtk_tree_view_column_pack_start(column1, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column1, renderer, "stock-id", TT_PAGE_PIXBUF, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(ttwindow.pagesview), column1);
	
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, "ellipsize-set", TRUE, NULL);
	column2 = gtk_tree_view_column_new_with_attributes("Page", renderer, "markup", TT_PAGE_NAME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(ttwindow.pagesview), column2);
		
	ttwindow.pagesstore = gtk_list_store_new(TT_PAGE_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(ttwindow.pagesview), GTK_TREE_MODEL(ttwindow.pagesstore));
	
	g_object_unref(ttwindow.pagesstore);
	
	ttwindow.pagessortable = GTK_TREE_SORTABLE(ttwindow.pagesstore);
	
	gtk_tree_sortable_set_sort_func(ttwindow.pagessortable, TT_PAGE_ID, tt_sort_iter_compare_func, NULL, NULL);
	
	/* set initial sort order */
	gtk_tree_sortable_set_sort_column_id(ttwindow.pagessortable, TT_PAGE_ID, GTK_SORT_ASCENDING);
	
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(ttwindow.pagesview), FALSE);
	
	ttwindow.listcursignal = g_signal_connect(G_OBJECT(ttwindow.pagesview), "cursor-changed", G_CALLBACK(tt_page_selected), NULL);
	
	return ttwindow.pagesview;
}

static GtkWidget *ttwindow_create_text_view(void)
{
	GtkTextBuffer *buffer;
	//GtkTextTag *pagefonttag;
	GtkTextIter iter;
	ttcolour_t  colours;
	gchar tagname[12];
	gchar colourstr[8];
	gint i;
	
	ttwindow.pagetext = gtk_text_view_new();
	
	gtk_text_view_set_editable(GTK_TEXT_VIEW(ttwindow.pagetext), FALSE);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(ttwindow.pagetext), FALSE);
	gtk_text_view_set_justification(GTK_TEXT_VIEW(ttwindow.pagetext), GTK_JUSTIFY_CENTER);
		
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ttwindow.pagetext));
	
	ttwindow.deffonttag = gtk_text_buffer_create_tag(buffer, "deffont", "font", "Monospace 10", "justification", GTK_JUSTIFY_LEFT, "wrap-mode", GTK_WRAP_NONE, NULL);
	ttwindow.sizetag = gtk_text_buffer_create_tag(buffer, "size", "weight", PANGO_WEIGHT_BOLD, NULL);
	ttwindow.flashtag = gtk_text_buffer_create_tag(buffer, "flash", "style", PANGO_STYLE_ITALIC, NULL);
	
	colours = tt_page_get_colormap(ttwindow.parser, 0);
	
	if (colours != NULL) {
		for (i=0; i<16; i++) {
			tt_colour_to_string(&colours[i], (gchar *)&colourstr, sizeof(colourstr));
			g_sprintf(tagname, "fgcolour_%u", i);
			ttwindow.fgcolormap[i] = gtk_text_buffer_create_tag(buffer, tagname, "foreground", colourstr, NULL);
			g_sprintf(tagname, "bgcolour_%u", i);
			ttwindow.bgcolormap[i] = gtk_text_buffer_create_tag(buffer, tagname, "background", colourstr, NULL);
		}
		g_free(colours);
	}
	
	tt_system_message(_("Waiting for initial page"));
	
	return ttwindow.pagetext;
}

static gboolean tt_parser_page_add(gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	gchar pagelabel[10];
	gushort pagenum, subpagenum;
	GtkTreeSelection *selection;
	ttpage_t page;
	
	if (data == NULL) return FALSE;
	
	page = (ttpage_t)data;
	
	pagenum = tt_get_pagenum(page->pageid);
	subpagenum = tt_get_subpagenum(page->pageid);
	
	if (subpagenum == 0) {
		g_sprintf(pagelabel, "%u", pagenum);
	} else {
		g_sprintf(pagelabel, "%u/%u", pagenum, subpagenum);
	}
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(ttwindow.pagesview));
	
	if (model != NULL) {
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, TT_PAGE_PIXBUF, GTK_STOCK_FILE, TT_PAGE_NAME, pagelabel, TT_PAGE_ID, page->pageid, -1);
		
		if (ttwindow.currentpageid == 0) {
			if (pagenum == ttwindow.parser->initialpage) {
				if ((ttwindow.parser->initialsubpage == 0) || (subpagenum == ttwindow.parser->initialsubpage)) {
					selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ttwindow.pagesview));
					if (selection != NULL) {
						gtk_tree_selection_select_iter(selection, &iter);
						ttwindow.currentpageid = page->pageid;
						tt_page_open(page->pageid);
					}
				}
			}
		}
		
	}
	
	return FALSE;
}

static gboolean tt_parser_page_refresh(gpointer data)
{
	ttpage_t page;
	
	if (data == NULL) return FALSE;
	
	page = (ttpage_t)data;
	
	if (page->pageid == ttwindow.currentpageid) {
		tt_page_open(page->pageid);
	}
	
	return FALSE;
}

static gboolean tt_parser_page_parsing(gpointer data)
{
	gushort pagenum, subpagenum;
	gchar pagelabel[10];
	ttpage_t page;
	
	if (data == NULL) return FALSE;
	
	page = (ttpage_t)data;
	
	pagenum = tt_get_pagenum(page->pageid);
	subpagenum = tt_get_subpagenum(page->pageid);
	
	if (subpagenum == 0) {
		g_sprintf(pagelabel, "%u", pagenum);
	} else {
		g_sprintf(pagelabel, "%u/%u", pagenum, subpagenum);
	}
	
	gtk_label_set_text(GTK_LABEL(ttwindow.parsingpage), pagelabel); 
	
	return FALSE;
}

static gboolean tt_network_event(gpointer data)
{
	ttparser_t parser;
	gchar timestr[32];
	gchar statusstr[64];
	time_t gmttime;
	struct tm *timestruct;
	gint sym;
	
	if (data == NULL) return FALSE;
	
	parser = (ttparser_t)data;
	
	gmttime = parser->svctime;
	timestruct = gmtime(&gmttime);
	
	if (timestruct == NULL) return FALSE;
	
	if (strftime(timestr, sizeof(timestr), "%H:%M:%S %d-%m-%Y", timestruct) == 0) {
		sym = g_sprintf(statusstr, "%s", parser->networkname);
	} else {
		sym = g_sprintf(statusstr, "%s %s", parser->networkname, timestr);
	}
	statusstr[sym] = '\0';
	
	gtk_statusbar_pop(GTK_STATUSBAR(ttwindow.statusbar), ttwindow.statusbarcontext);
	gtk_statusbar_push(GTK_STATUSBAR(ttwindow.statusbar), ttwindow.statusbarcontext, statusstr);
	
	return FALSE;
}

static void tt_stop_event_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog));
	
	if (response_id == GTK_RESPONSE_CLOSE) {
		g_signal_emit_by_name(ttwindow.win, "destroy", NULL);
	}
}

static gboolean tt_stop_event(gpointer data)
{
	GtkWidget *dialog;
	ttparser_t parser;
	gchar *errorstring;
	
	parser = (ttparser_t)data;
	
	if (parser != NULL) {
		switch (parser->error) {
			case TT_ERROR_TIMEOUT:
				errorstring = _("Error: timeout after multiple data read attempts");
				break;
			case TT_ERROR_NO_DATA:
				errorstring = _("Error: data stream does not contain teletext");
				break;
			case TT_ERROR_NULL:
			default:
				errorstring = _("Error while reading teletext");
				break;
		}
	} else {
		errorstring = _("Error: Teletext PID not found");
	}
		
	
	dialog = gtk_message_dialog_new(GTK_WINDOW(ttwindow.win),
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
									errorstring,
									NULL);
	
	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(tt_stop_event_response), NULL);
	gtk_widget_show(dialog);
	
	return FALSE;
}

static void tt_parser_event_cb(gint event, gpointer data)
{
	ttparser_t parser;
	ttpage_t page;
	
	switch (event) {
		case TT_EVENT_PAGE_NEW:
			page = (ttpage_t)data;
			g_idle_add(tt_parser_page_add, page);
			break;
		case TT_EVENT_PAGE_REFRESH:
			page = (ttpage_t)data;
			g_idle_add(tt_parser_page_refresh, page);
			break;
		case TT_EVENT_PAGE_PARSING:
			page = (ttpage_t)data;
			g_idle_add(tt_parser_page_parsing, page);
			break;
		case TT_EVENT_NETWORK:
			parser = (ttparser_t)data;
			g_idle_add(tt_network_event, parser);
			break;
		case TT_EVENT_STOP:
			parser = (ttparser_t)data;
			g_idle_add(tt_stop_event, parser);
			break;
		default:
			
			break;
	}
}

void ttwindow_create(GtkWindow *rootwindow)
{
	GtkWidget *vbox;
    GtkWidget *paned;
    GtkWidget *frame1, *frame2;
    GtkWidget *pagesscroll;
    GtkWidget *pagesview;
    GtkWidget *textscroll;
    GtkWidget *pagetext;
    GtkWidget *separator;
    frontend_t frontend;
    gint language;
	gboolean langdetect;
    gushort ttpid;
    guint stream[3];
		
	if (!ttwindow.visible) {
		ttwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_transient_for(GTK_WINDOW(ttwindow.win), GTK_WINDOW(rootwindow));
		gtk_window_set_position(GTK_WINDOW(ttwindow.win), GTK_WIN_POS_CENTER_ON_PARENT);
		gtk_window_set_title(GTK_WINDOW(ttwindow.win), _("Teletext"));
		gtk_widget_set_size_request(ttwindow.win, 500, 465);
		gtk_window_set_default_size(GTK_WINDOW(ttwindow.win), 500, 465);
		
		g_signal_connect(ttwindow.win, "destroy", G_CALLBACK(ttwindow_close), NULL);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
		#else
			vbox = gtk_vbox_new(TRUE, 0);
		#endif
		
		ttwindow.markupenabled = settings_read_boolean("tt_markup_enabled", TRUE);
		
		ttwindow.toolbar = gtk_toolbar_new();
		
		gtk_toolbar_set_show_arrow(GTK_TOOLBAR(ttwindow.toolbar), TRUE);
		gtk_toolbar_set_style(GTK_TOOLBAR(ttwindow.toolbar), GTK_TOOLBAR_BOTH_HORIZ);
		
		ttwindow.settings_tb = gtk_tool_button_new_from_stock(GTK_STOCK_PREFERENCES);
		gtk_widget_set_tooltip_text(GTK_WIDGET(ttwindow.settings_tb), _("Set language"));
		g_object_set(ttwindow.settings_tb, "is-important", TRUE, NULL);
		ttwindow.markup_tb = gtk_toggle_tool_button_new_from_stock(GTK_STOCK_SELECT_COLOR);
		gtk_widget_set_tooltip_text(GTK_WIDGET(ttwindow.markup_tb), _("Enable/Disable colored text"));
		g_object_set(ttwindow.markup_tb, "is-important", TRUE, NULL);
		ttwindow.sep_tb = gtk_separator_tool_item_new();
		ttwindow.close_tb = gtk_toggle_tool_button_new_from_stock(GTK_STOCK_CLOSE);
		gtk_widget_set_tooltip_text(GTK_WIDGET(ttwindow.close_tb), _("Close teletext window"));
		g_object_set(ttwindow.close_tb, "is-important", TRUE, NULL);
		
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ttwindow.markup_tb), ttwindow.markupenabled);
		
		gtk_toolbar_insert(GTK_TOOLBAR(ttwindow.toolbar), ttwindow.settings_tb, 0);
		gtk_toolbar_insert(GTK_TOOLBAR(ttwindow.toolbar), ttwindow.markup_tb, 1);
		gtk_toolbar_insert(GTK_TOOLBAR(ttwindow.toolbar), ttwindow.sep_tb, 2);
		gtk_toolbar_insert(GTK_TOOLBAR(ttwindow.toolbar), ttwindow.close_tb, 3);
		
		g_signal_connect_swapped(G_OBJECT(ttwindow.settings_tb), "clicked", G_CALLBACK(ttwindow_settings_dialog), NULL);
		g_signal_connect_swapped(G_OBJECT(ttwindow.markup_tb), "toggled", G_CALLBACK(ttwindow_markup_toggle), NULL);
		g_signal_connect_swapped(G_OBJECT(ttwindow.close_tb), "clicked", G_CALLBACK(ttwindow_close_toolbar), NULL);
		
		gtk_box_pack_start(GTK_BOX(vbox), ttwindow.toolbar, FALSE, FALSE, 0);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			paned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
		#else
			paned = gtk_hpaned_new();
		#endif
		
		pagesview = ttwindow_create_pages_list();
		
		pagesscroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pagesscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(pagesscroll), GTK_SHADOW_IN);
		gtk_container_add(GTK_CONTAINER(pagesscroll), pagesview);
		
		gtk_widget_set_size_request(pagesscroll, 150, -1);
		gtk_paned_add1(GTK_PANED(paned), pagesscroll);
		
		pagetext = ttwindow_create_text_view();
		
		textscroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(textscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(textscroll), GTK_SHADOW_IN);
		gtk_container_add(GTK_CONTAINER(textscroll), pagetext);
		
		gtk_paned_add2(GTK_PANED(paned), textscroll);
				
		gtk_box_pack_start(GTK_BOX(vbox), paned, TRUE, TRUE, 0);
		
		ttwindow.statusbar = gtk_statusbar_new();
		
		ttwindow.statusbarcontext = gtk_statusbar_get_context_id(GTK_STATUSBAR(ttwindow.statusbar), "network_events");
		
		ttwindow.parsingpage = gtk_label_new(NULL);
		
		#if GTK_CHECK_VERSION(3, 0, 0)
			separator = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
		#else
			separator = gtk_vseparator_new();
		#endif
			
		gtk_box_pack_start(GTK_BOX(ttwindow.statusbar), separator, FALSE, TRUE, 5);
		
		gtk_box_pack_start(GTK_BOX(ttwindow.statusbar), ttwindow.parsingpage, FALSE, TRUE, 5);
		
		gtk_box_pack_start(GTK_BOX(vbox), ttwindow.statusbar, FALSE, FALSE, 0);
		
		gtk_container_add(GTK_CONTAINER(ttwindow.win), vbox);
		
		gtk_widget_show_all(ttwindow.win);
		
		ttwindow.visible = TRUE;
		
		ttwindow.currentpageid = 0;
		
		frontend = gsfcore_get_frontend();
		
		if ((frontend != NULL) && (gsfcore_service_has_pid_type(PID_TYPE_TELETEXT, &ttpid))) {
			stream[0] = frontend->adapter_nr;
			stream[1] = frontend->frontend_nr;
			stream[2] = ttpid;
			
			language = settings_read_int("tt_language_id", TT_LANGUAGE_RUSSIAN);
			langdetect = settings_read_boolean("tt_language_detect", TRUE);
			
			ttwindow.parser = tt_parser_new(language, langdetect, stream, tt_parser_event_cb);
		} else {
			g_idle_add(tt_stop_event, NULL);
		}
	
	} else {
	
		 gtk_window_present(GTK_WINDOW(ttwindow.win));
	
	}
}
