/*
 *      udpintctl.h
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "udpintctl.h"

float udpintctl_calc(struct _rtt *rtt)
{
	return rtt->smooth_rtt + (4.0 * rtt->variable_rtt);
}

float udpintctl_round(float rtt)
{
	if (rtt > MAX_RETS_TIME) rtt = MAX_RETS_TIME; else
	if (rtt < MAX_RETS_TIME) rtt = MIN_RETS_TIME;
	
	return rtt;
}

void udpintctl_new(struct _rtt *rtt)	
{
	struct timeval tv;
	
	gettimeofday(&tv, NULL);
	rtt->unixtime = tv.tv_sec;
	rtt->last_rtt = 0;
	rtt->smooth_rtt = 0;
	rtt->variable_rtt = 0.75;
	rtt->present_rto = udpintctl_round(udpintctl_calc(rtt));
}

time_t udpintctl_timestamp(struct _rtt *rtt)
{
	time_t timestamp;
	struct timeval tv;
	
	gettimeofday(&tv, NULL);
	timestamp = ((tv.tv_sec - rtt->unixtime) * 1000) + (tv.tv_usec / 1000);
	
	return timestamp;
}

void udpintctl_packet_new(struct _rtt *rtt)	
{
	rtt->transmissions = 0;
}

int udpintctl_packet_start(struct _rtt *rtt)	
{
	return ((int)(rtt->present_rto + 0.5));
}

void udpintctl_packet_stop(struct _rtt *rtt, unsigned int ms)	
{
	double delta;
	
	rtt->last_rtt = ms / 1000.0;
	delta = rtt->last_rtt - rtt->smooth_rtt;
	rtt->smooth_rtt += delta / 8;
	if (delta < 0.0) delta = -delta;
	rtt->variable_rtt += (delta - rtt->variable_rtt) / 4;
	rtt->present_rto = udpintctl_round(udpintctl_calc(rtt));
}

int udpintctl_timeout(struct _rtt *rtt)
{
	rtt->present_rto *= 2;
	
	if (++rtt->transmissions > MAX_TS_ATTEMPTS) return -1;
	return 0;
}

